/* USMID @(#)libscs/include/scsl_cblas.h	11.1	07/09/98 10:51:17 */
/*
 *      (C) COPYRIGHT SILICON GRAPHICS, INC.
 *      UNPUBLISHED PROPRIETARY INFORMATION.
 *      ALL RIGHTS RESERVED.
 */
#ifndef CBLAS_H
#define CBLAS_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define  NOTRANSPOSE  0
#define  TRANSPOSE    1
#define  CONJTRANS    2
#define  UPPER        0
#define  LOWER        1
#define  UNIT         0
#define  NONUNIT      1
#define  LEFT         0
#define  RIGHT        1

typedef struct { float real; float imag; } CBLAS_COMPLEX;
typedef struct { double real; double imag; } CBLAS_ZOMPLEX;

/*
 * Enumerated and derived types
 */
#define CBLAS_INDEX size_t  /* this may vary between platforms */

enum CBLAS_ORDER {CblasRowMajor=101, CblasColMajor=102};
enum CBLAS_TRANSPOSE {CblasNoTrans=111, CblasTrans=112, CblasConjTrans=113};
enum CBLAS_UPLO {CblasUpper=121, CblasLower=122};
enum CBLAS_DIAG {CblasNonUnit=131, CblasUnit=132};
enum CBLAS_SIDE {CblasLeft=141, CblasRight=142};
/*
 * ===========================================================================
 * Prototypes for level 1 BLAS functions (complex are recast as routines)
 * ===========================================================================
 */
static __inline float cblas_sdot(const int N, const float  *X, const int incX,
                const float  *Y, const int incY) {
		float sdot_();
   		return sdot_(&N, X, &incX, Y, &incY);
}
static __inline double cblas_ddot(const int N, const double *X, const int incX,
                const double *Y, const int incY) {
		double ddot_();
   		return ddot_(&N, X, &incX, Y, &incY);
}

/*
 * Functions having prefixes Z and C only
*/
static __inline CBLAS_COMPLEX cblas_cdotu(const int N, const CBLAS_COMPLEX *X, 
		const int incX, const CBLAS_COMPLEX *Y, const int incY) {
		CBLAS_COMPLEX cdotu_();
		return cdotu_(&N, X, &incX, Y, &incY);
}
static __inline CBLAS_COMPLEX cblas_cdotc(const int N, const CBLAS_COMPLEX *X, 
		const int incX, const CBLAS_COMPLEX *Y, const int incY) {
		CBLAS_COMPLEX cdotc_();
		return cdotc_(&N, X, &incX, Y, &incY);
}
static __inline CBLAS_ZOMPLEX cblas_zdotu(const int N, const CBLAS_ZOMPLEX *X, 
		const int incX, const CBLAS_ZOMPLEX *Y, const int incY) {
		CBLAS_ZOMPLEX zdotu_();
		return zdotu_(&N, X, &incX, Y, &incY);
}
static __inline CBLAS_ZOMPLEX cblas_zdotc(const int N, const CBLAS_ZOMPLEX *X, 
		const int incX, const CBLAS_ZOMPLEX *Y, const int incY) {
		CBLAS_ZOMPLEX zdotc_();
		return zdotc_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_cdotu_sub(const int N, const void *X, const int incX,
                       const void *Y, const int incY, CBLAS_COMPLEX *dotu) {
		CBLAS_COMPLEX cdotu_();
		*dotu = cdotu_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_cdotc_sub(const int N, const void *X, const int incX,
                       const void *Y, const int incY, CBLAS_COMPLEX *dotc) {
		CBLAS_COMPLEX cdotc_();
		*dotc = cdotc_(&N, X, &incX, Y, &incY);
}

static __inline void cblas_zdotu_sub(const int N, const void *X, const int incX,
                       const void *Y, const int incY, CBLAS_ZOMPLEX *dotu) {
		CBLAS_ZOMPLEX zdotu_();
		*dotu = zdotu_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_zdotc_sub(const int N, const void *X, const int incX,
                       const void *Y, const int incY, CBLAS_ZOMPLEX *dotc) {
		CBLAS_ZOMPLEX zdotc_();
		*dotc = zdotc_(&N, X, &incX, Y, &incY);
}
/*
 * Functions having prefixes S D SC DZ
 */
static __inline float cblas_snrm2(const int N, const float  *X, const int incX){
		float snrm2_();
   		return snrm2_(&N, X, &incX);
}
static __inline float cblas_sasum(const int N, const float  *X, const int incX){
		float sasum_();
   		return sasum_(&N, X, &incX);
}
static __inline float cblas_ssum(const int N, const float  *X, const int incX){
		float ssum_();
   		return ssum_(&N, X, &incX);
}
static __inline double cblas_dnrm2(const int N, const double  *X, 
		const int incX) {
		double dnrm2_();
                return dnrm2_(&N, X, &incX);
}
static __inline double cblas_dasum(const int N, const double  *X, 
		const int incX) {
		double dasum_();
   		return dasum_(&N, X, &incX);
}
static __inline double cblas_dsum(const int N, const double  *X, 
		const int incX) {
		double dsum_();
   		return dsum_(&N, X, &incX);
}
static __inline float cblas_scnrm2(const int N, const void *X, 
		const int incX) {
		float scnrm2_();
   		return scnrm2_(&N, X, &incX);
}
static __inline float cblas_scasum(const int N, const void *X, 
		const int incX) {
		float scasum_();
   		return scasum_(&N, X, &incX);
}

static __inline float cblas_dznrm2(const int N, const void *X,
		const int incX) {
		float dznrm2_();
   		return dznrm2_(&N, X, &incX);
}
static __inline float cblas_dzasum(const int N, const void *X,
		const int incX) {
		float dzasum_();
   		return dzasum_(&N, X, &incX);
}
/*
 * Functions having standard 4 prefixes (S D C Z)
 */
static __inline CBLAS_INDEX cblas_isamax(const int N, const float  *X, 
		const int incX) {
   		CBLAS_INDEX isamax_();
   		return (isamax_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_ismax(const int N, const float  *X, 
		const int incX) {
   		CBLAS_INDEX ismax_();
   		return (ismax_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_isamin(const int N, const float  *X, 
		const int incX) {
   		CBLAS_INDEX isamin_();
   		return (isamin_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_ismin(const int N, const float  *X, 
		const int incX) {
   		CBLAS_INDEX ismin_();
   		return (ismin_(&N, X, &incX) - 1);
}

static __inline CBLAS_INDEX cblas_idamax(const int N, const double  *X, 
		const int incX) {
   		CBLAS_INDEX idamax_();
   		return (idamax_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_idmax(const int N, const double  *X, 
		const int incX) {
   		CBLAS_INDEX idmax_();
   		return (idmax_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_idamin(const int N, const double  *X, 
		const int incX) {
   		CBLAS_INDEX idamin_();
   		return (idamin_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_idmin(const int N, const double  *X, 
		const int incX) {
   		CBLAS_INDEX idmin_();
   		return (idmin_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_icamax(const int N, const void *X, 
		const int incX) {
		CBLAS_INDEX icamax_();
   		return (icamax_(&N, X, &incX) - 1);
}
static __inline CBLAS_INDEX cblas_izamax(const int N, const void *X, 
		const int incX) {
		CBLAS_INDEX izamax_();
   		return (izamax_(&N, X, &incX) - 1);
}
/*
 * ===========================================================================
 * Prototypes for level 1 BLAS routines
 * ===========================================================================
 */
/*
 * Routines with standard 4 prefixes (s, d, c, z)
 */
static __inline void cblas_sswap(const int N, float *X, const int incX,
                float *Y, const int incY) {
		sswap_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_scopy(const int N, const float *X, const int incX,
                float *Y, const int incY) {
		scopy_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_saxpy(const int N, const float alpha, const float *X,
                const int incX, float *Y, const int incY) {
		saxpy_( &N, &alpha, X, &incX, Y, &incY );
}
static __inline void cblas_saxpby(const int N, const float alpha, 
		const float *X, const int incX, const float beta, float *Y, 
		const int incY) {
		saxpby_( &N, &alpha, X, &incX, &beta, Y, &incY );
}
static __inline void cblas_dswap(const int N, double *X, const int incX,
                double *Y, const int incY) {
   		dswap_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_dcopy(const int N, const double *X, const int incX,
                double *Y, const int incY) {
   		dcopy_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_daxpy(const int N, const double alpha, 
		const double *X,
                const int incX, double *Y, const int incY) {
   		daxpy_( &N, &alpha, X, &incX, Y, &incY );
}
static __inline void cblas_daxpby(const int N, const double alpha, 
		const double *X,
                const int incX, const double beta, double *Y, const int incY) {
   		daxpby_( &N, &alpha, X, &incX, &beta, Y, &incY );
}
static __inline void cblas_cswap(const int N, void *X, const int incX,
                void *Y, const int incY) {
		cswap_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_ccopy(const int N, const void *X, const int incX,
                void *Y, const int incY) {
		ccopy_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_caxpy(const int N, const void *alpha, 
		const void *X,
                const int incX, void *Y, const int incY) {
		caxpy_(&N, alpha, X, &incX, Y, &incY);
}
static __inline void cblas_zswap(const int N, void *X, 
		const int incX, void *Y, const int incY) {
		zswap_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_zcopy(const int N, const void *X, 
		const int incX,
                void *Y, const int incY) {
		zcopy_(&N, X, &incX, Y, &incY);
}
static __inline void cblas_zaxpy(const int N, const void *alpha, 
		const void *X,
                const int incX, void *Y, const int incY) {
		zaxpy_(&N, alpha, X, &incX, Y, &incY);
}
/*
 * Routines with S and D prefix only
 */
static __inline void cblas_srotg(float *a, float *b, float *c, float *s) {
		srotg_(a, b, c, s);
}
static __inline void cblas_srotmg(float *d1, float *d2, float *b1, 
		const float b2, float *P) {
		srotmg_(d1, d2, b1, &b2, P);
}
static __inline void cblas_srot(const int N, float *X, const int incX,
                float *Y, const int incY, const float c, const float s) {
		srot_(&N, X, &incX, Y, &incY, &c, &s);
}
static __inline void cblas_srotm(const int N, float *X, const int incX,
                float *Y, const int incY, const float *P) {
		srotm_(&N, X, &incX, Y, &incY, P);
}
static __inline void cblas_drotg(double *a, double *b, double *c, double *s) {
   		drotg_(a, b, c, s);
}
static __inline void cblas_drotmg(double *d1, double *d2, double *b1, 
		const double b2, double *P) {
   		drotmg_(d1, d2, b1, &b2, P);
}
static __inline void cblas_drot(const int N, double *X, const int incX,
                double *Y, const int incY, const double c, const double s) {
   		drot_(&N, X, &incX, Y, &incY, &c, &s);
}
static __inline void cblas_drotm(const int N, double *X, const int incX,
                double *Y, const int incY, const double *P) {
   		drotm_(&N, X, &incX, Y, &incY, P);
}
/*
 * Routines with S D C Z CS and ZD prefixes
 */
static __inline void cblas_sscal(const int N, const float alpha, float *X, 
		const int incX) {
		sscal_( &N, &alpha, X, &incX);
}
static __inline void cblas_dscal(const int N, const double alpha, double *X, 
		const int incX) {
   		dscal_( &N, &alpha, X, &incX );
}
static __inline void cblas_cscal(const int N, const CBLAS_COMPLEX *alpha, 
		CBLAS_COMPLEX *X, const int incX) {
		cscal_( &N, alpha, X, &incX);
}
static __inline void cblas_zscal(const int N, const CBLAS_ZOMPLEX *alpha, 
		CBLAS_ZOMPLEX *X, const int incX) {
		zscal_( &N, alpha, X, &incX);
}
static __inline void cblas_csscal(const int N, const float alpha, 
		CBLAS_COMPLEX *X, const int incX) {
		csscal_( &N, &alpha, X, &incX);
}
static __inline void cblas_zdscal(const int N, const double alpha, 
		CBLAS_ZOMPLEX *X, const int incX) {
		zdscal_( &N, &alpha, X, &incX);
}
/*
 * ===========================================================================
 * Prototypes for level 2 BLAS
 * ===========================================================================
 */
/*
 * Routines with standard 4 prefixes (S, D, C, Z)
 */
void cblas_sgemv(const enum CBLAS_ORDER order,
                 const enum CBLAS_TRANSPOSE TransA, const int M, const int N,
                 const float alpha, const float *A, const int lda,
                 const float *X, const int incX, const float beta,
                 float *Y, const int incY);
void cblas_strmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const float *A, const int lda,
                 float *X, const int incX);
void cblas_strsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const float *A, const int lda, float *X,
                 const int incX);
void cblas_sgbmv(const enum CBLAS_ORDER order, 
                 const enum CBLAS_TRANSPOSE TransA,
                 const int M, const int N, const int Kl, const int Ku,
                 const float alpha, const float *A, const int lda,
                 const float *X, const int incX, const float beta,
                 float *Y, const int incY);
void cblas_stbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const float *A, const int lda,
                 float *X, const int incX);
void cblas_stbsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const float *A, const int lda,
                 float *X, const int incX);
void cblas_sspmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const float  alpha, const float *AP,
                 const float  *X, const int incX, const float  beta, 
                 float  *Y, const int incY);
void cblas_stpsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const float *Ap, float *X, const int incX);
void cblas_stpmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const float *Ap, float *X, const int incX);

void cblas_dgemv(const enum CBLAS_ORDER order,
                 const enum CBLAS_TRANSPOSE TransA, const int M, const int N,
                 const double alpha, const double *A, const int lda,
                 const double *X, const int incX, const double beta,
                 double *Y, const int incY);
void cblas_dtrmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const double *A, const int lda,
                 double *X, const int incX);
void cblas_dtrsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const double *A, const int lda, double *X,
                 const int incX);
void cblas_dgbmv(const enum CBLAS_ORDER order, 
                 const enum CBLAS_TRANSPOSE TransA,
                 const int M, const int N, const int Kl, const int Ku,
                 const double alpha, const double *A, const int lda,
                 const double *X, const int incX, const double beta,
                 double *Y, const int incY);
void cblas_dtbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const double *A, const int lda,
                 double *X, const int incX);
void cblas_dtbsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const double *A, const int lda,
                 double *X, const int incX);
void cblas_dspmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const double  alpha, const double *AP,
                 const double  *X, const int incX, const double  beta, 
                 double  *Y, const int incY);
void cblas_dtpsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const double *Ap, double *X, const int incX);
void cblas_dtpmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const double *Ap, double *X, const int incX);

void cblas_cgemv(const enum CBLAS_ORDER order,
                 const enum CBLAS_TRANSPOSE TransA, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A,
                 const int lda, CBLAS_COMPLEX *X, const int incX,
                 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *Y, const int incY);
void cblas_cgbmv(const enum CBLAS_ORDER order,
                 const enum CBLAS_TRANSPOSE TransA,
                 const int M, const int N, const int KL, const int KU,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A,
                 const int lda, CBLAS_COMPLEX *X, const int incX,
                 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *Y, const int incY);
void cblas_ctrmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_COMPLEX *A, const int lda,
                 CBLAS_COMPLEX *X, const int incX);
void cblas_ctbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const CBLAS_COMPLEX *A,
                 const int lda, CBLAS_COMPLEX *X, const int incX);
void cblas_ctpmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_COMPLEX *Ap, CBLAS_COMPLEX *X,
                 const int incX);
void cblas_ctrsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_COMPLEX *A, const int lda,
                 CBLAS_COMPLEX *X, const int incX);
void cblas_ctbsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const CBLAS_COMPLEX *A,
                 const int lda, CBLAS_COMPLEX *X, const int incX);
void cblas_ctpsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_COMPLEX *Ap, CBLAS_COMPLEX *X,
                 const int incX);

void cblas_zgemv(const enum CBLAS_ORDER order,
                 const enum CBLAS_TRANSPOSE TransA, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A,
                 const int lda, CBLAS_ZOMPLEX *X, const int incX,
                 const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *Y, const int incY);
void cblas_zgbmv(const enum CBLAS_ORDER order,
                 const enum CBLAS_TRANSPOSE TransA,
                 const int M, const int N, const int KL, const int KU,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A,
                 const int lda, CBLAS_ZOMPLEX *X, const int incX,
                 const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *Y, const int incY);
void cblas_ztrmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_ZOMPLEX *A, const int lda,
                 CBLAS_ZOMPLEX *X, const int incX);
void cblas_ztbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const CBLAS_ZOMPLEX *A,
                 const int lda, CBLAS_ZOMPLEX *X, const int incX);
void cblas_ztpmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_ZOMPLEX *Ap, CBLAS_ZOMPLEX *X,
                 const int incX);
void cblas_ztrsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_ZOMPLEX *A, const int lda,
                 CBLAS_ZOMPLEX *X, const int incX);
void cblas_ztbsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const int K, const CBLAS_ZOMPLEX *A,
                 const int lda, CBLAS_ZOMPLEX *X, const int incX);
void cblas_ztpsv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA, const enum CBLAS_DIAG Diag,
                 const int N, const CBLAS_ZOMPLEX *Ap, CBLAS_ZOMPLEX *X,
                 const int incX);

/*
 * Routines with S and D prefixes only
 */
void cblas_sger(const enum CBLAS_ORDER order, const int M, const int N,
                const float alpha, const float *X, const int incX,
                const float *Y, const int incY, float *A, const int lda);
void cblas_ssymv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const float  alpha, const float  *A,
                 const int lda, const float  *X, const int incX,
                 const float  beta, float  *Y, const int incY);
void cblas_ssbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const int K, const float  alpha,
                 const float *A, const int lda, const float  *X, 
                 const int incX, const float  beta, float  *Y, const int incY);
void cblas_ssyr(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo, 
                const int N, const float  alpha, const float  *X, 
                const int incX, float  *A, const int lda);
void cblas_ssyr2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo, 
                 const int N, const float  alpha, const float  *X, 
                 const int incX, const float  *Y, const int incY, float  *A, 
                 const int lda);
void cblas_sspr(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo, 
                const int N, const float  alpha, const float  *X, 
                const int incX, float  *Ap);
void cblas_sspr2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                const int N, const float  alpha, const float  *X, 
                const int incX, const float  *Y, const int incY, float  *A);
void cblas_dsymv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const double  alpha, const double  *A,
                 const int lda, const double  *X, const int incX,
                 const double  beta, double  *Y, const int incY);
void cblas_dger(const enum CBLAS_ORDER order, const int M, const int N,
                const double alpha, const double *X, const int incX,
                const double *Y, const int incY, double *A, const int lda);
void cblas_dsbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const int K, const double  alpha,
                 const double *A, const int lda, const double  *X, 
                 const int incX, const double  beta, double  *Y, 
                 const int incY);
void cblas_dsyr(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo, 
                const int N, const double  alpha, const double  *X, 
                const int incX, double  *A, const int lda);
void cblas_dsyr2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo, 
                 const int N, const double  alpha, const double  *X, 
                 const int incX, const double  *Y, const int incY, double  *A,
                 const int lda);
void cblas_dspr(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo, 
                const int N, const double  alpha, const double  *X, 
                const int incX, double  *Ap);
void cblas_dspr2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                const int N, const double  alpha, const double  *X, 
                const int incX, const double  *Y, const int incY, double  *A);

/*
 * Routines with C and Z prefixes only
 */
void cblas_chemv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_COMPLEX *alpha,
                 const CBLAS_COMPLEX *A, const int lda, CBLAS_COMPLEX *X,
                 const int incX, const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *Y,
                 const int incY);
void cblas_chbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const int K, const CBLAS_COMPLEX *alpha,
                 const CBLAS_COMPLEX *A, const int lda, CBLAS_COMPLEX *X,
                 const int incX, const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *Y,
                 const int incY);
void cblas_chpmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_COMPLEX *alpha,
                 const CBLAS_COMPLEX *Ap, CBLAS_COMPLEX *X, const int incX,
                 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *Y, const int incY);
void cblas_cgeru(const enum CBLAS_ORDER order, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *X,
                 const int incX, const CBLAS_COMPLEX *Y, const int incY,
                 CBLAS_COMPLEX *A, const int lda);
void cblas_cgerc(const enum CBLAS_ORDER order, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *X,
                 const int incX, CBLAS_COMPLEX *Y, const int incY,
                 CBLAS_COMPLEX *A, const int lda);
void cblas_cher(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                const int N, const CBLAS_COMPLEX *alpha, CBLAS_COMPLEX *X,
                const int incX, const CBLAS_COMPLEX *A, const int lda);
void cblas_chpr(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                const int N, const CBLAS_COMPLEX *alpha, CBLAS_COMPLEX *X,
                const int incX, const CBLAS_COMPLEX *Ap);
void cblas_cher2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_COMPLEX *alpha,
                 CBLAS_COMPLEX *X, const int incX, CBLAS_COMPLEX *Y,
                 const int incY, const CBLAS_COMPLEX *A, const int lda);
void cblas_chpr2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_COMPLEX *alpha, CBLAS_COMPLEX *X,
                 const int incX, CBLAS_COMPLEX *Y,
                 const int incY, const CBLAS_COMPLEX *Ap);

void cblas_zhemv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_ZOMPLEX *alpha,
                 const CBLAS_ZOMPLEX *A, const int lda, CBLAS_ZOMPLEX *X,
                 const int incX, const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *Y,
                 const int incY);
void cblas_zhbmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const int K, const CBLAS_ZOMPLEX *alpha,
                 const CBLAS_ZOMPLEX *A, const int lda, CBLAS_ZOMPLEX *X,
                 const int incX, const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *Y,
                 const int incY);
void cblas_zhpmv(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_ZOMPLEX *alpha,
                 const CBLAS_ZOMPLEX *Ap, CBLAS_ZOMPLEX *X, const int incX,
                 const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *Y, const int incY);
void cblas_zgeru(const enum CBLAS_ORDER order, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *X,
                 const int incX, const CBLAS_ZOMPLEX *Y, const int incY,
                 CBLAS_ZOMPLEX *A, const int lda);
void cblas_zgerc(const enum CBLAS_ORDER order, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *X,
                 const int incX, CBLAS_ZOMPLEX *Y, const int incY,
                 CBLAS_ZOMPLEX *A, const int lda);
void cblas_zher(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                const int N, const CBLAS_ZOMPLEX *alpha, CBLAS_ZOMPLEX *X,
                const int incX, const CBLAS_ZOMPLEX *A, const int lda);
void cblas_zhpr(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                const int N, const CBLAS_ZOMPLEX *alpha, CBLAS_ZOMPLEX *X,
                const int incX, const CBLAS_ZOMPLEX *Ap);
void cblas_zher2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_ZOMPLEX *alpha,
                 CBLAS_ZOMPLEX *X, const int incX, CBLAS_ZOMPLEX *Y,
                 const int incY, const CBLAS_ZOMPLEX *A, const int lda);
void cblas_zhpr2(const enum CBLAS_ORDER order, const enum CBLAS_UPLO Uplo,
                 const int N, const CBLAS_ZOMPLEX *alpha, CBLAS_ZOMPLEX *X,
                 const int incX, CBLAS_ZOMPLEX *Y,
                 const int incY, const CBLAS_ZOMPLEX *Ap);

/*
 * ===========================================================================
 * Prototypes for level 3 BLAS
 * ===========================================================================
 */
/*
 * Routines with standard 4 prefixes (S, D, C, Z)
 */
void cblas_sgemm(const enum CBLAS_ORDER Order, 
                 const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_TRANSPOSE TransB, const int M, const int N,
                 const int K, const float alpha, const float *A,
                 const int lda, const float *B, const int ldb,
                 const float beta, float *C, const int ldc);
void cblas_ssymm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const int M, const int N,
                 const float alpha, const float *A, const int lda,
                 const float *B, const int ldb, const float beta,
                 float *C, const int ldc);
void cblas_ssyrk(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                 const float alpha, const float *A, const int lda,
                 const float beta, float *C, const int ldc);
void cblas_ssyr2k(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                  const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                  const float alpha, const float *A, const int lda,
                  const float *B, const int ldb, const float beta,
                  float *C, const int ldc);
void cblas_strmm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const float alpha, const float *A, const int lda,
                 float *B, const int ldb);
void cblas_strsm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const float alpha, const float *A, const int lda,
                 float *B, const int ldb);

void cblas_dgemm(const enum CBLAS_ORDER Order, 
                 const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_TRANSPOSE TransB, const int M, const int N,
                 const int K, const double alpha, const double *A,
                 const int lda, const double *B, const int ldb,
                 const double beta, double *C, const int ldc);
void cblas_dsymm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const int M, const int N,
                 const double alpha, const double *A, const int lda,
                 const double *B, const int ldb, const double beta,
                 double *C, const int ldc);
void cblas_dsyrk(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                 const double alpha, const double *A, const int lda,
                 const double beta, double *C, const int ldc);
void cblas_dsyr2k(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                  const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                  const double alpha, const double *A, const int lda,
                  const double *B, const int ldb, const double beta,
                  double *C, const int ldc);
void cblas_dtrmm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const double alpha, const double *A, const int lda,
                 double *B, const int ldb);
void cblas_dtrsm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const double alpha, const double *A, const int lda,
                 double *B, const int ldb);

void cblas_cgemm(const enum CBLAS_ORDER Order, 
		 const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_TRANSPOSE TransB, const int M, const int N,
                 const int K, const CBLAS_COMPLEX *alpha, 
		 const CBLAS_COMPLEX *A,
                 const int lda, const CBLAS_COMPLEX *B, const int ldb,
                 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *C, const int ldc);
void cblas_csymm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A, 
		 const int lda, const CBLAS_COMPLEX *B, const int ldb, 
		 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *C, const int ldc);
void cblas_csyrk(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A, 
		 const int lda,
                 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *C, const int ldc);
void cblas_csyr2k(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                  const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                  const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A, 
		  const int lda, const CBLAS_COMPLEX *B, const int ldb, 
		  const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *C, const int ldc);
void cblas_ctrmm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A, 
		 const int lda, CBLAS_COMPLEX *B, const int ldb);
void cblas_ctrsm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A, 
		 const int lda, CBLAS_COMPLEX *B, const int ldb);
void cblas_cherk(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA,
                 const int N, const int K, const float alpha,
                 const CBLAS_COMPLEX *A, const int lda, const float beta,
                 CBLAS_COMPLEX *C, const int ldc);
void cblas_cher2k(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE TransA,
                 const int N, const int K, const CBLAS_COMPLEX *alpha,
                 const CBLAS_COMPLEX *A, const int lda, 
                 const CBLAS_COMPLEX *B, const int ldb, const float beta,
                 CBLAS_COMPLEX *C, const int ldc);
void cblas_chemm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const int M, const int N,
                 const CBLAS_COMPLEX *alpha, const CBLAS_COMPLEX *A, 
		 const int lda, const CBLAS_COMPLEX *B, const int ldb, 
		 const CBLAS_COMPLEX *beta, CBLAS_COMPLEX *C, const int ldc);

void cblas_zgemm(const enum CBLAS_ORDER Order, 
		 const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_TRANSPOSE TransB, const int M, const int N,
                 const int K, const CBLAS_ZOMPLEX *alpha, 
		 const CBLAS_ZOMPLEX *A,
                 const int lda, const CBLAS_ZOMPLEX *B, const int ldb,
                 const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *C, const int ldc);
void cblas_zsymm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		 const int lda, const CBLAS_ZOMPLEX *B, const int ldb, 
		 const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *C, const int ldc);
void cblas_zsyrk(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		 const int lda, const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *C, 
		 const int ldc);
void cblas_zsyr2k(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                  const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                  const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		  const int lda, const CBLAS_ZOMPLEX *B, const int ldb, 
		  const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *C, const int ldc);
void cblas_ztrmm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		 const int lda, CBLAS_ZOMPLEX *B, const int ldb);
void cblas_ztrsm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const enum CBLAS_TRANSPOSE TransA,
                 const enum CBLAS_DIAG Diag, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		 const int lda, CBLAS_ZOMPLEX *B, const int ldb);
void cblas_zhemm(const enum CBLAS_ORDER Order, const enum CBLAS_SIDE Side,
                 const enum CBLAS_UPLO Uplo, const int M, const int N,
                 const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		 const int lda, const CBLAS_ZOMPLEX *B, const int ldb, 
		 const CBLAS_ZOMPLEX *beta, CBLAS_ZOMPLEX *C, const int ldc);
void cblas_zherk(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                 const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                 const double alpha, const CBLAS_ZOMPLEX *A, 
		 const int lda, const double beta, CBLAS_ZOMPLEX *C, 
		 const int ldc);
void cblas_zher2k(const enum CBLAS_ORDER Order, const enum CBLAS_UPLO Uplo,
                  const enum CBLAS_TRANSPOSE Trans, const int N, const int K,
                  const CBLAS_ZOMPLEX *alpha, const CBLAS_ZOMPLEX *A, 
		  const int lda, const CBLAS_ZOMPLEX *B, const int ldb, 
		  const double beta, CBLAS_ZOMPLEX *C, const int ldc);

/*
 * auxiliary functions
 */
void get_transpose_type(char *type, enum CBLAS_TRANSPOSE *trans);
void get_uplo_type(char *type, enum CBLAS_UPLO *uplo);
void get_diag_type(char *type, enum CBLAS_DIAG *diag);
void get_side_type(char *type, enum CBLAS_SIDE *side);
void cblas_xerbla(char *type, int);

#ifdef __cplusplus
}
#endif

#endif /* CBLAS_H */
