#ifndef __SGCVT_H__
#define __SGCVT_H__

/*
 * convert/SgCvt.h
 *
 *	GoldenGate data conversion API
 *
 * Copyright 1995, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 *
 * UNPUBLISHED -- Rights reserved under the copyright laws of the United
 * States.   Use of a copyright notice is precautionary only and does not
 * imply publication or disclosure.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights
 * in Technical Data and Computer Software clause at DFARS 252.227-7013 and/or
 * in similar or successor clauses in the FAR, or the DOD or NASA FAR
 * Supplement.  Contractor/manufacturer is Silicon Graphics, Inc.,
 * 2011 N. Shoreline Blvd. Mountain View, CA 94039-7311.
 *
 * THE CONTENT OF THIS WORK CONTAINS CONFIDENTIAL AND PROPRIETARY
 * INFORMATION OF SILICON GRAPHICS, INC. ANY DUPLICATION, MODIFICATION,
 * DISTRIBUTION, OR DISCLOSURE IN ANY FORM, IN WHOLE, OR IN PART, IS STRICTLY
 * PROHIBITED WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SILICON
 * GRAPHICS, INC.
 */

#ident "$Revision: 1.17 $"

/*
 * APIs we depend on
 */
#include <sys/types.h>
#include <dmedia/dm_params.h>		/* for DMparams */

/*
 * Preprocessor stuff
 */

/*
 * Flags for registry fields.
 */
#define	SG_CVT_ATTR_FLAG_NAME		(1L<<0)
#define	SG_CVT_ATTR_FLAG_INPUT_FORMAT	(1L<<1)
#define	SG_CVT_ATTR_FLAG_OUTPUT_FORMAT	(1L<<2)
#define SG_CVT_ATTR_FLAG_IO_METHOD	(1L<<3)
#define SG_CVT_ATTR_FLAG_INPUT_LABEL	(1L<<4)
#define SG_CVT_ATTR_FLAG_OUTPUT_LABEL	(1L<<5)
#define	SG_CVT_ATTR_FLAG_VENDOR		(1L<<6)
#define	SG_CVT_ATTR_FLAG_VERSION	(1L<<7)
#define	SG_CVT_ATTR_FLAG_DSO_NAME	(1L<<8)
#define	SG_CVT_ATTR_FLAG_FUNCTION_NAME	(1L<<9)
#define	SG_CVT_ATTR_FLAG_DESCRIPTION	(1L<<10)

/*
 * Where SGI recommends installing converters
 */
#define SG_CVT_DEFAULT_CONVERTER_DIR	"/usr/lib/convert"

/*
 * Data Type Definitions
 */

/* Return codes form SgCvt... functions */
typedef enum {
	SG_CVT_E_SUCCESS,
	SG_CVT_E_NOMEM,
	SG_CVT_E_AGAIN,
	SG_CVT_E_FAILURE,
	SG_CVT_E_ACCEPT,
	SG_CVT_E_REJECT,
	SG_CVT_E_END_OF_STREAM,
	SG_CVT_E_QUEUE_EMPTY,
	SG_CVT_E_READ_FAILED,
	SG_CVT_E_WRITE_FAILED,
	SG_CVT_E_BAD_CONVERTER_TYPE,
	SG_CVT_E_MISSING_COMMAND,
	SG_CVT_E_NO_CONVERTER,
	SG_CVT_E_NO_TARGET,
	SG_CVT_E_UNKNOWN_TYPE
} SgCvtStatus;

/* Collation operands for registry lookup */
typedef enum {
	SG_CVT_OP_EQ,	/* EQual to 			*/
	SG_CVT_OP_NE,	/* Not Equal to			*/
	SG_CVT_OP_LT,	/* Less Than			*/
	SG_CVT_OP_LE,	/* Less than or Equal to	*/
	SG_CVT_OP_GT,	/* Greater Than 		*/
	SG_CVT_OP_GE	/* Greater than or Equal to	*/
} SgCvtOpCode;

/* Names of fields that may constrain registry lookup */
typedef enum {
	SG_CVT_ATTR_NAME,
	SG_CVT_ATTR_INPUT_FORMAT,
	SG_CVT_ATTR_OUTPUT_FORMAT,
	SG_CVT_ATTR_IO_METHOD,
	SG_CVT_ATTR_INPUT_LABEL,
	SG_CVT_ATTR_OUTPUT_LABEL,
	SG_CVT_ATTR_VENDOR,
	SG_CVT_ATTR_VERSION,
	SG_CVT_ATTR_DSO_NAME,
	SG_CVT_ATTR_FUNCTION_NAME,
	SG_CVT_ATTR_DESCRIPTION
} SgCvtAttributeName;

typedef enum {
	SG_CVT_IO_FILE_TO_FILE,
	SG_CVT_IO_STREAM_TO_STREAM
} SgCvtIOMethod;

/* Lookup constraint */
typedef struct {
	SgCvtAttributeName	attribute;
	void			*value;
	SgCvtOpCode		opcode;
} SgCvtQueryConstraint;

/* Registered converter info that can be obtained */ 
typedef struct {
	char	*name;
	SgCvtIOMethod io_method;
	char	*input_format;
	char	*input_label;
	char	*output_format;
	char	*output_label;
	char	*vendor;
	char	*version;
	char	*dso_name;
	char	*function_name;
	char	*description;
} SgCvtConverterAttrs;

typedef void	*SgCvtConverterId;
typedef struct	_SgCvtConversionContext *SgCvtConversionContext;
typedef void	*SgCvtRegistry;

typedef enum {
	SG_CVT_REQ_EVALUATE,
	SG_CVT_REQ_CONVERT
} SgCvtRequestType;

typedef struct {
	SgCvtRequestType	operation;	/* evaluate or convert */
	SgCvtConversionContext	context;
	DMparams		*output_params;	/* evaluation results */
	SgCvtStatus		status_return;	/* evaluation decision */
} SgCvtConverterData;

typedef void
   (*SgCvtConverterProc)(void *data);

/* Conversion Callbacks */

#define SG_CVT_CB_FLAG_CONVERSION_DONE	(1L<<0)
#define SG_CVT_CB_FLAG_STAGE_DONE	(1L<<1)

typedef enum {
	SG_CVT_CB_CONVERSION_DONE,
	SG_CVT_CB_STAGE_DONE,
	SG_CVT_CB_CONVERSION_FAILED	/* non-maskable */
 } SgCvtCallbackName ;

typedef struct _SgCvtCallbackStruct {
        SgCvtCallbackName       name;
        SgCvtStatus             status;
        char                    *output_file;
	void			*extension;
} SgCvtCallbackStruct;

typedef void (*SgCvtCallback)(
        SgCvtConversionContext, /* context */
        void *,                 /* client data */
        void *                  /* callback-specific data */
);

typedef struct {
	DMparams		*input_params;
	DMparams		*output_params;
	DMparams		*meta_params;
	char			*input_file;
	char			*input_host;
	char			*output_file;
	char			*output_host;
} SgCvtContextInfo;

/* Masks for setting/getting SgCvtContextData */
#define	SG_CVT_INFO_INPUT_PARAMS	(1L<<0)
#define	SG_CVT_INFO_OUTPUT_PARAMS	(1L<<1)
#define	SG_CVT_INFO_META_PARAMS		(1L<<2)
#define	SG_CVT_INFO_INPUT_FILE		(1L<<3)
#define SG_CVT_INFO_INPUT_HOST		(1L<<4)
#define SG_CVT_INFO_OUTPUT_FILE		(1L<<5)
#define SG_CVT_INFO_OUTPUT_HOST		(1L<<6)

/*
 * Function Prototypes
 */

#ifdef __cplusplus
extern "C" {
#endif

extern SgCvtStatus SgCvtSetQueryConstraint
	(
	SgCvtQueryConstraint	*constraint,
	SgCvtAttributeName	attribute,
	void			*value,
	SgCvtOpCode		opcode
	);

extern boolean_t SgCvtIsPipeline
	(
	SgCvtConverterId *translator
	);

extern SgCvtStatus SgCvtQueryRegistry
	(
	SgCvtQueryConstraint	*constraints,
	int			num_constraints,
	SgCvtRegistry		*registry,		/* ouptut */
	SgCvtConverterId	**converters,		/* output */
	int			*num_converters		/* output */
	);

extern void SgCvtFreeRegistry
	(
	SgCvtRegistry		registry
	);

extern SgCvtStatus SgCvtGetConverterAttributes
	(
	SgCvtConverterId	converter_id,
	unsigned long		converter_attr_mask,
	SgCvtConverterAttrs	*attributes		/* in/out */
	);

extern SgCvtStatus SgCvtCreateConversionContext
	(
	SgCvtConversionContext	*context
	);

extern SgCvtStatus SgCvtDestroyConversionContext
	(
	SgCvtConversionContext  context
	);

extern SgCvtStatus SgCvtSetContextInfo
	(
	SgCvtConversionContext  context,
	unsigned long 		valuemask,
	SgCvtContextInfo  	*context_data
	);

extern SgCvtStatus SgCvtGetContextInfo
	(
	SgCvtConversionContext  context,
	unsigned long valuemask,
	SgCvtContextInfo  *context_data
	);

extern SgCvtStatus SgCvtEvaluateConverter
	(
	SgCvtConversionContext  context,
	SgCvtConverterId	converter_id,
	DMparams		**output_params
	);

extern SgCvtStatus SgCvtInitializePipeline
	(
	SgCvtConversionContext  context,
	SgCvtConverterId	converter_id
	);

extern SgCvtStatus SgCvtTerminatePipeline
	(
	SgCvtConversionContext  context
	);

extern SgCvtStatus SgCvtSendData
	(
	SgCvtConversionContext	context,
	void			*data,
	size_t			length,
	DMparams		*params,
	boolean_t		canwait
	);

extern SgCvtStatus SgCvtGetData
	(
	SgCvtConversionContext	context,
	size_t			buf_len,
	void			*buffer,		/* output */
	size_t			*length_return,		/* output */ 
	DMparams		**params_return,	/* output */
	boolean_t		canwait
	);

extern SgCvtStatus SgCvtSendEndOfStream
        (
        SgCvtConversionContext  ctx
        );

extern SgCvtStatus SgCvtFree
	(
	SgCvtConversionContext	context,
	void *ptr
	);

extern SgCvtStatus SgCvtEncodeParams
	(
	DMparams	*params,
	void		**streamed,	/* output */
        unsigned int	*len		/* output */
	);

extern SgCvtStatus SgCvtDecodeParams
	(
	void		*streamed,
	DMparams	**params	/* output */
	);

extern SgCvtStatus SgCvtFreeEncodedParams
	(
	void	*addr
	);

extern SgCvtStatus SgCvtConvertFileToFile
        (
        SgCvtConversionContext  context,
        SgCvtConverterId        converter_id,
        char                    *input_file,
        char                    *output_file,
	unsigned long		callback_mask,
        SgCvtCallback           callback,
        void                    *client_data
        );

extern SgCvtStatus SgCvtGetFileSelectionTarget
	(
	char *fname,
	char **targetname		/* output */
	);

#ifdef __cplusplus
}
#endif

#endif	/* ! __SGCVT_H__ */
