/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _FS_NAMEFS_NAMENODE_H	/* wrapper symbol for kernel use */
#define _FS_NAMEFS_NAMENODE_H	/* subject to change without notice */

/*#ident	"@(#)uts-3b2:fs/namefs/namenode.h	1.4"*/
#ident	"$Revision: 1.5 $"

/*
 * This structure is used to pass a file descriptor from user
 * level to the kernel. It is first used by fattach() and then
 * by NAMEFS.
 */
struct namefd {
	int fd;
};

#ifdef _KERNEL

#include <sys/types.h>
#include <sys/sema.h>
#include <sys/vnode.h>

/*
 * Each NAMEFS object is identified by a struct namenode/vnode pair.
 *
 * MP locking protocols:
 *	nm_vattr			namelock/nameunlock
 *	nm_filevp,nm_filep,nm_mountpt	read-only
 *	nm_nextp, nm_backp		nameallocmon
 */
struct namenode {
	struct vnode	*nm_vnode;	/* represents mounted file desc.*/
	mutex_t		nm_lock;	/* mutual exclusion for get/setattr */
	struct vattr    nm_vattr;	/* attributes of mounted file desc.*/
	struct vnode	*nm_filevp;	/* file desc. prior to mounting */
	struct file	*nm_filep;	/* file pointer of nm_filevp */
	struct vnode	*nm_mountpt;	/* mount point prior to mounting */
	struct namenode *nm_nextp;	/* next link in the linked list */
	struct namenode *nm_backp;	/* back link in linked list */
};


/*
 * Macros to convert a vnode to a namenode, and vice versa.
 */
#define VTONM(vp) ((struct namenode *)((vp)->v_data))
#define NMTOV(nm) ((nm)->nm_vnode)

extern struct namenode	*namefind(struct vnode *, struct vnode *);
extern void		nameinsert(struct namenode *);
extern void		nameremove(struct namenode *);
extern void		namefree(struct namenode *);
extern void		nmclearid(struct namenode *);
extern u_short		nmgetid(void);

extern mutex_t	nameallocmon;

#endif	/* _KERNEL */

#endif	/* _FS_NAMEFS_NAMENODE_H */
