/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifndef __SYS_CAPABILITY_H__
#define __SYS_CAPABILITY_H__

#ifdef __cplusplus
extern "C" {
#endif

#ident	"$Revision: 1.3 $"

/*
 * Data types for capability sets.
 * capabilities were called privileges prior to P1003.6D14
 */

/*
 * Capability attributes for a file system object.
 */
#define EAG_CAP_FILE		"cap-file"	/* caps on a file */

/*
 * Capability attributes for the current process.
 */
#define EAG_CAP_PROCESS		"cap-process"	/* process caps */
#define EAG_CAP_REQUEST		"cap-request"	/* request a process cap */
#define EAG_CAP_SURRENDER	"cap-surrender"	/* surrender a process cap */

/*
 * Capability attributes for the system.
 */
#define EAG_CAP_SUPERUSER	"cap-superuser"		/* empower uid 0 */
#define EAG_CAP_NO_SUPERUSER	"cap-no-superuser"	/* disempower uid 0 */

/*
 * Plan G Database, two files:
 *	CAP_EAG_SET - capability set indexs, one per inode.
 *	CAP_EAG_SET - capability sets
 */
#define	CAP_EAG_DIR		"attribute"
#define	CAP_EAG_SET		"attribute/cap_set"
#define	CAP_EAG_SET_FILE	"cap_set"
#define	CAP_EAG_INDEX		"attribute/cap_index"
#define	CAP_EAG_INDEX_FILE	"cap_index"

/*
 * System capability states
 */
#define CAP_SYS_DISABLED	0	/* Traditional SuperUser */
#define CAP_SYS_SUPERUSER	1	/* Caps, Plus Traditional SuperUser */
#define CAP_SYS_NO_SUPERUSER	2	/* Caps, no SuperUser */
/*
 * Capabilities required by P1003.6D13
 * Capabilities required by Appendix B of the CMW spec.
 */
#define CAP_NOT_A_CID		0
#define	CAP_CHOWN		1
#define	CAP_DAC_OVERRIDE	2
#define	CAP_DAC_READ_SEARCH	3
#define	CAP_FOWNER		4
#define	CAP_FSETID		5
#define	CAP_KILL		6
#define	CAP_LINK_DIR		7
#define	CAP_SETFPRIV		8
#define	CAP_SETPPRIV		9
#define	CAP_SETGID		10 /* includes gid, group list, pgid */
#define	CAP_SETUID		11

#define	CAP_MAC_DOWNGRADE	12
#define	CAP_MAC_READ		13
#define	CAP_MAC_RELABEL_SUBJ	14
#define	CAP_MAC_WRITE		15
#define	CAP_MAC_UPGRADE		16

#define	CAP_INF_NOFLOAT_OBJ	17
#define	CAP_INF_NOFLOAT_SUBJ	18
#define	CAP_INF_DOWNGRADE	19
#define	CAP_INF_UPGRADE		20
#define	CAP_INF_RELABEL_SUBJ	21

#define	CAP_AUDIT_CONTROL	22
#define	CAP_AUDIT_WRITE		23

#define	CAP_MAC_MLD		24
#define	CAP_MEMORY_MGT		25
#define	CAP_SWAP_MGT		26
#define	CAP_TIME_MGT		27
#define	CAP_SYSINFO_MGT		28
#define	CAP_NVRAM_MGT		CAP_SYSINFO_MGT
#define	CAP_MOUNT_MGT		29
#define	CAP_QUOTA_MGT		30
#define	CAP_PRIV_PORT		31
#define	CAP_STREAMS_MGT		32
#define	CAP_SCHED_MGT		33
#define	CAP_PROC_MGT		34
#define	CAP_SVIPC_MGT		35
#define	CAP_NETWORK_MGT		36
#define	CAP_DEVICE_MGT		37
#define	CAP_MKNOD		CAP_DEVICE_MGT
#define	CAP_ACCT_MGT		38
#define	CAP_SHUTDOWN		39
#define	CAP_CHROOT		40
/*
 * CAP_MAX_ID must be <= CAP_SET_SIZE * sizeof(char) 
 */
#define	CAP_MAX_ID	40
#define CAP_SET_SIZE	8


typedef __uint64_t cap_value_t;
typedef	__uint64_t cap_flag_t;
typedef __uint64_t cap_flag_value_t;

struct cap_set {
	cap_value_t	cap_effective;	/* use in capability checks */
	cap_value_t	cap_permitted;	/* combined with file attrs */
	cap_value_t	cap_inheritable;/* pass through exec */
};
typedef struct cap_set cap_set_t;
typedef struct cap_set * cap_t;

typedef unsigned int cap_id_t;

struct capability {
	char	cap_set[CAP_SET_SIZE];	/* basic capabilities set */
};
typedef struct capability capability_t;

struct cap_proc {
	capability_t	cap_effective;	/* use in capability checks */
	capability_t	cap_inheritable;/* pass through exec */
	capability_t	cap_permitted;	/* combined with file attrs */
};
typedef struct cap_proc cap_proc_t;

struct cap_eag {
	capability_t	cap_allowed;	/* allow these ones */
	capability_t	cap_forced;	/* turn these on */
	int		cap_flags;	/* for set_effective bit */
};
typedef struct cap_eag cap_eag_t;
typedef int cap_eag_index_t;

/*
 * Flag values for the file capability set
 */
#define	CAP_SET_EFFECTIVE	0x00000001

#ifdef _KERNEL
/*
 * Data to add to the vfs entry. Includes the vnode pointer for CAP_EAG_SET
 */
struct cap_vfs {
	struct vnode	*cv_index;	/* capability set index */
	struct vnode	*cv_set;	/* capability sets */
};
typedef struct cap_vfs cap_vfs_t;

#endif /* _KERNEL */

/* function prototypes */

#ifdef _KERNEL
struct vfs;
struct proc;
struct cred;

extern void cap_eag_loadattr( struct vfs * );
extern void cap_eag_unload( struct vfs * );
extern void cap_eag_init( struct vfs *, char * );
extern void cap_eag_free( struct vfs * );
extern void cap_confignote( void );
extern void cap_init_cred( struct cred * );
extern void cap_crcopy( struct cred * );
extern void cap_crfree( struct cred * );

extern int cap_able( cap_id_t );
extern int cap_crable( struct cred *, cap_id_t );
extern int cap_recalc( struct vnode *, struct proc * );
extern int cap_eag_set(struct vfs *, int, cap_eag_t *);

/* Define macros choosing stub functions or real functions here */
extern int cap_enabled;

#define _CAP_EAG_LOADATTR(s)	((cap_enabled)? cap_eag_loadattr(s): (void)0)
#define _CAP_EAG_UNLOAD(s)	((cap_enabled)? cap_eag_unload(s): (void)0)
#define _CAP_EAG_INIT(v,e)	((cap_enabled)? cap_eag_init(v,e): (void)0)
#define _CAP_EAG_SETCAP(v,i,c)	((cap_enabled)? cap_eag_set(v,i,c): 0)
#define _CAP_EAG_FREE(v)	((cap_enabled)? cap_eag_free(v): (void)0)
#define _CAP_CONFIGNOTE()	((cap_enabled)? cap_confignote(): (void)0)
#define _CAP_INIT_CRED(cr)	((cap_enabled)? cap_init_cred(cr): (void)0)

#define _CAP_ABLE(c)		((cap_enabled)? cap_able(c): suser())
#define _CAP_CRABLE(cr,c)	((cap_enabled)? cap_crable(cr,c): crsuser(cr))
#define _CAP_RECALC(v,p)	((cap_enabled)? cap_recalc(v,p): 0)

#endif /* _KERNEL */

#ifdef __cplusplus
}
#endif

#endif	/* __SYS_CAPABILITY_H_ */
