/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _IO_CONF_H	/* wrapper symbol for kernel use */
#define _IO_CONF_H	/* subject to change without notice */

/*#ident	"@(#)uts-3b2:io/conf.h	1.4"*/
#ident	"$Revision: 3.23 $"

/*
 * Declaration of block device switch. Each entry (row) is
 * the only link between the main unix code and the driver.
 * The initialization of the device switches is in the file conf.c.
 */
struct bdevsw {
	int	*d_flags;
	int	d_cpulock;
	int	(*d_open)();
	int	(*d_close)();
	int	(*d_strategy)();
	int	(*d_print)();
	int	(*d_map)();
	int	(*d_unmap)();
	int	(*d_dump)();
	int	(*d_size)();
};

/*
 * Character device switch.
 */
struct cdevsw {
	int	*d_flags;
	int	d_cpulock;
	int	(*d_open)();
	int	(*d_close)();
	int	(*d_read)();
	int	(*d_write)();
	int	(*d_ioctl)();
	int	(*d_mmap)();
	int	(*d_map)();
	int	(*d_unmap)();
	int	(*d_poll)();
	struct	tty *d_ttys;
	struct	streamtab *d_str;
};

#define	DC_OPEN		1
#define DC_CLOSE	2
#define	DC_READ		3
#define DC_WRITE	4
#define DC_IOCTL	5
#define DC_PRINT	6
#define DC_STRAT	7
#define DC_DUMP		8
#define DC_SIZE		9
#define DC_MAP		10
#define DC_UNMAP	11
#define DC_POLL		12
#define DC_MMAP		13

#define bdopen(maj,devp,a2,a3,a4)	bdrv(maj,DC_OPEN,devp,a2,a3,a4)
#define bdstrat(maj,a1)			bdrv(maj,DC_STRAT,a1)
#define bdprint(maj,a1,a2)		bdrv(maj,DC_PRINT,a1,a2)
#define bdclose(maj,a1,a2,a3,a4)	bdrv(maj,DC_CLOSE,a1,a2,a3,a4)
#define bddump(maj,a1,a2,a3,a4,a5)	bdrv(maj,DC_DUMP,a1,a2,a3,a4,a5)
#define bdsize(maj,a1)			bdrv(maj,DC_SIZE,a1)
#define cdopen(maj,devp,a2,a3,a4)	cdrv(maj,DC_OPEN,devp,a2,a3,a4)
#define cdclose(maj,dev,a2,a3,a4)	cdrv(maj,DC_CLOSE,dev,a2,a3,a4)
#define cdread(maj,dev,uio,cr)		cdrv(maj,DC_READ,dev,uio,cr)
#define cdwrite(maj,dev,uio,cr)		cdrv(maj,DC_WRITE,dev,uio,cr)
#define cdioctl(maj,dev,a2,a3,a4,a5,a6)	cdrv(maj,DC_IOCTL,dev,a2,a3,a4,a5,a6)
#define cdpoll(maj,dev,a2,a3,a4,a5)	cdrv(maj,DC_POLL,dev,a2,a3,a4,a5)

struct vhandl;
int bdmap(int, struct vhandl *, off_t, u_int, u_int);
int bdunmap(int, struct vhandl *);
int cdmap(int, dev_t, struct vhandl *, off_t, u_int, u_int);
int cdunmap(int, dev_t, struct vhandl *);
int cdmmap(int, dev_t, off_t, u_int);

/*
 * driver semaphoring types
 * and cdevsw/bdevsw flags
 */
#define DLOCK_MASK	0x7
#define	D_PROCESSOR	0x0	/* driver executed on master processor */
#define	D_MP		0x1	/* driver does own semaphoring */
#define	D_MT		0x2	/* driver is thread-aware */
/* #define DLOCK_xx	0x4	unused */
#define	D_WBACK		0x8	/* write back cache before strategy routine */

/*
 * Device flags.
 *
 * Bit 0 to bit 15 are reserved for kernel.
 * Bit 16 to bit 31 are reserved for different machines.
 */
#define	D_OLD		0x10	/* old-style driver */
/*
 * Added for UFS.
 */
#define D_SEEKNEG       0x20    /* negative seek offsets are OK */
#define D_TAPE          0x40    /* magtape device (no bdwrite when cooked) */
/*
 * Added for pre-4.0 drivers backward compatibility.
 */
#define D_NOBRKUP	0x80	/* no breakup needed for new drivers */

#define	FMNAMESZ	8

struct fmodsw {
	char	f_name[FMNAMESZ+1];
	struct  streamtab *f_str;
	int	*f_flags;	/* same as device flags */
};

#define	GROUPNAMESZ	20

/*
 * tunable table 
 */
struct tunetable {
	char		t_name[GROUPNAMESZ+1];
	unsigned int	t_flag;
	int     	(*t_sanity)();
};
	

#define N_STATIC 1
#define N_RUN    2

#ifdef _KERNEL

extern struct bdevsw bdevsw[];
extern struct cdevsw cdevsw[];
extern struct fmodsw fmodsw[];

extern int	bdevcnt;
extern int	cdevcnt;
extern int	fmodcnt;
extern int	bdevmax;
extern int	cdevmax;
extern int	fmodmax;
extern int	vfsmax;
extern int	nfstype;
extern int	bdrv(int, int, ...);
extern int	cdrv(int, int, ...);

#define bdstatic(bmaj)  ((bmaj)< bdevcnt)
#define cdstatic(cmaj)  ((cmaj)< cdevcnt)
#define fmstatic(fmaj)  ((fmaj)< fmodcnt)

#define bdvalid(bm)  (bdstatic(bm) || ((bm)< bdevmax && bdevsw[bm].d_flags))
#define cdvalid(cm)  (cdstatic(cm) || ((cm)< cdevmax && cdevsw[cm].d_flags))
#define fmvalid(fm)  (fmstatic(fm) || ((fm)< fmodmax && fmodsw[fm].f_flags))

extern int cdhold(major_t);
extern void cdrele(major_t);
extern int bdhold(major_t);
extern void bdrele(major_t);
extern int fmhold(int);
extern void fmrele(int);

#endif	/* _KERNEL */
#endif	/* _IO_CONF_H */
