/**************************************************************************
 *									  *
 *		 Copyright (C) 1990, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
 *  dispmode.h $Revision: 1.3 $
 *
 *  Description of a generic display mode bitfield.
 *
 *  NOTE although this bitfield is generic not all bits will have the
 *  same meaning on all machines and some bits have no meaning on
 *  some machine.
 *
 *  For example on a 24 bit ECLIPSE if double buffer bit affects how
 *  RGBmode is displayed (12 bits vs. 24 bits).  On a CLOVER2 or STAPUFT
 *  it has no effect on how RGBmode is displayed.
 */


/*
 *  mode info
 *
 *  bit		0			1
 *
 *  NORMAL BUFFER
 *  0		cmode			RGBmode
 *  1		single buffer		double buffer
 *  2		display buffer 0	display buffer 1
 *  3		reserved
 *  4		onemap			multimap
 *  5-8		multi map number
 *  9		cmap A			cmap B
 *  
 *  AUX BUFFER
 *  10-13	bits of overlay
 *  14		underlay true		underlay false
 *
 *  ALPHA BUFFER
 *  15		alpha over true		alpha over false
 *  16		alpha under true	alpha under false
 *  17		alpha single buffer	alpha double buffer
 *  18		alpha disp buf 0	alpha disp buf 1
 *  19		reserved
 *  20-23	alpha multi map number
 *  24		alpha cmap A		alpha cmap B
 */


#define M_RGBMODE	0x00000001
#define M_DOUBLEBUFFER	0x00000002
#define M_DISPBUFFER	0x00000004
#define M_MULTIMAP	0x00000010
#define M_MAPNUM	0x000001e0
#define M_CMAP		0x00000200

#define M_OVERLAY	0x00003c00
#define M_UNDERLAY	0x00004000

#define M_A_OVERLAY	0x00008000
#define M_A_UNDERLAY	0x00010000
#define M_A_DOUBLEBUF	0x00020000
#define M_A_DISPBUFFER	0x00040000
#define M_A_MAPNUM	0x00f00000
#define M_A_CMAP	0x01000000


/*
 *  useful bit manipulation defines
 */
#define M_GET_MAPNUM(m)		((m >> 5) & 0xf)
#define M_SET_MAPNUM(m,n)	m = (m & ~M_MAPNUM) | ((n & 0xf) << 5)
#define M_GET_OVERLAY(m)	((m & M_OVERLAY) >> 10)
#define M_SET_OVERLAY(m,o)	m = (m & ~M_OVERLAY) | ((o & 0xf) << 10)
#define M_GET_UNDERLAY(m)	(m & M_UNDERLAY)
#define M_SET_UNDERLAY(m,u)	m = (u)? (m | M_UNDERLAY): (m & ~M_UNDERLAY)
#define M_GET_A_MAPNUM(m)	((m >> 20) & 0xf)
#define M_SET_A_MAPNUM(m,n)	m = (m & ~M_A_MAPNUM) | ((n & 0xf) << 20)
