/*
 * Directory Name Lookup Cache.
 *
 * Copyright 1992, Silicon Graphics, Inc.
 * All Rights Reserved.
 */

/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 * +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * 		PROPRIETARY NOTICE (Combined)
 * 
 * This source code is unpublished proprietary information
 * constituting, or derived under license from AT&T's UNIX(r) System V.
 * In addition, portions of such source code were derived from Berkeley
 * 4.3 BSD under license from the Regents of the University of
 * California.
 * 
 * 
 * 
 * 		Copyright Notice 
 * 
 * Notice of copyright on this source code product does not indicate 
 * publication.
 * 
 * 	(c) 1986,1987,1988,1989  Sun Microsystems, Inc
 * 	(c) 1983,1984,1985,1986,1987,1988,1989  AT&T.
 * 	          All rights reserved.
 *  
 */

#ifndef _FS_DNLC_H	/* wrapper symbol for kernel use */
#define _FS_DNLC_H	/* subject to change without notice */

/*#ident	"@(#)uts-3b2:fs/dnlc.h	1.4"*/
#ident	"$Revision: 1.4 $"

/*
 * This structure describes the elements in the cache of recently
 * looked up names.
 */

#define	NC_NAMLEN	31	/* maximum name segment length we bother with */

struct ncache {
	struct ncache *hash_next; 	/* hash chain, MUST BE FIRST */
	struct ncache **hash_prevp;	/* pointer to previous hash_next */
	struct ncache *lru_next; 	/* LRU chain */
	struct ncache *lru_prev;
	struct vnode *vp;		/* vnode the name refers to */
	struct vnode *dp;		/* vnode of parent of name */
	u_short vp_cap;			/* vnode pointer capability */
	u_short dp_cap;			/* parent directory capability */
	u_char namlen;			/* length of name */
	char name[NC_NAMLEN];		/* segment name */
	off_t offset;			/* offset in directory if fast */
	struct cred *cred;		/* credentials */
};

/*
 * Stats on usefulness of name cache.
 */
struct ncstats {
	u_int	hits;		/* hits that we can really use */
	u_int	misses;		/* cache misses */
	u_int	enters;		/* number of enters done */
	u_int	dbl_enters;	/* number of enters tried when already cached */
	u_int	long_enter;	/* long names tried to enter */
	u_int	long_look;	/* long names tried to look up */
	u_int	lru_empty;	/* LRU list empty */
	u_int	purges;		/* number of purges of cache */
	u_int	vfs_purges;	/* number of filesystem purges */
	u_int	removes;	/* number of removals by name */
	u_int	searches;	/* number of hash lookups */
	u_int	stale_hits;	/* hits that found old vnode stamp */
	u_int	steps;		/* hash chain steps for all searches */
};

#define	ANYCRED	((struct cred *) -1L)
#define	NOCRED	((struct cred *) 0L)

#ifdef _KERNEL
/*
 * External routines.
 */
struct cred;
struct vfs;
struct vnode;
void		dnlc_init(void);
void		dnlc_enter(struct vnode *, char *, struct vnode *,
			   struct cred *);
struct vnode	*dnlc_lookup(struct vnode *, char *, struct cred *);
void		dnlc_purge(void);
int		dnlc_purge1(void);
void		dnlc_purge_vp(struct vnode *);
int		dnlc_purge_vfsp(struct vfs *, int);
void		dnlc_remove(struct vnode *, char *);
void		dnlc_remove_vp(struct vnode *);

struct ncfastdata {
	char	*name;		/* name being operated upon */
	u_short	namlen;		/* name length, precomputed by lookuppn */
	u_short	flags;		/* name flags -- see pathname.h */
	off_t	offset;		/* offset in directory if invariant */
	void	*hash;		/* hash bucket pointer hint */
};

struct pathname;
struct vnode	*dnlc_lookup_fast(struct vnode *, char *, struct pathname *,
				  struct ncfastdata *, struct cred *);
void		dnlc_enter_fast(struct vnode *, struct ncfastdata *,
				struct vnode *, struct cred *);
void		dnlc_remove_fast(struct vnode *, struct ncfastdata *);

#endif	/* _KERNEL */
#endif	/* _FS_DNLC_H */
