#ifndef __SYS_ELF_H__
#define __SYS_ELF_H__

/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/
/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/
/*
 * Copyright 1992 Silicon Graphics,  Inc.
 * ALL RIGHTS RESERVED
 * 
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SGI
 * The copyright notice above does not evidence any  actual  or
 * intended  publication of this source code and material is an
 * unpublished work by Silicon  Graphics,  Inc.  This  material
 * contains CONFIDENTIAL INFORMATION that is the property and a
 * trade secret of Silicon Graphics, Inc. Any use,  duplication
 * or  disclosure  not  specifically  authorized  in writing by
 * Silicon Graphics is  strictly  prohibited.  THE  RECEIPT  OR
 * POSSESSION  OF  THIS SOURCE CODE AND/OR INFORMATION DOES NOT
 * CONVEY ANY RIGHTS TO REPRODUCE, DISCLOSE OR  DISTRIBUTE  ITS
 * CONTENTS,  OR  TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT
 * MAY DESCRIBE, IN WHOLE OR IN PART.
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND
 * Use, duplication or disclosure by the Government is  subject
 * to  restrictions  as  set  forth  in  FAR 52.227.19(c)(2) or
 * subparagraph (c)(1)(ii) of the Rights in Technical Data  and
 * Computer  Software  clause  at  DFARS 252.227-7013 and/or in
 * similar or successor clauses in the FAR, or the DOD or  NASA
 * FAR  Supplement.  Unpublished  --  rights reserved under the
 * Copyright Laws of the United States. Contractor/manufacturer
 * is Silicon Graphics, Inc., 2011 N. Shoreline Blvd., Mountain
 * View, CA 94039-7311
 */

/* WARNING:  This file is included in, and logically a part of,
 * /usr/include/elf.h.  The distribution of material between them
 * is artificial, generally reflecting the distinction between generic
 * and system-dependent, and may change.  Do not include this file
 * directly -- include /usr/include/elf.h instead.
 */

/*
 *	elf.h	file organization for Irix on MIPS and IA64.
 *	----------------------------------------------------
 *
 *	(1) /usr/include/elf.h
 *		This is generally processor and operating system independent
 *		information, typically common to all elf files.
 *
 *	(2) /usr/include/sys/elf.h
 *		This is generally Irix-specific but not processor-specific.  
 *		Names are often of the form <something>_IRIX_<something>.
 *
 *	(3) /usr/include/sys/mips/elf.h
 *		This file holds mips-specific information.  Names in the
 *		file are typically of the form <something>_MIPS_<something>.
 *
 *	(4) /usr/include/sys/ia64/elf.h
 *		This file holds ia64-specific information.  Names in the
 *		file are typically of the form <something>_IA64_<something>.
 *		Any names defined by Intel are also included in this file,
 *		and their form is <something>_IA_64_<something>.
 *
 *		Irix code should never reference anything with 
 *		an Intel defined name: <something>_IA_64_<something>.
 *	
 */

#include <sys/elftypes.h> 

/* 
 * Random constants
 */

#define _TEXT_ALIGN 0x10000
#define _DATA_ALIGN 0x10000
#define ELF_IRIX_MAXPGSZ (64*1024)
#define ELF_IRIX_MINPGSZ (0x1000)

#define MS_ALIAS        0x1	/* ???? */


/* ====================================================================
 *
 * Elf header
 *
 * ====================================================================
 */

/*
 * e_type
 */

#define ET_IR	(ET_LOPROC + 0)	 /* file is a 32-bit or 64-bit WHIRL file*/



/* ====================================================================
 *
 * Elf auxilliary header  
 *
 * ====================================================================
 *
 * The Elf auxilliary header is new to Irix as of v7.4 of the compilers.
 *
 * This is an Irix-specific extension to the elf header; it is used to
 * hold information that should go into the elf header (because it describes
 * the format of the file AND needs to be in the first page) but for which
 * there is no reasonable field in the elf header that we can use.
 *
 * This record, if present, immediately follows the Ehdr.  It's presence
 * is indicated by either (or both) of two distinct methods:
 *
 *	(1) e_ehsize in the Ehdr is larger than the actual elf header;
 *	    the difference being the size of the auxilliary header.
 *
 *	(2) The immediate 4 bytes after the Ehdr holding the bytes 'IRIX'
 *
 * Or stated a little more precisely:
 *
 *	Let EH_P  be a pointer to the elfheader.
 *	Let EHX_P be a pointer to the first byte after the elfheader.
 *	
 *	(1)  EHX_P		== EH_P + sizeof(Elf_Ehdr)
 *	(2)  *EHX_P		== 'Irix'
 *	(3)  EHX_P->ex_exhsize	== EH_P->e_ehsize    - sizeof(Elf_Ehdr)
 *	(2)  EH_P->e_ehsize	== sizeof(Elf_EXhdr) + sizeof(Elf_Ehdr) 
 * 
 * ====================================================================
 */

#define	EX_NIDENT	4

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS) )
typedef struct {
	unsigned char	ex_ident[EX_NIDENT];	/* magic string = 'IRIX'*/
	Elf32_Half	ex_exhsize;		/* exhdr length in bytes*/
	Elf32_Half	ex_version;		/* exhdr version number	*/

	/* added in ex_version 1 */
	Elf32_Word	ex_eversion;		/* elf file version number */

} Elf_EXhdr;
#endif

/*
 *	ex_ident
 *
 *	Always defined to be the string 'IRIX'
 */
#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS) )
#define	IS_ELFX(exhdr) ( 			\
		(exhdr).ex_ident[0]='I'  &&	\
		(exhdr).ex_ident[0]='R'  &&	\
		(exhdr).ex_ident[0]='I'  &&	\
		(exhdr).ex_ident[0]='X'  &&	\
		)
#endif

/*
 *	ex_version
 *
 *	Indicates the revision level of the Elf_EXhdr record.  In
 *	particular, whenever a new field is added to the Elf_EXhdr
 *	record, we bump the ex_version number.  This allows programs
 *	to know which fields in the record are valid.
 *
 *	EX_VERSION	CHANGE
 *	----------	-----------------------------------------------
 *	0		Initial checkin.
 *	1		Added	ex_version field.
 *
 */
#define	EX_VERSION	1

/*
 *	ex_eversion
 *
 *	This is the version number of the elf file format.  Every
 *	time we change/modify/add/delete anthing from the elf file
 *	format we increas this number.
 *
 *	It would be better to use the e_version field out of the actual
 *	Elf header record, but that field is, unfortunately, defined
 *	to be 1.  Therefore, we created a new version field.
 *
 *	Most object file utilities should not have to look at this
 *	field.
 *
 *	EX_EVERSION	CHANGE
 *	-----------	-----------------------------------------------
 *	0		Initial checkin
 *	1		Removed new section types from SHT_MIDPROC
 *			range and put back in the SHT_LOPROC range.
 *
 */
#define	EX_EVERSION	1



/*
 * e_flags -- used by Irix
 */

#define	EF_IRIX_ABI64		0x00000010	
	/* If the bit is set   (1) then this is a 64bit ABI file
	 * If the bit is clear (0) then this is a 32bit ABI file.
	 *
	 * If set, this is a 64bit ABI elf file using a LP64 data model.
	 * And if this is a WHIRL file, (ie. e_type == ET_IR) then this 
	 * holds 64bit-format WHIRL code.
	 *
	 * Similarly, if not set, this is a 32bit ABI elf file using a ILP32 
	 * data model.  And if it is a WHIRL file, (ie. e_type == ET_IR) this 
	 * holds 32bit format WHIRL code.
	 *
	 * NOTE: In the past, we implied 32bit ABI vs. 64bit ABI from
	 * the value of the ELFCLASS32 or ELFCLASS64.  
	 */


/* ====================================================================
 *
 * Program header
 *
 * ====================================================================
 */

/* ====================================================================
 *
 * Section Headers
 *
 * ====================================================================
 */

/* 
 * Special Irix section indices
 */

#define SHN_IRIX_ACOMMON	(SHN_LOPROC + 0)
#define SHN_IRIX_TEXT		(SHN_LOPROC + 1)
#define SHN_IRIX_DATA		(SHN_LOPROC + 2)
#define SHN_IRIX_SCOMMON	(SHN_LOPROC + 3)
#define SHN_IRIX_SUNDEFINED	(SHN_LOPROC + 4)
#define SHN_IRIX_LCOMMON	(SHN_LOPROC + 5)
#define SHN_IRIX_LUNDEFINED	(SHN_LOPROC + 6)


/*
 * sh_type
 *
 *	Irix section types are allocated starting at 
 *	SHT_LOPROC == 0x70000000 and increasing values.
 *
 *	NOTE: Not all of the defined section types are
 *	valid on all processor type nor on all versions
 *	of Irix.
 */
#define SHT_IRIX_LIBLIST	(SHT_LOPROC + 0)
#define SHT_IRIX_MSYM		(SHT_LOPROC + 1)
#define SHT_IRIX_CONFLICT	(SHT_LOPROC + 2)
#define SHT_IRIX_GPTAB		(SHT_LOPROC + 3)
#define SHT_IRIX_UCODE		(SHT_LOPROC + 4)
#define SHT_IRIX_DEBUG          (SHT_LOPROC + 5)
#define SHT_IRIX_REGINFO        (SHT_LOPROC + 6)

#ifdef __osf__
#define	SHT_IRIX_PACKAGE	(SHT_LOPROC + 7)
#define	SHT_IRIX_PACKSYM	(SHT_LOPROC + 8)
#endif /* __osf__ */

#define SHT_IRIX_RELD		(SHT_LOPROC + 9)
#define SHT_IRIX_DONTUSE	(SHT_LOPROC + 10)

#define SHT_IRIX_IFACE		(SHT_LOPROC + 11)
#define SHT_IRIX_CONTENT	(SHT_LOPROC + 12)
#define SHT_IRIX_OPTIONS	(SHT_LOPROC + 13)

#define SHT_IRIX_SHDR		(SHT_LOPROC + 16)
#define SHT_IRIX_FDESC		(SHT_LOPROC + 17)
#define SHT_IRIX_EXTSYM		(SHT_LOPROC + 18)
#define SHT_IRIX_DENSE		(SHT_LOPROC + 19)
#define SHT_IRIX_PDESC		(SHT_LOPROC + 20)
#define SHT_IRIX_LOCSYM		(SHT_LOPROC + 21)
#define SHT_IRIX_AUXSYM		(SHT_LOPROC + 22)
#define SHT_IRIX_OPTSYM		(SHT_LOPROC + 23)
#define SHT_IRIX_LOCSTR		(SHT_LOPROC + 24)
#define SHT_IRIX_LINE		(SHT_LOPROC + 25)
#define SHT_IRIX_RFDESC		(SHT_LOPROC + 26)

#define SHT_IRIX_DELTASYM	(SHT_LOPROC + 27)
#define SHT_IRIX_DELTAINST	(SHT_LOPROC + 28)
#define SHT_IRIX_DELTACLASS	(SHT_LOPROC + 29)

#define SHT_IRIX_DWARF		(SHT_LOPROC + 30)
#define SHT_IRIX_DELTADECL	(SHT_LOPROC + 31)
#define SHT_IRIX_SYMBOL_LIB	(SHT_LOPROC + 32)
#define SHT_IRIX_EVENTS        	(SHT_LOPROC + 33)
#define SHT_IRIX_TRANSLATE     	(SHT_LOPROC + 34)
#define SHT_IRIX_PIXIE     	(SHT_LOPROC + 35)
#define SHT_IRIX_XLATE		(SHT_LOPROC + 36)
#define SHT_IRIX_XLATE_DEBUG	(SHT_LOPROC + 37)
#define SHT_IRIX_WHIRL		(SHT_LOPROC + 38)
#define SHT_IRIX_EH_REGION	(SHT_LOPROC + 39)
#define SHT_IRIX_XLATE_OLD	(SHT_LOPROC + 40)
#define SHT_IRIX_PDR_EXCEPTION	(SHT_LOPROC + 41)

/* The next five section types are valid only on irix/ia64  */
#define SHT_IRIX_EXT            (SHT_LOPROC + 42)
#define SHT_IRIX_UNWIND         (SHT_LOPROC + 43)
#define SHT_IRIX_UNWIND_INFO    (SHT_LOPROC + 44)
#define SHT_IRIX_PLT            (SHT_LOPROC + 45)
#define SHT_IRIX_PLTOFF         (SHT_LOPROC + 46)


#define SHT_IRIX_NUM		47
	/* SHT_IRIX_NUM is one more than the highest offset to SHT_LOCPROC;
	 * it is the number of extensions, but is *NOT* a section type.
	 */

/*
 * sh_flags
 *
 *	We'll try and use the same flags for both irix/mips and irix/ia64.
 *	If this ever becomes a problem, we'll have to split them into
 *	two different namespaces.
 */
#define SHF_IRIX_GPREL          0x10000000
#define SHF_IRIX_MERGE          0x20000000
#define SHF_IRIX_ADDR           0x40000000	/* Reserved for future	*/
#define SHF_IRIX_STRINGS        0x80000000	/* Reserved for future	*/
#define SHF_IRIX_NOSTRIP        0x08000000
#define SHF_IRIX_LOCAL          0x04000000
#define SHF_IRIX_NAMES          0x02000000	/* Reserved for future	*/
#define SHF_IRIX_NODUPE         0x01000000	/* Reserved for future	*/


/* ====================================================================
 *
 * Symbol table
 *
 * ====================================================================
 */

/*
 * Special Irix st_other
 */
#define STO_DEFAULT		0x0
#define STO_INTERNAL		0x1
#define STO_HIDDEN		0x2
#define STO_PROTECTED		0x3
#define STO_OPTIONAL		0x4
#define STO_SC_ALIGN_UNUSED	0xff	/* No longer used 		*/

/*
 * Special Irix st_info
 */
#define STB_SPLIT_COMMON	(STB_LOPROC+0)

/* ====================================================================
 *
 * .IRIX.options Section
 *
 * sh_type:	SHT_MIPS_OPTIONS or SHT_IA64_OPTIONS
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct
{
	Elf64_Byte	kind;		/* determines interpretation of the */
					/* variable part of descriptor	    */
	Elf64_Byte	size;		/* size of descriptor, incl. header */
	Elf64_Section	section;	/* section header index of section  */
					/* affected, 0 for global options   */
	Elf64_Word	info;		/* Kind-specific information	    */
} Elf_Options;
#endif


/*
 * 	Elf_Options.kind		Options descriptor kinds
 */
#define ODK_NULL                 0      /* Undefined */
#define ODK_MIPS_REGINFO         1      /* MIPS Register usage information */
#define ODK_REGINFO         1      /* MIPS Register usage information */
#define ODK_MIPS_EXCEPTIONS      2      /* MIPS Exception processing options  */
#define ODK_EXCEPTIONS		 2      /* MIPS Exception processing options  */
#define ODK_PAD                  3      /* Section padding options */
#define ODK_MIPS_HWPATCH         4      /* MIPS Hardware workarounds performed*/
#define ODK_HWPATCH		 4      /* MIPS Hardware workarounds performed*/
#define ODK_FILL                 5      /* The fill value used by the linker */
#define ODK_TAGS                 6      /* Space for desktop tools to write */
#define ODK_MIPS_HWAND           7      /* HW workarounds.'AND' when merging */
#define ODK_HWAND		 7      /* HW workarounds.'AND' when merging */
#define ODK_MIPS_HWOR            8      /* HW workarounds.'OR'  when merging */
#define ODK_HWOR		 8      /* HW workarounds.'OR'  when merging */
#define ODK_GP_GROUP             9      /* GP group for text/data sections */
#define ODK_IDENT               10      /* ID information */
#define ODK_IA64_REGINFO        11      /* TO BE CHANGED! */
#define ODK_PAGESIZE            12      /* Alternate segment page size */


#define ODK_IA64_EXCEPTIONS     12      /* NOT USED! */
#define ODK_IA64_HWAND          13      /* NOT USED! */
#define ODK_IA64_HWOR           14      /* NOT USED! */


/*
 *      Elf_Options.kind == ODK_PAD
 *
 *      masks for Elf_Options.info
 */
#define OPAD_PREFIX	0x1	
#define OPAD_POSTFIX	0x2
#define OPAD_SYMBOL	0x4

/*
 *      Elf_Options.kind == ODK_GP_GROUP
 *
 *      masks for Elf_Options.info
 */
#define OGP_GROUP	0x0000ffff	/* GP group number */
#define OGP_SELF	0x00010000	/* Self-contained GP groups */

/*
 *      Elf_Options.kind == ODK_PAGESIZE
 *
 *      masks for Elf_Options.info
 */
#define OPS_DATA	0x000000ff	/* Data (static/heap) pagesize */
#define OPS_STACK	0x0000ff00	/* Stack data pagesize */
#define OPS_TEXT	0x00ff0000	/* Text pagesize */
/*
 *	These bytes are exponents of 2, so 2**b gives the actual
 *	pagesize desired, except for the following three special
 *	values:
 */
#define OPS_NOSPEC	0x00	/* No specification */
#define OPS_DEFAULT	0x01	/* Use system default */
#define OPS_MAXSIZE	0x02	/* Use system maximum */
/* Other values smaller than the system minimum are to be ignored. */

/*
 *      Elf_Options.kind == ODK_MIPS_HWAND
 *      Elf_Options.kind == ODK_MIPS_HWOR
 *      Elf_Options.kind == ODK_IA64_HWAND
 *      Elf_Options.kind == ODK_IA64_HWOR
 *
 *	These option descriptors are used by the kernel and 
 *	various tools to track HW specific bugs/issues/features.
 *
 *	In addition to using the info word, it can also use the
 *	the following 8-bytes for more bit masks, if needed.
 *	This record is defined below as Elf_Options_Hw.
 *
 *	The ODK_HWAND option is used for those bits that should be
 *	AND'd by the linker when merged and the ODK_HWOR option is
 *	used for those bits that should be OR'd by the linker.
 *
 *	EXAMPLE:
 *      --------
 *	Suppose the kernel wanted to know if a program used CMOV 
 *	instructions.  We might define the following :
 *		#define	OHWA0_CMOV_CHECKED	0x00000001
 *		#define OHWO0_CMOV_PRESENT	0x00000001
 *
 *	Assume, for this example, that  'eo_hw_and'   and   'eo_hw_or'
 *	are pointers to the two different options.  If the compiler 
 *	checked an object and found it generated no CMOV instructions 
 *	it would generate the following:
 *		eo_hw_and->info |= OHWA0_CMOV_CHECKED;
 *		eo_hw_or->info  |= OHWO0_CMOV_PRESENT;
 *	
 *	The linker would merge these two options by AND'ing all HW_AND
 *	options and OR'ing all HW_OR options.  After all copies are merged
 *	by the linker the meaning of the fields would be:
 *
 *	    CHECKED  PRESENT	MEANING
 *		1	1	All objects checked, CMOVS in binary.
 *		1	0	All objects checked, no CMOVS in binary.
 *		0	1	Some objects checked, CMOVS in binary.
 *		0	0	Some objects checked, CMOVS may be in binary.
 *
 *	NOTES:
 *	------
 *	(1) Dummy defines are included below to indicate naming conventions
 *	    for these fields.
 *	(2) Often it will be appropriate to have both a CHECKED bit and
 *	    a PRESENT bit, but it is not required.  It is up to the 
 *	    implementors to make the choice.
 *	(3) This is a SGI specific feature.  These option words should
 *	    be ignored by any other vendor.
 *	(4) {A|O}0 refers to the info field;
 *	    {A|O}1 refers to the hwp_flags1 field;
 *	    {A|O}2 refers to the hwp_flags2 field.
 *      (5) Any name can be used for bit define's but here is a suggested
 *          naming convention:
 *              OHW{A|O}{0|1|2}_<topic>_{CHECKED|PRESENT}
 *
 * ====================================================================
 *	The defines for HWAND and HWOR are found in:
 *		/usr/include/sys/mips/elf.h
 *		/usr/include/sys/ia64/elf.h
 * ====================================================================
 */
#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct
{
	Elf32_Word	hwp_flags1;	/* extra flags */
	Elf32_Word	hwp_flags2;	/* extra flags */
} Elf_Options_Hw;
#endif


/* ====================================================================
 *
 * .interfaces Section
 *
 * sh_type:	SHT_*_IFACE
 * sh_link:	section header index of associated symbol table
 * sh_info:	0
 * attributes:	SHF_ALLOC, SHF_IRIX_NOSTRIP
 *
 * --- WARNING ---- WARNING ---- WARNING ---- WARNING ---- WARNING ----
 *
 *      TBD:  .interface section needs to be processor specific
 *		In particular the Elf_Ifd.fpmask won't work.
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))

typedef struct {
    Elf64_Word symbol;		/* symbol table index of subprogram, or 0 */
    Elf64_Half attrs;		/* Attributes: See list below */
    Elf64_Byte pcnt;		/* Parameter count */
    Elf64_Byte fpmask;		/* bit on indicates an FP parameter register */
} Elf_Ifd;

typedef Elf_Ifd Elf_Interface_Descriptor;  /* for compatibility */

/* Flags that can be set in the 'attrs' field of Elf_Interface_Descriptor */
#define SA_PROTOTYPED	0x8000	/* Does def or ref have prototype ? */
#define SA_VARARGS	0x4000	/* Is this a varargs subprogram ? */
#define SA_PIC		0x2000	/* Are memory references PIC? */
#define SA_DSO_ENTRY	0x1000	/* Is subprogram valid DSO entry? */
#define SA_ADDRESSED	0x0800	/* Is subprogram address taken? */
#define SA_FUNCTION	0x0400	/* Does subprogram return a result? */
#define SA_NESTED	0x0200	/* Is subprogram nested? */
#define SA_IGNORE_ERROR	0x0100	/* Ignore consistency errors? */
#define SA_DEFINITION	0x0080	/* Is this a definition (no just call)? */
#define SA_AT_FREE	0x0040	/* Is the at register free at all branches? */
#define SA_FREE_REGS	0x0020	/* Free register mask precedes parm profile */
#define SA_PARAMETERS	0x0010	/* Parameter profile follows descriptor? */
#define SA_ALTINTERFACE 0x0008	/* Alternate descriptor follows? */

/* Parameter descriptor masks */
#define PDM_TYPE	0x00ff	/* Fundamental type of parameter */
#define PDM_REFERENCE	0x4000	/* Reference parameter ? */
#define PDM_SIZE	0x2000	/* Followed by explicit 32-bit byte count? */
#define PDM_Qualifiers	0x0f00	/* Count of type qualifiers << 8 */

/* Parameter descriptor mask flags */
#define PDMF_REFERENCE  0x40
#define PDMF_SIZE       0x20
#define PDMF_Qualifiers 0x0f

/* Fundamental Parameter Types */
#define FT_unknown         0x0000
#define FT_signed_char     0x0001
#define FT_unsigned_char   0x0002
#define FT_signed_short    0x0003
#define FT_unsigned_short  0x0004
#define FT_signed_int32    0x0005
#define FT_unsigned_int32  0x0006
#define FT_signed_int64    0x0007
#define FT_unsigned_int64  0x0008
#define FT_pointer32       0x0009
#define FT_pointer64       0x000a
#define FT_float32         0x000b
#define FT_float64         0x000c
#define FT_float128        0x000d
#define FT_complex64       0x000e
#define FT_complex128      0x000f
#define FT_complex256      0x0010
#define FT_void            0x0011
#define FT_bool32          0x0012
#define FT_bool64          0x0013
#define FT_label32         0x0014
#define FT_label64         0x0015
#define FT_struct          0x0020
#define FT_union           0x0021
#define FT_enum            0x0022
#define FT_typedef         0x0023
#define FT_set             0x0024
#define FT_range           0x0025
#define FT_member_ptr      0x0026
#define FT_virtual_ptr     0x0027
#define FT_class           0x0028

/* Parameter Qualifiers (aka Modifiers)  */
#define MOD_pointer_to     0x01
#define MOD_reference_to   0x02
#define MOD_const          0x03
#define MOD_volatile       0x04
#define MOD_function       0x80
#define MOD_array_of       0x81

#endif

/* ====================================================================
 *
 * .liblist Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct
{
	Elf32_Word	l_name;
	Elf32_Word	l_time_stamp;
	Elf32_Word	l_checksum;
	Elf32_Word	l_version;
	Elf32_Word	l_flags;
} Elf32_Lib;
#endif


/*
 * l_flags  These flags exist in each liblist entry.
 * The dynamic section flag bit RHF_SGI_ONLY
 * must be on for most of these to be honored by rld(1).
 * LL_DELAY_LOAD is honored regardless of RHF_SGI_ONLY.
 * If RHF_SGI_ONLY is not on no version matches are done
 * (just as if LL_IGNORE_INT_VER were on).
 * 
 * Recall that a liblist entry has a version string with a single version,
 * while the dynamic section on a DSO has a version string with
 * possibly multiple versions listed (separated by :).  man dso for
 * more details on multiple versions.
 *
 * By default version matches do NOT consider the minor version number.
 *
 * LL_EXACT_MATCH means (if on) that only a DSO with exactly the
 * checksum and timestamp and version from the liblist entry
 * (which were gotten at static-link time but might have
 * been updated by rqs(1)) will match.
 *
 * LL_REQUIRE_MINOR means (if on) that the version match must
 * match the minor version number.
 *
 * LL_NONE is just a convenient name requesting the default checks.
 *
 * LL_DELAY_LOAD means  (if on) that  rld will not load the DSO
 * refered to by the liblist entry till something in the DSO
 * is called.
 *
 * LL_EXPORTS is ignored by rld. It informs that ld exported symbols 
 * from the liblist-entry DSO into the main DSO (that this
 * liblist entry is part of).
 *
 * LL_IGNORE_INT_VER means (if on) that all version checks
 * should be suppressed and the DSO accepted based on
 * name alone.
 *
 * LL_DELTA means that the DSO referenced is Delta C++.
 *
 */

#define LL_NONE			0
#define LL_EXACT_MATCH		0x1 
#define LL_IGNORE_INT_VER	0x2
#define LL_REQUIRE_MINOR	0x4
#define LL_EXPORTS		0x8
#define LL_DELAY_LOAD		0x10

#define LL_DELTA                0x20

/* ====================================================================
 *
 * .conflict Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef Elf32_Addr Elf32_Conflict;
#endif

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
extern Elf32_Conflict	_ConflictList [];
#endif

#define RLD_VERSION            1


/* ====================================================================
 *
 * .got Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct
{
	Elf32_Addr	g_index;
} Elf32_Got;


typedef struct
{
	Elf64_Addr	g_index;
} Elf64_Got;

extern Elf32_Got	_GlobalOffsetTable [];
extern Elf64_Got	_GlobalOffsetTable64 [];

#endif


/* ====================================================================
 *
 * .package Section
 *
 * Multiple package entries for the same package are allowed
 * in order to express out of order symbols in a package.
 *
 * ====================================================================
 */

#ifdef __osf__

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct {
	Elf32_Word	pkg_name;	/* index into String Space of name */
	Elf32_Word	pkg_version;	/* index into String Space of version string */
	Elf32_Half	pkg_flags;	/* package flags */
} Elf32_Package;
#endif

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
extern Elf32_Package	_PackageList [];
#endif


/*
 * pkg_name --
 * index of a string that identifies the name of this package
 * implementation, which cannot be the null string; the offset is in
 * bytes of a zero terminated string from the start of the .dynstr section
 * pkg_version --
 * index of a string that identifies the version of this package
 * implementation, which may be the null string; the offset is in
 * bytes of a zero terminated string from the start of the .dynstr section
 * pkg_flags --
 * export flag means package is exported, import flag means package is imported,
 * both flags must be set if a package is exported and is also used by other
 * packages within the shared library.  continuance flag means that this
 * package entry defines additional symbols for a previously defined
 * package.  continuance entries must exactly match the original entry in each
 * field, except for the pkg_start, pkg_count, and continuance flag in the pkg_flags.
 * The conflict flag is a possibility for future support for symbol preemption.
 */

/*
 * pkg_flags
 */

#define PKGF_EXPORT	0x1
#define PKGF_IMPORT	0x2
/* #define PKGF_CONFLICT	0x8 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef Elf32_Word Elf32_Package_Symbol;
#endif

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
#define	PACKSYM_NULL_INDEX	((Elf32_Word) 0)
#endif

#endif /* __osf__ */


/* ====================================================================
 *
 * .dynamic Section
 *
 * --- WARNING ---- WARNING ---- WARNING ---- WARNING ---- WARNING ----
 *
 *      TBD:  The .dynamic section has not yet been prepared for
 *            use in both Irix/mips and Irix/ia64
 *
 *      I believe that most of the tags will be the same in both
 *      Irix/mips and Irix/ia64, but it still needs to be reviewed.
 * 
 *      For now, we'll document them only in mips/elf.h
 * ====================================================================
 */


/* ====================================================================
 * ====================================================================
 *
 * 64-bit declarations
 *
 * ====================================================================
 * ====================================================================
 */

#define ELF64_FSZ_ADDR		8
#define ELF64_FSZ_HALF		2
#define ELF64_FSZ_OFF		8
#define ELF64_FSZ_SWORD		4
#define ELF64_FSZ_WORD		4
#define ELF64_FSZ_SXWORD	8
#define ELF64_FSZ_XWORD		8


/* ====================================================================
 *
 * ELF header
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct {
	unsigned char	e_ident[EI_NIDENT];	/* ident bytes */
	Elf64_Half	e_type;			/* file type */
	Elf64_Half	e_machine;		/* target machine */
	Elf64_Word	e_version;		/* file version */
	Elf64_Addr	e_entry;		/* start address */
	Elf64_Off	e_phoff;		/* phdr file offset */
	Elf64_Off	e_shoff;		/* shdr file offset */
	Elf64_Word	e_flags;		/* file flags */
	Elf64_Half	e_ehsize;		/* sizeof ehdr */
	Elf64_Half	e_phentsize;		/* sizeof phdr */
	Elf64_Half	e_phnum;		/* number phdrs */
	Elf64_Half	e_shentsize;		/* sizeof shdr */
	Elf64_Half	e_shnum;		/* number shdrs */
	Elf64_Half	e_shstrndx;		/* shdr string index */
} Elf64_Ehdr;
#endif


/* ====================================================================
 *
 * Program header
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct {
	Elf64_Word	p_type;		/* entry type */
	Elf64_Word	p_flags;	/* entry flags */
	Elf64_Off	p_offset;	/* file offset */
	Elf64_Addr	p_vaddr;	/* virtual address */
	Elf64_Addr	p_paddr;	/* physical address */
	Elf64_Xword	p_filesz;	/* file size */
	Elf64_Xword	p_memsz;	/* memory size */
	Elf64_Xword	p_align;	/* memory/file alignment */
} Elf64_Phdr;
#endif


/* ====================================================================
 *
 * Section Headers
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct {
	Elf64_Word	sh_name;	/* section name */
	Elf64_Word	sh_type;	/* SHT_... */
	Elf64_Xword	sh_flags;	/* SHF_... */
	Elf64_Addr	sh_addr;	/* virtual address */
	Elf64_Off	sh_offset;	/* file offset */
	Elf64_Xword	sh_size;	/* section size */
	Elf64_Word	sh_link;	/* misc info */
	Elf64_Word	sh_info;	/* misc info */
	Elf64_Xword	sh_addralign;	/* memory alignment */
	Elf64_Xword	sh_entsize;	/* entry size if table */
} Elf64_Shdr;
#endif


/* ====================================================================
 *
 * .symtab Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct {
	Elf64_Word	st_name;
	unsigned char	st_info;	/* bind, type: ELF_64_ST_... */
	unsigned char	st_other;
	Elf64_Half	st_shndx;	/* SHN_... */
	Elf64_Addr	st_value;
	Elf64_Xword	st_size;
} Elf64_Sym;
#endif


/*	The macros compose and decompose values for S.st_info
 *
 *	bind = ELF64_ST_BIND(S.st_info)
 *	type = ELF64_ST_TYPE(S.st_info)
 *	S.st_info = ELF64_ST_INFO(bind, type)
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
#define ELF64_ST_BIND(info)		((info) >> 4)
#define ELF64_ST_TYPE(info)		((info) & 0xf)
#define ELF64_ST_INFO(bind,type)	(((bind)<<4)+((type)&0xf))
#endif


/* ====================================================================
 *
 * .rel, .rela Section
 *
 * WARNING:  The Elf64_Rel and Elf64_Rela structures must be identical
 * except for the r_addend field.
 *
 * --- WARNING ---- WARNING ---- WARNING ---- WARNING ---- WARNING ----
 *
 *      TBD:  .rel, .rela sections needs to be processor specific
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))

typedef struct {
	Elf64_Addr	r_offset;
	Elf64_Word	r_sym;		/* Symbol index */
	Elf64_Byte	r_ssym;		/* Special symbol */
	Elf64_Byte	r_type3;	/* 3rd relocation op type */
	Elf64_Byte	r_type2;	/* 2nd relocation op type */
	Elf64_Byte	r_type;		/* 1st relocation op type */
} Elf64_Rel;

typedef struct {
	Elf64_Addr	r_offset;
	Elf64_Word	r_sym;		/* Symbol index */
	Elf64_Byte	r_ssym;		/* Special symbol */
	Elf64_Byte	r_type3;	/* 3rd relocation op type */
	Elf64_Byte	r_type2;	/* 2nd relocation op type */
	Elf64_Byte	r_type;		/* 1st relocation op type */
	Elf64_Sxword	r_addend;
} Elf64_Rela;

/* Values for the r_ssym field: */
typedef enum {
    RSS_UNDEF	= 0,	/* Undefined */
    RSS_GP	= 1,	/* Context pointer (gp) value */
    RSS_GP0	= 2,	/* gp value used to create object being relocated */
    RSS_LOC	= 3	/* Address of location being relocated */
} Elf64_Rel_Ssym;

#endif

/*	The macros compose and decompose values for Rel.r_info, Rela.f_info
 *
 *	sym = ELF64_R_SYM(R.r_info)
 *	type = ELF64_R_TYPE(R.r_info)
 *	R.r_info = ELF64_R_INFO(sym, type)
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
#define ELF64_R_SYM(info)	((info)>>8)
#define ELF64_R_TYPE(info)	((unsigned char)(info))
#define ELF64_R_INFO(sym,type)	(((sym)<<8)+(unsigned char)(type))
#endif


/* ====================================================================
 *
 * .conflict Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef Elf64_Addr Elf64_Conflict;

extern Elf64_Conflict   _ConflictList64 [];
#endif


/* ====================================================================
 *
 * .dynamic Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct {
	Elf64_Xword	d_tag;
	union {
		Elf64_Xword	d_val;
		Elf64_Addr	d_ptr;
	} d_un;
} Elf64_Dyn;
#endif


/* ====================================================================
 *
 * .liblist Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct
{
        Elf64_Word      l_name;
        Elf64_Word      l_time_stamp;
        Elf64_Word      l_checksum;
        Elf64_Word      l_version;
        Elf64_Word      l_flags;
} Elf64_Lib;
#endif



/* ====================================================================
 *
 * Processor Specific Information
 *
 *	Processor specific information is in contained in the files
 *
 *		/usr/include/sys/mips/elf.h
 *		/usr/include/sys/ia64/elf.h
 *
 *	By default, only the correct file for the specified target
 *	processor is include; but for those situations where programs
 *	need declarations for both mips and ia64 target objects,
 *	define the following variable on the compile commandline:
 *
 *		-D__MIPS_AND_IA64_ELF_H
 *
 *	in order to include both processor specific files.
 *
 * ====================================================================
 */
#if defined(__mips) || defined(__MIPS_AND_IA64_ELF_H)
#include <sys/mips/elf.h>
#endif

#if defined(__ia64) || defined(__MIPS_AND_IA64_ELF_H)
#include <sys/ia64/elf.h>
#endif


#define DT_IRIX_LTR			0x70000033
#define DT_IRIX_LPT			0x70000034
#define DT_IRIX_LPT_IDX			0x70000035
#define DT_IRIX_LPT_NO			0x70000036
#define DT_IRIX_PLT			0x70000037
#define DT_IRIX_PLT_NON_PREEMPT_IDX	0x70000038
#define DT_IRIX_PLT_NON_PREEMPT_NO	0x70000039
#define DT_IRIX_PLT_IN_DYNSYM_IDX	0x7000003A
#define DT_IRIX_PLT_IN_DYMSYM_NO	0x7000003B
#define DT_IRIX_PLT_PROTECTED_IDX	0x7000003C
#define DT_IRIX_PLT_PROTECTED_NO	0x7000003D
#define DT_IRIX_PLT_PREEMPT_IDX		0x7000003E
#define DT_IRIX_PLT_PREEMPT_NO		0x7000003F
#define DT_IRIX_DLT			0x70000040
#define DT_IRIX_DLT_NON_PREEMPT_IDX	0x70000041
#define DT_IRIX_DLT_NON_PREEMPT_NO	0x70000042
#define DT_IRIX_DLT_IN_DYNSYM_IDX	0x70000043
#define DT_IRIX_DLT_IN_DYMSYM_NO	0x70000044
#define DT_IRIX_DLT_PROTECTED_IDX	0x70000045
#define DT_IRIX_DLT_PROTECTED_NO	0x70000046
#define DT_IRIX_DLT_PREEMPT_IDX		0x70000047
#define DT_IRIX_DLT_PREEMPT_NO		0x70000048
#define DT_IRIX_DSYM_PLT_IDX		0x70000049
#define DT_IRIX_DSYM_DLT_IDX		0x7000004A



#endif /* __SYS_ELF_H__ */
