/*
 * Copyright 1995, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef	__SYS_EXTACCT_H
#define	__SYS_EXTACCT_H
#ifdef __cplusplus
extern "C" {
#endif

#ident "$Revision: 1.6 $"

#include <sys/types.h>

/*
 * Structures and other useful information about SGI extended accounting
 *
 * NOTE: the structures for the actual extended accounting records (both
 *       process and session) are contained in sat.h.
 */

#define PROC_ACCT_VERSION  1
#define SESS_ACCT_VERSION  1

/*
 * Service Provider Information. Strings should be null-terminated.
 */
typedef struct acct_spi {	/* Service provider information */
	char	spi_company[8];		/* Name of company providing service */
	char	spi_initiator[8];	/* Name of machine initiating job */
	char	spi_origin[16];		/* Name of queue */
	char	spi_spi[16];		/* Available for service provider */
	char	spi_spare[16];		/*   reserved */
} acct_spi_t;

/*
 * Accounting timers. These are all in units of nanoseconds.
 */
typedef struct acct_timers {	/* Accounting times */
	accum_t	 ac_utime;		/* User time */
	accum_t	 ac_stime;		/* System time */
	accum_t	 ac_bwtime;		/* Time waiting for block I/O */
	accum_t	 ac_rwtime;		/* Time waiting for raw I/O */
	accum_t	 ac_qwtime;		/* Time waiting on run queue */
	accum_t	 ac_spare[3];		/*   reserved */
} acct_timers_t;

/*
 * Accounting counters
 */
typedef struct acct_counts {	/* Accounting counts */
	accum_t	 ac_mem;		/* "Memory usage" (weird acct units) */
	accum_t  ac_swaps;		/* # swaps */
	accum_t  ac_chr;		/* # bytes read */
	accum_t  ac_chw;		/* # bytes written */
	accum_t  ac_br;			/* # blocks read */
	accum_t  ac_bw;			/* # blocks written */
	accum_t  ac_syscr;		/* # read syscalls */
	accum_t  ac_syscw;		/* # write syscalls */
	accum_t  ac_disk;		/* not implemented */
	accum_t  ac_spare[3];		/*   reserved */
} acct_counts_t;


#ifdef __cplusplus
}
#endif  /* __cplusplus */
#endif  /* _SYS_EXTACCT_H */
