/**************************************************************************
 *                                                                        *
 *               Copyright (C) 1993, Silicon Graphics, Inc.               *
 *                                                                        *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *                                                                        *
 **************************************************************************/

/*
 * fpanel.h
 * Header for ioctl to pass commands to the i2c bus for the Presenter
 * flat panel display.
 * $Revision: 1.11 $
 */

/* for newport and xz ioctl */
#define PANEL_I2C_CMD 21011

/* The ioctl call to send a command to the I2C bus from an application 
 * (after it's opened /dev/graphics and attached the graphics board) is:
 *
 * ioctl("/dev/graphics", PANEL_I2C_CMD, arg);
 *
 * Where "arg" must be an I2C_CMD_ARG struct.  "cmd" contains the operation
 * to perform (see #define's below), "args" is a pointer to the arguments
 * implicitly defined by "cmd", and the called I2C operation may return an
 * integer value by placing it in "return_val".
 */

typedef struct I2C_CMD_ARG
{
  int cmd;
  void *args;
  int return_val;
} I2C_CMD_ARG;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_I2C_CMD_ARG{
  	int cmd;
  	app32_ptr_t args;
  	int return_val;
} irix5_I2C_CMD_ARG;
#endif


/* valid i2c cmd's */

#define I2C_PANEL_ON          0
#define I2C_PANEL_OFF         1
#define I2C_BL_UP             2
#define I2C_BL_DOWN           3
#define I2C_AUDIO_VOL_LEFT    4
#define I2C_AUDIO_VOL_RIGHT   5
#define I2C_AUDIO_BASS        6
#define I2C_AUDIO_TREBLE      7
#define I2C_AUDIO_SWITCH      8
#define I2C_BL_VAL            9
#define I2C_LEFT_VAL         10
#define I2C_RIGHT_VAL        11
#define I2C_BASS_VAL         12
#define I2C_TREBLE_VAL       13
#define I2C_MUTE_VAL         14
#define I2C_SET_BIT          15
#define I2C_BIT_VAL          16

/* id report from flat panel 3=conrona and 4=ichiban */
#define PANEL_XGA_MITSUBISHI    0x03
#define PANEL_EWS_MITSUBISHI    0x04

/* monitor id report by gfxinfo */
#define MONITORID_CORONA	130
#define MONITORID_ICHIBAN	131
/* paneltype if none connected */
#define MONITORID_NOFPD         255


#define CORONA_BL_MIN         0
#define CORONA_BL_MAX       100

/* valid i2c args:

no args:
   I2C_PANEL_ON
   I2C_PANEL_OFF

int:
   I2C_BL_UP 
   I2C_BL_DOWN 
   I2C_AUDIO_VOL_LEFT 
   I2C_AUDIO_VOL_RIGHT
   I2C_AUDIO_BASS 
   I2C_AUDIO_TREBLE 
   I2C_SWITCH
   
*/

#if defined (_KERNEL) || defined (_STANDALONE)
struct gr2_hw;
extern int gr2_i2cProbe(struct gr2_hw *);
extern int gr2_i2cSetup(void);
extern int gr2_i2cPanelID(int *);
extern int gr2_i2cPanelOn(void);
extern int gr2_i2cPanelOff(void);
extern int gr2_i2c_command(struct I2C_CMD_ARG *);

struct rex3chip;
extern int ng1_i2cProbe(struct rex3chip *);
extern int ng1_i2cSetup(void);
extern int ng1_i2cPanelID(int *);
extern int ng1_i2cPanelOn(void);
extern int ng1_i2cPanelOff(void);
extern int ng1_i2c_command(struct I2C_CMD_ARG *);

struct mgras_hw;
extern int mgras_i2cProbe(struct mgras_hw *);
extern int mgras_i2cSetup(void);
extern int mgras_i2cPanelID(int *);
extern int mgras_i2cPanelOn(void);
extern int mgras_i2cPanelOff(void);
extern int mgras_i2c_command(struct I2C_CMD_ARG *);
#endif

#define PANEL_XGA_MITSUBISHI    0x03

#if defined (_STANDALONE)
extern int      i2cPanelOn(void);
extern void     i2cPanelOff(void);
extern int      i2cSetup(void);
extern int      i2cPanelID(int *);
extern void     logoOff(void);
#endif

