#ifndef __SYS_GFX_H__
#define __SYS_GFX_H__

/**************************************************************************
 *		 Copyright (C) 1990, Silicon Graphics, Inc.		  *
 *  These coded instructions, statements, and computer programs	 contain  *
 *  unpublished	 proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may	not be disclosed  *
 *  to	third  parties	or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 **************************************************************************/

/*
 *   $Revision: 1.123 $
 */
#include <sys/rrm.h>
#ifdef _KERNEL
#ifdef _OLD_GFX
#include <sys/pda.h>
#include <sys/user.h>
#include <sys/proc.h>
#include <sys/region.h>
#define set_user_rval(x)     u.u_rval1 = (x)
#define set_user_error(e)    (u.u_error = e)
#define get_user_error()     (u.u_error)
#define curproc()       (u.u_procp)
#define v_regionva(prp) (((struct pregion *)(prp))->p_regva)
#define gfx_skobj_t     shaddr_t
#define proc_handl_t     proc_t
#define shaddr_handl_t   shaddr_t
#define get_cursproc_shaddr()        	(u.u_procp->p_shaddr)
#define sproc_buflocate(x, y, pp)       (pp)->p_shaddr
#define cursproc_buflocate(x, y)        (u.u_procp->p_shaddr)
#define proc_buflocate(x, y, pp)        (pp)->p_gfx
#define curproc_buflocate(x, y)         (u.u_procp->p_gfx)
#define proc_isgraph(pp)                (pp->p_flag & SGRAPH)
#define curproc_abi()                   (u.u_procp->p_abi)
#define HOLD_NONFATALSIG(x)             hold_nonfatalsig(u.u_procp, (x))
#define RELEASE_NONFATALSIG(x)          release_nonfatalsig(u.u_procp, (x))
#define curabi_is_irix5_64()		(ABI_IS_IRIX5_64(curproc_abi()))
#define GFX_WAITC_ON()			private.p_gfx_waitc = 1
#define GFX_WAITC_OFF()			private.p_gfx_waitc = 0
#define GFX_WAITF_ON()			private.p_gfx_waitf = 1
#define GFX_WAITF_OFF()			private.p_gfx_waitf = 0
#define GFXINFO				SYSINFO
#define sysinfo_outch_add(x)		SYSINFO.outch += x
#define sysinfo_rawch_add(x)		SYSINFO.rawch += x
typedef struct v_mapent {
	caddr_t	start;
	caddr_t	end;
	int	mode;	/* always map at 0? */
} v_mapent_t;

#else
#include <sys/kabi.h>
#include <sys/sema.h>

/*
 * initialize gfx data for the current sproc
 */
typedef struct gfx_skobj        {
        int             s_gfxruncount;  /* num of gfx gp members running */
        int             s_gfxreaders;   /* num of readers reading */
        sv_t          	s_gfxwait;      /* waiting readers or writers */
        lock_t          s_gfxlock;      /* lock for above fields */
} gfx_skobj_t;


/* 
   huy nguyen doesn't want us to include proc.h so we
   have to put our own prototypes here... but there's a
   caveat: gfx.h is included in master.c over in kernel
   land and so if this stuff is defined you get compiler
   errors.  To work around all this self-imposed "structure"
   there's an extra #define that gets defined by the gfx.sm
   file which tells us not to declare this stuff.
*/
#ifndef   GFX_MASTER_C
extern void psignal(proc_handl_t *, int);
extern void hold_nonfatalsig(k_sigset_t *);
extern void release_nonfatalsig(k_sigset_t *);
extern int procscan(int ((*)(proc_handl_t *, void *, int)), void *);
#endif /* GFX_MASTER_C */

#define HOLD_NONFATALSIG(x)             hold_nonfatalsig((x))
#define RELEASE_NONFATALSIG(x)          release_nonfatalsig((x))
extern struct gfxinfo *gfxinfo;
#define GFXINFO		gfxinfo[cpuid()]
#define GFX_WAITC_ON()	gfx_waitc_on();
#define GFX_WAITC_OFF()	gfx_waitc_off();
#define GFX_WAITF_ON()	gfx_waitf_on();
#define GFX_WAITF_OFF()	gfx_waitf_off();
#endif /* _OLD_GFX */
#endif /* _KERNEL */

/*
 *   I_STR ioctl commands for gfx STREAMS
 *
 *   All Gfx ioctls share the same cmd token space to
 *   avoid unnecessary indirections.
 *
 *   There are three types of ioctl.
 *     - GFX device independant gfx operations (defined here)
 *     - RRM rendering resource manager commands (defined in rrm.h)
 *     - Graphics board specific cmds (defined in a board-specific header e.g. gr1.h)
 */

#ifndef NULL
#define	NULL	0L
#endif
/*
 *   GFX commands
 */

#define GFX_SYNC                (1)
#define GFX_BOARD_SYNC          (10)

#define GFX_BASE		100
#define GFX_GETNUM_BOARDS	(GFX_BASE+1)
#define GFX_GETBOARDINFO	(GFX_BASE+2)
#define GFX_ATTACH_BOARD	(GFX_BASE+3)
#define GFX_DETACH_BOARD	(GFX_BASE+4)
#define GFX_IS_MANAGED		(GFX_BASE+5)

	/* Have to be the board owner to use these */
#define GFX_INITIALIZE		(GFX_BASE+6)
#define GFX_START		(GFX_BASE+7)
#define GFX_DOWNLOAD		(GFX_BASE+8)
#define GFX_BLANKSCREEN		(GFX_BASE+9)
#define GFX_MAPALL		(GFX_BASE+10)
#define GFX_LABEL		(GFX_BASE+11)
#define GFX_REATTACH_BOARD	(GFX_BASE+12)
#define GFX_SET_DIAG_FLAG	(GFX_BASE+13)

	/* XXX for debugging, remove later */
#define GFX_DEBUGMAP		(GFX_BASE+100)
#define GFX_DUMMY		(GFX_BASE+101)

        /* Start of board private commands */
#define GFX_PRIVATE_BASE        10000

/*
 *	gfx ioctl arguments
 */

struct gfx_getboardinfo_args {
	unsigned int board; /* board number (contiguous from zero) */
	void *buf;		/* buffer for reply */
	unsigned int len;	/* max length, length of reply (bytes) */
};

#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gfx_getboardinfo_args {
	unsigned int board;	/* board number (contiguous from zero) */
	app32_ptr_t  buf;	/* buffer for reply */
	unsigned int len;	/* max length, length of reply (bytes) */
};
#endif


struct gfx_attach_board_args {
	unsigned int board;
	void        *vaddr;     /* this is a user space address */
};


#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gfx_attach_board_args {
	unsigned int board;
	app32_ptr_t  vaddr;     /* this is a user space address */
};
#endif

struct gfx_attach_dm_table_args {
	void *addr;
	int  len;
};


#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gfx_attach_dm_table_args {
	app32_ptr_t addr;
	int  len;
};
#endif

struct gfx_download_args {		/* GFX_DOWNLOAD */
	int len;			/* length in bytes of ucode buffer */
	void *addr;			/* address of ucode buffer */
	int magic;			/* magic number in ucode file */
	int unit;			/* unit number to load ucode into */
	int off;			/* offset in memory space where ucode goes */
	int width;			/* width in bytes of ucode word */
	int entry;			/* entry point (if applicable) */
	int run;			/* run flag (should the engine be started?) */		
};


#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gfx_download_args {	/* GFX_DOWNLOAD */
	int len;			/* length in bytes of ucode buffer */
	app32_ptr_t addr;		/* address of ucode buffer */
	int magic;			/* magic number in ucode file */
	int unit;			/* unit number to load ucode into */
	int off;			/* offset in memory space where ucode goes */
	int width;			/* width in bytes of ucode word */
	int entry;			/* entry point (if applicable) */
	int run;			/* run flag (should the engine be started?) */		
};
#endif

#define GFX_DOWNLOAD_LIMIT 4096

struct gfx_private_args {
	unsigned int cmd;
	char data[1];
};

/*
 *   M_PROTO message types for gfx STREAM
 */
#define GFX_PROTO_ACK		1
#define GFX_PROTO_NACK		-1
#define GFX_PROTO_BASE		100
#define GFX_POSCURSOR		(GFX_PROTO_BASE+2)

/*
 *   PROTO message args
 */

struct gfx_poscursor {
	int cmd;			/* GFX_POSCURSOR */
	int x ;
	int y ;
};

union gfx_proto	{
	int cmd;			/* may be unknown */
	struct gfx_poscursor poscursor;
};


/*
 *   Graphics board description
 */

#define GFX_INFO_NAME_SIZE 	16
#define GFX_INFO_LABEL_SIZE 	16

struct gfx_info {
	char name[GFX_INFO_NAME_SIZE]; 	/* unique name for this graphics */
	char label[GFX_INFO_LABEL_SIZE];/* manager's name for this board */
    	unsigned short int xpmax;	/* screen x size in pixels */
	unsigned short int ypmax;	/* screen y size in pixels */
	unsigned int length;		/* size of a complete info struct */
	                                /* for this board */
};

#define GFX_SYNC_VALUE  3

#ifdef _KERNEL

/*
 * Device independent interface to device dependent graphics board functions.
 */

struct gfx_data;
struct gfx_gfx;
struct rrm_rnode;
struct RRM_ValidateClip;

struct gfx_fncs {
			 /* functions for the Graphics Driver */
	int (*gf_Info)(struct gfx_data *, void *, unsigned int);    
	int (*gf_Attach)(struct gfx_gfx *, caddr_t);
	int (*gf_Detach)(struct gfx_gfx *);
	int (*gf_Initialize)(struct gfx_gfx *);
	int (*gf_Download)(struct gfx_gfx *, struct gfx_download_args *);
	int (*gf_Start)(struct gfx_gfx *);
	int (*gf_PositionCursor)(struct gfx_data *, int, int);
	int (*gf_Sync)(unsigned int);

			/* functions for the Rendering Resource Manager */
	int (*gf_CreateDDRN)(struct gfx_data *, struct rrm_rnode *);
	int (*gf_DestroyDDRN)(struct gfx_data *, struct gfx_gfx *, struct rrm_rnode *);
	int (*gf_ValidateClip)(struct gfx_gfx *, struct rrm_rnode *,
		struct rrm_rnode *, struct RRM_ValidateClip *);
	int (*gf_SetNullClip)(struct RRM_ValidateClip *);
	int (*gf_MapGfx)(struct gfx_gfx *, __psunsigned_t, int);  /* was int (*gf_MapGfx)(struct gfx_gfx *, int, int); */
	int (*gf_UnMapGfx)(struct gfx_gfx *);
	int (*gf_InvalTLB)(struct gfx_gfx *);
	int (*gf_PcxSwap)(struct gfx_data *, struct rrm_rnode *,
		struct rrm_rnode *, struct rrm_rnode *);
	int (*gf_PcxSwitch)(struct gfx_data *, struct rrm_rnode *,
		struct rrm_rnode *);
	int (*gf_SchedSwapBuf)(struct gfx_data *, struct rrm_rnode *,int,int);
	int (*gf_UnSchedSwapBuf)(struct gfx_gfx *, struct rrm_rnode *, int);
	int (*gf_SchedRetraceEvent)(struct gfx_data *, struct rrm_rnode *);
	int (*gf_SetDisplayMode)(struct gfx_gfx *, int, unsigned int);
	int (*gf_SchedMGRSwapBuf)(struct gfx_gfx *, int, int, int, int);
	int (*gf_Suspend)(struct gfx_data*, struct gfx_gfx *);
	int (*gf_Resume)(struct gfx_data*, struct gfx_gfx *);

	/*
	 * routine to call graphics board driver specific functions.
	 */
	int (*gf_Private)(struct gfx_gfx*, struct rrm_rnode *,
		unsigned int, void *);

	/*
	 * provide a pointer to appropriate spl?() routines for
	 * the given hardware.
	 */
	int	(*gf_SplRetrace)(void);

	/*
	 * Functions to install and uninstall
	 * link to Frame Scheduler
	 */
	int (*gf_FrsInstall)(struct gfx_data*, void* intrgroup);
	int (*gf_FrsUninstall)(struct gfx_data*);
};


struct gfx_board { 
	struct gfx_info *gb_info ;
	struct gfx_gfx *gb_manager;
	struct gfx_gfx *gb_owner;
	int gb_number;	   /* same as index in GfxBoards */
	struct queue *gb_rq_ptr;
	struct queue *gb_wq_ptr;
	/* gunk for talking to shimq  */
	short int gb_devminor ;		/* Identifies the shmiq */
	short int gb_index ;		/* Identifies the device */
	struct gfx_fncs *gb_fncs ;
	struct gfx_data *gb_data ;
} ;

/*
 *   gfx driver structure
 *   one of these per open board, per process
 */

struct gfx_gfx {

	/*  generic stuff  */
	struct gfx_board *gx_board; /* pointer to board */
	proc_handl_t *gx_pp; /* proc pointer */
	struct gfx_gfx *gx_next;	/* ptr to next open gfx */

	/*  board driver stuff	*/
	struct gfx_fncs *gx_fncs;
	struct gfx_data *gx_bdata;	/* ptr to driver private data */
					/* there are many of these pointers */
					/* but only one copy of the data */
	struct pregion *gx_preg;
	unsigned int gx_flags;		/* flags needed by char side only */
	unsigned int gx_sflags;		/* flags needed by both sides */
	unsigned int gx_tlbmask;	/* for late TLB flushing on (MP) */

	/*  rrm stuff  */
	void *gx_openrns;		/* RN's this gfx has open */
	void *gx_boundrn;		/* RN this gfx is currently bound to */

	/* share group fields */

	sema_t gx_sema;			/* semaphore for multiple sg procs */
};


/*
 *   Values for gfx_gfx.gx_flags
 */
#define GFX_MAPPED	0x00000001
#define GFX_DIAGMAPALL	0x00000002	/* The pipe is mapped in a special way for diagnostic use */
#define GFX_DIAG_FLAG	0x00000004	/* the process has identified itself as a diagnostic */
#define GFX_OPENGL_PROC 0x00000008	/* the process is an OpenGL process */
#define GFX_INTRANSIT	0x00000010	/* the process is in the middle of a graphics context switch.  Currently used only on Reality Engines*/

/*
 *   Values for gfx_gfx.gx_sflags
 */
#define GFXS_STREAM	0x00000001	/* referenced by streams side of driver */
#define GFXS_CHAR	0x00000002	/* referenced by character side of driver */

struct gfx_data {
		/* for the Rendering Resource Manager */
	struct gfx_gfx *manager;	/* pointer to manager, if managed */
	void *currentrn;		/* Current rendering node */
	int numrns;			/* number of RN's open on this board */
	int numpcx;			/* number of pipe contexts in hardware*/
	struct rrm_rnode **loadedpcxs;	/* table of pipe contexts loaded
					   in hardware */
	struct rrm_rnode *newestrn;	/* most recently used rnp
					   (pcxid allocation) */
	struct rrm_rnode *oldestrn;	/* least recently used rnp
					   (pcxid allocation) */
	lock_t lock;			/* per board locking mechanism */
	sema_t		gfxsema;	/* The gfx semaphore is held during all
					   times the process has the pipe
					   mapped.  It is also necessary to
					   hold it during modification of
					   rnp->validmask, even if the process
					   doesn't have the pipe.  Unless the
					   gfxlock is non-zero, the process will
					   give up gfxsema during times when it
					   is _not_ running (see rrmSuspend()
					   and rrmResume()).
					 */
	struct gfx_gfx	*gfxsema_owner; /* When the semaphore is grabbed, this
					   is set to the current gfxp.  It is
					   used to avoid re-grabbing gfxsema.
					 */
	int		gfxlock;	/* The lock is used so that even when a
					   process sleeps it will hold gfxsema.
					   This is necessary in ValidateMyPcx,
					   and in some machine dependent code.
					 */
	int		gfxbusylock;	/* when a process switches out, if the
					   device dependent graphics driver decides
					   that its not an opportune time to context
					   switch the graphics (based on user activity)
					   then we can force the state of gfxlock to
					   hold off a graphics context switch.
					*/
	int nummodes;			/* number of mode registers in
					   hardware*/
	unsigned int *loadedmodes;	/* table of mode registers loaded
					   in hardware */
	volatile int gfxbackedup;	/* flag set by fifo interrupt handler when
					 * fifo is near full and we shouldn't drop 
					 * anymore tokens 
					 */
	int (*gf_ClearBackupFnc)(struct gfx_data *);
					/* When gfx is backed up, it is sometimes
					 * possible to clear the backup by 
					 * using a board-dependent function. 
					 * This function is called from 
					 * gfx_disp if gfx is backed up and
					 * the function is defined.
					 */
	int gfxresched;
	unsigned int vsync_cnt;		/* vsync counter for IrisGL; writeable
					 * by apps */
	unsigned int vsync_cnt_ogl;	/* vsync counter for OpenGL; not
					 * writeable by apps */
	struct rrm_rnode *vsync_sleep_list;
					/* The list of vsync sleepers; they are
					 * inserted by the RRM when a process
					 * calls the vsync sleep ioctl, and
					 * deleted by the vertical retrace
					 * interrupt handler via an rrm routine.
					 */
	int   frs_vsync;                 /* Flag to manage linkage to Frame
					  * Scheduler. True if the vsync
					  * interrupt has to be forwarded to
					  * some other cpus.
					  */
	void* frs_intrgroup;             /* Interrupt group descriptor to be used
					  * to forward the vsync interrupt to
					  * cpus managed by the Frame Scheduler.
					  */
	int wait_rrm;			  /* wait list implemented in rrm */
	int wait_time;			  /* retrace counter for wait_list */
	struct rrm_rnode *wait_list;	  /* rrm's wait list (for swapinterval) */
};




/*
 *   args to GfxKiller() routine called by procscan()
 */

struct GfxKiller_args {		/* arg struct given to procscan() */
	int	signal;		/* signal to send */
	struct gfx_data	*bdata;	/* pointer to board data; identifies a board */
	proc_handl_t	*pp;	/* your proc pointer if you shouldn't be killed */
				/* (e.g. board manager exits), NULL otherwise */
};


/*
 * Multiple reader, single writer lock routines for list rooted at pp->p_gfx.
 */
struct shaddr_s;
struct gfx_rrm_message;
extern int  GfxKiller(proc_handl_t *, void *, int);
extern int  GfxListWriterLock(gfx_skobj_t *);
extern void GfxListWriterUnlock(gfx_skobj_t *, int);
extern void GfxListReaderLock(gfx_skobj_t *);
extern void GfxListReaderUnlock(gfx_skobj_t *);
extern int  GfxMessage( struct gfx_rrm_message *);
extern void GfxIntrMessage( struct gfx_rrm_message *);
extern int  GfxRegisterBoard(struct gfx_fncs *, struct gfx_data *, struct gfx_info *);

#include <sys/region.h>    /* for vhandl_t in the gfx_fault() prototype */

#ifdef _OLD_GFX
extern int
gfx_fault(vhandl_t *vt, void *_gfxp, proc_t *pp, caddr_t vaddr, int rw);
#else /* !_OLD_GFX */
extern int gfx_fault(vhandl_t *vt, void *_gfxp, caddr_t vaddr, int rw);
#endif /* _OLD_GFX */



/*

If you define HARDCORE_DEBUG when compiling the code in
gfx/kern/graphics (and below), you pick up a definition for the
GFX_LOG macro and a debugger command, gfxlog [num].  These two in
combination can give you a reverse chronological event log of all
"interesting" graphics related events that have occurred (up to a
maximum of MAX_LOG).  The GFX_LOG() macro is used to record
timestamped events in a per-cpu circular buffer.  The log records a
"command", some piece of data along with line number and file name
information.  The "command" is usually just some number which you can
then correspond back to some place in the code.  For example, the
Venice macros that put tokens in the graphics pipe have been modified
to also log the event with GFX_LOG() and they pass the token number as
the "command".  It is possible to pass any number, you just have to
make sense of it.  Some useful #define's are already here for parts of
gfx.c and rrm.c that have been instrumented with GFX_LOG().  Feel free
to add as much logging/instrumentation as is necessary.  It doesn't
get compiled if HARDCORE_DEBUG is off, so it doesn't really affect
anything. 

Generally the way I use this is to drop into the debugger when some
event occurs that is bad (pipe crash, bad CP command, kernel panic,
etc).  Then I'll do something like 'gfxlog 100' to see the last 100
gfx events that the kernel has seen.  Then I take the output of that
command, filter it through a sed script that converts the commands
from numbers to a more readable format.  Then using other debugger
commands like "plist" and "gfx" I can convert most of the remaining
hex values into more symbolic values.  This results in a nicely
annotated traceback of exactly what the kernel did and to whom.
That's incredibly handy for debugging complicated MP related issues.

NOTE WELL: These macros are useful, but how you use them influences
           how much info they'll give you.  If you have a tough MP
	   related bug,  then when the event that you want to
	   trigger on occurs,  you'll want to be sure to call the
	   GFX_STOP_LOGGING macro and then (possibly) call
	   stop_all_cpus().  If you don't do this, then what can
	   happen is that the other cpu's will continue on, spinning
	   on various events and in a very short time wrap around in
	   their buffers.  Then when you ask for a traceback, you'll
	   see lots of junk from the other cpu's, but not what you're
	   interested in.  This can be very misleading, so generally
	   when you reach the point you want to freeze the machine, do
	   a GFX_STOP_LOGGING followed by a call to stop_all_cpus().

*/

#ifdef HARDCORE_DEBUG

typedef struct gfx_event_log
{
  unsigned long timestamp;   /* the EV_RTC is really a 64bit register, we get the low 32 bits */
  char *cmd_string;
  unsigned long value;
  unsigned int line_num;
  char *fname;
} gfx_event_log;


#define MAX_CPU 24
#define MAX_LOG 512

extern int stop_logging;
extern int cpu_counter[MAX_CPU];
extern gfx_event_log event_history[MAX_CPU][MAX_LOG];

#if defined(IP19) || defined(IP21) || defined(IP25)
#define PRE_GET_TIME
#define GET_TIME  EV_GET_LOCAL(EV_RTC)
#elif defined(IP22) || defined(IP26) || defined(IP28)
extern unsigned int update_ust_i(void);
extern void get_ust_nano(unsigned long long *);
#define PRE_GET_TIME unsigned long long ust; update_ust_i(); get_ust_nano(&ust);
#define GET_TIME (ust >> 10) /* gives us units roughly in microseconds */
#else
/* we need to pick up a definition for IOTIMER_ADDR for the debug macros */
/* #include <sys/clover2.h> */
/* wholesale inclusion of clover2.h causes build problems here (IP20) */
/* Since, we only need this one value... and it's not going to change */
/* (at least not in clover2.h), let's just grab the value             */
#define PRE_GET_TIME
#define IOTIMER_ADDR    0xb7f44000      /* Free running timer register(IO3) */
#define GET_TIME  *(int *)IOTIMER_ADDR
#endif			     

#define GFX_LOG(cmd, val)  {\
			      int _cpuid = cpuid();\
                              PRE_GET_TIME; \
			      if (stop_logging == 0) {\
			        event_history[_cpuid][cpu_counter[_cpuid]].cmd_string = #cmd;\
			        event_history[_cpuid][cpu_counter[_cpuid]].value  = (unsigned long)val;\
			        event_history[_cpuid][cpu_counter[_cpuid]].timestamp = GET_TIME;\
			        event_history[_cpuid][cpu_counter[_cpuid]].line_num = __LINE__;\
			        event_history[_cpuid][cpu_counter[_cpuid]].fname = __FILE__;\
			        \
			        cpu_counter[_cpuid] = (cpu_counter[_cpuid]+1) % MAX_LOG;\
			      }\
			   }

#define GFX_STOP_LOGGING   stop_logging = 1

/*
   This is a utility function you can call to make sure all
CPUs have dropped into the debugger.  This prevents cases
where one cpu drops into the debugger and the rest continue
on and overflow the logging buffers with events that happened
long after the point in time we care about.  The cpu that calls
this routine should then call debug(0) or something.
*/
extern void stop_all_cpus(char *str);  

#else  /* we're not doing HARDCORE_DEBUG, so just make it a nop */

#define GFX_LOG(cmd, val)
#define GFX_STOP_LOGGING

#endif /* HARDCORE_DEBUG */


#endif /* _KERNEL */

#endif /* __SYS_GFX_H__ */
