#ifndef __SYS_GR2_DEV_H__
#define __SYS_GR2_DEV_H__

/*
 * $Revision: 1.93 $
 */
#include "gfx.h"
#include "rrm.h"

#ifdef _KERNEL
#ifndef OLDLOCKS
#include "sys/sema.h" /* for definition of mutex */
#endif
#endif

#ifdef IP17
#define GFX_NAME_GR2		"GR2MP"	/* Unique name for GR2 type boards */
#else
#if defined(IP19)
#define GFX_NAME_GR2		"GR2EV" /* Unique name for GR2 type boards */
#else
#if defined(IP21)
#define GFX_NAME_GR2		"GR2TP" /* Unique name for GR2 type boards */
#else
#if defined(IP25)
#define GFX_NAME_GR2		"GR2SH" /* Unique name for GR2 type boards */
#else
#define GFX_NAME_GR2		"GR2"	/* Unique name for GR2 type boards */
#endif
#endif
#endif
#endif

struct gr2_info {
	struct gfx_info gfx_info; 	/* device independent information */

	unsigned char BoardType;	/* 0 for a GR2, 1 for HI1 */
	unsigned char Bitplanes; 	/* 8 or 24 */
 	unsigned char Auxplanes;	/* 2 or 4 */
	unsigned char Wids;		/* 2 or 4 */
	unsigned char Zbuffer;		/* true if zbuffer option installed */
	unsigned char MonitorType;	/* monitor id */
	unsigned char GfxBoardRev;
	unsigned char PICRev;
	unsigned char HQ2Rev;
	unsigned char GE7Rev;
	unsigned char RE3Rev;
	unsigned char VC1Rev;
	unsigned char VidBckEndRev; 
	unsigned char Xabnormalexit;	/* X killed without closing file
					   descriptor. */
	unsigned char GEs;		
	unsigned char MonTiming;	/* monitor timinig */
	unsigned char VidBrdRev;	/* video board revision */
	unsigned char PanelType;	/* flat panel type */
};

/*
 * XXX If change Gr2PixelDma() or any of these defines, check the others!
 */
#define DMAMAXXLEN 	1280	/* Max x length for pixel DMA */
#define DMAMAXYLEN 	1024	/* Max y length for pixel DMA */

#if defined(IP21) || defined(IP25)
#define	GR2_GFXPG_VADDR	0x4000	/* IP21 uses 16KB pages, and gfx addr has to be page aligned */
#else
#if defined(IP26) || defined(IP28) || defined(I2_64) || ( defined(IP22) && _PAGESZ == 16384 ) || (defined(IP19) && _PAGESZ==16384)
#define	GR2_GFXPG_VADDR	0x4000	/* Used in gl - virtual graphics address */
#else
#define	GR2_GFXPG_VADDR	0x2000	/* Used in gl - virtual graphics address */
#endif
#endif

#ifdef _KERNEL

					/* physical addr */
#ifndef GR2_BASE_PHYS
#define GR2_BASE_PHYS	0x1F000000	/* private bus GR2 board family base */ 
#define GR2_BASE1_PHYS	0x1F400000	/* private bus GR2 board family base */ 
#define GR2_BASE2_PHYS	0x1F600000 	/* private bus GR2 board family base */ 
#endif

#define GR2_3WAY_SIZE	0x2000
#define GR2_GFX_SIZE	0x6D000
#define GR2_MAP_SIZE	(GR2_GFX_SIZE + GR2_3WAY_SIZE)
					/* size of region to map into graphics
					 * process's address space */

/* GR2 variants; -1 means probe failed */
#define GR2_TYPE_NOT_THERE -1		/* probe failed */
#define GR2_TYPE_GR2	0		/* Express  GR2 */
#define GR2_TYPE_HI1	1		

/*
 * Change the timing table as screen x coord of cursor crosses this x value.
 * This value must be < 1024, since for high res. 1024+250bp = 1280, 
 * which is the max range of CUR_XL.
 * Be aware when changing this value which side of the "border" the cursor
 * will be on when the xserver starts up. Set the GR2_VID_EP for the correct
 *      timing table in the VC1 initialization, accordingly.
 */
#define GR2_CURS_CHNGE_TIMING 32 

/*
 * XXX If change Gr2PixelDma() or any of these defines, check the others!
 */
#define PGSPERSHOT	32	/* Number pages map/lock at a shot*/
#define MAXPERSHOT	(NBPC * PGSPERSHOT) /* Max bytes map/lock at a shot */

#define CXDMAPAGES	20

struct Gr2Retrace;

struct gr2_data {
	struct gfx_data	gfx_data;	/* Common gfx driver above */

	struct gr2_info *info;	/* gr2 inventory */

	struct gr2_hw	*base;		/* K1 seg board base address */
	unsigned int	attaches;
	int 		cursor_x;	/* x passed by Xserver*/
	int 		cursor_y;	/* y passed by Xserver*/
	int 		cursor_xhot;	/* x hot position */
	int 		cursor_yhot;	/* y hot position  */
	int 		mon_xoffset;	/* mon. dependent horizontal offset*/
	int 		mon_yoffset;	/* mon. dependent vertical offset*/
	struct Gr2Retrace *Retrace;	/* retrace handler structure */
	sema_t		dmalock;
	void		*dmadap;	/* DMA descriptor array for pixels */
	void		*cxdmadap;	/* DMA descriptor array for context
					   switching */
					/* The owner of the data buffer */
	struct rrm_rnode	*scratchrn;
	struct rrm_rnode	*nurbsrn;
	struct rrm_rnode	*spotlightrn;
	struct rrm_rnode	*volumern;

	unsigned int loadedmodes[32];
	lock_t		gr2lock;	/* spinlock to be used in MP systems, noop-ed in SP systems */
#ifdef IP17
	void		*adapter_base;	/* kernel address of base address of adapter board */
	int		adapter_slot;	/* MP slot id of adapter (only on MP bus systems)*/
	int		gio_slot;	/* MP slot id of GIO (only on MP bus systems)*/
#endif
#if defined(IP19) || defined(IP21) || defined(IP25)
	int		window;
	int		adapter;
	void		*pipebase;	/* Base virtual address of the write gatherer */
	long long	*lwinbase;	/* Base virtual address of large window of dang chip, where everything is accessed as 64bit entity */
	long long	*swinbase;	/* Base virtual address of large window of dang chip, where everything is accessed as 64bit entity */
	int		cpu_id;		/* CPU id of the process last running the board manager */
#endif
        int             fifo_timeout_callid;    /* The handle for the fifo full callback */
	int		fifo_full_shadow;	/* Shadow of the (write-only) hq2 fifo_full register */

#ifdef FLAT_PANEL
        unsigned char   panelalive;
#endif
};

/*
 * device dependent rendering node
 */
struct gr2_rnode {
	/* Context switch stuffs */
	int user_finish;
	int pcxtype;	/* New, GE, Host, Dump */
	unsigned int pcxsize;	/* Small, Light */
	unsigned int newpcxsize;
	struct RRM_ValidateClip validateclip;
	int pcxdatatype;	 /*
				   bit 0 - DATA_NURBS
				   bit 1 - DATA_SPOTLIGHT
				   bit 2 - DATA_VOLUME
				   bit 3 - DATA_TEXTURE
				   bit 4 - DATA_VOLUME8K
				   bit 5 - DATA_VOLUME16K
				   bit 6 - DATA_SCRATCH
				*/
				/* all kinds of data buffer */
	void		*scratchp;
	unsigned int	scratch_datasize;
	unsigned int	scratch_bufsize;

	void		*nurbsp;
	unsigned int	nurbs_size;

	void		*spotlightp;
	unsigned int	spotlight_size;

	void		*volumep;
	unsigned int	volume_size;

	/* XXX add VOLUME8K, VOLUME16K later */

	/* Retrace stuffs */
	unsigned int swapinterval; /* Min. retraces between swapbuffers */


#ifdef IP17
	/* MG1 board control registers */
	unsigned int mg1_control1;
	unsigned int mg1_control2;
	/* IP17 3way registers */
	unsigned int ip17p3way_cntcmdreg; /* only 16 bits are valid */
	unsigned int ip17p3way_ABCDregs[4];
#endif
#if defined(IP19) || defined(IP21) || defined(IP25)
	/* DANG chip register to be saved and restored during ctx switch. */
	int dang_stream_cnt;		/* non-zero if in streaming mode */
	int dang_addr_saved;		/* 1 if dang_addr is valid, 0 otherwise */
	long long dang_addr;
#endif
};

struct gr2boards {
        struct gr2_hw   *base;                  /* K1 seg addresses for non IP19, for IP19 K2 non-cached address */
        struct gr2_data *data;
        struct gr2_info *info;
        int             found;
#if defined(IP22) || defined(IP26) || defined(IP28)
	int		slot;			/* gio slot number */
	int		index;			/* index into this array */
	int		mask;			/* retrace intr mask */
#endif
#if defined (IP19) || defined(IP21) || defined(IP25)
	void		*swin;			/* DANG regs and GIO bus regs are accessed via small window */
	int		window;
	int		adapter;
#endif
};

/* Prototypes */
#ifndef _STANDALONE
struct mcout;
struct gdmada;
#if defined(IP22) || defined(IP26) || defined(IP28)
extern void Gr2Reset(int);
#else
#if defined(IP19) || defined(IP21) || defined(IP25)
extern void Gr2Reset(long long *);
#else
extern void Gr2Reset(void);
#endif
#endif
extern void Gr2BlankScreen(register struct gr2_hw *, int);
extern int Gr2DownloadGE7(register struct gr2_hw *, struct mcout *, int);
extern int Gr2DownloadHQ2(register struct gr2_hw *, struct mcout *, int);
extern void Gr2LoadVC1SRAM(register struct gr2_hw *, unsigned short *, unsigned int, unsigned int);
extern int Gr2PositionCursor(struct gfx_data *, register int, register int);
extern int Gr2SetCursorHotspot(struct gfx_gfx *, register int, register int);
extern int Gr2SetDisplayMode(struct gfx_gfx *, int, unsigned int);
extern int Gr2SchedSwapBuf(struct gfx_data *, struct rrm_rnode *, int, int);
extern int Gr2UnSchedSwapBuf(struct gfx_gfx *, struct rrm_rnode *, int);
extern int Gr2SchedRetraceEvent(struct gfx_data *, struct rrm_rnode *);
extern int Gr2StartClock(register struct gr2_hw *, struct gr2_info *);
extern int Gr2Probe(struct gr2_hw *, struct gr2_info *);
extern int Gr2Init(struct gr2_hw *, struct gr2_info *);
extern int Gr2TpSetup(struct gr2_hw *, int, int, int);
extern void initGr2Cursor(struct gr2_hw *);
extern void Gr2ProbeVC1(register struct gr2_hw *, struct gr2_info *);
extern void Gr2RetraceInit(struct gr2_data *);
extern void Gr2RetraceCloseRN(struct gr2_data *, struct rrm_rnode *);
extern int Gr2SetGammaRamp(struct gr2_data *, unsigned char *, unsigned char *, unsigned char *);
extern int Gr2PixelDma(struct gfx_gfx *, struct gr2_pixeldma_args *);
extern void gr2_error(struct gr2_data *);
extern void Gr2RetraceHandler(register struct gr2_data *);
extern int _Gr2CrossPage(struct gdmada *);
extern void Gr2FillVc1Shadow(struct gr2_data *, int);
extern void Gr2ReleaseVc1Shadow(struct gr2_data *);
#ifdef FLAT_PANEL
extern void gr2_fp_probe (struct gr2_info *);
extern void gr2_fp_reinit (struct gr2_data *);
#endif

#endif

#ifdef IP21
#define EVEN_OFFSET	0
#define ODD_OFFSET	1
#define FLUSH_OFFSET	4
#define SYNC_OFFSET	5
#endif /*IP21*/
#endif /* _KERNEL */

/*
 *   GR2 specific (gf_Private) Ioctls
 */
#define GR2_BASE                15000
#define GR2_SETCURSOR_HOTSPOT   (GR2_BASE+3)
#define GR2_SETMONITOR 		(GR2_BASE+5)
#define GR2_SETGAMMARAMP	(GR2_BASE+6)
#define GR2_GETMONITOR		(GR2_BASE+8)
#define GR2_PIXELDMA            (GR2_BASE+10)
#define GR2_CHANGECXSIZE	(GR2_BASE+11)
#define GR2_TEXTUREDMA		(GR2_BASE+12)
#define GR2_ACCUMULATE		(GR2_BASE+13)
#define GR2_ACRETURN		(GR2_BASE+14)
#define GR2_ALLOCCXDATA		(GR2_BASE+15)
#define GR2_SETDISPLAYMODE	(GR2_BASE+16)
#define GR2_ADJGENLOCK		(GR2_BASE+17)
#define GR2_VSYNC_SET		(GR2_BASE+18)
#define GR2_VSYNC_GET		(GR2_BASE+19)
#define GR2_VSYNC_SLEEP		(GR2_BASE+20)
    /* for setting flat panel backlight */
#define GR2_I2C_CMD		(GR2_BASE+21)

/*
 *   GR2 Ioctl args
 */

struct gr2_colorentry {
	unsigned short index;
	unsigned char red;
	unsigned char green;
	unsigned char blue;
	unsigned char junk;	 /*XXX What is this?*/
};

struct gr2_setcolor_args {
	short numberofcells;
	struct gr2_colorentry colors[1];    
};

struct gr2_setdisplaymode_args {
   	int wid;
	unsigned int displaymode;
};

struct gr2_blankscreen_args {
   	int blank;		/* 0 on, 1 off, 2 query */
};

/* Flags for graphics DMA */
#define GR2_WRITE       0x01            /* DMA direction: host to gfx */
#define GR2_STRIDE      0x02            /* Stride DMA */
#define GR2_TTOB        0x04            /* Pixels dma'ed from top to bottom */
#define GR2_RTOL        0x08            /* Pixels dma'ed from right to left */
#define GR2_ZOOM        0x10            /* Zoomed pixels */

#define MAXDMADA        200             /* The maximum DMA descriptor arrays 
					   for simulation */
struct gr2_pixeldma_args {
	int token;			/* Token to send down pipe */
	int x;				/* Rectangular coordinate in pixels */
	int xlen;
	int y;
	int ylen;
	unsigned short flags;		/* Bit flags defined below */
	int pmstride;			/* Number of 32-bit CPU words per
					   scanline */
	int yzoom;			/* Zoom factor in Y */
	int nbytes;			/* Total bytes of DMA,
					   stride = pmstride * ylen * 4;
					   non-stride = wlen * ylen * 4
					 */
	int wlen;			/* # of dma words (in words) */
	char *buf;			/* Data buffer */
};

#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gr2_pixeldma_args {
	int token;			/* Token to send down pipe */
	int x;				/* Rectangular coordinate in pixels */
	int xlen;
	int y;
	int ylen;
	unsigned short flags;		/* Bit flags defined below */
	int pmstride;			/* Number of 32-bit CPU words per
					   scanline */
	int yzoom;			/* Zoom factor in Y */
	int nbytes;			/* Total bytes of DMA,
					   stride = pmstride * ylen * 4;
					   non-stride = wlen * ylen * 4
					 */
	int wlen;			/* # of dma words (in words) */
	app32_ptr_t buf;		/* Data buffer */
};
#endif
struct gr2_setcursor_hotspot_args{
	unsigned short xhot;    /* X cursor hotspot */
	unsigned short yhot;	/* Y cursor hotspot */
};

struct gr2_acbufdma_args {
	int	token;			/* GL token */
	int	x;			/* accumulation buffer area */
	int	xlen;	
	int	y;
	int	ylen;
	int	scale;
	int	*buf;
};
#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gr2_acbufdma_args {
	int	token;			/* GL token */
	int	x;			/* accumulation buffer area */
	int	xlen;	
	int	y;
	int	ylen;
	int	scale;
	app32_ptr_t buf;
};
#endif
struct gr2_alloccxdata_args {
	int datatype;
};

struct gr2_getcmap_args {
        unsigned int start;
        unsigned int size;
        unsigned char *buffer;
};
#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gr2_getcmap_args {
        unsigned int start;
        unsigned int size;
        app32_ptr_t buffer;
};
#endif

struct gr2_getoverlaycmap_args {
        unsigned int size;
        unsigned char *buffer;
};
#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gr2_getoverlaycmap_args {
        unsigned int size;
        app32_ptr_t buffer;
};
#endif

struct gr2_setgammaramp_args {
        unsigned char red[256];
        unsigned char green[256];
        unsigned char blue[256];
};

/*  Monitor types - for setmonitor  */
/* IMPT: Xserver assumes that interlaced monitors have the low bit set to 1.*/
#define	GR2_HZ60	0 
#define	GR2_NTSC	1 /* Interlaced */	
#define	GR2_STR_RECT	2	
#define	GR2_PAL		3 /* Interlaced */	
#define	GR2_HZ72	4
#define	GR2_A343	5 /* Interlaced */	

/* Flags for genlock */ 
#define GR2_GENLOCK_IN		0x01
#define GR2_SYNCR		0x02
#define GR2_SYNCG		0x04
#define GR2_SYNCB		0x08
#define GR2_FRAMELOCK		0x10
#define GR2_SYNCNONE		0x20
#define	GR2_FLOCK_INTERLACED 	0x40 /* Framelock input signal is interlaced*/

struct gr2_monitor_args {
	unsigned int monitortype;
	unsigned char flag;
};


struct gr2_adjgenlock_args {
        int adj_board;
        int ltablesize;
        void *ltableaddr;
        int frtablesize;
        void *frtableaddr;
        unsigned char glvflag;
};
#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gr2_adjgenlock_args {
        int adj_board;
        int ltablesize;
        app32_ptr_t ltableaddr;
        int frtablesize;
        app32_ptr_t frtableaddr;
        unsigned char glvflag;
};
#endif

struct gr2_setmon_args {                /* GR2_SETMONITOR*/
        int len;	                /* length in bytes of mon info file*/
        void *addr;                     /* address of mon info file*/
};
#if defined(_K64U64) && defined(_KERNEL)
struct irix5_gr2_setmon_args {                /* GR2_SETMONITOR*/
        int len;	                /* length in bytes of mon info file*/
        app32_ptr_t addr;                     /* address of mon info file*/
};
#endif


struct gr2_vsync_sleep_args {
	int	interval;
	int	mod;
};


#endif /* __SYS_GR2_DEV_H__ */
