/* * Copyright 1995 Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef __SYS_HWPERFMACROS_H__
#define __SYS_HWPERFMACROS_H__

#ident	"$Revision: 1.15 $"

/*
 * Some commands access the event counters via syssgi.
 * For entry via syssgi, there is only one syssgi value, 
 * and the actual operation on the counters is specified 
 * through the following sub-commands:
 */
#define HWPERF_PROFENABLE	0x1	/* acquire counters for profiling */
#define HWPERF_ENSYSCNTRS	0x2	/* grab counters globally */
#define HWPERF_GET_CPUCNTRS	0x4	/* read cpu's private counters */
#define HWPERF_RELSYSCNTRS	0x8	/* release system counters */
#define HWPERF_GET_SYSCNTRS	0x10 	/* read all the counters together */
#define HWPERF_GET_SYSEVCTRL	0x20 	/* get global counter state */
#define HWPERF_SET_SYSEVCTRL	0x40 	/* set global counter state */


/*
 * Some useful macros for extracting the event selects of the 2 hardware
 * performance counters
 *
 *
 * Performance control register
 *         --------------------------------------------------
 *         | event select <8:5> | IE <4> | count enable <3:0>|
 *         --------------------------------------------------
 */


#define HWPERF_EVENTSPEC_SHIFT 16 /* Amount of shift to get the most
			           * sig. specifier(16-bit) out of a
		 	           * 32-bit compound specifier
			           */
#define HWPERF_EVENTSPEC_HALFMASK 0x0000ffff /* Mask for the least
		     		              * sig. specifier(16-bit)
				              */

/* Event specifier for perf. counter0 */

#define HWPERF_EVENT_SPEC0(spec) (spec >> HWPERF_EVENTSPEC_SHIFT) 

/* Event specifier for perf. counter1 */

#define HWPERF_EVENT_SPEC1(spec) (spec & HWPERF_EVENTSPEC_HALFMASK)


/* 
 * To extract the events that are being counted from event index
 */

#define HWPERF_INDEX_SHIFT 4  /* Event index is a byte long & each half     
		    	       * byte corresponds to index for each of the
			       * two virtual counters 
			       */
			
#define HWPERF_INDEX_MASK 0x0f/* Mask for the  lower half byte */

/* Event of counter 0 */
#define HWPERF_CNTEVENT0(index) (index >> HWPERF_INDEX_SHIFT)   
/* Event of counter 1 */
#define HWPERF_CNTEVENT1(index) (index & HWPERF_INDEX_MASK)

/*
 * To extract the appropriate fields given a specifier
 */

#define HWPERF_EVMASK  		0x1E0   /* Mask for the event bits */
#define HWPERF_MODEMASK  	0xF   /* Mask for the count mode bits */
#define HWPERF_EVSHIFT 		5 

/* Event corresponding to the event specifier */
#define HWPERF_EVENTSEL(spec) (((spec) & HWPERF_EVMASK) >> HWPERF_EVSHIFT)

/* Count mode corresponding to the event specifier */ 
#define HWPERF_EVENTCNT(spec) ((spec) & HWPERF_CNTMASK)

/* Concatenates two half bytes(event numbers of the
 * the 2 counters into a byte .
 */

/* First 16 virtual counters correspond to the 
 * hardware counter 0.
 * Next 16 corr. to hardware counter 1
 */
#define HWPERF_CNT1BASE 16

#define HWPERF_CNTEN_U  0x8  /* user mode counting */
#define HWPERF_CNTEN_S  0x4  /* supervisor mode    */
#define HWPERF_CNTEN_K  0x2  /* kernel mode        */
#define HWPERF_CNTEN_E  0x1  /* exception level    */
#define HWPERF_CNTEN_A  0xA  /* all "real" modes   */
#define HWPERF_CNTEN_IE 0x10  /* enable overflow interrupt */

#define HWPERF_MINEVENT 0  /* Minimum event number for a counter */
#define HWPERF_MAXEVENT 15 /* Maximum event number for a counter */

#define HWPERF_OVERFLOW_THRESH	0x80000000

/*
 * Following are the various kinds of events that can be counted in the T5
 * performance counters.
 */

#define HWPERF_C0PRFCNT0_CYCLES   0  /* Cycles */
#define HWPERF_C0PRFCNT0_IINSTR   1  /* Issued instructions */
#define HWPERF_C0PRFCNT0_ILD      2  /* Issued loads */
#define HWPERF_C0PRFCNT0_IST      3  /* Issued stores */
#define HWPERF_C0PRFCNT0_ISC      4  /* Issued store conditionals */
#define HWPERF_C0PRFCNT0_FAILSC   5  /* Failed store conditionals */
#define HWPERF_C0PRFCNT0_DECBR    6  /* Decoded branches */
#define HWPERF_C0PRFCNT0_QWWSC    7  /* Quadwords written back to scache */
#define HWPERF_C0PRFCNT0_SCDAECC  8  /* Correctable scache data array errors */
#define HWPERF_C0PRFCNT0_PICMISS  9  /* Primary instruction cache misses */
#define HWPERF_C0PRFCNT0_SICMISS  10 /* Secondary instruction cache misses */
#define HWPERF_C0PRFCNT0_IMISPRED 11 /* Instruction misprediction from scache
					way predication table */
#define HWPERF_C0PRFCNT0_EXTINT   12 /* External interventions */
#define HWPERF_C0PRFCNT0_EXTINV   13 /* External invalidates */
#define HWPERF_C0PRFCNT0_VCOH     14 /* Virtual coherency conditon */
#define HWPERF_C0PRFCNT0_GINSTR   15 /* Graduated instructions */


#define HWPERF_C0PRFCNT1_CYCLES   0  /* Cycles */
#define HWPERF_C0PRFCNT1_GINSTR   1  /* Graduated instructions */
#define HWPERF_C0PRFCNT1_GLD      2  /* Graduated loads */
#define HWPERF_C0PRFCNT1_GST      3  /* Graduated stores */
#define HWPERF_C0PRFCNT1_GSC      4  /* Graduated store conditionals */
#define HWPERF_C0PRFCNT1_GFINSTR  5  /* Graduated floating point instructions */
#define HWPERF_C0PRFCNT1_QWWPC    6  /* Quadwords written back from primary
					data cache */
#define HWPERF_C0PRFCNT1_TLBMISS  7  /* Translation lookaside buffer misses */
#define HWPERF_C0PRFCNT1_BRMISS   8  /* Mispredicted branches */
#define HWPERF_C0PRFCNT1_PDCMISS  9  /* Primary data cache misses */
#define HWPERF_C0PRFCNT1_SDCMISS  10 /* Secondary data cache misses */
#define HWPERF_C0PRFCNT1_DMISPRED 11 /* Data misprediction from scache
					way predication table */
#define HWPERF_C0PRFCNT1_EXTINTHIT 12 /* External intervention hits in scache */
#define HWPERF_C0PRFCNT1_EXTINVHIT 13 /* External invalidate hits in scache */
#define HWPERF_C0PRFCNT1_SPEXCLEAN 14 /* Store/prefetch exclusive to clean
					 block in secondary cache */
#define HWPERF_C0PRFCNT1_SPEXSHR   15 /* Store/prefetch exclusive to shared
					 block in secondary cache */

#ifdef _KERNEL

#ifdef R10000

#include <sys/hwperftypes.h>
#include <sys/pda.h>

#define	SUSPEND_HWPERF_COUNTERS(p)	{				\
		if (Cpu_hwperf_owner) {					\
			hwperf_stop_cntrs();				\
			hwperf_read_cntrs(&u.u_cpu_mon, 1, 0);		\
			u.u_cpu_mon.cm_flags |=	HWPERF_CNTRS_UNLCKED;	\
			Cpu_hwperf_owner = NULL;			\
			mrunlock(&evcntrs_lock);			\
		}							\
	}

#define	START_HWPERF_COUNTERS(p) 	{			 	\
		if (!IS_KTHREAD((p)) &&					\
		    ((u.u_cpu_mon.cm_events0 > 0) || 			\
		    (u.u_cpu_mon.cm_events1 > 0))) {			\
			if (!Sys_hwperf_owner) {			\
				ASSERT(u.u_cpu_mon.cm_flags &		\
						HWPERF_CNTRS_UNLCKED); 	\
				if (!cmrlock(&evcntrs_lock, MR_ACCESS))	{\
					u.u_cpu_mon.cm_gen++;		\
					u.u_cpu_mon.cm_flags |=		\
						HWPERF_LOST_CNTRS;	\
					u.u_cpu_mon.cm_events0 = 0;	\
					u.u_cpu_mon.cm_events1 = 0;	\
				} else {				\
					Cpu_hwperf_owner = 		\
					   &curprocp->p_user->u_cpu_mon;\
					u.u_cpu_mon.cm_flags &=		\
						~HWPERF_CNTRS_UNLCKED;	\
					hwperf_start_cntrs();		\
				}					\
			} else {					\
				u.u_cpu_mon.cm_gen++;			\
				u.u_cpu_mon.cm_flags |=	HWPERF_LOST_CNTRS;\
				u.u_cpu_mon.cm_events0 = 0;		\
				u.u_cpu_mon.cm_events1 = 0;		\
			}						\
		 }						 	\
	}

#define	STOP_HWPERF_COUNTERS(p)	{			 		\
	if (!Sys_hwperf_owner && !IS_KTHREAD((p)) && 			\
	    ((u.u_cpu_mon.cm_events0 > 0) || (u.u_cpu_mon.cm_events1 > 0)))\
		(void) hwperf_diseventctr(&curprocp->p_user->u_cpu_mon);\
	}

#define USING_HWPERF_COUNTERS(p) 					\
		 (!IS_KTHREAD((p)) && ((u.u_cpu_mon.cm_events0 > 0) ||	\
			(u.u_cpu_mon.cm_events1 > 0)))		

#define VALID_HWPERF_COUNTERS(p)					\
		(!IS_KTHREAD((p)) && 					\
		((p)->p_user->u_cpu_mon.cm_flags & HWPERF_CNTRS_VALID))

#define MULTIPLEX_HWPERF_COUNTERS()					\
	{								\
		cpu_mon_t *cmp = NULL;					\
		if (Cpu_hwperf_owner)					\
			cmp = Cpu_hwperf_owner;				\
		else if (Sys_hwperf_owner) 				\
			cmp = private.p_cpu_mon;			\
		if (cmp) {						\
			if ((cmp->cm_events0 > 1) || 			\
			    (cmp->cm_events1 > 1))			\
				hwperf_multiplex_events();		\
		}							\
	}

struct proc;
struct eframe_s;

extern int hwperf_eneventctr(cpu_mon_t *, hwperf_profevctrarg_t *);
extern int hwperf_diseventctr(cpu_mon_t *);
extern int hwperf_get_cntrs(cpu_mon_t *, hwperf_cntr_t *);
extern int hwperf_get_cpucntrs(int, hwperf_cntr_t *);
extern int hwperf_getctrl(cpu_mon_t *, hwperf_eventctrl_t *);
extern int hwperf_setctrl(cpu_mon_t *, hwperf_profevctrarg_t *);
extern int hwperf_read_cntrs(cpu_mon_t *, int, int);
extern void hwperf_inc_cntrs(cpu_mon_t *, int, int);
extern int hwperf_getprusage(struct proc *, hwperf_prusage_t *);
extern void hwperf_stop_cntrs(void);
extern int hwperf_start_cntrs(void);
extern void hwperf_multiplex_events(void);
extern void hwperf_add_cntrs(struct proc *, struct proc *);
extern int hwperf_set_flag(cpu_mon_t *, int);
extern int hwperf_syscntrs(hwperf_profevctrarg_t *);
extern void hwperf_release_cntrs(void);
extern void hwperf_intr(struct eframe_s *);
extern int get_C0PRFCNT0(void);
extern int get_C0PRFCNT1(void);
extern int get_C0PRFCRT0(void);
extern int get_C0PRFCRT1(void);
extern int set_C0PRFCNT0(int);
extern int set_C0PRFCNT1(int);
extern int set_C0PRFCRT0(int);
extern int set_C0PRFCRT1(int);

extern cpu_mon_t System_hwperf_mon;
extern struct proc *Sys_hwperf_owner;
extern mrlock_t evcntrs_lock;
extern lock_t evcntrs_upd_lock;


#else /* R10000 */

#define	SUSPEND_HWPERF_COUNTERS(p)
#define	START_HWPERF_COUNTERS(p)
#define	STOP_HWPERF_COUNTERS(p)
#define USING_HWPERF_COUNTERS(p) 	0
#define VALID_HWPERF_COUNTERS(p)	0
#define MULTIPLEX_HWPERF_COUNTERS()

#endif /* R10000 */

#endif /* _KERNEL */

#endif /* __SYS_HWPERFMACROS_H__ */
