/*
 * Copyright 1995 Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef __SYS_HWPERFTYPES_H__
#define __SYS_HWPERFTYPES_H__

#ident	"$Revision: 1.9 $"

/*
 * Type definitions for performance monitoring
 */


/*
 * Hub performance monitoring
 */

typedef struct hub_mon {
	int dummy;
} hub_mon_t;


#include <sys/time.h>

#define HWPERF_CNTRS		2	/* number of counters */
#define HWPERF_EVENTMAX		32	/* Max. number of events that can be
					 * counted together in the counters
					 */
#define HWPERF_CNTEVENTMAX	16	/* Max # events per counter */

/*
 * R10000 performance monitoring
 */

typedef struct cpu_mon {
	short	cm_events0;	/* number of events monitored by cntr0 */
	short	cm_events1;	/* number of events monitored by cntr1 */
	short	cm_evindx0;	/* current event for ctr 0 */
	short	cm_evindx1;	/* current event for ctr 1 */
	short	cm_evspec[HWPERF_EVENTMAX];	/* specifiers for counters */
	short	cm_flags;
	short	cm_sig;		/* signal to deliver upon cntr overflow */
	int	cm_gen;		/* genration number for specifiers */
	uint	cm_preloadcnt[HWPERF_EVENTMAX]; /* preload values */
	uint	cm_savecnt[HWPERF_EVENTMAX]; /* saved values */
	__uint64_t cm_eventcnt[HWPERF_EVENTMAX];  /* performance counters */
} cpu_mon_t;

/* flags for cm_flags */
#define HWPERF_CNTRS_VALID	0x1
#define HWPERF_OVFLW_THRESH	0x2
#define HWPERF_SAVE_CCNTRS	0x4
#define HWPERF_CNTRS_UNLCKED	0x8
#define HWPERF_LOST_CNTRS	0x10

typedef struct perf_mon {

	cpu_mon_t pm_cpu_mon;    /* cpu performance monitoring counters */
	hub_mon_t *pm_hub_mon;   /* hub performance monitoring counters */ 
} perf_mon_t;

/*
 * structure used to retrieve the counts 
 */
typedef struct {
	__uint64_t hwp_evctr[HWPERF_EVENTMAX];
} hwperf_cntr_t;


/*
 * The following is used to set/retrieve the information
 * on which events are being tracked.
 */
struct hwperf_ctrlreg {
	ushort_t	hwp_ev  :11,	/* event to be counted */
			hwp_ie	:1,	/* overflow interrupt enable */
			hwp_mode:4;	/* user/kernel mode */
};

typedef union {
	short			hwperf_spec;
	struct hwperf_ctrlreg	hwperf_creg;
} hwperf_ctrl_t;

typedef struct {
	hwperf_ctrl_t 	hwp_evctrl[HWPERF_EVENTMAX];
} hwperf_eventctrl_t;

typedef struct hwperf_profevctrarg {
	hwperf_eventctrl_t	hwp_evctrargs;
	int			hwp_ovflw_freq[HWPERF_EVENTMAX];
	int			hwp_ovflw_sig; /* SIGUSR1,2 */
} hwperf_profevctrarg_t;
	
typedef struct hwperf_prusage {

	timestruc_t pu_tstamp;	/* time stamp */ 
	timestruc_t pu_starttime;	/* time process was started */ 
	timestruc_t pu_utime;	/* user CPU time */ 
	timestruc_t pu_stime;	/* system CPU time */ 
	__uint64_t pu_minf;	/* minor (mapping) page faults */
	__uint64_t pu_majf;	/* major (disk) page faults */
	__uint64_t pu_utlb;	/* user TLB misses */
	__uint64_t pu_nswap;	/* swaps (process only) */
	__uint64_t pu_gbread;	/* gigabytes ... */ 
	__uint64_t pu_bread;	/* and bytes read */
	__uint64_t pu_gbwrit;	/* gigabytes ... */ 
	__uint64_t pu_bwrit;	/* and bytes written */
	__uint64_t pu_sigs;	/* signals received */
	__uint64_t pu_vctx;	/* voluntary context switches */
	__uint64_t pu_ictx;	/* involuntary context switches */
	__uint64_t pu_sysc;	/* system calls */ 
	__uint64_t pu_syscr;	/* read() system calls */ 
	__uint64_t pu_syscw;	/* write() system calls */ 
	__uint64_t pu_syscps;	/* poll() or select() system calls */
	__uint64_t pu_sysci;	/* ioctl() system calls */ 
	__uint64_t pu_graphfifo;	/* graphics pipeline stalls */
	__uint64_t pu_graph_req[8];	/* graphics resource requests */
	__uint64_t pu_graph_wait[8];/* graphics resource waits */
	__uint64_t pu_size;	/* size of swappable image in pages */
	__uint64_t pu_rss;		/* resident set size */
	__uint64_t pu_inblock;	/* block input operations */
	__uint64_t pu_oublock;	/* block output operations */
	__uint64_t pu_vfault;	/* total number of vfaults */
	__uint64_t pu_ktlb;	/* kernel TLB misses */
	cpu_mon_t pu_cpu_mon;   /* cpu monitoring stats */
} hwperf_prusage_t;

#endif /* __SYS_HWPERFTYPES_H__ */
