/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989-1992, Silicon Graphics, Inc.	  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/
#ifndef __SYS_IMMU_H__
#define __SYS_IMMU_H__

#ident	"$Revision: 3.172 $"

#include <sys/sbd.h>
#include <sys/mips_addrspace.h>
#if IP32
#include <sys/cpu.h>
#endif

#define	MIN_NBPP	4096		/* smallest acceptable page size */
#define MIN_PNUMSHFT	12		/* Shift for page number from addr. */
#define MIN_POFFMASK	(MIN_NBPP - 1)	/* Mask for offset into page. */

#define	IO_NBPP		4096		/* Number of bytes per page (for DMA) */
#define IO_PNUMSHFT	12		/* Shift for page number from addr. */
#define IO_POFFMASK	(IO_NBPP - 1)	/* Mask for offset into page. */

#if _PAGESZ == 4096
#define PNUMSHFT	12		/* Shift for page number from addr. */
#endif
#if _PAGESZ == 16384
#define PNUMSHFT	14		/* Shift for page number from addr. */
#endif

#define	NBPP		_PAGESZ		/* Number of bytes per page (virtual) */
#define	PGFCTR		(NBPP / MIN_NBPP)
#define	PGSHFTFCTR	(PNUMSHFT - MIN_PNUMSHFT)

#if !LANGUAGE_ASSEMBLY

/*
 * This structure is used to define the sizes of various constants that
 * are shared between kernel and administrative utilities.
 */
struct pageconst {
	int		p_pagesz;	/* _PAGESZ */
	int		p_pnumshft;	/* PNUMSHFT */
	int		p_nbpp;		/* NBPP */
};

#if _KERNEL
#include <sys/sema.h>
#endif

/*
 * Page Descriptor (Table) Entry Definitions for MIPS CPU.
 */

/* For the R2000/R3000
 *      	  <- hard bits  -> <- soft bits ->
 *  +--------------------+-+-+--+-+--+--+--+--+--+
 *  |        pfn         |n|m|vr|g|  |sv| d|  |nr|
 *  +--------------------+-+-+--+-+--+--+--+--+--+
 *             20         1 1  1 1  1  1  1  2  3  
 *
 * For the R4000 IP17/IP19/IP20/IP22
 *       <-             hard bits               ->
 *  +---+-------------------------------+--+-+-+-+
 *  |   |              pfn		|cc|m|v|g|
 *  +---+-------------------------------+--+-+-+-+
 *    2                  24		  3 1 1 1
 *
 * For the IP17/MIPS-II, we use 4 address bits from pfn
 * together with 2 unused bit for the software bits BUT we left shift
 * the HW bits.  This lets us clear the SW bits with a single srl of
 * 6 bits.
 *
 *  +---------------------+--+-+-+-+--+---+---+--+
 *  |        pfn          |cc|m|v|g|nr|eop|d/w|sv|
 *  +---------------------+--+-+-+-+--+---+---+--+
 *  	    20		    3 1 1 1  3   1   1  1
 *
 * For the IP19(R4000A)/MIPS-II and IP30 R4x00, only two address bits are
 * unused.  As above, we left shift the HW bits so that SW bits can be
 * cleared with a single srl, this time of 4 bits.
 *
 *  +-------------------------+--+-+-+-+--+---+--+
 *  |		 pfn          |cc|m|v|g|nr|d/w|sv|
 *  +-------------------------+--+-+-+-+--+---+--+
 *     	    23			3 1 1 1  1  1   1
 *
 * For the IP21/MIPS-III
 *
 * #ifdef TFP_PTE64
 *
 *  <- soft bits -> <-    hard bits               <-soft
 *  +--------------+----------------------+--+-+-+--+-------+
 *  |              |		pfn	  |cc|m|v|  |nr|d|sv|
 *  +--------------+----------------------+--+-+-+--+--+-+--+
 *        24			28	    3 1 1  2  3 1  1 
 * #else
 *
 *     <-    hard bits            <-soft
 *  +-+-------------------+--+-+-+--+-+--+
 *  |0|		pfn	  |cc|m|v|nr|d|sv|
 *  +-+-------------------+--+-+-+--+-+--+
 *   1 		23	    3 1 1  1 1  1 
 * #endif
 *
 * For the IP26/TFP (hard bits the same as R4000 for VDMA engine compat).
 *      soft   <-       hard bits            -> soft
 *  +-+--+--+-+-+-----------------------+--+-+-+-+
 *  |0|sv|nr|0|0|      pfn		|cc|m|v|d|
 *  +-+--+--+-+-+-----------------------+--+-+-+-+
 *   1 1  3  1 1         19		  3 1 1 1
 *
 * For R10000:
 *
 *      soft <-       hard bits            -> soft
 *  +--+-+--+--------------------------+--+-+-+-+
 *  |nr|d|sv|          pfn             |cc|m|v|g|
 *  +--+-+--+--------------------------+--+-+-+-+
 *   1  1 1            23               3  1 1 1
 *
 */

/*
 * Hardware bits:
 *
 *  pfn	- Page frame number
 *  n	- if set is non-cacheable
 *  m	- modified bit if clear a write will cause fault
 *  vr	- Valid and referenced bit.  If clear a fault will occur.
 *	  Used as reference bit also because there is no HW reference bit.
 *  g	- Global so ignore pid in translations
 *  cc  - (R4000) Cache coherency algorithm.
 *
 * Software bits:
 *
 *  d/w - page is dirty (with respect to pte's mapping),
 *	  or process wants wakeup after read of (virtual) page.
 *  sv	- software valid bit set if page is valid
 *  nr  - Reference count. Set to all ones on reference.
 *	  Decremented (not below zero) when page is aged by vhand.
 *        Pages with lower nr will be taken by getpages.
 */

#if NBPP == MIN_NBPP
#define	PTE_PFN(num)	pte_pfn:num
#define PFNWIDTH	PFNDWIDTH
#define PFNSHIFT	0
#elif !defined(_KERNEL)
#define	PTE_PFN(num)	pte_pfn:num
#define PFNWIDTH	PFNDWIDTH
#define PFNSHIFT	0
#else
#define	PTE_PFN(num)	 pte_pfn:num-PGSHFTFCTR, \
				:PGSHFTFCTR
#define PFNWIDTH	(PFNDWIDTH-PGSHFTFCTR)
#define PFNSHIFT	PGSHFTFCTR
#endif


#if R4000 || IP28
#ifdef IP17 
typedef struct pte {
	uint	PTE_PFN(20),	/* HW: core page frame number or 0 */
		pte_cc:3,	/* HW: coherency algorithm for this page */
		pte_m:1,	/* HW: modified (dirty) bit */
		pte_vr:1,	/* HW: valid bit (& SW referenced)*/
		pte_g:1,	/* HW: ignore pid bit */
		pte_nr : 3,	/* SW: page reference count */
		pte_eop : 1,	/* SW: end-of-page bug */
		pte_dirty : 1,	/* SW: page dirty bit */
		pte_sv : 1;	/* SW: page valid bit */
} pte_t;
#define PFNDWIDTH	20
#define PFNOFFSET	0
#endif	/* IP17 */
#if defined(IP20) || defined(IP22) || defined(IP28) || defined(IPMHSIM)
typedef struct pte {
        uint    pte_dirty : 1,  /* SW: page dirty bit */
                pte_sv : 1,     /* SW: page valid bit */
                pte_nr : 3,     /* SW: page reference count */
                pte_eop : 1,    /* SW: end-of-page bug */
                PTE_PFN(20),    /* HW: core page frame number or 0 */
                pte_cc:3,       /* HW: coherency algorithm for this page */
                pte_m:1,        /* HW: modified (dirty) bit */
                pte_vr:1,       /* HW: valid bit (& SW referenced)*/
                pte_g:1;        /* HW: ignore pid bit */
} pte_t;
#define PFNDWIDTH	20
#define PFNOFFSET	6
#endif	/* IP20 || IP22 || IP28 */
#if defined(IP32)
typedef struct pte {
        uint    pte_dirty : 1,  /* SW: page dirty bit */
                pte_sv : 1,     /* SW: page valid bit */
                pte_nr : 1,     /* SW: page reference count */
                pte_eop : 1,    /* SW: end-of-page bug */
                PTE_PFN(22),    /* HW: core page frame number or 0 */
                pte_cc:3,       /* HW: coherency algorithm for this page */
                pte_m:1,        /* HW: modified (dirty) bit */
                pte_vr:1,       /* HW: valid bit (& SW referenced)*/
                pte_g:1;        /* HW: ignore pid bit */
} pte_t;
#define PFNDWIDTH       22
#define PFNOFFSET       4
#endif	/* IP32 */
#if IP19 || IP30
typedef struct pte {
#ifdef JUMP_WAR
	uint	PTE_PFN(22),	/* HW: core page frame number or 0 */
		pte_cc : 3,	/* HW: coherency algorithm for this page */
		pte_m : 1,	/* HW: modified (dirty) bit */
		pte_vr : 1,	/* HW: valid bit (& SW referenced)*/
		pte_g : 1,	/* HW: ignore pid bit */
		pte_nr : 1,	/* SW: page reference count */
		pte_eop : 1,	/* SW: end-of-page bug */
#define PFNDWIDTH	22
#else
	uint	PTE_PFN(23),	/* HW: core page frame number or 0 */
		pte_cc : 3,	/* HW: coherency algorithm for this page */
		pte_m : 1,	/* HW: modified (dirty) bit */
		pte_vr : 1,	/* HW: valid bit (& SW referenced)*/
		pte_g : 1,	/* HW: ignore pid bit */
		pte_nr : 1,	/* SW: page reference count */
#define PFNDWIDTH	23
#endif
		pte_dirty : 1,	/* SW: page dirty bit */
		pte_sv : 1;	/* SW: page valid bit */
} pte_t;
#define PFNOFFSET	0
#endif /* IP19 */
#endif	/* R4000 */

#ifdef R10000
#if IP25 || IP30
typedef struct pte {
	__uint32_t
		pte_nr : 1,	/* SW: page reference count */
		pte_dirty : 1,	/* SW: page dirty bit */
		pte_sv : 1,	/* SW: page valid bit */
		PTE_PFN(23),	/* HW: core page frame number or 0 */
		pte_cc : 3,	/* HW: coherency algorithm for this page */
		pte_m : 1,	/* HW: modified (dirty) bit */
		pte_vr : 1,	/* HW: valid bit (& SW referenced)*/
		pte_g : 1;	/* HW: ignore pid bit */
} pte_t;
#define PFNDWIDTH	23
#define PFNOFFSET	3
#endif /* IP25 || IP30 */
#endif /* R10000 */

#if TFP
#if IP21
typedef struct pte {
#ifdef TFP_PTE64
	__uint64_t
		pte_swun1: 24,	/* SW: unused */
		PTE_PFN(28),	/* HW: core page frame number or 0 */
		pte_cc : 3,	/* HW: coherency algorithm for this page */
		pte_m : 1,	/* HW: modified (dirty) bit */
		pte_vr : 1,	/* HW: valid bit (& SW referenced)*/
		pte_swun2: 2,	/* SW: unused */
		pte_nr : 3,	/* SW: page reference count */
		pte_dirty : 1,	/* SW: page dirty bit */
		pte_sv : 1;	/* SW: page valid bit */
#define PFNDWDITH	28
#define PFNOFFSET	24
#else
	/* This definition lets ptes fit in 32 bits.  Fields are same sizes
	 * as for the other EVEREST cpu board -- IP19.
	 */
	__uint32_t
		pte_unused: 1,	/* SW: unused - zero sign extension */
		PTE_PFN(23),	/* HW: core page frame number or 0 */
		pte_cc : 3,	/* HW: coherency algorithm for this page */
		pte_m : 1,	/* HW: modified (dirty) bit */
		pte_vr : 1,	/* HW: valid bit (& SW referenced)*/
		pte_nr : 1,	/* SW: page reference count */
		pte_dirty : 1,	/* SW: page dirty bit */
		pte_sv : 1;	/* SW: page valid bit */
#define PFNDWIDTH	23
#define PFNOFFSET	1
#endif
} pte_t;
#endif	/* IP21 */
#ifdef IP26		/* HW bits need to be the same as IP20/IP22 */
typedef struct pte {
	uint	pte_unused : 1,	/* SW: bit zero to avoid sign extension */
		pte_sv : 1,	/* SW: page valid bit */
		pte_nr : 3,	/* SW: page reference count */
		pte_unused2 : 1,/* SW: unused */
		pte_zero : 1,	/* HW: bit 31 of pfn -- must be zero */
		PTE_PFN(19),	/* HW: core page frame number or 0 */
		pte_cc : 3,	/* HW: coherency algorithm for this page */
		pte_m : 1,	/* HW: modified (dirty) bit */
		pte_vr : 1,	/* HW: valid bit (& SW reference) */
		pte_dirty : 1;	/* SW: page dirty bit (R4000 global bit) */
} pte_t;
#define PFNDWIDTH	19
#define PFNOFFSET	7
#endif	/* IP26 */
#endif	/* TFP */

#if !R4000 && !TFP && !R10000
typedef struct pte {
	uint_t	opaque_pte;
} pte_t;
#endif

typedef union pde {
	pte_t		pte;		/* pte structure */
	uint_t		pgi;		/* Full page table entry */
} pde_t;

#define pg_pfn	pte_pfn
#define pg_m	pte_m
#define pg_vr	pte_vr
#define pg_g	pte_g
#define pg_sv	pte_sv
#define pg_nr	pte_nr
#define pg_waiter pte_dirty
#define pg_dirty pte_dirty

#if R4000 || TFP || R10000
#define pg_cc	pte_cc
#endif

/*
 * For the R4000, a single tlb maps 2 consecutive virtual pages.
 * This structure is used in the upage, for wired entries.
 */
typedef struct {
	pde_t	pde_low;
	pde_t	pde_hi;
} tlbpde_t;

#endif /* !LANGUAGE_ASSEMBLY */

					/* A disk block is 512 bytes (sector) */
#define NDPP		(8*PGFCTR)	/* Number of disk blocks per page */
#define DPPSHFT		(3+PGSHFTFCTR)	/* Shift for disk blocks per page. */

/*
 * IO map registers (usually) map 4KB chunks while system
 * virtual memory may be using a larger page size.
 */
#if NBPP == IO_NBPP
#define	SYSTOIOPFN(x,i)	(x)
#else
#define	SYSTOIOPFN(x,i)	(((x) << PGSHFTFCTR) + ((i) % PGFCTR))
#endif

/* Page descriptor (table) entry dependent constants */

					/* Nbr of page tbls to map user space */
#define NPGTBLS		btos(HIUSRATTACH_32)
#define	NBPPT		NBPP		/* Number of bytes per page table */
					/* Nbr of pages per page table (seg). */
#define NPGPT		(NBPPT/sizeof(pte_t))
#define	BPTSHFT		PNUMSHFT	/* LOG2(NBPPT) if exact */

#define POFFMASK	(NBPP - 1)	/* Mask for offset into page. */
#define PTOFFMASK	(NPGPT - 1)	/* Mask for offset into page table. */
#define SOFFMASK	((NPGPT * NBPP) - 1)/* Mask for offset into segment. */
#define soff(x)		((__psunsigned_t)(x) & SOFFMASK)
#define ptoff(x)	((__psunsigned_t)(x) & PTOFFMASK)

#define PG_SENTRY	0x7f		/* physical address <= this */
					/* sentinel special handling */
#define PDEFER_SENTRY	0x01		/* defer to next pmap handler */
#define WAIT_SENTRY_LO	0x08		/* physical address as wait key ... */
#define WAIT_SENTRY_HI	0x7f		/* ... for page fault READMAP */
#ifdef _VM_WS_SWAPPING
#define WS_SENTRY 0x02			/* page swapped out and in */
					/* working set */
#define WS_SENTINEL (WS_SENTRY << PTE_PFNSHFT)
#endif /* _VM_WS_SWAPPING */

/* Constants for three-level segment table
 *
 * A three level mapping scheme is composed of a base table, which
 * has pointers to the 2nd level segment tables, which in turn point
 * to the 1st level page tables.
 */

/* Number of entries in a 2nd level table. Currently, this is the
 * same as a segment table, which is enough to map 0x80000000 per
 * segment table
 */
#define SEGTABSIZE		NPGTBLS

/* The number of segment tables required to map extended user space. */
#define BASETABSIZE		howmany(HIUSRATTACH_64, stob(SEGTABSIZE))

/* Select a segment pointer from the base table - this is log2(SEGTABSIZE) */
#if _PAGESZ == 4096
#define BASETABSHIFT		9
#endif
#if _PAGESZ == 16384
#define BASETABSHIFT		5
#endif

/* mask off high order bits to select a page table pointer within a segment */
#define SEGTABMASK		(SEGTABSIZE-1)

/* Number of bytes mapped by a segment table */
#define NBPSEGTAB		(SEGTABSIZE * NBPS)

/* Number of pages mapped by a segment table */
#define NPGSEGTAB		(NPGPT * SEGTABSIZE)

/* Given a page number, find its page number within a space
 * mapped by a segment table
 */
#define SEGPGMASK		(NPGSEGTAB - 1)

/* Bytes per base tab shift - turn a vaddr into a base table index */
#define BPBASETABSHIFT		(BPSSHIFT+BASETABSHIFT)

/* A shift amount to scale a pointer into a base table -
 * log2(sizeof pcom_t) - for assembly coding.
 */
#define PCOM_TSHIFT		4

/* given vaddr x, find index into base table (or segment table number) */
#define	btobasetab(x)		((__psunsigned_t)(x) >> BPBASETABSHIFT)

/* given vaddr x, find index into a segment table */
#define btosegtabindex(x)	(btost(x) & SEGTABMASK)

/* given page number x, find index into a segment table */
#define ctosegtabindex(x)	(ctost(x) & SEGTABMASK)

/* given segment x, find index into a segment table */
#define segtosegtabindex(x)	((__psunsigned_t)(x) & SEGTABMASK)

/* given page number x, find its page number within a space mapped
 * by a segment table
 */
#define ctosegpg(x)		((__psunsigned_t)(x) & SEGPGMASK)

/* Page descriptor (table) entry field masks */


#ifdef IP17
#define	PG_CACHMASK	0x00000e00	/* cache coherency algorithm */
#define	PG_CACHSHFT	9		/* position in pte_t */
#define PG_UNCACHED	0x00000400	/* non-cached */
#define PG_TRUEUNCACHED	PG_UNCACHED	/* non-cached */
#if _RUN_UNCACHED
#define	PG_NONCOHRNT	PG_UNCACHED
#define PG_COHRNT_EXLWR	PG_UNCACHED
#else
#define	PG_NONCOHRNT	0x00000600	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x00000a00	/* exclusive write */
#endif
#define PG_M		0x00000100	/* modified bit	*/
#define PG_VR		0x00000080	/* valid and referenced  bit */
#define PG_G		0x00000040	/* global (ignore pid)  bit */
#define PG_EOP		0x00000004	/* end-of-page bit */
#define PG_NDREF	0x00000038	/* reference count (software) */
#define NDREF		7		/* positionless count */
#define PG_NDREFNORM	3		/* normalize NDREF for arithmetic */
#define PG_ADDR		(0xfffff000<<PGSHFTFCTR) /* physical page address */
#define PTE_PFNSHFT	(12+PGSHFTFCTR)	/* shift pfn to place in pte */

#define pg_isnoncache(P)	(pg_cachemode(P) == PG_UNCACHED)
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_UNCACHED)

#define PG_D		0x00000002	/* page is dirty w.r.t. pte */
#define PG_SV		0x00000001	/* software valid bit */
#define PG_N		PG_UNCACHED
#define PG_PFNUM	PG_ADDR		/* physical page address */
#endif	/* IP17 */

#if defined(IP20) || defined(IP22) || defined(IP28) || defined(IP32) || defined(IPMHSIM)

#define	PG_CACHMASK	0x00000038	/* cache coherency algorithm */
#define	PG_CACHSHFT	3		/* position in pte_t */
#define PG_UNCACHED	0x00000010	/* non-cached */
#define PG_TRUEUNCACHED	PG_UNCACHED	/* non-cached */
#if _RUN_UNCACHED
#define	PG_NONCOHRNT	PG_UNCACHED
#define PG_COHRNT_EXLWR	PG_UNCACHED
#else
#define	PG_NONCOHRNT	0x00000018	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x00000028	/* exclusive write */
#endif
#define PG_M		0x00000004	/* modified bit	*/
#define PG_VR		0x00000002	/* valid and referenced  bit */
#define PG_G		0x00000001	/* global (ignore pid)  bit */
#if IP32
#define PG_EOP		0x10000000	/* end-of-page bit */
#define PG_NDREF	0x20000000	/* reference count (software) */
#define NDREF		1		/* positionless count */
#define PG_NDREFNORM	29		/* normalize NDREF for arithmetic */
#define PG_ADDR		(0x0fffffc0<<PGSHFTFCTR) /* physical page address */
#else
#define PG_EOP		0x04000000	/* end-of-page bit */
#define PG_NDREF	0x38000000	/* reference count (software) */
#define NDREF		7		/* positionless count */
#define PG_NDREFNORM	27		/* normalize NDREF for arithmetic */
#define PG_ADDR		(0x03ffffc0<<PGSHFTFCTR) /* physical page address */
#endif  /* IP32 */
#define PTE_PFNSHFT	(6+PGSHFTFCTR)	/* shift pfn to place in pte */

#define pg_isnoncache(P)	(pg_cachemode(P) == PG_UNCACHED)
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_UNCACHED)

#define PG_D		0x80000000	/* page is dirty w.r.t. pte */
#define PG_SV		0x40000000	/* software valid bit */
#define PG_N		PG_UNCACHED
#define PG_PFNUM	PG_ADDR		/* physical page address */

#ifdef R10000
#define	FRAMEMASK_MASK	0x0000fe00	/* Frame Mask register value */
#endif

#endif	/* IP20 || IP22 || IP28 || IP32 */

#if IP19 || (IP30 && R4000)
#ifdef JUMP_WAR
#define PG_EOP		0x00000004	/* end-of-page bit */
#define PG_NDREF	0x00000008	/* reference count (software) */
#define NDREF		1		/* positionless count */
#define PG_NDREFNORM	3		/* normalize NDREF for arithmetic */
#define	PG_CACHMASK	0x00000380	/* cache coherency algorithm */
#define	PG_CACHSHFT	7		/* position in pte_t */
#define PG_TRUEUNCACHED	0x00000100	/* non-cached */
#define PG_M		0x00000040	/* modified bit	*/
#define PG_VR		0x00000020	/* valid and referenced  bit */
#define PG_G		0x00000010	/* global (ignore pid)  bit */
#define PG_ADDR		(0xfffffc00<<PGSHFTFCTR) /* physical page address */
#define PTE_PFNSHFT	(10+PGSHFTFCTR)	/* shift pfn to place in pte */

#if _RUN_UNCACHED
#define	PG_NONCOHRNT	PG_UNCACHED
#define PG_COHRNT_EXLWR	PG_UNCACHED
#else
#define	PG_NONCOHRNT	0x00000180	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x00000280	/* exclusive write */
#if IP19
#define	PG_COHRNT_EXCL	0x00000200	/* exclusive access */
#endif
#endif

#else   /* No JUMP_WAR */
#define PG_NDREF	0x00000004	/* reference count (software) */
#define NDREF		1		/* positionless count */
#define PG_NDREFNORM	2		/* normalize NDREF for arithmetic */
#define	PG_CACHMASK	0x000001c0	/* cache coherency algorithm */
#define	PG_CACHSHFT	6		/* position in pte_t */
#define PG_TRUEUNCACHED	0x00000080	/* non-cached */
#define PG_M		0x00000020	/* modified bit	*/
#define PG_VR		0x00000010	/* valid and referenced  bit */
#define PG_G		0x00000008	/* global (ignore pid)  bit */
#define PG_ADDR		(0xfffffd00<<PGSHFTFCTR) /* physical page address */
#define PTE_PFNSHFT	(9+PGSHFTFCTR)	/* shift pfn to place in pte */

#if _RUN_UNCACHED
#define	PG_NONCOHRNT	PG_UNCACHED
#define PG_COHRNT_EXLWR	PG_UNCACHED
#else
#define	PG_NONCOHRNT	0x000000c0	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x00000140	/* exclusive write */
#if IP19
#define	PG_COHRNT_EXCL	0x00000100	/* exclusive access */
#endif
#endif

#endif	/* no JUMP_WAR */

#define PG_D		0x00000002	/* page is dirty w.r.t. pte */
#define PG_SV		0x00000001	/* software valid bit */
#define PG_PFNUM	PG_ADDR		/* physical page address */

#if IP30
#define PG_N		PG_UNCACHED
#define PG_UNCACHED	PG_TRUEUNCACHED
#define pg_isnoncache(P)	(pg_cachemode(P) == PG_UNCACHED)
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_UNCACHED)
#endif	/* IP30 */
#endif /* IP19 || (IP30 && R4000) */

#if IP21
#ifdef TFP_PTE64	/* in case we ever use 64-bit ptes */
#define PG_ADDR		(0x000000fffffff000<<PGSHFTFCTR)	/* physical page address */
#define PTE_PFNSHFT	(12+PGSHFTFCTR)	/* shift pfn to place in pte */
#define PG_SV		0x00000001	/* software valid bit */
#define PG_D		0x00000002	/* page is dirty w.r.t. pte */
#define PG_NDREF	0x0000001c	/* reference count (software) */
#define NDREF		7		/* positionless count */
#define PG_NDREFNORM	2		/* normalize NDREF for arithmetic */
#define	PG_CACHMASK	0x00000e00	/* cache coherency algorithm */
#define	PG_CACHSHFT	9		/* position in pte_t */
#define PG_TRUEUNCACHED	0x00000400	/* non-cached */
#define PG_UNC_PROCORD  0x00000000      /* un-cached processor ordered */
#define	PG_NONCOHRNT	0x00000600	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x00000a00	/* Cacheanle coherent, excl on write */
#define PG_M		0x00000100	/* modified bit	*/
#define PG_VR		0x00000080	/* valid and referenced  bit */

#else	/* !TFP_PTE64 -- still using 32-bit ptes */

#define PG_ADDR		(0x7fffff00<<PGSHFTFCTR) /* physical page address */
#define PTE_PFNSHFT	(8+PGSHFTFCTR)	/* shift pfn to place in pte */
#define PG_SV		0x00000001	/* software valid bit */
#define PG_D		0x00000002	/* page is dirty w.r.t. pte */
#define PG_NDREF	0x00000004	/* reference count (software) */
#define NDREF		1		/* positionless count */
#define PG_NDREFNORM	2		/* normalize NDREF for arithmetic */
#define	PG_CACHMASK	0x000000e0	/* cache coherency algorithm */
#define	PG_CACHSHFT	5		/* position in pte_t */
#define PG_TRUEUNCACHED	0x00000040	/* non-cached */
#define PG_UNC_PROCORD  0x00000000      /* un-cached processor ordered */
#define	PG_NONCOHRNT	0x00000060	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x000000a0	/* Cacheable coherent, excl on write */
#define PG_M		0x00000010	/* modified bit	*/
#define PG_VR		0x00000008	/* valid and referenced  bit */
#endif	/* !TFP_PTE64 */

#define PG_G		0x00000000	/* global (ignore pid)  bit */
					/* PG_G doesn't really exist -- */
					/* determined by virtual address */
#endif	/* IP21 */

#if EVEREST
#define PG_N		PG_UNCACHED
#define PG_UNCACHED	PG_COHRNT_EXLWR	/* (Actually cached) */
#define pg_isnoncache(P)	(pg_cachemode(P) == PG_TRUEUNCACHED)
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_TRUEUNCACHED)
#endif	/* EVEREST */

#if IP26
#define PG_ADDR		(0x01ffffc0<<PGSHFTFCTR) /* physical page address */
#define PTE_PFNSHFT	(6+PGSHFTFCTR)	/* shift pfn into place in pte */

#define PG_D		0x00000001	/* page is dirty wrt pte */
#define PG_VR		0x00000002	/* valid and referenced bit */
#define PG_M		0x00000004	/* modified bit */
#define PG_CACHMASK	0x00000038	/* cache coherency algorithm */
#define PG_NDREF	0x38000000	/* reference count (software) */
#define PG_SV		0x40000000	/* software valid bit */

#define PG_CACHSHFT	3		/* position in pte_t */
#define NDREF		7		/* positionless count */
#define PG_NDREFNORM	27		/* normalize NDREF for arithmetic */

#define PG_TRUEUNCACHED	0x00000010	/* non-cached */
#define PG_UNC_PROCORD	0x00000000	/* un-cached processor ordered */
#define PG_NONCOHRNT	0x00000018	/* Cacheable non-coherent */
#define PG_COHRNT_EXLWR	0x00000028	/* Cacheable coherent, excl on write */

#define PG_N			PG_UNCACHED
#define PG_UNCACHED		PG_TRUEUNCACHED
#define pg_isnoncache(P)	(pg_cachemode(P) == PG_UNCACHED)
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_UNCACHED)

#define PG_G			0		/* for compat with R4k */
#endif	/* IP26 */

#if IP25 || (IP30 && R10000)
#define PG_NDREF	0x80000000	/* reference count (software) */
#define NDREF		1		/* positionless count */
#define PG_NDREFNORM	30		/* normalize NDREF for arithmetic */
#define	PG_CACHMASK	0x00000038	/* cache coherency algorithm */
#define	PG_CACHSHFT	3		/* position in pte_t */
#define PG_TRUEUNCACHED	0x00000010	/* non-cached */
#define	PG_NONCOHRNT	0x00000018	/* Cacheable non-coherent */
#define	PG_COHRNT_EXCL	0x00000020	/* exclusive access */
#define PG_COHRNT_EXLWR	0x00000028	/* exclusive write */
#define PG_UNCACHED_ACC	0x00000038	/* Uncached Accelerated */
#define PG_M		0x00000004	/* modified bit	*/
#define PG_VR		0x00000002	/* valid and referenced  bit */
#define PG_G		0x00000001	/* global (ignore pid)  bit */
#define PTE_PFNSHFT	(6+PGSHFTFCTR)	/* shift pfn to place in pte */
#define PG_D		0x40000000	/* page is dirty w.r.t. pte */
#define PG_SV		0x20000000	/* software valid bit */
#define PG_ADDR		(0x1fffffc0<<PGSHFTFCTR) /* physical page address */
#define	FRAMEMASK_MASK	0x0000f800	/* Frame Mask register value */
#endif /* IP25 || (IP30 && R10000) */

#if IP30 && R10000
#define PG_N		PG_UNCACHED
#define PG_UNCACHED	PG_TRUEUNCACHED
#define pg_isnoncache(P)	(pg_cachemode(P) == PG_UNCACHED)
#define	pte_iscached(pte)	(((pte) & PG_CACHMASK) != PG_UNCACHED)
#endif


/* byte addr to virtual page */

#define pnum(X)		((__psunsigned_t)(X) >> PNUMSHFT) 
#define io_pnum(X)	((__psunsigned_t)(X) >> IO_PNUMSHFT) 

/* page offset */

#define poff(X)		((__psunsigned_t)(X) & POFFMASK)
#define io_poff(X)	((__psunsigned_t)(X) & IO_POFFMASK)

/*	Disk blocks (sectors) and pages.
*/
#define	ptod(PP)	((PP) << DPPSHFT)		/* pages to sectors */
#define	dtop(DD)	(((DD) + NDPP - 1) >> DPPSHFT)  /* sectors to pages */
#define	dtopt(DD)	((DD) >> DPPSHFT)  		/* sectors to pages,*/
							/* truncated	    */
#define dpoff(DD)	((DD) & (NDPP-1))		/* sector page offset */

/*	Disk blocks (sectors) and bytes.
*/

#define	btod(BB)	(((BB) + NBPSCTR - 1) >> SCTRSHFT)

/*	Form page descriptor (table) entry from modes and page frame number.
*/

#define	mkpde(mode,pfn)	(mode | ((pfn) << PTE_PFNSHFT))

/*	The following macros are used to check the value
 *	of the bits in a page descriptor (table) entry 
 */

/*	Software bits:
*/
#define pg_isvalid(P) 	((P)->pte.pg_sv)

/*	Set, clear dirty bit.
*/
#define pg_isdirty(P)	((P)->pte.pg_dirty)
#define pg_setdirty(P)	((P)->pgi |= PG_D)
#define pg_clrdirty(P)	((P)->pgi &= ~PG_D)


/*	The following macros are used to set the value
 *	of the bits in a page descrptor (table) entry 
 */

/*	Hardware bits:
*/

/*	Set, clear hardware valid bit.
*/
#define pg_ishrdvalid(P)	((P)->pgi & PG_VR)
#define pg_sethrdvalid(P)	((P)->pgi |= PG_VR)
#define pg_clrhrdvalid(P)	((P)->pgi &= ~PG_VR)

/*	Set, clear writeable bit.
*/
#define pg_ismod(P)	((P)->pte.pg_m)
#define pg_setmod(P)	((P)->pgi |= PG_M|PG_D)
#define pg_clrmod(P)	((P)->pgi &= ~PG_M)

/*	Set, clear global (ignore tlb pid) bit.
*/
#define pg_setglob(P)	((P)->pgi |= PG_G)
#define pg_clrglob(P)	((P)->pgi &= ~PG_G)
#define	pg_isglob(P)	((P)->pgi & PG_G)


/*	Test, set, clear, copy cache attributes.
*/

#if R4000 || TFP || R10000
#define	pg_cachemode(P)		((P)->pgi & PG_CACHMASK)
#define	pg_nomode(P)		((P)->pgi & ~PG_CACHMASK)
#define pg_setnoncache(P)	((P)->pgi = pg_nomode(P) | PG_UNCACHED)
#define pg_setnoncohrnt(P)	((P)->pgi = pg_nomode(P) | PG_NONCOHRNT)
#define pg_setcohrnt_exlwr(P)	((P)->pgi = pg_nomode(P) | PG_COHRNT_EXLWR)
#define pg_copycachemode(T,S)	((T)->pgi = pg_nomode(T) | pg_cachemode(S))
#define	pg_setcc(P,cc)		((P)->pte.pg_cc = (cc))
#if !LANGUAGE_ASSEMBLY && _KERNEL
void pg_setcachable(pde_t *P);
#endif /* !LANGUAGE_ASSEMBLY && KERNEL */
#if JUMP_WAR
#define pg_seteop(P)	((P)->pgi |= PG_EOP)
#define pg_iseop(P)	((P)->pgi & PG_EOP)
#endif
#endif	/* R4000 */

/*	Set cache alg. for pages in RG_PHYS regions.
*/
#if EVEREST
#define pg_setcache_phys(P)	((P)->pgi = pg_nomode(P) | PG_TRUEUNCACHED)
#else
#define pg_setcache_phys(P)	pg_setnoncache(P)
#endif

/*	Some 'bit' macros (rather than pde * macros) to
 *	manipulate cache attributes of pte's.
 */

#if R4000 || TFP || R10000
#if !LANGUAGE_ASSEMBLY && _KERNEL
uint	pte_cachebits(void);
#endif /* !LANGUAGE_ASSEMBLY && _KERNEL */
#define	pte_cached(pte)		(((pte) & ~PG_CACHMASK) | pte_cachebits())
#define	pte_noncached(pte)	(((pte) & ~PG_CACHMASK) | PG_UNCACHED)
#if EVEREST
#define	pte_uncachephys(pte)	(((pte) & ~PG_CACHMASK) | PG_TRUEUNCACHED)
#else	/* !EVEREST */
#define	pte_uncachephys(pte)	pte_noncached(pte)
#endif	/* EVEREST */

#endif


/*	Software bits:
*/

/*	Note that the valid and refernce bits are the same.
*/
#define pg_setref(P)	pg_setvalid(P)	/* Set ref bit.	*/
#define pg_clrref(P)	pg_clrvalid(P)	/* Clear ref bit.	*/

/*	Set valid bit.
**	Note: due to lack of reference bits, "valid bit" is vr, nr, and sv.
*/
#define pg_setvalid(P)	((P)->pgi |= PG_VR|PG_NDREF|PG_SV)

/*	Clear valid bit.
*/
#define pg_clrvalid(P)	((P)->pgi &= ~(PG_VR|PG_NDREF|PG_SV))

/*	Set, clear soft valid bit.
*/
#define pg_setsftval(P)	((P)->pgi |= PG_SV)
#define pg_clrsftval(P)	((P)->pgi &= ~PG_SV)

/*	Set, clear reference bits.
*/
#define pg_setndref(P)	((P)->pgi |= PG_NDREF)
#define pg_clrndref(P)	((P)->pgi &= ~PG_NDREF)


/*	Virtual addresses.
*/
/*
 * KPTE window is 4 megabytes long and is placed
 * 8 Meg from the top of kseg2, to leave space for upage).
 * KPTE window must be on a 4 Meg boundary.
 * (Note: this definition relies upon 2's complement arithmetic!)
 *
 * KPTEBASE holds the segment table for regular processes and sproc
 * shared address processes that have no local mappings.  It is
 * 2 Mb long (512 pages).  If a shared address process has local
 * mappings (as well as shared mappings) within one logical
 * segment (page of page tables), the shared page table will be
 * mapped at KPTESHRD bytes beyond the private mapping.  A sentinal
 * value is placed in the private mapping's page table entry to
 * indicate that the KPTESHRD entry should be used.
 */
#if R4000 || R10000
#if _K64U64
#ifdef R10000
#define	KPTEBASE	(0xc0000fc000000000)
#else
#define	KPTEBASE	(0xc00000fc00000000)
#endif /* R10000 */
#else /* _K64U64 */
#define	KPTEBASE	(-0x800000)
#endif /* _K64U64 */
#endif
/*
 * Note that for TFP we must relocate the KPTE to kernel private space
 * in order to allow the mappings to be ASID dependent.  This is required
 * because TFP has no concept of a "global" bit in the tlb.
 */
#if TFP
#define	KPTEBASE	(0x4000fff000000000)
#endif	/* TFP */

#if R4000 || R10000
#define KPTE_TLBPRESHIFT 1		/* R4000 ctxt assumes 8 byte pte's */
#else
#define KPTE_TLBPRESHIFT 0
#endif

#define	PDESHFT		2		/* 4 byte pte's */

#define KPTE_USIZE	(KUSIZE / NBPS * NBPC)
#define KPTE_SHDUBASE	(KPTEBASE - KPTE_USIZE)
#ifndef _LANGUAGE_ASSEMBLY
#if (R4000 || R10000) && _K64U64
#define KPTE_KBASE	\
	(KPTEBASE + 	\
	(K2BASE >> (TLBEXTCTXT_REGIONSHIFT + KPTE_TLBPRESHIFT + PGSHFTFCTR)))
#else
#define KPTE_KBASE	(KPTEBASE + ((unsigned int)K2BASE / NBPS * NBPC))
#endif
#endif

#if _K64U64
#define KPTE_REGIONSHIFT (TLBEXTCTXT_REGIONSHIFT+KPTE_TLBPRESHIFT+PGSHFTFCTR)
#define KPTE_REGIONMASK	(TLBEXTCTXT_REGIONMASK >> (KPTE_TLBPRESHIFT+PGSHFTFCTR))
#define KPTE_VPNMASK	((TLBEXTCTXT_VPNMASK >> (KPTE_TLBPRESHIFT+PGSHFTFCTR)) & ~KPTE_REGIONMASK)
#endif

/*	First 64K reserved for system - graphics, etc.
*/
#define PRDAADDR	0x00200000L	/* process private data area */
#define LOWUSRATTACH	0x00000000L	/* lowest addr allowed for USR attach */
#define USRCODE		0x10000000L	/* default location of user coe */
#define USRDATA		0x10000200L	/* default location of user data */

					/* avoid address wrap problem */
#define USERSTACK_32	0x7fff8000L	/* location of user's stack */
#define TOPOFSTACK_32	0x7fff8000L	/* segment top of users stack */
#define HIUSRATTACH_32	0x7fff8000L	/* highest allowable user address */

#ifdef _MIPS3_ADDRSPACE
#define USERSTACK_64	0x10000000000	/* location of user's stack */
#define TOPOFSTACK_64	0x10000000000	/* segment top of users stack */
#define HIUSRATTACH_64	0x10000000000	/* highest allowable user address */
#else
#define USERSTACK_64	USERSTACK_32	/* location of user's stack */
#define TOPOFSTACK_64	TOPOFSTACK_32	/* segment top of users stack */
#define HIUSRATTACH_64	HIUSRATTACH_32	/* highest allowable user address */
#endif	/* _MIPS3_ADDRSPACE */

/*
 * R4000_DADDIU_WAR for user N32 programs:
 *
 * This avoids an address wrap problem on R4000. The compiler uses addu
 * instructions to reference arrays on the stack which may end up being
 * above 0x80000000L and sign extended. For example, the code sequence:
 *	addu    v0,sp,v0
 *	lw      v0,-25600(v0)
 * can generate a bus error. If sp+v0 sign extends (e.g., 0xffffffff800012f0),
 * then -25600(v0) will be 0xffffffff7fffaef0 which causes a bus error.
 */
#define USERSTACK_N32	0x7fff8000L	/* location of user's stack (N32) */
#define TOPOFSTACK_N32	0x7fff8000L	/* segment top of users stack */
#define HIUSRATTACH_N32	0x7fff8000L	/* highest allowable user address */

#define ABI_USERSTACK(abi) (ABI_IS(ABI_IRIX5_64, abi) \
			    ? USERSTACK_64 \
			    : (ABI_IS(ABI_IRIX5_N32, abi) \
			       ? USERSTACK_N32 \
			       : USERSTACK_32))
#define ABI_TOPOFSTACK(abi) (ABI_IS(ABI_IRIX5_64, abi) \
			     ? TOPOFSTACK_64 \
			     : (ABI_IS(ABI_IRIX5_N32, abi) \
				? TOPOFSTACK_N32 \
				: TOPOFSTACK_32))
#define ABI_HIUSRATTACH(abi) (ABI_IS(ABI_IRIX5_64, abi) \
			      ? HIUSRATTACH_64 \
			      : (ABI_IS(ABI_IRIX5_N32, abi) \
				 ? HIUSRATTACH_N32 \
				 : HIUSRATTACH_32))

/*
 * Largest possible legal user address.  Used in copyin/out and friends to
 * protect against copying over kernel space.  They could use K0BASE, but
 * MAXHIUSRATTACH is more conservative.
 */
#define MAXHIUSRATTACH	HIUSRATTACH_64

/*
 * _S_EXT_ should only be used for UADDR and friends, not for
 * real 64-bit register addresses.
 */
#ifndef _S_EXT_
#if _K32U32 || _K32U64
#define _S_EXT_(addr)		(addr)
#else	/* _K64U64 */
#define _S_EXT_(addr)		((addr) | 0xffffffff00000000)
#endif	/* _K32U32 || _K32U64 */
#endif	/* ifndef _S_EXT_ */

#if NBPP == 4096
#if TFP
/* On TFP we need to avoid negative addressing, which only works from r0
 * and can get an Address Error exception when attempted from other registers
 * when the region bits of the effective address differ from the base reg.
 */
#define UADDR		_S_EXT_(0xffff0000)	/* U area loc. - above stack */
#else
#define UADDR		_S_EXT_(0xffffd000)	/* U area loc. - above stack */
#endif
#define KERNELSTACK	_S_EXT_(0xffffd000)	/* location of kernel stack */
#define PDAPAGE		_S_EXT_(0xffffa000)	/* private data area page */
#define PDAADDR		PDAPAGE
#define PDASIZE		NBPP
#endif
#if NBPP == 16384
#if TFP
/* On TFP we need to avoid negative addressing, which only works from r0
 * and can get an Address Error exception when attempted from other registers
 * when the region bits of the effective address differ from the base reg.
 */
#define UADDR		_S_EXT_(0xffff0000)	/* U area loc. - above stack */
#define KERNELSTACK	_S_EXT_(0xffff4000)	/* location of kernel stack */
#else
#define UADDR		_S_EXT_(0xffff8000)	/* U area loc. - above stack */
#define KERNELSTACK	_S_EXT_(0xffffc000)	/* location of kernel stack */
#endif
#define PDAPAGE		_S_EXT_(0xffffc000)	/* private data area */
#define PDAADDR		PDAPAGE
#define PDASIZE		NBPP
#endif

#define PDAOFFSET	(PDAADDR-PDAPAGE)
#define PRDASIZE	NBPP		/* private proc data area size */
#define UVPN		btoct(UADDR)	/* virtual page number of U */
#define KUSEG		KUBASE		/* kuseg user text/data */

#define K0SEG		K0BASE		/* k0seg kernel text/data cached */
#define K1SEG		K1BASE		/* k1seg kernel text/data uncached */
#define K2SEG		K2BASE		/* k2seg kernel mapped data */
#define K0SEG_SIZE	K0SIZE

#if !_LANGUAGE_ASSEMBLY
/* Include param.h since EXTUSIZE is set in there */
#include <sys/param.h>
#endif	/* !_LANGUAGE_ASSEMBLY */

/*
 * TLBWIREDBASE is set to 1 - pda
 */
#define PDATLBINDEX	0		/* processor PDA - set once */
#define UPGTLBINDEX	1		/* upage wired index */

#if R4000 || R10000
#define UKSTKTLBINDEX	1		/* ukstk wired index */
#define TLBKSLOTS	1		/* kernel tlb slots for upg/kstk */
#endif	/* R4000 || R10000 */


#define IS_UAREA(x)	((__psunsigned_t)(x) > (__psunsigned_t)&u)

/*
 * Flags for the nofault handler. 0 means no fault is expected.
 */
#define	NF_BADADDR	1
#define	NF_COPYIO	2
#define	NF_ADDUPC	3
#define	NF_FSUMEM	4
#define	NF_USERACC	5
#define NF_SOFTFP	6
#define NF_REVID	7
#define NF_SOFTFPI	8
#define NF_EARLYBADADDR	9
#define NF_IDBG		10
#define NF_FIXADE	11
#define	NF_SUERROR	12
#define NF_BADVADDR	13
#define	NF_NENTRIES	14

#if !LANGUAGE_ASSEMBLY && _KERNEL

/*	The following variables describe the memory managed by
**	the kernel.  This includes all memory above the kernel
**	itself.
*/
extern pfn_t	kpbase;		/* The address of the start of	*/
				/* the first physical page of	*/
				/* memory above the kernel.	*/
				/* Physical memory from here to	*/
				/* the end of physical memory	*/
				/* is represented in the pfdat.	*/
extern pde_t	*kptbl;		/* Kernel page table.		*/
extern int	mmem;		/* Number of resident mapped	*/
				/* file pages.			*/
				/* currently in core.		*/
extern int	mmemlim;	/* Maxsystmimum number of resident	*/
				/* mapped file pages.		*/
extern pfn_t	freemem;	/* Current free memory.		*/
extern pgno_t	usermem;	/* Memory available to users.	*/
extern pgno_t	availrmem;	/* Available resident (not	*/
				/* swappable) memory in pages.	*/
extern pgno_t	availsmem;	/* Available swappable memory	*/
				/* in pages.			*/
extern ulong	freeswap;	/* free swap space		*/
extern lock_t	avail_lock;	/* avail[rs]mem spinlock	*/


#define availmemlock()          mutex_spinlock(&avail_lock)
#define	availmemunlock(T)	mutex_spinunlock(&avail_lock, T)

#endif /* !LANGUAGE_ASSEMBLY && _KERNEL*/

/*	Conversion macros
*/

/*	convert system address to physical address
 *	(works only for k0seg and k1seg)
 *	64 bit kernels support up to 16 GB of memory but we need to
 *	keep 40 bits because the Everest CC uses 0x400000000 range
 *	of addresses.
 */

#if _K64U64
#define svirtophys(x)	((__psunsigned_t)(x) & 0x0FFFFFFFFFFll)
#else
#define svirtophys(x)	((__psunsigned_t)(x) & 0x1FFFFFFF)
#endif

/*	convert pde to physical address */

#define pdetophys(x)	((x)->pte.pg_pfn<<PNUMSHFT)

/*	Get page frame number from pde	*/
#define pdetopfn(x)	((x)->pte.pg_pfn)
#define pdetopfdat(x)	pfntopfdat(pdetopfn(x))

#if !LANGUAGE_ASSEMBLY
/*
 * This structure is used to define the sizes of various constants that
 * are shared between kernel and administrative utilities.
 */
struct pteconst {
	int		pt_ptesize;	/* sizeof(pte_t) */
	int		pt_pdesize;	/* sizeof(pde_t) */
	int		pt_pfnwidth;	/* Width of pfn */
	int		pt_pfnshift;	/* Shift for pfn */
	int		pt_pfnoffset;	/* Offset of pfn */
};
#endif

#ifdef	_KERNEL

/*	The following routines are involved with the pfdat
**	table described in pfdat.h
*/

#define	pfntopfdat(pfn)	(&pfdat[pfn])
#define	pfdattopfn(pfd)	(pfd - pfdat)
#define	pfdattophys(pfd)	((pfd - pfdat) << PNUMSHFT)

/*	Test whether a virtual address is in the kernel dynamically
**	allocated area.
*/

#define	iskvir(va)	((caddr_t)(va) >= (caddr_t)K2SEG && \
			 (caddr_t)(va) < (caddr_t)(K2SEG + ctob(syssegsz)))

/* The following defines let us manage large memory configurations.
 * Kernel Virtual addresses will be K0 when possible 
 * (i.e. when < 256 MB) otherwise will be K2.  We generally handle
 * addresses as Virtual Frame Numbers.
 */

#define vatovfn(x)		((__psunsigned_t)(x) >> PNUMSHFT)
#define vfntova(x)		((caddr_t)((x) << PNUMSHFT))

#define VFNMIN			((__psunsigned_t)K0SEG >> PNUMSHFT)

#ifdef _K64U64
#define VFNMAX			((__psunsigned_t)0xffffffffffffffff >> PNUMSHFT)
#else
#define VFNMAX			((__psunsigned_t)0xffffffff >> PNUMSHFT)
#endif
/*
 * SMALLMEM_PFNMAX	is largest PFN directly mappable in both K0 & K1
 * SMALLMEM_K0_PFNMAX	is largest PFN directly mappable in K0
 */
#if EVEREST
#define	SMALLMEM_K0_PFNMAX	((K0SEG_SIZE >> PNUMSHFT) - 1)
#define	SMALLMEM_K1_PFNMAX	((0x10000000 >> PNUMSHFT) - 1)
#define	SMALLMEM_PFNMAX		SMALLMEM_K1_PFNMAX
#else
#if IP20 || IP22 || IP26 || IP28 || IP30 || IPMHSIM
#define SMALLMEM_PFNMAX		((K0SEG_SIZE >> PNUMSHFT) - 1)
#define SMALLMEM_K0_PFNMAX	SMALLMEM_PFNMAX
#define SMALLMEM_K1_PFNMAX	SMALLMEM_PFNMAX
#elif IP32
#define SMALLMEM_PFNMAX         ((K1SIZE >> PNUMSHFT) - 1)
#define SMALLMEM_K0_PFNMAX      (((K0SIZE/2) >> PNUMSHFT) - 1)
#define SMALLMEM_K1_PFNMAX      SMALLMEM_PFNMAX
#else
#define SMALLMEM_PFNMAX		(((K0SEG_SIZE/2) >> PNUMSHFT) - 1)
#define	SMALLMEM_K0_PFNMAX	SMALLMEM_PFNMAX
#define	SMALLMEM_K1_PFNMAX	SMALLMEM_PFNMAX
#endif
#endif
#define SMALLMEM_VFNMIN_K0	VFNMIN
#define SMALLMEM_VFNMAX_K0	(SMALLMEM_VFNMIN_K0 + SMALLMEM_K0_PFNMAX)
#define SMALLMEM_VFNMIN_K1	((__psunsigned_t)K1SEG >> PNUMSHFT)
#define SMALLMEM_VFNMAX_K1	(SMALLMEM_VFNMIN_K1 + SMALLMEM_K1_PFNMAX)
#define K2MEM_VFNMIN		((__psunsigned_t)K2SEG >> PNUMSHFT)

#define IS_SMALLVFN_K0(x)	(((x) >= SMALLMEM_VFNMIN_K0) && \
				 ((x) <= SMALLMEM_VFNMAX_K0))
#define IS_SMALLVFN_K1(x)	(((x) >= SMALLMEM_VFNMIN_K1) && \
				 ((x) <= SMALLMEM_VFNMAX_K1))
#define IS_SMALLVFN(x)		(IS_SMALLVFN_K0(x) || IS_SMALLVFN_K1(x))
#define IS_K2VFN(x)		(((x) >= K2MEM_VFNMIN) &&((x) <= VFNMAX))
#define IS_VFN(x)		(IS_SMALLVFN(x) || IS_K2VFN(x))
#if _K64U64
/*
 * On 64-bit kernels, all pfns are accessible with "small" addresses.
 * Note that this is NOT true for K1 space on EVEREST some of the
 * uncached K1 space maps to CPU board local registers.
 */
#define small_K0_pfn(x)		1
#if SMALLMEM_K0_PFNMAX == SMALLMEM_K1_PFNMAX	/* catch EVEREST case */
#define small_pfn(x)		1
#define small_K1_pfn(x)		1
#else
#define small_pfn(x)		((x) <= SMALLMEM_PFNMAX)
#define small_K1_pfn(x)		((x) <= SMALLMEM_K1_PFNMAX)
#endif
#else
/*
 * On 32-bit kernels, only "small" memory addresses are accessible
 */
#define small_pfn(x)		((x) <= SMALLMEM_PFNMAX)
#define small_K0_pfn(x)		((x) <= SMALLMEM_K0_PFNMAX)
#define small_K1_pfn(x)		((x) <= SMALLMEM_K1_PFNMAX)
#endif
#define small_pfntovfn_K0(x)	((x) + SMALLMEM_VFNMIN_K0)
#define small_pfntovfn_K1(x)	((x) + SMALLMEM_VFNMIN_K1)
#define small_pfntova_K0(x)	(vfntova(small_pfntovfn_K0(x)))
#define small_pfntova_K1(x)	(vfntova(small_pfntovfn_K1(x)))
#define vfntopfn(x)	((IS_SMALLVFN_K0(x) ? ((x)-SMALLMEM_VFNMIN_K0) : \
			  (IS_SMALLVFN_K1(x) ? ((x)-SMALLMEM_VFNMIN_K1) : \
			   (IS_K2VFN(x) ? kptbl[(x)-K2MEM_VFNMIN].pte.pg_pfn:\
			    bad_vfn(x)))))
#define vfntopfdat(x)	(pfntopfdat(vfntopfn(x)))
/*
 * kvatopfn -- replaces svtop and svtopfn with a macro which handles K2 address
 *             as well as K0/K1 address
 * kvatopfdat -- kvtopfdat with a macro which handles K0/K1/K2 address
 */
#define kvatopfn(x)	(vfntopfn(vatovfn(x)))
#define kvatopfdat(x)	(vfntopfdat(vatovfn(x)))

#if !LANGUAGE_ASSEMBLY
extern int bad_vfn(pgno_t);
#endif
#endif /* _KERNEL */

/*	Convert dynamically allocated kernel virtual address 
 *	(in k2seg) to address of kernel page table entry.
 *	(dbx needs this so its outside _KERNEL)
 */
#define kvtokptbl(X) (&kptbl[pnum((__psunsigned_t)(X) - (__psunsigned_t)K2SEG)])

/*	Test whether an address is (secondary) cache-aligned.
 *	On a machine with no secondary cache, this always returns
 *	"aligned".
 */
#define SCACHE_ALIGNED(x)	(((__psunsigned_t)(x) & scache_linemask) == 0)
#if !LANGUAGE_ASSEMBLY && _KERNEL
extern int scache_linemask;
#endif /* !LANGUAGE_ASSEMBLY && _KERNEL */

/*	Flags used by various vm routines.
*/

#define VM_NOSLEEP	0x0001	/* do not wait (sleep) for pages. */
#define VM_TLBNOSET	0x0002	/* do not set wired entries to tlbpid */
#define VM_ONSWAP	0x0004	/* backing store already allocated */
#define VM_FLUSHWIRED	0x0008	/* flush wired pdes. */
				/* On EVEREST systems we avoid uncached access
				 * as it should not be needed and not all
				 * physical addresses have an uncached cpu
				 * address.  Many older systems use VM_UNCACHED
				 * to obtain an "IO coherent" address.
				 */
#ifndef EVEREST
#define VM_UNCACHED	0x0010	/* return uncached virtual address */
#else
#define VM_UNCACHED	0x0000	/* EVEREST allows cached access only */
#endif
#define VM_TLBINVAL	0x0020	/* invalidate all tlbpids */
#define VM_TLBVALID	0x0040	/* TLB ids on other processors still valid */
#define VM_UNSEQ	0x0080	/* unsequential access */
#define VM_DIRECT	0x0100	/* direct map -- that is, k0 or k1 seg */
#define VM_ATTACH	0x0200	/* attach page */
#define VM_ANY		0x0400	/* ??? */
#define VM_CACHEALIGN	0x0800	/* guarantee that memory is cache aligned */
#define	VM_NOAVAIL	0x1000	/* avail[rs]mem already accounted */
#define	VM_BREAKABLE	0x2000	/* breakable VM call (e.g.: user dma) */
#define	VM_STALE	0x4000	/* physical contents can be in other's icache */
#define VM_PHYSCONTIG	0x8000	/* physically contiguous */
#ifndef R10000
#define VM_VACOLOR	0x20000	/* color virtual addresses */
#else
#define VM_VACOLOR	0x00000	/* color virtual addresses - not required */
#endif /* R10000 */
#define VM_PACOLOR	0x40000	/* color physical addresses */
#define VM_VM		0x80000	/* Allocate in K2 space, please */
#define VM_UNCACHED_PIO	0x100000/* Return uncached space for all platforms */
#define VM_MVOK		0x200000/* Ok to move, if necessary */

#if !LANGUAGE_ASSEMBLY && _KERNEL
extern uint cachecolormask;

#define vcache_color(x) (((unsigned int) (x)) & cachecolormask)
#endif /* !LANGUAGE_ASSEMBLY && _KERNEL */

#define colorof(address)	\
		(((__psunsigned_t)(address) >> PNUMSHFT) & cachecolormask)

/*	Flags used by cache flushing routines.
 */
/* Which cache to flush */
#define	CACH_ICACHE		0x10000
#define	CACH_DCACHE		0x20000
#define CACH_PRIMARY (CACH_ICACHE | CACH_DCACHE)
#define CACH_SCACHE		0x40000
#define CACH_ALL_CACHES (CACH_PRIMARY | CACH_SCACHE)

/* cache ops and address types */
#define	CACH_OPMASK		0xff
#define	CACH_INVAL		0x1
#define	CACH_WBACK		0x2

#define	CACH_NOADDR		0x100
#define	CACH_VADDR		0x200
#define	CACH_LOCAL_ONLY		0x400	/* operate only on current CPU */

#define	CACH_IO_COHERENCY	0x1000	/* flush for I/O coherency */
#define CACH_ICACHE_COHERENCY	0x2000	/* flush for icache coherency - text
					 * written in dcache */
#define CACH_AVOID_VCES		0x4000	/* flush to avoid VCEs */
#define	CACH_FORCE		0x8000	/* flush because I said so */
#define CACH_REASONS (CACH_IO_COHERENCY|CACH_ICACHE_COHERENCY|CACH_AVOID_VCES|CACH_FORCE)

#if !LANGUAGE_ASSEMBLY && _KERNEL

/* Function Prototypes */
extern void	*kvpalloc(uint, int, int);
extern void	*kvalloc(uint, int, int);
extern void	kvpfree(void *, uint);
extern void	kvpffree(void *, uint, int);
extern void	kvpfree_defer_cleanup(void);
extern void	kvfree(void *, uint);
extern char	*vmap(paddr_t, uint);
extern void	vunmap(char *);
extern pgno_t	contmemall(int, int, int);
extern pgno_t	contig_memalloc(int, int, int);

extern pfn_t	kvtophyspnum(void *);
extern paddr_t	kvtophys(void *);
extern paddr_t	kvtophyspg(void *);

extern void	kvswappable(void *, pgno_t);
extern int 	kvswapin(void *, register uint, int);

struct eframe_s;
extern int	vfault(caddr_t, int, struct eframe_s *);
extern int	tfault(struct proc *, struct eframe_s *, uint, caddr_t);
extern int	pfault(caddr_t, struct eframe_s *);

/* Low level TLB management routines */
extern void unmaptlb(unsigned char, __psunsigned_t);
extern void invaltlb(int);
extern uint tlbdropin(unsigned char, caddr_t, pte_t);
extern void flush_tlb(int);
extern void set_tlbpid(unsigned char);
extern unsigned char tlbgetpid(void);
extern int tlbgetrand(void);
extern int getrand(void);
struct proc;
#if _K64U64
#define WIRESEGTBL(p) ASSERT((!p->p_segtbl || IS_KSEGDM(p->p_segtbl)) && \
			     (!p->p_shrdsegtbl || IS_KSEGDM(p->p_shrdsegtbl)))
#else
#define WIRESEGTBL(p) wiresegtbl(p)
extern void wiresegtbl(struct proc *);
#endif /* !_K64U64 */
#if R4000 || R10000
extern void tlbwired(int, unsigned char, caddr_t, pte_t, pte_t);
extern void tlbdrop2in(unsigned char, caddr_t, pte_t, pte_t);
#endif
#if TFP
extern void set_icachepid(unsigned char);
extern unsigned char icachegetpid(void);
/* This will probably change, but for now it lets us compile */
extern void tlbwired(int, unsigned char, caddr_t, pte_t);
#endif

struct pregion;
struct pfdat;
extern int handlepd(caddr_t, struct pfdat *, pde_t *, struct pregion *, int);

#ifdef USE_LGPAGES
extern void tlbwire_lpgs(int, unsigned char, caddr_t, uint, uint, uint);
extern void zap_tlbentry(char, caddr_t);
extern void tlbdropin_lpgs(unsigned char, caddr_t, pte_t, pte_t, uint);
#endif

#if JUMP_WAR
extern void _invaltlb(int);
extern void clr_jump_war_wired(void);
extern int sexc_eop(struct eframe_s *, uint);
#endif

#if defined(USE_LGPAGES) || defined(JUMP_WAR)
extern void setwired(int);
#endif 

#endif /* !LANGUAGE_ASSEMBLY && _KERNEL */

#endif /* __SYS_IMMU_H__ */
