#ifndef __SYS_IOBUS_H
#define __SYS_IOBUS_H

/**************************************************************************
 *									  *
 * 		   Copyright (C) 1995, Silicon Graphics, Inc.	          *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#ident "$Revision: 1.3 $"

struct piomap;
struct dmamap;

/* bus handle */
typedef __psunsigned_t iobush_t;

/* generic physical/IO address */
typedef union {
	paddr_t paddr;
	iopaddr_t ioaddr;
} xaddr_t;

/* iobus virtual functions */
typedef struct iobusops {
	/* free low-level pio handle */
        void            (*pio_freeh)(struct piomap *);
	/* fix pio mapping */
        int             (*pio_fix)(struct piomap *); 
	/* translate UNFIXED mode pio address */
        int             (*pio_trans)(struct piomap *, iopaddr_t, int bytes,
                        int can_block);  
	/* undo pio translate; UNFIXED mode only */
        void            (*pio_untrans)(struct piomap *);
	/* free low-level dma handle */
	void		(*dma_freeh)(struct dmamap *);
	/* map/translate for dma */
	int		(*dma_map)(struct dmamap*, xaddr_t *phys, xaddr_t *io, 
			int p_cnt);
	/* translate 1 physical address */
	iopaddr_t	(*dma_trans1addr)(struct dmamap*, paddr_t);
	/* bus configuration management*/
	int		(*bus_config)(iobush_t, int opcode);
} iobusops_t;

/* IO state */
#define IO_STATE_EMPTY			0x01	/* non-existent */
#define IO_STATE_INITIALIZING		0x02	/* being initialized */
#define IO_STATE_ATTACHING   		0x04    /* becoming active */
#define IO_STATE_ACTIVE      		0x08    /* active */
#define IO_STATE_DETACHING   		0x10    /* becoming inactive */
#define IO_STATE_INACTIVE    		0x20    /* not in use */
#define IO_STATE_ERROR			0x40    /* problems */
#define IO_STATE_BAD_HARDWARE		0x80	/* broken hardware */

/* return codes */
#define RC_OK				0	
#define RC_ERROR			1

/* bus configuration management op code */
#define IOBUS_CONFIG_ATTACH		0	/* vary on */
#define IOBUS_CONFIG_DETACH		1	/* vary off */
#define IOBUS_CONFIG_RECOVER		2	/* clear error then vary on */

/* get low-level PIO handle */
extern int pio_geth(struct piomap*, int bus, int bus_id, int subtype, 
	iopaddr_t addr, int size);	

/* get low-level DMA handle */
extern int dma_geth(struct dmamap*, int bus_type, int bus_id, int dma_type, 
	int npages, int page_size, int flags);	

#ifdef __cplusplus
}
#endif

/*
 * Macros for page number and page offsets, using ps as page size
 */
#define x_pnum(addr, ps) ((__psunsigned_t)(addr) / (__psunsigned_t)(ps))
#define x_poff(addr, ps) ((__psunsigned_t)(addr) & ((__psunsigned_t)(ps) - 1))

/* 
 * Convenience macros
 */
#define PIO_FREEH(map) (*(map)->iobusops->pio_freeh)(map)
#define PIO_FIX(map) (*(map)->iobusops->pio_fix)(map)
#define PIO_TRANS(map, a, b, c) (*(map)->iobusops->pio_trans)(map, a, b, c)
#define PIO_UNTRANS(map) (*(map)->iobusops->pio_untrans)(map)
#define DMA_FREEH(map) (*(map)->iobusops->dma_freeh)(map)
#define DMA_MAP(map, a, b, c) (*(map)->iobusops->dma_map)(map, a, b, c)
#define DMA_TRANS1(map, a) (*(map)->iobusops->dma_map)(map, a)
#define BUS_CONFIG(map, bus_h, opcode) \
	(*(map)->iobusops->bus_config)(bus_h, opcode)

#endif /* __SYS_IOBUS_H */
