/**************************************************************************
 *									  *
 * 		 Copyright (C) 1988, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ident "$Revision: 1.33 $"

/***************************************************************************
 *
 *  kdsp_kern.h - Hollywood DSP device driver internal structs.
 *  Only device-independent structs should be here.
 *  (application programmer visible kdsp structs are in hdsp.h).
 *  (device-dependent structs go in header files for the device).
 *
 ****************************************************************************/

#ifndef HRB_STRLEN
#define HRB_STRLEN	20	/* maximum length for a port/HRB name string */
#endif

#define HDSP_MAXUNIQID	32767	/* max value for a port unique ID */
#define HDSP_MAXRATES	7	/* number of entries in rate table */

#define HRB_DEFAULT	    ((hrb_t *)-2L) /* -2 in conn indicates default RB */
#define HRB_WILDCARD	    ((hrb_t *)-1L) /* -1 arg to connlist_delete is wildcard */
#define HRB_END		    -1	/* -1 is list terminator */

/* 
 * The following struct describes what the kdsp driver knows about a host ring
 * buffer.
 *
 * NOTE: offsets are incorrect when compiled _K64U64.
 */
typedef struct hrb_s {
    u_int *vheader;	 /* 0 Virtual address of the header part of the hrb   */
    caddr_t pheader;	 /* 4 Physical address of the header part of the hrb  */
    void *vdata;	 /* 8 Virtual address of the data part of the hrb     */
    caddr_t pdata;	 /* 12 Physical address of the data part of the hrb   */
    caddr_t vaddr;	 /* 16 Virtual address of data in user's addr space  */
    int nlocs;		 /* 20 Number of locations in data part of the hrb   */
    int nbytes;		 /* 24 Number of bytes in the hrb, including header  */
    int minordev;	 /* 28 Minor device number associated with this hrb. */
    struct hrb_s *next;  /* 32 Pointer to the next free or allocated hrb struct  */
    struct pollhead pq;  /* 36 Poll queue (for select, poll)                 */
    int maplen;		 /* 40 Length of the mmaped region                       */
    pid_t pid;		 /* 56 pid of application using port		      */
    int uniqid;		 /* 60 unique sequence number to identify port	      */
    int newhdtl;	 /* 64 new head/tail value for this RB after curr intr*/
    int refcnt;	    	 /* 68 reference count: # conns into/out of RB */
    int state;		 /* 72 State of the hrb.                             */
    char type;		 /* 76 Type of hrb (TOMIPS, FROMMIPS, FOLDED, etc.)   */
    char rbnum;		 /* 77 Ring Buffer Number */
    char wakeup;	 /* 78 True if this RB needs a reader/writer woken    */
    char dma_active;	 /* 79 True if RB is currently being DMAd from/to */
    char name[HRB_STRLEN]; /* 80 name of this rb (== port name from app)      */
    char needs_dma;	 /* 100 This hrb requires DMA to service it */
    char dma_chan;	 /* 101 DMA channel if applicable */
    char errstate;	 /* 102 error state: 1 = error, 0 = no error */
    u_int errcnt;	 /* 104 total number of errors on port */
    u_int errtype;	 /* 108 type of error that last occurred on port */
    u_int errlength;	 /* 112 length of current error */
    int	frameshift;	 /* 116 shift to convert between samples, frames */
    long long errloc;	 /* 120 location of current error on port */
    long long sampoffset;   /* 128 sample offset of this HRB from reference */
    int	framemask;	 /* 136 mask to convert to even # of frames */
    struct hrb_s *nxtuhrb;  /* 140 Pointer to the next allocated USER hrb struct  */
    void *pend;	 	 /* 144 physical address of end of rb */
    int alversion;	 /* 148 AL version number for this HRB */
    int nchans;	    	 /* 152 Number of interleaved channels in this RB */
    int drop_samps;	 /* 154 number of samples to drop off head */
    void *rate_entry;	 /* 160 pointer to rate table entry for this RB */
    struct hrb_s *master;/* 164 If we are a slave this is our master */
    int dma_latency; 	 /* 168 DMA latency if needs_dma */
} hrb_t;

/* 
 * The following defines indicate the state of a host ring buffer.  
 */
#define HRB_WAITING 1

/* 
 * The following flags control the behavior of kernel writes to the dsp 
 *	-- needed by streams drivers to prevent sleeping 
 */
#define HDSPSLEEPOK 0
#define HDSPNOSLEEP 1

/* 
 * The following macros are convenient ways to query the type of a host ring
 * buffer. 
 */
#define HRB_ISFOLDED(x)		(((x)&HRB_FOLDED)!=0)
#define HRB_ISSPLIT(x)		(((x)&HRB_SPLIT)!=0)
#define HRB_ISFROMMIPS(x)	(((x)&HRB_FROMMIPS)!=0)
#define HRB_ISEXTEND(x)		(((x)&HRB_EXTEND)!=0)
#define HRB_ISUSERSPACE(x)	(((x)&HRB_USERSPACE)!=0)
#define HRB_ISDIRECT(x)		(((x)&HRB_DIRECT)!=0)
#define HRB_ISMASTER(x)		(((x)&HRB_MASTER)!=0)

#define HRB_ISUNFOLDED(x)	(((x)&HRB_FOLDED)==0)
#define HRB_ISTOMIPS(x)		(((x)&HRB_FROMMIPS)==0)
#define HRB_ISNOEXTEND(x)	(((x)&HRB_EXTEND)==0)
#define HRB_ISKERNELSPACE(x)	(((x)&HRB_USERSPACE)==0)
#define HRB_ISCONTIGUOUS(x)	(((x)&HRB_SPLIT)==0)
#define HRB_ISSERIAL(x)		(((x)&HRB_DIRECT)==0)
#define HRB_ISSLAVE(x)		(((x)&HRB_MASTER)==0)

/* 
 * limits 
 */
#define MAX_USER_RBS	16
#define NUM_KERNEL_RBS	5
#define MAXHRB          21      /* maximum # of hrbs (MAX_USER + NUM_KERNEL) */
#define MAXCONN		21	/* maximum number of connections */

/* 
 * Configuration constants 
 */
#define CMDBUFSIZE   300
#define REPLYBUFSIZE 20
/*
 * 404 samples is the absolute minimum for 4-channel operation.
 * The maximum number of frames per 2 ms is 100, frame size is 4,
 * and there needs to be one extra sample frame in the ring buffer.
 */
#define AUDIOKERNBUFSIZE 404	
#define HDSPSPRI (PZERO-1)

/*
 * This struct defines an entry in the driver's rate table. The rate
 * table keeps track of which rates are in use, and what each rate is
 * synced to. (Each connection has an index into the rate table).
 */
typedef struct kdsp_rate_entry_s {
	unsigned long long   elapsed; /* elapsed samples (64 bits) */
	unsigned long long   ust; /* ust at which elapsed changed (64 bits) */
	int	rate;		/* sample rate for this entry */
	hrb_t  *syncsource;	/* synced to what RB? (if synced to RB) */
	int 	frames_to_move;	/* # frames to move this interrupt */
	int	usecs_per_frame;/* usecs per frame (if synced to unix clock) */
	int	servosamps;	/* samples to servo */
	int	inuse;		/* is this rate in use? */
	int	synctype;	/* synced to RB or unix clock? */
	int	pad[5];		/* pad to power of two */
} kdsp_rate_entry_t;

#define HDSP_CLOCK_SYNC		0
#define HDSP_RBDRAIN_SYNC	1
#define HDSP_RBFILL_SYNC	2
#define HDSP_DELETED		-1

/*
 * The following struct defines the information required for a single
 * HRB->HRB connection. 
 */
typedef struct kdsp_connection_s {
	int next;			/* next free or used conn */
	int prev;			/* previous free or used conn */
	hrb_t *in;			/* input port */
	hrb_t *out;			/* output port */
	kdsp_rate_entry_t *inrate;	/* rate table entry for input */
	kdsp_rate_entry_t *outrate;	/* rate table entry for output */
	void (*t_func)();		/* transfer func: e.g. copy or mix? */
	unsigned int flags;		/* various flags (see below for defs) */
} kdsp_connection_t;

/*
 * values for connection flags
 * if you change / add to these values you'll need to look at
 * the calls to connlist_reconfig.
 */

#define KDSP_MIX       		0x1	/* conn should mix */
#define KDSP_COPY       	0x0	/* conn should copy */
#define KDSP_CLIP		0x2	/* conn should clip */
#define KDSP_MONO_LEFT		0x4	/* conn is mono, left channel only */
#define KDSP_WDEFAULT		0x100	/* writer is default */
#define KDSP_RDEFAULT		0x200	/* reader is default */

/*
 * The following struct defines a "list-of-connections" type.
 */
typedef struct kdsp_connlist_s {
	kdsp_connection_t conns[MAXCONN];	/* array of conns */
	int firstused;				/* first used conn */
	int firstfree;				/* first free conn */
	int numconns;				/* number of conns */
} kdsp_connlist_t;

#define KDSP_MAX_DEVICES	1   /* maximum number of devices */
#define KDSP_MAX_FRAMESIZE	4   /* maximum number of channels */

typedef struct kdsp_param_info_s {
    int param;
    int kind;
    int min;
    int max;
    int initial;
    char *name;
} kdsp_param_info_t;

/* 
 * The following struct describes a physical audio device.  
 * Functions must be provided for device specific operations.
 */
typedef struct kdsp_device_s {
	int  (*probe)();
	int (*init)(struct kdsp_device_s *);
	void (*pre_trans_ops)(struct kdsp_device_s *);
	void (*post_trans_ops)(struct kdsp_device_s *);
	void (*reset)(struct kdsp_device_s *);
	int (*activate)(struct kdsp_device_s *);
	int  (*start_dma)(hrb_t *);
	void (*stop_dma)(hrb_t *);
	void (*set_audio_params)(struct kdsp_device_s *,unsigned int *,int);
	void (*get_audio_params)(struct kdsp_device_s *,unsigned int *,int);
	int (*connect)(struct kdsp_device_s *, hrb_t *, hrb_t *);
	void (*halt_intr_ops)(void);
	void (*shutdown)(struct kdsp_device_s *);
	hrb_t *default_in;
	hrb_t *default_out;
	void *data;
	kdsp_param_info_t *params; /* buffer full of parameter information */
	int nparams;		/* # entries in parameter buffer */
} kdsp_device_t;

typedef void (*kdsp_tfunc_t)();

/*
 * Client locks state 
 */
#define HDSP_CLIENT_UNOPENED	(-2)
#define HDSP_CLIENT_OPENED	(-1)

/* 
 * Priority at which the we blocks interrupts. 
 */
#ifdef USE_MUTEX_SPINLOCK
extern lock_t kdsp_mutex;
#else
#define splkdsp()	splhi()
#endif

/*
 * transfer functions.
 * all of these take a source, a target, and a number of frames to
 * transfer, except fastclip, which takes a target and a number of samples
 * to clip. 
 */

extern void fastcopy22(void*, void*, int);
extern void fastcopyl22(void*, void*, int);
extern void fastmix22(void*, void*, int);
extern void fastmixl22(void*, void*, int);
extern void fastmixclip22(void*, void*, int);
extern void fastmixclipl22(void*, void*, int);
extern void fastcopyclip22(void*, void*, int);
extern void fastcopyse22(void*, void*, int);
extern void fastmixse22(void*, void*, int);
extern void fastmixclipse22(void*, void*, int);

extern void fastclip(void *, int);

extern void fastcopy44(void*, void*, int);
extern void fastcopyl44(void*, void*, int);
extern void fastmix44(void*, void*, int);
extern void fastmixl44(void*, void*, int);
extern void fastmixclip44(void*, void*, int);
extern void fastcopyclip44(void*, void*, int);
extern void fastmixclipl44(void*, void*, int);

extern void fastcopy24(void*, void*, int);
extern void fastmix24(void*, void*, int);
extern void fastcopyl24(void*, void*, int);
extern void fastmixclip24(void*, void*, int);
extern void fastcopyclip24(void*, void*, int);
extern void fastcopyse24(void*, void*, int);
extern void fastmixse24(void*, void*, int);
extern void fastmixclipse24(void*, void*, int);

extern void fastcopy42(void*, void*, int);
extern void fastcopyl42(void*, void*, int);
extern void fastmix42(void*, void*, int);
extern void fastmixclip42(void*, void*, int);
extern void fastcopyclip42(void*, void*, int);

/*
 * local functions
 */
void kdsp_timercallback(void);

void hrb_usercreate(int nlocs, int type, int minordev);
hrb_t *hrb_kernalloc(void);
int  hrb_kerncreate(int, int, int, int, void *);
int  hrb_destroy(int);

int  connlist_delete(hrb_t *, hrb_t *);
int  connlist_insert(hrb_t *, hrb_t *, int);
int  connlist_reconnect(hrb_t *, hrb_t *, hrb_t *, hrb_t *);
int  connlist_reconfig(hrb_t *, hrb_t *, int, int);
void connlist_init(void);

#ifdef EVEREST
#define KVTOPHYS(xxx) kdsp_evkvtophys(xxx)
void *kdsp_evkvtophys(void *kv);
#else
#define KVTOPHYS(xxx) kvtophys(xxx)
#endif
