/**************************************************************************
 *									  *
 * 		 Copyright (C) 1990-1992 Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifndef __SYS_KMEM_H__
#define __SYS_KMEM_H__

#ifdef __cplusplus
extern "C" {
#endif

#ident	"$Revision: 1.19 $"

#include <sys/immu.h>
#include <sys/sema.h>
/*
 * Definitions for kmem_*, must match VM_*
 */
#define KM_SLEEP	0
#define KM_NOSLEEP	VM_NOSLEEP	/* must match VM_NOSLEEP */
#define KM_PHYSCONTIG	VM_PHYSCONTIG	/* physically contiguous */
#define KM_CACHEALIGN	VM_CACHEALIGN	/* guarantee that memory is */
					/* cache aligned */

/*
 * Function prototypes.
 */

extern	void	*kmem_alloc(size_t, int);
extern	void	*kmem_zalloc(size_t, int);
extern	void	*kmem_realloc(void *, size_t, int);
extern	void	kmem_free(void *, size_t);
extern	ulong	kmem_avail(void);


typedef struct zone {
	struct zone	*zone_next;
	void		*zone_free;
	int		zone_freesize;
	short		zone_unitsize;
	short		zone_units_pp;
	int		zone_totalsize;
	int		zone_kvm;
	short		zone_usecnt;
	unsigned short	zone_pages_freed;
	char		*zone_name;
} zone_t;

/*
 * Zone routines:
 *	kmem_zone_init(unit-size, zone-name) -- connect to a zone manager;
 *	kmem_zone_alloc(zone, zone-flags) -- alloc a zone unit from the zone;
 *	kmem_zone_zalloc(zone, flags) -- alloc a zeroed zone unit from the zone;
 *	kmem_zone_free(zone, data-pointer) -- free a zone unit into the zone.
 */
extern zone_t	*kmem_zone_init(int, char *);
extern void	*kmem_zone_alloc(zone_t *, int);
extern void	*kmem_zone_zalloc(zone_t *, int);
extern void	kmem_zone_free(zone_t *, void *);

/*
 * Private zone interfaces: these zones are never shared;
 * the clients can pre-fill the zones and the zones
 * never get shaken (trimmed).
 *	kmem_zone_private(unit-size, zone-name) -- connect to a zone manager;
 *	kmem_zone_fill(zone, pointer-to-memory, size-of-memory) -- pre-fill;
 *	kmem_zone_reserve(zone, number-of-items-wanted) -- pre-fill.
 */
extern zone_t	*kmem_zone_private(int, char *);
extern void	kmem_zone_fill(zone_t *, void *, int);
extern int	kmem_zone_reserve(zone_t *, int);

#define kmem_zone_freesize(z)	((z)->zone_freesize)


/*
 * shake manager routines
 *
 * A resource manager that would like to have its resource
 * trimmed (shaken) when the general resource class is
 * depleted registers its shake routine with the shake manager
 * via shake_register.
 *
 * The resource class manager invokes the various registered
 * routines via shake_shake, passing the resource class id.
 */

extern void	shake_register(int, int (*)(int));
extern int	shake_shake(int);

/*
 * General resource classes defined.
 */
#define SHAKEMGR_MEMORY	1
#define SHAKEMGR_SWAP	2
#define SHAKEMGR_VM	3


#ifdef __cplusplus
}
#endif

#endif /* __SYS_KMEM_H__ */
