/**************************************************************************
 *                                                                        *
 *               Copyright (C) 1994-1995 Silicon Graphics, Inc.           *
 *                                                                        *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *                                                                        *
 **************************************************************************/
#ident	"$Revision: 1.7 $"

#ifndef	_KTHREAD_H
#define	_KTHREAD_H

#include <sys/types.h>
#include <sys/pcb.h>
#include <sys/pda.h>
#include <sys/debug.h>
#include <sgidefs.h>

#define	KTHREAD_MAGIC	0x53544844

/* for flags */
#define	KTHREAD_STACK_MALLOC	1

/* stack management */
#define	KTHREAD_DEF_STACKSZ	NBPC	/* one click */
#define ARGSAVEFRM	(((18*sizeof(k_machreg_t))+15) & ~0xf)

/* for state */
#define	KTSTATE_RUN	0x01	/* thread is runnable */
#define	KTSTATE_STOP	0x02	/* thread hasn't started */

#define KTSTATE_STOPPED	0x01	/* thread is stopped */
#define	KTSTATE_BLOCKED	0x02	/* thread is blocked */
#define KTSTATE_QUEUED	0x04	/* thread is queued */
#define KTSTATE_RUNNING	0x08	/* thread is running */

/* for species */
#define	KTSPEC_FREE	0	/* free thread */
#define	KTSPEC_INTR	1	/* an interrupt thread */
#define	KTSPEC_USER	2	/* thread with possibly null user state */

/*
 * Mutex priority inheritance records.
 */
typedef struct mirec_s {
	mutex_t		*i_mp;
	pri_t		i_pri;
	struct mirec_s	*i_next;
	struct mirec_s	*i_back;
} minherit_t;

/*
 * general kernel thread stucture 
 *
 */

typedef struct kthread {
	k_machreg_t	k_regs[NPCBREGS];
	struct proc	*k_flink;	/* linked list of sleeping threads */
	struct proc	*k_blink;	/* linked list of sleeping threads */
	uint		k_flag;		/* flags defined below.	     */
	uint		k_flag2;	/* more flags defined below.	     */
	pid_t		k_pid;		/* unique process id */
	time_t		k_stime;  	/* system time in ticks */
	caddr_t		k_wchan;	/* Sleeping process's sync object */
	caddr_t		k_w2chan;	/* Wait addr for sleeping process.   */
	char		k_stat;		/* status of thread */

	/* parameters used by the scheduler. try to keep together, so that
	   hitting one of them fetches a whole cache line containing what
	   we need */
	/*---*/
	pri_t		k_pri;		/* priority, lower # is high */
	pri_t		k_upri;		/* user-level priority */
	pri_t		k_saveupri;	/* saved user-level priority */
	mutex_t		*k_mblock;	/* mutex we're blocked on */
	minherit_t	*k_pilist;	/* mutexes we've inherited through */
	lock_t		k_pilck;	/* inherit list lock */
	cpuid_t		k_sonproc;	/* processor process is running on */
	cpuid_t		k_mustrun;	/* cpu to run on, set by user/kernel */	
	unchar		k_onrq;		/* set if on runq, runq.lock protects */
	cpuid_t		k_lastrun;	/* last processor flag */
	char		k_sqself;	/* If I'm using my stack */
	pri_t		k_rtpri;	/* real-time priority */	
	ushort		k_tslice;	/* process time slice size */
	uint		k_dvec;		/* scheduling discipline vector */
	struct runqinfo *k_runqinfo;	/* private Runq information */
	/*---*/
	struct monitor	*k_monitor;	/* pointer to current monitor */	
	int		k_error;	/* return error code */
	int		k_code;		/* trap error code */
	caddr_t		k_depth;	/* allow nesting */
	time_t		k_start;	/* Time of thread creation */
	
	uint_t		k_stacksize;	/* in bytes */ 
	caddr_t		k_stack;	/* stack area */
	void		(* k_func)(void *);
					/* thread code */
	void		*k_tpda;	/* thread private data */
	uint_t		k_flags;	/* thread flags */
	int		k_nofault;	/* index into table of functions to */
					/* handle user space external memory */
					/* faults encountered in kernel */
	char		*k_name;	/* thread name */
	__uint32_t	k_magic;		/* thread magic number */
} kthread_t;

/* some macros */

#define curthreadp ((kthread_t *)(private.p_curproc))
#define curprocp (private.p_curproc)

#define proctokt(p) ((kthread_t *)(p))
#define kttoproc(k) ((proc_t *)(k))

/* static initialization */

typedef
struct kthread_init {
    void (* func)(void *);
    uint_t stack_size;
    void *arg;
    short flags;
    kthread_t *threadp;
} kthread_init_t;

/* function prototypes */
kthread_t *
	kthread_create(char *, caddr_t, uint_t, void (*)( void *), void *,
			uint_t);
void	kthread_exit(void);
int	kthread_kill(kthread_t *);
int	kthread_stop(kthread_t *);
int	kthread_continue(kthread_t *);
int	kthread_priority(kthread_t *, pri_t);
void	kthread_yield(void);

void	kthread_start_daemons(void);

#endif
