
/**************************************************************************
 *									  *
 * 		 Copyright (C) 1988, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifndef	__MAC_LABEL_
#define	__MAC_LABEL_

#ifdef __cplusplus
extern "C" {
#endif

#ident "$Revision: 1.28 $"

/*
 * Allow for 26 (A-Z) MSEN label types.
 * Expanding this will require a bigger table in mac_dom.c
 */
#define MSEN_MIN_LABEL_NAME	'A'	/* No label name less than this */
#define MSEN_MAX_LABEL_NAME	'Z'	/* No label name greater than this */
#define MSEN_LABEL_NAME_COUNT	26	/* Number of label types */
#define MSEN_MIN_LEVEL		0	/* No level less than this 	*/
#define MSEN_MAX_LEVEL		255	/* No level greater than this 	*/
#define MSEN_MIN_CATEGORY	0	/* No category less than this 	*/
#define MSEN_MAX_CATEGORY	65534	/* No category greater than this */

/*
 * MSEN label type names. Choose an upper case ASCII character.
 */
#define MSEN_ADMIN_LABEL	'A'	/* Admin: low < admin != tcsec < high */
#define MSEN_EQUAL_LABEL	'E'	/* Wildcard - always equal */
#define MSEN_HIGH_LABEL		'H'	/* System High - always dominates */
#define MSEN_MLD_HIGH_LABEL	'I'	/* System High, multi-level dir */
#define MSEN_LOW_LABEL		'L'	/* System Low - always dominated */
#define MSEN_MLD_LABEL		'M'	/* TCSEC label on a multi-level dir */
#define MSEN_MLD_LOW_LABEL	'N'	/* System Low, multi-level dir */
#define MSEN_TCSEC_LABEL	'T'	/* TCSEC label */
#define MSEN_UNKNOWN_LABEL	'U'	/* unknown label */

/*
 * Allow for 26 (a-z) MINT label types.
 * Expanding this will require a bigger table in mac_dom.c
 */
#define MINT_MIN_LABEL_NAME	'a'	/* No label name less than this */
#define MINT_MAX_LABEL_NAME	'z'	/* No label name greater than this */
#define MINT_LABEL_NAME_COUNT	26	/* Number of label types */
#define MINT_MIN_GRADE		0	/* No grade less than this 	*/
#define MINT_MAX_GRADE		255	/* No grade greater than this 	*/
#define MINT_MIN_DIVISION	0	/* No division less than this 	*/
#define MINT_MAX_DIVISION	65534	/* No division greater than this */

/*
 * MINT label type names. Choose a lower case ASCII character.
 */
#define MINT_BIBA_LABEL		'b'	/* Dual of a TCSEC label */
#define MINT_EQUAL_LABEL	'e'	/* Wildcard - always equal */
#define MINT_HIGH_LABEL		'h'	/* High Grade - always dominates */
#define MINT_LOW_LABEL		'l'	/* Low Grade - always dominated */

/*
 * Layout of a label.
 *
 * ml_list contains the list of categories (MSEN) followed by the list of
 * divisions (MINT). This is actually a header for the data structure which
 * will have an ml_list with more than one element.
 *
 *	-------------------------------
 *	| ml_msen_type | ml_mint_type |
 *	-------------------------------
 *	| ml_level     | ml_grade     |
 *	-------------------------------
 *	| ml_catcount                 |
 *	-------------------------------
 *	| ml_divcount                 |
 *	-------------------------------
 *	| category 1                  |
 *	| . . .                       |
 *	| category N                  | (where N = ml_catcount)
 *	-------------------------------
 *	| division 1                  |
 *	| . . .                       |
 *	| division M                  | (where M = ml_divcount)
 *	-------------------------------
 */
#define MAC_MAX_SETS	250

typedef struct	mac_label {
	unsigned char	ml_msen_type;	/* MSEN label type */
	unsigned char	ml_mint_type;	/* MINT label type */
	unsigned char	ml_level;	/* Hierarchical level  */
	unsigned char	ml_grade;	/* Hierarchical grade  */
	unsigned short	ml_catcount;	/* Category count */
	unsigned short	ml_divcount;	/* Division count */
					/* Category set, then Division set */
	unsigned short	ml_list[MAC_MAX_SETS];
} mac_label;

/*
 * Plan G Database: Two files:
 *	MAC_EAG_INDEX - index and size of an entry in ...
 *	MAC_EAG_LABEL - the label itself.
 */
#define MAC_EAG_DIR		"attribute"
#define MAC_EAG_INDEX_FILE	"mac_index"
#define MAC_EAG_LABEL_FILE	"mac_label"
#define MAC_EAG_INDEX		"attribute/mac_index"
#define MAC_EAG_LABEL		"attribute/mac_label"

/*
 * Attribute names known to PlanG and especially eag_parseattr().
 */
#define EAG_MAC		"mac"           /* MAC label of a file */
#define EAG_MAC_IP	"mac-ip"        /* MAC label for NFS */
#define EAG_MAC_NOPLANG "mac-noplang"   /* MAC label of unlabeled filesys */
#define EAG_MAC_PROC	"mac-process"	/* MAC label of this process */

/*
 * index file entry flags.
 *
 * If MAC_EAG_DIRECT the label is stored in the index
 * entry rather than the label file.
 */
#define MAC_EAG_DIRECT	0x00000001	/* Direct entry */
#define MAC_EAG_MOLDY	0x00000002	/* Multi-level Directory */

/*
 * The maximum size of data to put into a direct index file entry.
 * Use as much as possible, but leave the flags alone.
 */
#define MAC_EAG_DIRECT_MAX	(sizeof (int) + sizeof (int))

/*
 * Structure of an entry in the MAC_EAG_INDEX file.
 */
struct mac_eag_index_s {
	unsigned int	mei_index;	/* Index of label in the label file */
	unsigned int	mei_size;	/* Size of label, in bytes */
	unsigned int	mei_flags;	/* Flags reguarding this entry */
};
typedef struct mac_eag_index_s mac_eag_index_t;

#ifdef _KERNEL
/*
 * Mode definitions for mac_access().
 * These need to be the same as VWRITE/IWRITE and VREAD/IREAD, respectivly.
 */
#define	MACEXEC		00100	/* VEXEC in vnode.h */
#define	MACWRITE	00200	/* VWRITE in vnode.h */
#define MACREAD		00400	/* VREAD in vnode.h */
#define	MACWRITEUP	01000	/* VSVTX in vnode.h */

/*
 * Moldy flags
 */
#define MAC_NOMOLD	0	/* Moldy not permitted */
#define MAC_MOLDOK	1	/* Moldy permitted */

/*
 * Structure of the in-core index to label pointer mapping.
 */
struct mac_eag_incore {
	struct mac_eag_incore	*mei_next;	/* Next in this list */
	struct mac_label	*mei_label;	/* Pointer to label table */
	unsigned int		mei_index;	/* Index in the label file */
	int			mei_size;	/* size of label */
};
typedef struct mac_eag_incore mac_eag_incore_t;

/*
 * Data to add to the vfs entry. Includes the vnode pointers for
 * MAC_EAG_INDEX and MAC_EAG_LABEL and the in-core index-label list.
 */
struct mac_vfs {
	struct vnode	*mv_index;      /* inode->data map file */
	struct vnode	*mv_label;      /* label data file */
	mac_eag_incore_t *mv_list;       /* in-core mapping */
	struct mac_label *mv_noplang;   /* use if not plang */
	struct mac_label *mv_ipmac;     /* use for network access */
};
typedef struct mac_vfs mac_vfs_t;

#endif /* _KERNEL */

/* function prototypes */

#ifdef _KERNEL
struct cred;
struct inode;
struct mac_vfs;
struct mounta;
struct snode;
struct vattr;
struct vfs;
struct vnode;
struct pathname;
union rval;
extern int mac_getplabel( mac_label * );
extern int mac_setplabel( mac_label * );
extern int mac_getlabel( char *, mac_label * );
extern int mac_setlabel( char *, mac_label * );
extern int mac_revoke( char * );
extern void mac_init_cred(void);
extern void mac_confignote(void);
extern void mac_setumac( mac_label * );

extern int mac_access( mac_label *, struct cred *, mode_t );
extern int mac_vaccess( struct vnode *, struct cred *, mode_t );
extern int mac_copyin_label( mac_label *, mac_label ** );
extern mac_label *mac_add_label( mac_label * );
extern mac_label *mac_unmold( mac_label * );
extern int mac_inrange( mac_label *, mac_label *, mac_label * );
extern int mac_mint_equ( mac_label *, mac_label * );
extern int mac_cat_equ( mac_label *, mac_label * );
extern int mac_moldy_path( struct vnode *, char *, struct pathname *, struct cred * );

extern mac_label * mac_spec_getlabel( struct vattr *, struct snode * );

extern mac_label * mac_efs_getlabel( struct inode * );
extern int mac_efs_setlabel( mac_label *, struct inode *, int );
extern int mac_efs_iaccess( struct inode *, struct cred *, mode_t );

extern mac_label * mac_nfs_getlabel( ino_t, struct vnode * );
extern int mac_nfs_setlabel( mac_label *, struct vnode *, struct cred *, int );

extern void mac_sem_init( int );
extern void mac_sem_setlabel( int, mac_label * );
extern int mac_sem_access( int, struct cred * );

extern void mac_shm_init( int );
extern void mac_shm_setlabel( int, mac_label * );
extern int mac_shm_access( int, struct cred * );

extern void mac_msg_init( int );
extern void mac_msg_setlabel( int, mac_label * );
extern int mac_msg_access( int, struct cred * );

extern mac_label *mac_eag_getlabel(struct vfs *, int);
extern int mac_eag_setlabel(struct vfs *, int, mac_label *);

#else  /* _KERNEL */
extern int sgi_getplabel( mac_label * );
extern int sgi_setplabel( mac_label * );
extern int sgi_getlabel( char *, mac_label * );
extern int sgi_setlabel( char *, mac_label * );
extern int sgi_revoke( char * );
#endif /* _KERNEL */

extern int mac_dom( mac_label *, mac_label * );
extern int mac_equ( mac_label *, mac_label * );
extern int mac_is_moldy( mac_label * );
extern mac_label *mac_dup( mac_label * );
extern int mac_size( mac_label * );
extern mac_label *mac_demld( mac_label * );
extern mac_label *mac_set_moldy( mac_label * );
extern int mac_ltomld( mac_label *, char *, int );
extern char *mac_labeltostr( mac_label *, int );
extern mac_label *mac_strtolabel( const char * );
extern int mac_invalid( mac_label * );
extern char *mac_getname( int, int );
extern int mac_getvalue( int, char * );
extern char *mac_getlblname( char * );
extern char *mac_getlblcomp( char * );
extern int mac_same( mac_label *, mac_label * );
extern int mac_copy( mac_label *, mac_label * );
/* extern int mac_label_devs( char *, mac_label *, uid_t ); */
#ifdef _KERNEL
/* Define macros choosing stub functions or real functions here */
extern int mac_enabled;

#define _MAC_GETLABEL(a,b)	((mac_enabled)? mac_getlabel(a,b): ENOPKG)
#define _MAC_SETLABEL(a,b)	((mac_enabled)? mac_setlabel(a,b): ENOPKG)
#define _MAC_GETPLABEL(a)	((mac_enabled)? mac_getplabel(a): ENOPKG)
#define _MAC_SETPLABEL(a)	((mac_enabled)? mac_setplabel(a): ENOPKG)
#define _MAC_REVOKE(a)		((mac_enabled)? mac_revoke(a): ENOPKG)

#define _MAC_DOM(a,b)	((mac_enabled)? mac_dom(a,b): 1)
#define _MAC_EQU(a,b)	((mac_enabled)? mac_equ(a,b): 1)
#define _MAC_ACCESS(l,c,i) ((mac_enabled)? mac_access(l,c,i): 0)
#define _MAC_VACCESS(v,c,m) ((mac_enabled)? mac_vaccess(v,c,m): 0)
#define _MAC_SIZE(l)	((mac_enabled)? mac_size(l): 0)
#define _MAC_IS_MOLDY(l)	((mac_enabled)? mac_is_moldy(l): 0)
#define _MAC_MOLDY_PATH(v,s,p,c)	((mac_enabled)? mac_moldy_path(v,s,p,c): 0)

#define _MAC_EAG_GETLABEL(v,s) ((mac_enabled)? mac_eag_getlabel(v,s): NULL)
#define _MAC_SPEC_GETLABEL(v,s) ((mac_enabled)? mac_spec_getlabel(v,s): NULL)

#define _MAC_EFS_GETLABEL(i) ((mac_enabled)? mac_efs_getlabel(i): NULL)
#define _MAC_EFS_SETLABEL(v,i,m) ((mac_enabled)? mac_efs_setlabel(v,i,m): 0)
#define _MAC_EFS_IACCESS(i,c,m) ((mac_enabled)? mac_efs_iaccess(i,c,m): 0)

#define _MAC_NFS_GETLABEL(i,vn) ((mac_enabled)? mac_nfs_getlabel(i,vn): NULL)
#define _MAC_NFS_SETLABEL(l,v,c,m) ((mac_enabled)? mac_nfs_setlabel(l,v,c,m): 0)

#define _MAC_INIT_CRED() ((mac_enabled)? mac_init_cred(): (void)0)
#define _MAC_CONFIGNOTE() ((mac_enabled)? mac_confignote(): (void)0)
#define _MAC_SETUMAC(m) ((mac_enabled)? mac_setumac(m): (void)0)

#define _MAC_SEM_INIT(i) ((mac_enabled)? mac_sem_init(i): (void)0)
#define _MAC_SEM_ACCESS(i,c) ((mac_enabled)? mac_sem_access(i,c): 0)
#define _MAC_SEM_SETLABEL(i,l) ((mac_enabled)? mac_sem_setlabel(i,l): (void)0)

#define _MAC_SHM_INIT(i) ((mac_enabled)? mac_shm_init(i): (void)0)
#define _MAC_SHM_ACCESS(i,c) ((mac_enabled)? mac_shm_access(i,c): 0)
#define _MAC_SHM_SETLABEL(i,l) ((mac_enabled)? mac_shm_setlabel(i,l): (void)0)

#define _MAC_MSG_INIT(i) ((mac_enabled)? mac_msg_init(i): (void)0)
#define _MAC_MSG_ACCESS(i,c) ((mac_enabled)? mac_msg_access(i,c): 0)
#define _MAC_MSG_SETLABEL(i,l) ((mac_enabled)? mac_msg_setlabel(i,l): (void)0)
#endif /* _KERNEL */

#ifdef __cplusplus
}
#endif

#endif	/* __MAC_LABEL_ */
