#ifndef __SYS_MAP_H__
#define __SYS_MAP_H__

/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*#ident	"@(#)kern-port:sys/map.h	10.2"*/
#ident	"$Revision: 3.21 $"

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/sema.h>

/*
 *	struct map	X[]	.m_size		.m_addr
 *			---	------------	-----------
 *			[0]	mapsize(X)	unused
 *				# X[] unused
 *			[1]	map lock *	mapwant sv_t *
 *				map access	wait for free map space
 *
 *	  mapstart(X)->	[2]	# units		unit number
 *			 :	    :		  :
 *			[ ]	    0
 */

struct map
{
	unsigned long m_size;	/* number of units available */
	unsigned long m_addr;	/* address of first available unit */
};

#define	mapstart(X)		&X[2]		/* start of map array */

#define	mapsize(X)		X[0].m_size	/* number of empty slots */
						/* remaining in map array */
#define	mapdata(X)		{(X)-3, 0} , {0, 0}

#define	mapinit(X, Y, A, B)	X[0].m_size = (Y)-3;\
				X[1].m_size = (unsigned long) A;\
				X[1].m_addr = (unsigned long) B

#define maplock(X)		(((lock_t *) X[1].m_size))

#define mapout(X)		((sv_t *) X[1].m_addr)

#define	mapdefine(mapsize, nfree, start, maplock, mapout) \
	{ (mapsize)-3, 0 } ,\
	{ (unsigned long) maplock, (unsigned long) mapout },\
	{ (nfree), (start) },\
	{ 0, 0 }

extern uint malloc(struct map *, int);
extern uint malloc_wait(struct map *, int);
extern void mfree(struct map *, int, unsigned int);
extern ulong_t rmalloc(struct map *, size_t);
extern ulong_t rmalloc_wait(struct map *, size_t);
extern struct map *rmallocmap(ulong_t);
extern void rmfree(struct map *, size_t, ulong_t);
extern void rmfreemap(struct map *);
extern void sptfree(int, __psunsigned_t);
extern __psunsigned_t sptalloc(int, int, int);

#define BITSPERWORD	(NBBY * sizeof(int))
#define BITSPERLONG	(NBBY * sizeof(long))

struct bitmap {
	__psunsigned_t m_unit0;	/* value of unit[0] */
	char	*m_map;		/* the bitmap itself */
	uint_t	m_lock;		/* map protections */
	int	m_size;		/* map units available */
	int	m_lowbit;	/* low-unit mark (inclusive) */
	int	m_highbit;	/* high-unit mark (exclusive) */
	int	m_rotor;	/* bitmap rotor */
	int	m_count;	/* current count */
	uint_t	m_gen;		/* generation number */
};

#define MAP_LOCK	0x1	/* normal access requires two bits to lock */
#define MAP_URGENT	0x2	/* urgency bit asserted */

#define SPTROTORS 10
extern int sptrotors[];

extern void mergebitmaps(struct bitmap *, struct bitmap *);
extern void bmapswtch(struct bitmap *, struct bitmap *);

extern struct bitmap sptbitmap;		/* Map for system virtual space. */
extern struct bitmap stale_sptmap;	/* Map for stale system virtual space. */
extern struct bitmap temp_sptmap;	/* In-transit map. */
extern struct bitmap aged_sptmap;	/* Map for clock-aging sys virt space */
extern sema_t sptbitmaplock;

struct bitmap_wait {
	int	waiters;
	sv_t	wait;
};
extern struct bitmap_wait spt_wait;

#ifdef R4000
extern struct bitmap clean_colormap[];
extern struct bitmap aged_colormap[];
extern struct bitmap stale_colormap[];
extern struct bitmap temp_colormap[];
#endif

int sptwait(int);
void sptwakeup(void);

extern lock_t	msgmplock;
extern sema_t	msgmpout;
extern lock_t	semmplock;
extern sema_t	semmpout;

#ifdef __cplusplus
}
#endif

#endif /* __SYS_MAP_H__  */
