#ifndef __SYS_MGRAS_H__
#define __SYS_MGRAS_H__

/*
** Copyright 1993, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /hosts/bonnie/proj/pacecar/isms/gfx/kern/sys/RCS/mgras.h,v 1.106 1996/06/08 20:55:16 sarah Exp $
*/

#include "rrm.h"
#include "gfx.h"

typedef struct mgras_info {
    struct gfx_info gfx_info; 	/* device independent information */

    unsigned char ProductID;	/* chassis type */
    unsigned char MonitorType;	/* monitor id */
    unsigned char MonTiming;	/* monitor timing */

    unsigned char NumGEs;	/* number of GEs on gfx board */
    unsigned char NumREs;	/* number of REs on RA/RB boards */
    unsigned char NumTRAMs;	/* number of TRAMs on RA/RB boards */
    unsigned char NumRATRAMs;	/* number of TRAMs on RA board */
    unsigned char NumRBTRAMs;	/* number of TRAMs on RB board */
    unsigned char NumVCs;	/* number of VCs on gfx board */
    
    unsigned char GfxBoardRev;
    unsigned char RARev;	/* Raster A board revision */
    unsigned char RBPresent;	/* 1=Raster B board present, 0=not present */
    unsigned char RBRev;	/* Raster B board revision (if RB present) */
    unsigned char VidBdPresent; /* video board present*/
    unsigned char MCOBdPresent; /* MCO board present */
    
    unsigned char HQ3Rev;	/* HQ3 chip revision */
    unsigned char GE11Rev;	/* GE11 chip revision */
    unsigned char RE4Rev;	/* RE4 chip revision */
    unsigned char TE1Rev;	/* TE1 chip revision */
    unsigned char PP1Rev;	/* PP1 chip revision */
    unsigned char VC3Rev;	/* VC3 chip revision */
    unsigned char CmapRev;	/* CMAP chip revision */
    unsigned char TRAMRev;	/* TRAM chip revision */
    unsigned char MCRev;	/* MC chip revision */

    unsigned char MCOBdRev;     /* MCO board revision */
    
    unsigned char Xabnormalexit;/* X killed without closing file descriptor */
    unsigned char Bitplanes;	/*
				** Bitplanes depends on RA/RB config so
				** this field may never get used
				*/
    unsigned char PanelType;	/* flat panel type */
} mgras_info;

/* flags for use in flag field of timing table */
#define MGRAS_GENSYNC		BIT(1)
#define	MGRAS_INTERLACED	BIT(2)	
#define MGRAS_GENLOCK		BIT(3)
#define MGRAS_FULLSCREEN_STEREO BIT(4)
#define MGRAS_ZBUF_AVAIL        BIT(16)	/* Z-buffered visuals available */
#define MGRAS_32DB_AVAIL        BIT(17) /* 32-bit double buffering available */
#define MGRAS_SERRATED_VSYNC    BIT(18)
#define MGRAS_MCO_FORMAT        BIT(24)	/* This timing is used w/ MCO */
/* overload one bit in flag field to allow X to suppress reconfiguration of */
/* the framebuffer. */
#define MGRAS_SUPPRESS_FBCONFIG BIT(30) 

#define	MGRAS_DRB_MAX_NUM_BANKS	6

/* constants for use in determining which hardware can run a table */
#define MGRAS_OPTIONS_ANY       	0
#define MGRAS_OPTIONS_FLAT_PANEL_ONLY	1
#define MGRAS_OPTIONS_MCO_ONLY	        2

typedef struct mgras_DRBptrs {
    unsigned int dummy;		/* compatibility padding so 6.2 X server
				   can run on a 5.3 Operating System */
    unsigned int overlay;
    unsigned int main;
    unsigned int banks[MGRAS_DRB_MAX_NUM_BANKS];
} mgras_DRBptrs;

typedef struct mgras_timing_info
{
        unsigned int flag;	  /* see #defines above */
        unsigned int w, h;	  /* monitor resolution */
	unsigned int refresh_rate; /* refresh rate to nearest Hz */
        unsigned int cfreq;	  /* pixel clock frequency (MHz) */
        unsigned short *ftab;	  /* video timing frame table */
	unsigned int ftab_len;	  /* length (in shorts) of *ftab */
        unsigned short *ltab;	  /* video timing line table */
        unsigned int ltab_len;	  /* length (in shorts) of *ltab */
        unsigned char *dacplltab; /* ADV DAC  pixel clock parameters  */
	unsigned int num_fields;  /* # fields per frame */
	unsigned int DIBptr;
	unsigned int DIBskip; 	
	unsigned int DIBtopscan;
	unsigned int DIBctl0; 	
	unsigned int DIBctl1; 	
	unsigned int DIBctl2; 	
	unsigned int DIBctl3; 	
	unsigned int REFctl; 	
	mgras_DRBptrs DRBptrs;
} mgras_timing_info;

/*
** save/restore info structure
*/
typedef struct __MgrasGMTexDesc{
    struct __MgrasGMTexDesc 	*next;     /* Pointer to next in list */
    int	linebytes;          /* bytes per line */
    int	pagelines;          /* lines per page */
    unsigned char *data;              /* Pointer to data */
    unsigned short startpageidx, numpage, endpage; /* Start, #pages, end */
    unsigned char freedata;           /* Original pointer to malloced data */
    unsigned char saved, code;        /* Format codes */
    unsigned char overlapping;        /* Flag=> "in the process of being overlapped" */
    unsigned char vx2;                /* Vertical x2 mode */
    unsigned char numcomp;            /* # components 0-3 */
    unsigned char comptype;           /* 0=>1byte/1=>2byte */
    unsigned char buffer;             /* 0->lobyte/1=>hibyte (for double buffered) */
    unsigned char db_enab;            /* Double-buffer mode enable */
    unsigned char select;             /* TRAM selection 0-3 */
    unsigned char select_enab;        /* enable TRAM selection 0-3 */
    unsigned char resid;              /* hardware residency */
    unsigned short baseaddr_index;    /* Start addr in TRAM */
    }MgrasGMTexDesc_t;

#define MGRAS_MAXTRAMPAGES	256
#define MGRAS_INCRTEXSPACE 	16     /* local texture mgr requests more 
					* texture space in multiples 
					* of MGRAS_INCRTEXSPACE pages 
					*/

/* Number of blocks of texture pages. Each block has MGRAS_INCRTEXSPACE pages 
*/
#define MGRAS_NUMTEXBLOCK	(MGRAS_MAXTRAMPAGES/MGRAS_INCRTEXSPACE)

#ifdef _KERNEL
/* 
 * Commands which may be CPU-dependent
 */

#define MGRASFIFO(x) 	base->fifo[x]
#define SPLRETR   	splretr

#ifdef DEBUG
#define DEBUGprintf0(x,fmt)	\
	if (x)  printf(fmt)

#define DEBUGprintf1(x,fmt,a0)	\
	if (x)  printf(fmt,a0)

#define DEBUGprintf2(x,fmt,a0,a1) \
	if (x)  printf(fmt,a0,a1)

#define DEBUGprintf3(x,fmt,a0,a1,a2) \
	if (x)  printf(fmt,a0,a1,a2)

#else /* DEBUG */
#define DEBUGprintf0(x,fmt)
#define DEBUGprintf1(x,fmt,a0)
#define DEBUGprintf2(x,fmt,a0,a1) 
#define DEBUGprintf3(x,fmt,a0,a1,a2)
#endif /* DEBUG*/

#define MGRAS_XMAP_NUMMODES_MAIN      32
#define MGRAS_XMAP_NUMMODES_OVERLAY    8
#define	MGRAS_XMAP_NUMMODES	      (MGRAS_XMAP_NUMMODES_MAIN|MGRAS_XMAP_NUMMODES_OVERLAY)

/* DMA translation page size vs. process page size */
#define MGRAS_DMA_PNUMSHFT		12	/* 2^N bytes per tx page */
#define MGRAS_DMA_PAGESZ		(1 << (MGRAS_DMA_PNUMSHFT))
#define MGRAS_DMA_OFFSET_MASK		((MGRAS_DMA_PAGESZ) - 1)
#define MGRAS_DMA_TX_TO_PROC_PG_SHFT	((PNUMSHFT) - (MGRAS_DMA_PNUMSHFT))
#define MGRAS_DMA_TX_TO_PROC_RATIO	(1 << MGRAS_DMA_TX_TO_PROC_PG_SHFT)

/* DMA limits */
#define MGRAS_DMA_POOLS		5	/* Number of hw pools supported */
#define MGRAS_DMA_MIN_TXS ( 2 * 1024) 	/* Min trnsltion table size (entries)*/

typedef struct mgras_lock_cookies {
    int      fast_useracc_cookie;
#ifdef R10000_USER_SPECULATION_WAR
    void    *mpin_lockdown_cookie;
#endif /* R10000_USER_SPECULATION_WAR */
} mgras_lock_cookies;

typedef struct mgras_dma_pool {
	int		active;		/* Pool will be useable for dma */
	int 		teardownPending;/* Pool deactivation is underway */
	__uint32_t	syncBase;	/* Address used for syncronization */
	__uint32_t	numImPages;	/* The number of IM pages */
	void		**imPageAddrs;	/* Vector of IM page starting addrs */
	__uint32_t	*imPageLengths;	/* The size of IM pages (in bytes) */
	mgras_lock_cookies *imPageCookies; /* Cookies for fast page (un)locking */
	__uint32_t	imTxStride;	/* Entries for a page in txTbl */
	int		*txTbl;		/* Pointer to full translation table */
	__uint32_t	txTblLength;	/* Total number of entries in table */
	int		readOnly;	/* Dma accesses can be read only */
} mgras_dma_pool;


typedef struct mgras_dma {
  	mgras_dma_pool	pools[MGRAS_DMA_POOLS];
	sema_t		tx_tbl_sema;	  /* Translation table switch sema */
} mgras_dma;

typedef struct mgras_shadow {
	void           *uv_sh;	/* user-virtual region (for unuseracc) */
	void	       *kv_sh;	/* kernel virtual addr (for unmaputokv) */
	raster_shadow  *rss_sh;	/* kernel access to shadow region */
} mgras_shadow;

struct MgrasRetrace;

typedef struct _read_window {
  __uint32_t read_xywin;
  ppdrbpointersu read_drb_pointers;
} read_window;

struct m2_packet {
	int geEramGlContextPtr;
#	define NONE_ALLOCATED		-1
	unsigned int restoreIdleHq3 : 1;	/* restore an idle hq3 state */
	unsigned int newContext : 1;		/* call mRasterAddrInitialize */
	unsigned int validateClip : 1;		/* there is validateClip info */
	unsigned int suppressRSSRestore : 1;	/* do not restore RSS */
	unsigned int serverShadow : 1;		/* valid server shadow data */
	unsigned int validateBanks : 1;		/* there is display bank info */
	unsigned int unusedflags : 26;
				/* validate clip info */
	__uint32_t xywin;
	winmodeu winmode;
	ppwinmodeu pp1winmode;
	__uint32_t scrmsk4x;
	__uint32_t scrmsk4y;
	__uint32_t scrmsk3x;
	__uint32_t scrmsk3y;
	__uint32_t scrmsk2x;
	__uint32_t scrmsk2y;
	__uint32_t scrmsk1x;
	__uint32_t scrmsk1y;
	/* Added new field to support validatebanks (for buf swaps) in m2 packet.
	 * Added fields for drbptrs and read drbptrs and read origin in validate
	 * clip area of m2 packet to support makecurrentread.
	 * Added fields for display banks for kernel-directed swapping.
	 */
	ppdrbpointersu drb_pointers; /* added for pbuffers */
	/* used for MakeCurrentRead: */
	read_window read_window;
				/* end of validate clip info */
				/* validate banks info */
	int	drawDisplayBankIndex;
	int	readDisplayBankIndex;
				/* end of validate clip info */
	int spare2[1];
	struct server_regs {
		int block_xystarti;
		int block_xyendi;
		int xfrsize;
		int fillmode;
		int bkgrd_rg;
		int bkgrd_ba;
		int config;
		int pp1fillmode;
		int ColorMaskMSBs;
		int ColorMaskLSBsA;
		int ColorMaskLSBsB;
		int PIXcolorR;
		int PIXcolorG;
		int PIXcolorB;
		int PixCmd;
		int TAGmode;
		int z_hi;
		int z_low;
		int scissorx;
		int scissory;
	} server_regs;
	int spare3[1];
#	define MAX_ERAM_RES_ID 8
	int eramptr[MAX_ERAM_RES_ID];
	int spare[15];			/* 63 words in m2 packet */
};

struct eram_resource {
	int er_size;	/* size in words or zero if not requested */
	int er_addr;	/* ge11 eram physical address */
	char *er_host;	/* host buffer */
	int er_hq3va;	/* hq3 virtual addr */
	int er_hq3vs;	/* # of entries taken in tx table */
	int er_new;	/* freshly requested resource */
	struct ge11node *er_node;	/* node associated with er_addr */
};


typedef struct  mgras_texpage {
	struct	mgras_texpage	*next;
	int			vaddr;
	__uint32_t		pagenum;
}mgras_texpage;

typedef struct _mgras_texture
{
	unsigned short	 numoverlap;
	MgrasGMTexDesc_t *desc;
	mgras_texpage    *savedpages[MGRAS_NUMTEXBLOCK];
}mgras_texture;

typedef struct _GMSaveNode {
	struct mgras_rnode *rnp;
} GMSaveNode;

typedef struct mgras_texmem {
	struct mgras_texmem *next;
	mgras_texpage 	*headfreepage;
	void 		*headmem;
	__uint32_t	length;
	
}mgras_texmem;
typedef struct _mgras_texrefcount {
	int		refcount[MGRAS_NUMTEXBLOCK]; 
	int		maxrefcount;
} mgras_texrefcount;

typedef struct mgras_bdata_tex
{
	int		*txTbl;		/* Pointer to full translation table */
        __uint32_t	txTblLength;	/* Total number of entries in table */
	__uint32_t	texmode1tram;	/* hw encode value for TEXMODE1 reg */
	mgras_texpage 	*freepage;
	mgras_texmem	*texmemlist;
	unsigned char	resid_array[MGRAS_NUMTEXBLOCK];
	GMSaveNode 	resid_list[MGRAS_NUMTEXBLOCK];
	int 		mgras_num_texprocs;/* # of tex procs on this board */	
	mgras_texrefcount blockusage;
}mgras_bdata_tex;


typedef struct mgras_rnode { 
	struct mgras_rnode 	*prev;
	struct mgras_rnode 	*next;
	struct rrm_rnode	*rnp;
	unsigned int 		pcxtype;	
	unsigned int 		pcxsize;	
	/* context switch related state */
	int 			hq_config, gio_config, intr_enab_set, flag_enab_set;
	int 			cfifo_nonempty;
	struct m2_packet	m2_packet;
	struct eram_resource	eram_res[MAX_ERAM_RES_ID];
	char *			glContextHostBuf;
	int			glContextHq3va;	/* hq3 virtual addr */
	int			glContextHq3vs;	/* # of entries taken in tx table */
	/* DMA-related state */
	mgras_dma 		dma;
	/* Raster shadow registers (if this process needs them) */
	mgras_shadow            shadow;
	/* last buffer swap vsync count */
	unsigned int            last_swaptime;
	mgras_texture 		*texture;
	int			must_flush_before_swap_out;
        int                     decouple_swap_from_vr;
} mgras_rnode;
#endif /* _KERNEL */

struct mgras_ge11_config_args {
	int glContextMax;
	int glContextBase;
	int glContextSize;
	int ge_get_data_token;
	int blockxy_start_offset;
	int blockxy_end_offset;
	int xfrsize_offset;
	int re_fillmode_offset;
	int crambackup_offset;
	int texmode1_offset;
	int emh_offset;
	int em_screenres;
};

/*
 * for MGRAS_REQUEST_ERAM_RES,
 * inputs are size and data; erid is returned.
 * for MGRAS_RELEASE_ERAM_RES
 * inputs are size and erid; buffer contents returned.
 */
struct mgras_eram_resource_arg {
	int size;	/* in words */
	int erid;	/* eram resource id */
	int *data;	/* if not null, points to initial/final data */
};

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_eram_resource_arg {
	int size;	/* in words */
	int erid;	/* eram resource id */
	app32_ptr_t data; /* if not null, points to initial/final data */
} irix5_mgras_eram_resource_arg;
#endif

#ifdef _KERNEL
#ifndef _STANDALONE
#include "sys/map.h"
#endif

typedef struct mgras_data {
	struct gfx_data 	gfx_data;	/* Common gfx driver above */
	mgras_info 		*info;  	/* mgras inventory */
	mgras_hw		*base;		/* K1 seg board base address */
	struct MgrasRetrace 	*Retrace;
	unsigned char           *mgr_xmap_index_shadow;
	unsigned int            loadedmodes[MGRAS_XMAP_NUMMODES];
	int			cursor_xhot;
	int 			cursor_yhot;
	int			cursor_x;
	int 			cursor_y;
	lock_t			mgraslock;  /* spinlock to be used in MP systems, 
					     * no op-ed in SP systems 
					     */
	unsigned int		attaches;
	int			ftcallid;	/* Callback timeout  for LW*/

	struct mgras_rnode *active_rnode;
	struct mgras_rnode *last_rnode;		/* debugging aid */
	struct mgras_rnode irnode;	/* intermediate rnode */
	sema_t csmb, ctxdone;
	int /* boolean */	boardManagerHasRebus;
	int /* boolean */	waitingForContextDone;
	int /* boolean */	debugger_running;
	int glContextAvailMask;
	struct mgras_ge11_config_args ge11_config;
	int glContextBound;
	int er_max_size;	/* bound on sum of all eram resources */
	int er_cur_size;
	int openrn;		/* count of contexts created */
	struct mgras_rnode_anchor {
		struct mgras_rnode *prev;	/* must match 1st member of mgras_rnode */
		struct mgras_rnode *next;	/* must match 2nd member of mgras_rnode */
	} rnode_anchor;	/* head of doubly-linked list of opened rnodes */
	int *kpoolTxBase;	/* translation table for kernel dma */
	int kpoolTxSize;	/* # of entries in translation table */
	lock_t hq3VmMapLock;
	struct map *kpoolHq3VmMap; /* available hq3 virtual addr space */
	int kpoolHq3VmMapSize;
	int /* boolean */	kpoolTxNeedsFlushing;
	mgras_bdata_tex		*bdtexture;
	unsigned long loswap[2], hiswap[2];	/* heavy-weight stats */
	int /* boolean */ check_demarcation, readj_demarcation;
	unsigned long swap_history;
	sema_t sleeper;		/* for handling signals during ctxsw */
	sema_t dmadone;		/* for deferring ctxsw until dma done */
	int ctid;		/* callout id for ctxsw timer */
        sema_t hq3cfifodrain;   /* for draining hq3 cfifo before ctxsw */
        int /* boolean */ hq3cfifodrain_flag;
	int ccv_update_asked;	/* have we asked to adjust RAC cell current */
	int ccv_update_needed;	/* do we need to adjust RAC cell current now */
#ifdef FLAT_PANEL
        unsigned char   panelalive;
#endif
	sema_t init_sleeper;	/* waiting for gfx processes to die */
        int mco_fpga_mode;
        int mco_vc2_flags;
	int video_dma_in_progress;	/* prevent ctxsw during video texture dma */
	sema_t video_dma_done;
} mgras_data;

typedef struct mgrasboards {
	mgras_hw   *base; 
        mgras_data *data;
        mgras_info *info;
        int             found;
#if defined(IP22) || defined(IP26) || defined(IP28)
        int             slot;                   /* gio slot number */
        int             index;                  /* index into this array */
        int             mask;                   /* retrace intr mask */
#endif
} mgrasboards;

struct mgras_dma_util_single_args;
struct mgras_dma_util_multi_args;
struct mgras_dma_edit_append_args;
struct mgras_dma_edit_replace_args;

/* Standard gfx kernel routines  */
extern int MgrasPositionCursor(struct gfx_data *, register int, register int);
extern int MgrasSchedMGRSwapBuf(struct gfx_gfx *, int, int, int, int);  
extern int MgrasSchedSwapBuf(struct gfx_data *, struct rrm_rnode *,int,int);  
extern int MgrasUnSchedSwapBuf(struct gfx_gfx *, struct rrm_rnode *, int);
extern int MgrasSchedRetraceEvent(struct gfx_data *, struct rrm_rnode *);
extern int MgrasSetDisplayMode(struct gfx_gfx *, int , unsigned int);
extern int MgrasSetGammaRamp(struct gfx_data *, unsigned short *,
			     unsigned short *, unsigned short *);
extern int MgrasDmaUtil_Single(struct gfx_gfx *, struct rrm_rnode *, 
			       struct mgras_dma_util_single_args *);
extern int MgrasDmaUtil_Multi(struct gfx_gfx *, struct rrm_rnode *, 
			      struct mgras_dma_util_multi_args *);
extern int MgrasDmaEditAppend(struct gfx_gfx *, struct rrm_rnode *,
			      struct mgras_dma_edit_append_args *);
extern int MgrasDmaEditReplace(struct gfx_gfx *, struct rrm_rnode *,
			       struct mgras_dma_edit_replace_args *);


extern void MgrasReset(mgras_hw *);
extern int MgrasProbe(mgras_hw *);
extern int MgrasGatherInfo(mgras_hw *, mgras_info *);

extern void MgrasRetraceHandler(mgras_data *);
extern void MgrasDestroyDMA(struct mgras_rnode *rnp);
extern void MgrasInstallTxTbl_Kern(register mgras_hw *base, unsigned int pool, 
				   int *txTbl, unsigned int numEntries, 
				   int flush);
extern void MgrasResetTxTbl_Kern(register mgras_hw *base, unsigned int pool);
extern void MgrasCcvUpdate(mgras_data *bdata);


/* Initialization Functions */
extern int MgrasInit(mgras_hw *,mgras_info *);
extern void MgrasSetTiming(mgras_hw *base,
			   mgras_info *info,
			   int ucode_loaded,
			   struct mgras_timing_info *tinfo);
extern void MgrasInitHQ3(register mgras_hw *base, int numge);
extern void MgrasInitDCBctrl( mgras_hw *base);
extern int MgrasInitRasterBackend(mgras_hw *, mgras_info *);
extern void MgrasInitDMA(register mgras_hw *base);
extern int MgrasSyncREPP(register mgras_hw *base, int NumREs,
			  int re4_id, int pp1_id);
extern int MgrasSyncREPPs(register mgras_hw *base, mgras_info *info);
extern void MgrasSyncPPRDRAM(register mgras_hw *base, int re4_id,
			     int pp1_id, int rdram_id);
extern void MgrasSyncPPRDRAMs(register mgras_hw *base, mgras_info *info);
extern void MgrasInitRSS(register mgras_hw *base, mgras_info *info);
extern void MgrasInitDac(register mgras_hw *base);
extern void MgrasInitClock(mgras_hw *base, mgras_info *info);
extern void MgrasInitVC3(register mgras_hw *base);
extern void MgrasInitXMAP(register mgras_hw *base, mgras_info *info);
extern void MgrasInitCMAP(register mgras_hw *base);
extern int MgrasDownloadHQ3Ucode(mgras_hw *, void *, int, int);
extern int MgrasDownloadGE11Ucode(mgras_hw *, void *, int, int, int);
extern void MgrasEramManagerTimeout(mgras_data *);
#endif /* KERNEL */

/*
 *	MGRAS specific (gf_Private) Ioctls
 */
#define MGRAS_BASE		('m'<<24|'g'<<16)	
#define MGRASIOCTL(x)		(MGRAS_BASE|x)

#define MGRAS_SETDISPLAYMODE	MGRASIOCTL(1)
#define MGRAS_SETCURSOR_HOTSPOT MGRASIOCTL(2)
#define MGRAS_SETGAMMARAMP	MGRASIOCTL(3)
#define MGRAS_VSYNC_SET		MGRASIOCTL(4)
#define MGRAS_VSYNC_GET		MGRASIOCTL(5)
#define MGRAS_VSYNC_SLEEP	MGRASIOCTL(6)
#define MGRAS_DEBUG_GE11_BEGIN	MGRASIOCTL(7)
#define MGRAS_DEBUG_GE11_END	MGRASIOCTL(8)
#define MGRAS_GE11_CONFIG	MGRASIOCTL(9)
#define MGRAS_DMA_ALIGNMENT	MGRASIOCTL(10)
#define MGRAS_DMA_UTIL_SINGLE	MGRASIOCTL(11)
#define MGRAS_SET_SHADOW_RGN	MGRASIOCTL(12)
#define MGRAS_DMA_UTIL_MULTI	MGRASIOCTL(13)
#define MGRAS_DMA_EDIT_REPLACE	MGRASIOCTL(14)
#define MGRAS_DMA_EDIT_APPEND	MGRASIOCTL(15)
#define MGRAS_ADD_TEXTURE_SPACE MGRASIOCTL(16)
#define MGRAS_LOAD_TIMING_TABLE MGRASIOCTL(17)
#define MGRAS_REQUEST_ERAM_RES	MGRASIOCTL(18)
#define MGRAS_RELEASE_ERAM_RES	MGRASIOCTL(19)
#define MGRAS_SET_GENLOCK	MGRASIOCTL(20)
#define MGRAS_TEXTURE_READ      MGRASIOCTL(21)
#define MGRAS_VID_TEXTURE_XFER  MGRASIOCTL(22)
#define MGRAS_GET_DRBPTRS       MGRASIOCTL(23)
#define MGRAS_I2C_CMD		MGRASIOCTL(24)
#define MGRAS_FPREINIT_CMD	MGRASIOCTL(25)
#define MGRAS_DECOUPLE_SWAPBUF	MGRASIOCTL(26)
#define MGRAS_LOAD_VIDEO_FORMAT_COMBINATION    MGRASIOCTL(27)
#define MGRAS_LOAD_CHANNEL_GAMMA MGRASIOCTL(28)
#define MGRAS_MANAGE_CURSOR_GLYPH MGRASIOCTL(29)


/* physical address */
#ifndef MGRAS_BRINGUP
#define	MGRAS_BASE0_PHYS	0x1F000000	/* private bus MGRAS board family base */
#define	MGRAS_BASE1_PHYS	0x1F400000	
#define	MGRAS_BASE2_PHYS	0x1F800000	/* Needed for slot2 logic */	

#else  /* MGRAS_BRINGUP */

#define	MGRAS_BASE0_PHYS	0x1F0F0000	/* PX - access DCB thru REX3,
						 *    - access RE4 from base ptr
						 */
#endif /* ifndef MGRAS_BRINGUP*/

#define	MGRAS_SYNC_VALUE 	0 

#define	MGRAS_GIO_ID_REG	0x1F000000 	/* gio_id reg for GIO64 devices*/
#define GIO_ID_MGRAS    	0x10 		/* MGRAS gfx board */

#define MGRAS_PRODUCT           0x00
#define MGRAS_PRODUCT_MG10      0x01

/*
 * for Xserver to differentiate slowpath vs. fastpath machines
 */
#if defined(IP22)
#define GFX_NAME_MGRAS		"IMPACT"
#else
#define GFX_NAME_MGRAS		"IMPACTPC"
#endif

struct mgras_setcursor_hotspot{
	unsigned short xhot;	/* X cursor hotspot */
	unsigned short yhot;	/* Y cursor hotspot */
};

#define MGRAS_BUF_SELECT_BIT    BIT(30)

struct mgras_setdisplaymode_args{
	unsigned int wid;	
	unsigned int displaymode;
};

typedef struct mgras_setgammaramp_args {
	unsigned short red[256];
	unsigned short green[256];
	unsigned short blue[256];
} mgras_setgammaramp_args, mgras_gammatable;

struct mgras_vsync_sleep_args {
	int	interval;
	int 	mod;
};

/* Info for the dma utility ioctls */
#define MGRAS_IM_POOL_0		0
#define MGRAS_IM_POOL_1		1
#define MGRAS_IM_POOL_2		2
#define MGRAS_GL_POOL		3
#define MGRAS_DL_POOL		4
#define MGRAS_KERN_POOL		MGRAS_IM_POOL_2
#define MGRAS_IM_POOL_FIRST	MGRAS_IM_POOL_0
#define MGRAS_IM_POOL_LAST	MGRAS_IM_POOL_2

#define MGRAS_POOL_START_TEARDOWN	BIT(0)	/* Start destroying pool */
#define MGRAS_POOL_FINISH_TEARDOWN	BIT(1)	/* Finish destroying pool */
#define MGRAS_POOL_BUILD		BIT(2)	/* Build a new pool */
#define MGRAS_POOL_FLUSH		BIT(3)	/* Flush addrs from caches */
#define MGRAS_POOL_APPEND		BIT(4)	/* Append IM pages to pool */
#define MGRAS_POOL_REPLACE		BIT(5)	/* Replace IM pages in pool */

typedef struct mgras_dma_util_single_args {
	unsigned int	flags;		/* req'd - What to do */
	unsigned int	pool;		/* tear/build - dma pool affected */
	void 		*dma_start;	/* build - Start of mem for dma pool */
	unsigned int	dma_length;	/* build - Bytes of mem for dma pool */
	int		read_only;	/* build - Dmas cannot write to mem */
	void		*flush_start;	/* flush - Start of range to flush */
	unsigned int	flush_length;	/* flush - Bytes of range to flush */
} mgras_dma_util_single_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_dma_util_single_args {
	app32_uint_t	flags;		/* req'd - What to do */
	app32_uint_t	pool;		/* tear/build - dma pool affected */
	app32_ptr_t	dma_start;	/* build - Start of mem for dma pool */
	app32_uint_t	dma_length;	/* build - Bytes of mem for dma pool */
	app32_int_t	read_only;	/* build - Dmas cannot write to mem */
	app32_ptr_t	flush_start;	/* flush - Start of range to flush */
	app32_uint_t	flush_length;	/* flush - Bytes of range to flush */
} irix5_mgras_dma_util_single_args;
#endif

typedef struct mgras_dma_flush_defn {
	void		*start;		/* Address of user mem to be flushed */
	unsigned int	length;		/* Bytes of user mem to be flushed */
} mgras_dma_flush_defn;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_dma_flush_defn {
	app32_ptr_t	start;		/* Address of user mem to be flushed */
	app32_uint_t	length;		/* Bytes of user mem to be flushed */
} irix5_mgras_dma_flush_defn;
#endif

typedef struct mgras_dma_util_multi_args {
	unsigned int	flags;		/* req'd - What to do */
	unsigned int	pool;		/* tear/build - dma pool affected */
	unsigned int	num_impages;	/* build - How many im pages in list */
	void 		*impage_starts;	/* build - Im page start addrs list */
	unsigned int	impage_length;	/* build - Bytes of mem for im page */
	int		read_only;	/* build - Dmas cannot write to mem */
	unsigned int	num_flushes;	/* flush - How many flushes in list */
	mgras_dma_flush_defn *flush_defns;/* flush - List of ranges to flush */
} mgras_dma_util_multi_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_dma_util_multi_args {
	app32_uint_t	flags;		/* req'd - What to do */
	app32_uint_t	pool;		/* tear/build - dma pool affected */
	app32_uint_t	num_impages;	/* build - How many im pages in list */
	app32_ptr_t	impage_starts;	/* build - Im page start addrs list */
	app32_uint_t	impage_length;	/* build - Bytes of mem for im page */
	app32_int_t	read_only;	/* build - Dmas cannot write to mem */
	app32_uint_t	num_flushes;	/* flush - How many flushes in list */
	app32_ptr_t	flush_defns;	/* flush - List of ranges to flush */
} irix5_mgras_dma_util_multi_args;
#endif

typedef struct mgras_dma_replace_defn {
	unsigned int	impage_num;	/* Which im page is to be replaced */
	void		*start;		/* User mem address of new im page */
	unsigned int	length;		/* Bytes of user mem in im page */
} mgras_dma_replace_defn;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_dma_replace_defn {
	app32_uint_t	impage_num;	/* Which im page is to be replaced */
	app32_ptr_t	start;		/* User mem address of new im page */
	app32_uint_t	length;		/* Bytes of user mem in im page */
} irix5_mgras_dma_replace_defn;
#endif

typedef struct mgras_dma_edit_append_args {
	unsigned int	flags;		/* req'd - What to do */
	unsigned int	pool;		/* appnd - dma pool affected */
	unsigned int	num_appends;	/* appnd - How many im pages in list */
	void 		*impage_starts;	/* appnd - Im page start addrs */
	unsigned int	impage_length;	/* appnd - Bytes of mem for im page */
	unsigned int	num_flushes;	/* flush - How many flushes in list */
	mgras_dma_flush_defn *flush_defns;/* flush - List of ranges to flush */
} mgras_dma_edit_append_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_dma_edit_append_args {
	app32_uint_t	flags;		/* req'd - What to do */
	app32_uint_t	pool;		/* tear/build - dma pool affected */
	app32_uint_t	num_appends;	/* appnd - How many im pages in list */
	app32_ptr_t	impage_starts;	/* appnd - Im page start addrs */
	app32_uint_t	impage_length;	/* appnd - Bytes of mem for im page */
	app32_uint_t	num_flushes;	/* flush - How many flushes in list */
	app32_ptr_t	flush_defns;	/* flush - List of ranges to flush */
} irix5_mgras_dma_edit_append_args;
#endif

typedef struct mgras_dma_edit_replace_args {
	unsigned int	flags;		/* req'd - What to do */
	unsigned int	pool;		/* replc - dma pool affected */
	unsigned int	num_replaces;	/* replc - How many im pages in list */
	mgras_dma_replace_defn *replace_defns;/* replc - Im page start addrs */
	unsigned int	num_flushes;	/* flush - How many flushes in list */
	mgras_dma_flush_defn *flush_defns;/* flush - List of ranges to flush */
} mgras_dma_edit_replace_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_dma_edit_replace_args {
	app32_uint_t	flags;		/* req'd - What to do */
	app32_uint_t	pool;		/* replc - dma pool affected */
	app32_uint_t	num_replaces;	/* replc - How many im pages in list */
	app32_ptr_t	replace_defns;	/* replc - Im page start addrs */
	app32_uint_t	num_flushes;	/* flush - How many flushes in list */
	app32_ptr_t	flush_defns;	/* flush - List of ranges to flush */
} irix5_mgras_dma_edit_replace_args;
#endif

typedef struct mgras_set_shadow_rgn_args {
	void           *rss_sh;
} mgras_set_shadow_rgn_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_set_shadow_rgn_args {
	app32_ptr_t    rss_sh;
} irix5_mgras_set_shadow_rgn_args;
#endif

typedef struct mgras_timing_info mgras_load_timing_table_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_load_timing_table_args {
        app32_uint_t flag;	  /* see #defines above */
        app32_uint_t w, h;	  /* monitor resolution */
	app32_uint_t refresh_rate; /* refresh rate to nearest Hz */
        app32_uint_t cfreq;	  /* pixel clock frequency (MHz) */
        app32_ptr_t  ftab;	  /* video timing frame table */
	app32_uint_t ftab_len;	  /* length (in shorts) of *ftab */
        app32_ptr_t  ltab;	  /* video timing line table */
        app32_uint_t ltab_len;	  /* length (in shorts) of *ltab */
        app32_ptr_t  dacplltab; /* ADV DAC  pixel clock parameters  */
	app32_uint_t num_fields;  /* #fields per frame */
	app32_uint_t DIBptr; 	
	app32_uint_t DIBskip;
	app32_uint_t DIBtopscan;
	app32_uint_t DIBctl0; 	
	app32_uint_t DIBctl1; 	
	app32_uint_t DIBctl2; 	
	app32_uint_t DIBctl3; 	
	app32_uint_t REFctl; 	
	mgras_DRBptrs DRBptrs;
} irix5_mgras_load_timing_table_args;
#endif

typedef struct mgras_texspace_args {
        int numpages;
	int overlap;
} mgras_texspace_args;

#define MGRAS_GENLOCK_SOURCE_INTERNAL 0
#define MGRAS_GENLOCK_SOURCE_EXTERNAL 1

typedef struct mgras_set_genlock_args {
	unsigned int enable_genlock;
	unsigned int genlock_source;
} mgras_set_genlock_args;

struct mgras_texture_read_args {
    unsigned int ge_cmd_words;
    unsigned int ge_cmd_buffer[10];
    unsigned int hag_start_addr;
    unsigned int bytesPerRow;
    unsigned int stride;
    unsigned int width;
    unsigned int height;
    unsigned int xfr_mode;
    unsigned int xfr_ctrl;
    unsigned int format_control;
    unsigned int dma_pool;
};

/* video texture external_type values */
#define MGRAS_VIDTEX_ETYPE_RGB          0
#define MGRAS_VIDTEX_ETYPE_RGBA         1

/* video texture external_format values */
#define MGRAS_VIDTEX_EFORM_UBYTE        0

/* Video texture transfer status values */
#define MGRAS_VIDTEX_STAT_OK            0
#define	MGRAS_VIDTEX_STAT_NODRIVER	1
#define MGRAS_VIDTEX_STAT_NOTACTIVE     2
#define MGRAS_VIDTEX_STAT_BADSIZE       3
#define MGRAS_VIDTEX_STAT_BADMIPMAP     4
#define MGRAS_VIDTEX_STAT_BADTYPE       5
#define MGRAS_VIDTEX_STAT_BADFORMAT     6
#define MGRAS_VIDTEX_STAT_LOADING       7

typedef struct mgras_vid_texture_xfer_args {
    unsigned int portnum;
    unsigned int loadflag;
    unsigned int width;
    unsigned int height;
    unsigned int mipmap_on;
    unsigned int external_type;
    unsigned int external_format;
    unsigned int last_field;
    unsigned int stat;
    unsigned int dbflag;
} mgras_vid_texture_xfer_args;

typedef mgras_DRBptrs mgras_get_drbptrs_args;

typedef struct mgras_ccr_struct {
    unsigned int last_ccv;	/* last loaded ccv value */
} mgras_ccr;

typedef mgras_ccr mgras_ccr_array[4][2];

typedef struct mgras_mco_ramDAC7162_config {
    unsigned int mode;		/* MCO_ADV7162_MODE_INIT */
    unsigned int cmd1;		/* MCO_ADV7162_CMD1_INIT */
    unsigned int cmd2;		/* MCO_ADV7162_CMD2_INIT */
    unsigned int cmd3;		/* MCO_ADV7162_CMD3_INIT */
    unsigned int cmd4;		/* MCO_ADV7162_CMD4_INIT */
    unsigned int cmd5;		/* MCO_ADV7162_CMD5_INIT */
    unsigned int pllr;
    unsigned int pllv;
    unsigned int pllctl;	/* s, psel, vsel, rsel, PLL_ENA */
    unsigned int curs;		/* MCO_ADV7162_CURS_DISABLE */
} mgras_mco_ramDAC7162_config;

typedef struct mgras_mco_ramDAC473_config {
    unsigned int cmd;
} mgras_mco_ramDAC473_config;

typedef struct mgras_mco_vc2_config {
    unsigned int flag;		/* see #defines above */
    unsigned int  w, h;		/* monitor resolution */
    unsigned short *ftab;	/* video timing frame table */
    unsigned int ftab_len;	/* length (in shorts) of *ftab */
    unsigned int ftab_addr;	/* VC2_VID_FRAME_TAB_ADDR  */
    unsigned short *ltab;	/* video timing line table */
    unsigned int ltab_len;	/* length (in shorts) of *ltab */
    unsigned int config;
} mgras_mco_vc2_config;

#define MGRAS_MCO_ALTERA_PROG_SIZE   63112

typedef struct mgras_mco_altera_config {
    unsigned int mode;
    unsigned char progbits[MGRAS_MCO_ALTERA_PROG_SIZE];
} mgras_mco_altera_config;

typedef struct mgras_mco_config {
    mgras_mco_altera_config       altera;
    mgras_mco_ramDAC7162_config   ramDAC7162[2];
    mgras_mco_ramDAC473_config    ramDAC473[2];
    mgras_mco_vc2_config          vc2;
    unsigned int                  cntrl1;
    unsigned int                  cntrl2;
    unsigned int                  cntrl3;
} mgras_mco_config, mgras_load_video_format_combination_args;

#if defined(_K64U64) && defined(_KERNEL)
typedef struct irix5_mgras_mco_vc2_config {
    app32_uint_t flag;		/* see #defines above */
    app32_uint_t w, h;		/* monitor resolution */
    app32_ptr_t  ftab;		/* video timing frame table */
    app32_uint_t ftab_len;	/* length (in shorts) of *ftab */
    app32_uint_t ftab_addr;	/* VC2_VID_FRAME_TAB_ADDR  */
    app32_ptr_t  ltab;		/* video timing line table */
    app32_uint_t ltab_len;	/* length (in shorts) of *ltab */
    app32_uint_t config;
} irix5_mgras_mco_vc2_config;

typedef struct irix5_mgras_load_video_format_combination_args {
    mgras_mco_altera_config       altera;
    mgras_mco_ramDAC7162_config   ramDAC7162[2];
    mgras_mco_ramDAC473_config    ramDAC473[2];
    irix5_mgras_mco_vc2_config    vc2;
    app32_uint_t                  cntrl1;
    app32_uint_t                  cntrl2;
    app32_uint_t                  cntrl3;
} irix5_mgras_load_video_format_combination_args;
#endif

typedef struct mgras_load_channel_gamma_args {
    unsigned int     channel;
    mgras_gammatable gamma;
} mgras_load_channel_gamma_args;

#endif /* __SYS_MGRAS_H__ */
