#ifndef _SYS_MGRASHW_H_
#define _SYS_MGRASHW_H_

/*
** Copyright 1993, Silicon Graphics, Inc.
** All Rights Reserved.
**
** This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
** the contents of this file may not be disclosed to third parties, copied or
** duplicated in any form, in whole or in part, without the prior written
** permission of Silicon Graphics, Inc.
**
** RESTRICTED RIGHTS LEGEND:
** Use, duplication or disclosure by the Government is subject to restrictions
** as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
** and Computer Software clause at DFARS 252.227-7013, and/or in similar or
** successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
** rights reserved under the Copyright Laws of the United States.
**
** $Header: /hosts/bonnie/proj/pacecar/isms/gfx/kern/sys/RCS/mgrashw.h,v 1.132 1996/05/20 22:51:32 jquilici Exp $
*/
#include <MGRAS/gl/re4te1reg.h>
#include <MGRAS/gl/re4reg.h>
#include <MGRAS/gl/te1tr1reg.h>
#include <MGRAS/gl/te1reg.h>
#include <MGRAS/gl/tr1reg.h>
#include <MGRAS/gl/pp1reg.h>
#include <MGRAS/gl/ge11cmds.h>

/**************************************************************************
 * Utility Macros and Constants
 **************************************************************************/

#ifdef BIT
#undef BIT
#endif
#define BIT(n)	            (0x1 << (n))
#define BITRANGE_LE(n)      (((n)>= 31) ? ~0 : \
			     (((n) < 0) ? 0 : (BIT((n)+1)) - 1))
#define BITRANGE(h,l)       (BITRANGE_LE(h) - BITRANGE_LE((l)-1))

#define EXTRACT_BIT(s,n)    (((s) & BIT(n)) >> (n))
#define EXTRACT_BITS(s,h,l) (((s) & BITRANGE(h,l)) >> (l))

/**************************************************************************
 * Board-Level Structures and Macros
 **************************************************************************/

/**** Board revision registers *****/

#define MGRAS_GET_BV1(mgbase, bv1)          { bv1 = EXTRACT_BITS(mgbase->bdvers.bdvers1, 31, 24); }
#define MGRAS_BV1_NUM_GE(bv1)               EXTRACT_BITS(bv1, 1, 0)
#define MGRAS_BV1_BOARD_REV(bv1)            EXTRACT_BITS(bv1, 4, 2)
#define MGRAS_BV1_BOARD_REV_FLOATING        0x7
#define MGRAS_BV1_PRODUCT_ID(bv1)           EXTRACT_BITS(bv1, 6, 5)
#define MGRAS_BV1_SWAPREADY(bv1)            EXTRACT_BIT(bv1, 7)

#define MGRAS_GET_BV0(mgbase, bv0)          { bv0 = EXTRACT_BITS(mgbase->bdvers.bdvers0, 31, 24); }
#define MGRAS_BV0_DECODE_NUM_TRAM(n)        (0x1 << (n))
#define MGRAS_BV0_NUM_RATRAM(bv0)           EXTRACT_BITS(bv0, 1, 0)
#define MGRAS_BV0_NUM_RBTRAM(bv0)           EXTRACT_BITS(bv0, 3, 2)
#define MGRAS_BV0_RB_REV(bv0)               EXTRACT_BITS(bv0, 6, 4)
#define MGRAS_BV0_RB_NOT_INSTALLED          0x7
#define MGRAS_BV0_RB_NOT_INSTALLED_BAD_BIT2 0x3

/**** Board Configuration registers ****/
#define MGRAS_SET_BC1(mgbase, bc1)          { mgbase->bdvers.bdvers0 = bc1 << 24; }
#define MGRAS_BC1_SWAPREADY_BIT             BIT(7)
#define MGRAS_BC1_VC3_RESET_BIT             BIT(5)
#define MGRAS_BC1_DH_MASTER_BIT             BIT(3)
#define MGRAS_BC1_FORCE_DCB_STAT_BIT        BIT(1)
#define MGRAS_BC1_CBLANK_HACK_BIT           BIT(0)

#define MGRAS_BC1_DEFAULT                   (MGRAS_BC1_VC3_RESET_BIT)
#define MGRAS_BC1_CBLANK_0		    MGRAS_BC1_DEFAULT
#define MGRAS_BC1_CBLANK_1		    (MGRAS_BC1_DEFAULT | \
					    MGRAS_BC1_CBLANK_HACK_BIT)

typedef struct {
	volatile unsigned char pad2_bv0[8];
	volatile __uint32_t    bdvers0;                /* 0x1fx62408 */
	volatile unsigned char pad2_bv1[0x88-0x8-4];
	volatile __uint32_t    bdvers1;		       /* 0x1fx62488 */
	volatile unsigned char pad2_end[0x400-0x88-4];
} _mgrasbd;

/**************************************************************************
 * RSS Structures and Macros
 **************************************************************************/

/***** RE bit register definitions ********/

/*  RE Config register */
#define CFG_DEST_RE	BIT(0)
#define CFG_YFLIP	BIT(3)
#define CFG_BRES_RND	BIT(4)
#define CFG_DEFAULT     0x0
#define CFG_BCAST       0x4

/* RE Fillmode register */
#define FM_POLY_STIP_ENAB	BIT(0)
#define FM_CHAR_STIP_EN		BIT(3)
#define FM_CHAR_STIP_OPAQUE	BIT(4)
#define FM_LINE_STIP_EN		BIT(5)
#define FM_LINE_STIP_OPAQUE	BIT(6)
#define FM_LINE_STIP_ADVANCE	BIT(7)
#define FM_LINE_SKIP_LAST	BIT(10)
#define FM_FAST_FILL		BIT(20)
#define FM_BLOCK_TYPE		BIT(22)
#define FM_SCISSOR_EN           BIT(28)

/* RE IR register */
#define IR_OPCODE		BIT(0)
#define IR_SETUP		BIT(4)

/* xfrcontrol */
#define XFR_DMA_START		BIT(0)
#define XFR_PIO_START		BIT(1)
#define XFR_DEVICE		BIT(2)
#define XFR_DIRECTION		BIT(3)
#define XFR_STATUS		BIT(4)

/* xfrmode */
#define PIXEL_COMP_TYPE		BIT(0)
#define PIXEL_FORMAT		BIT(4)
#define BEGIN_SKIP_BYTES	BIT(8)
#define BEGIN_SKIP_BITS		BIT(11)
#define STRIDE_SKIP_BYTES	BIT(14)
#define BITMAP_OPAQUE		BIT(23)

/* PP1 Fillmode register */
#define PP_FM_LOGICOP_ENAB	BIT(2)
#define PP_FM_PIXTYPE		BIT(8)
#define PP_FM_BUFCOUNT		BIT(11)
#define PP_FM_BUFSIZE		BIT(13)
#define PP_FM_DRAWBUF		BIT(14)
#define PP_FM_READBUF		BIT(21)
#define PP_FM_LOGICOP		BIT(26)

/* Texmode1 register */
#define BLEND_UNIT_COUNT 	BIT(16)

/* RE4 DEVICE address register map */
#define MGRAS_RE4_DEVICE_PP1_A             0x00000000
#define MGRAS_RE4_DEVICE_PP1_B             0x10000000
#define MGRAS_RE4_DEVICE_PP1_BCAST         0x20000000
#define MGRAS_RE4_DEVICE_PP1_A_READ        0x20000000
#define MGRAS_RE4_DEVICE_RSRV_0            0x30000000
#define MGRAS_RE4_DEVICE_RACCONTROL_A      0x40000000
#define MGRAS_RE4_DEVICE_RACCONTROL_B      0x50000000
#define MGRAS_RE4_DEVICE_RACCONTROL_BCAST  0x60000000
#define MGRAS_RE4_DEVICE_RACCONTROL_B_READ 0x60000000
#define MGRAS_RE4_DEVICE_RSRV_1            0x70000000
#define MGRAS_RE4_DEVICE_FOG_TABLE         0x80000000
#define MGRAS_RE4_DEVICE_TEXTURE_LUT       0x90000000
#define MGRAS_RE4_DEVICE_POYSTIPPLE_RAM    0xa0000000
#define MGRAS_RE4_DEVICE_LINE_AA_TABLE     0xb0000000
#define MGRAS_RE4_DEVICE_RAM_BCAST         0xc0000000
#define MGRAS_RE4_DEVICE_ADDR_FIFO_HI_READ 0xc0000000
#define MGRAS_RE4_DEVICE_ADDR_FIFO_LO_READ 0xd0000000
#define MGRAS_RE4_DEVICE_Z_FIFO_A_READ     0xe0000000
#define MGRAS_RE4_DEVICE_Z_FIFO_B_READ     0xf0000000

/* RE/PP RAMBUS Synchronization values */
#define MGRAS_RE4_RACCONTROL_SYNC          0x74140
#define MGRAS_RE4_RACCONTROL_DEFAULT       0x54140
#define MGRAS_RE4_RACCONTROL(ctl,ccv,enb,ld) ( (0x4100)              | \
					      (((ccv) & 0x3f) <<  0) | \
					      (((enb) & 0x1)  <<  6) | \
					      (((ld)  & 0x1)  <<  7) | \
					      ((ctl)          << 16))
#define MGRAS_PP1_PLL_LOCKTIME             5000        /* 5 ms (tunable) */
#define MGRAS_MITSUBISHI_MONITOR_DELAY	   100000

/* RE4-PP1 Port address map */
#define MGRAS_PP1_RDRAM_N_REG(n)           (n << 24)
#define MGRAS_PP1_RDRAM_0_REG              0x0000000
#define MGRAS_PP1_RDRAM_1_REG              0x1000000
#define MGRAS_PP1_RDRAM_2_REG              0x2000000
#define MGRAS_PP1_RDRAM_BCAST_REG          0x3000000
#define MGRAS_PP1_RDRAM_N_MEM(n)           ((n+4) << 24)
#define MGRAS_PP1_RDRAM_0_MEM              0x4000000
#define MGRAS_PP1_RDRAM_1_MEM              0x5000000
#define MGRAS_PP1_RDRAM_2_MEM              0x6000000
#define MGRAS_PP1_RDRAM_BCAST_MEM          0x7000000
#define MGRAS_PP1_RACCONTROL               0x8000000
#define MGRAS_PP1_RENDERING_REG            0xc000000
#define MGRAS_PP1_TAGWINDOW_REG            0xe000000

/* RDRAM registers */
#define MGRAS_RDRAM_DEVICETYPE             0x000000
#define MGRAS_RDRAM_DEVICEID               0x000004
#define MGRAS_RDRAM_DELAY                  0x000008
#define MGRAS_RDRAM_MODE                   0x00000c
#define MGRAS_RDRAM_REFINTERVAL            0x000010
#define MGRAS_RDRAM_REFROW                 0x000014
#define MGRAS_RDRAM_RASINTERVAL            0x000018
#define MGRAS_RDRAM_MININTERVAL            0x00001c
#define MGRAS_RDRAM_ADDRESSSELECT          0x000020
#define MGRAS_RDRAM_MANUFACTURER           0x000024
#define MGRAS_RDRAM_ROW                    0x000200

/* PP1/RDRAM Synchronization values */
#define MGRAS_450MHz                       0x450
#define MGRAS_500MHz                       0x500
#define MGRAS_PP1_BUSCLK                   0x1		 /* 1 us > 4.44 ns */
#define MGRAS_PP1_BUSFREQ                  MGRAS_450MHz	 /* 1/BUSCLK */
#define MGRAS_PP1_TPAUSE                   200		 /* 200 us */
#define MGRAS_PP1_TLOCKRESET               3		 /* 3 us */
#define MGRAS_PP1_TMODEARMAX               (288*MGRAS_PP1_BUSCLK)
#define MGRAS_PP1_TINTRESET                (66*MGRAS_PP1_BUSCLK)
#define MGRAS_PP1_RDRAMNUM                 6
#define MGRAS_PP1_NEC                      0x0005
#define MGRAS_PP1_CURRVAL450               0x25
#define MGRAS_PP1_CURRVAL500               0x25
#define MGRAS_PP1_RACCONTROL_SYNC          0x00244
#define MGRAS_PP1_PIXCMD_SYNC              0x0
#define MGRAS_PP1_FILLMODE_SYNC            0x6300
#define MGRAS_PP1_WINMODE_SYNC             0x0
#define MGRAS_PP1_TAGMODE_SYNC             0x0
#define MGRAS_PP1_DRBPTRS_SYNC             0x0
#define MGRAS_PP1_DRBSIZE_SYNC             0x31e
#define MGRAS_PP1_RETRYTIME_SYNC           0x653
#define MGRAS_RASINTERVAL_NEC              0x040a1c10
#define MGRAS_RASINTERVAL_TOSHIBA          0x04120c08

/* PP1 registers */
#define MGRAS_PP1_CLEAN(x)	(x & 0xf)			
#define MGRAS_PP1_DIRTY(x)	(x << 4)

#define PP1_REFENABLE		BIT(0)
#define PP1_STARTREF(start)	(start 	<< 1)			
#define PP1_ENDREF(end)		(end	<< 11)			
#define PP1_REFLINECNT(line)	(line 	<< 21) 
#define PP1_REFBURSTCNT(line)	(line 	<< 21) 
#define MGRAS_XMAP_REFRESH(x,y) (PP1_REFLINECNT(x) | PP1_REFBURSTCNT(y))

#define PP1_REFDEFAULT		(PP1_REFENABLE | PP1_STARTREF(0) | \
				PP1_ENDREF(1023) | MGRAS_XMAP_REFRESH(1,1))

/*
 * Textport RSS register defaults
 */
/* Textport PP1 fillmode  macros */
#define	TP_PP1_CICLAMP		BIT(25)
#define	TP_PP1_INITCI		((PP_FILLMODE_CI8 << 8) | PP_FM_BUFSIZE | \
				 PP_FM_DRAWBUF | TP_PP1_CICLAMP)

/* Textport RE4 fillmode macros */
	/* CIMODE_INIT sets the CIMODE and CICLAMP bits in RE4 fillmode reg*/
#define	TP_RE4_CIMODE_INIT	(BIT(15) | BIT(16))

#define	TP_RE4_CIBLOCK		(TP_RE4_CIMODE_INIT | \
				 (FILLMODE_DRAW_BLOCK << 22))
#define	TP_RE4_BLOCK_IR_SETUP	(IR_OP_BLOCK | IR_SETUP)

#define	TP_RE4_CICHAR		(TP_RE4_CIMODE_INIT | \
				 (FILLMODE_DRAW_BLOCK_CHAR << 22) | \
				 FM_CHAR_STIP_EN)

#define TP_RE4_POINT_IR_SETUP	IR_OP_GL_POINT

/* Textport Screen size defaults */
#define TP_SCREEN_X_MIN         0x0
#define TP_SCREEN_X_MAX         0x4ff
#define TP_SCREEN_Y_MIN         0x0
#define TP_SCREEN_Y_MAX         0x3ff

#ifdef MG0
#define mgras_re4Set(base, reg, data) \
    (*(__uint32_t *)((char *)&((base)->rss.s) + 4 * ((reg) & 0x3ff)) = data)

#define mgras_re4Set64(base, reg, data) \
    (*(__uint64_t *)((__uint32_t *)&((base)->rss.s) + ((reg) & 0x3ff)) = data)

#define mgras_re4SetExec(base, reg, data) \
    (*(__uint32_t *)((char *)&((base)->rss.s) + 4 * ((reg) & 0x3ff) + 0x1000) = data)

#define mgras_re4Get(base, reg, stored) \
    (stored = *(__uint32_t *)((char *)&((base)->rss.s) + 4 * ((reg) & 0x3ff)))
#else
#define mgras_re4Set(base,reg,data) 		{ base->rss.s.reg = data; }
#define mgras_re4SetExec(base,reg,data) 	{ base->rss.go.s.reg = data; }
#define mgras_re4Get(base,reg,stored)		{ stored = base->rss.s.reg; }
#define mgras_re4Send(base,reg,dataval) { \
    __uint32_t addr = \
	((__uint32_t *)&((mgras_hw *) 0)->rss.s.reg - \
	 (__uint32_t *)&((mgras_hw *) 0)->rss.s.tri_x0) \
	| (HQ_RSS_ADDR_FIRST | HQ_RSS_WRITE_REG); \
    mgras_WRITECFIFO64(base, MGRAS_HQ_CMDTOKEN(addr, 4), dataval); \
}
#define mgras_re4SendExec(base,reg,dataval) { \
    __uint32_t addr = (__uint32_t) \
	((__uint32_t *)&((mgras_hw *) 0)->rss.s.reg - \
	(__uint32_t *)&((mgras_hw *) 0)->rss.s.tri_x0) \
	| (HQ_RSS_ADDR_FIRST | HQ_RSS_WRITE_REG | HQ_RSS_LOAD_AND_XCT) ; \
    mgras_WRITECFIFO64(base, MGRAS_HQ_CMDTOKEN(addr, 4), dataval); \
}
#endif

#define  mgras_re4SetDevice(base, dev, data) {     \
  mgras_re4Set(base, device_addr, dev);      \
  mgras_re4Set(base, device_data, data);     \
}

#define  mgras_re4GetDevice(base, dev, data) {     \
  mgras_re4Set(base, device_addr, dev);      \
  mgras_re4Get(base, device_data, data);     \
}
					

/*
** single RSS registers
*/
typedef struct {
    /* double buffered shared RE/TE registers */
    volatile __uint32_t tri_x0;					/* 0x0000 */
    volatile __uint32_t tri_x1;					/* 0x0001 */
    volatile __uint32_t tri_x2;					/* 0x0002 */
    volatile __uint32_t tri_ymax;				/* 0x0003 */
    volatile __uint32_t tri_ymid;				/* 0x0004 */
    volatile __uint32_t tri_ymin;				/* 0x0005 */
    volatile __uint32_t tri_dxdy0;				/* 0x0006 */
    volatile __uint32_t tri_dxdy0_low;				/* 0x0007 */
    volatile __uint32_t tri_dxdy1_hi;				/* 0x0008 */
    volatile __uint32_t tri_dxdy1_low;				/* 0x0009 */
    volatile __uint32_t tri_dxdy2_hi;				/* 0x000a */
    volatile __uint32_t tri_dxdy2_low;				/* 0x000b */
    volatile __uint32_t gline_xstartf;				/* 0x000c */
    volatile __uint32_t gline_ystartf;				/* 0x000d */
    volatile __uint32_t gline_xendf;				/* 0x000e */
    volatile __uint32_t gline_yendf;				/* 0x000f */
    volatile __uint32_t gline_dx;				/* 0x0010 */
    volatile __uint32_t gline_dy;				/* 0x0011 */
    volatile __uint32_t gline_adjust;				/* 0x0012 */
    volatile iru ir;						/* 0x0013 */
    volatile __uint32_t _pad014[0x40-0x14];

    /* double buffered RE registers */
    volatile __uint32_t xline_xystarti;				/* 0x0040 */
    volatile __uint32_t xline_xyendi;				/* 0x0041 */
    volatile __uint32_t xline_inc1;				/* 0x0042 */
    volatile __uint32_t xline_inc2;				/* 0x0043 */
    volatile __uint32_t xline_error_oct;			/* 0x0044 */
    volatile iru ir_alias;					/* 0x0045 */
    volatile __uint32_t block_xystarti;				/* 0x0046 */
    volatile __uint32_t block_xyendi;				/* 0x0047 */
    volatile __uint32_t block_xymove;				/* 0x0048 */
    volatile __uint32_t block_xsaveoct;				/* 0x0049 */
    volatile __uint32_t _pad04a[0x5b-0x4a];
    volatile __uint32_t packed_color;				/* 0x005b */
    volatile __uint32_t red;					/* 0x005c */
    volatile __uint32_t green;					/* 0x005d */
    volatile __uint32_t blue;					/* 0x005e */
    volatile __uint32_t alpha;					/* 0x005f */
    volatile __uint32_t dre;					/* 0x0060 */
    volatile __uint32_t drx;					/* 0x0061 */
    volatile __uint32_t dge;					/* 0x0062 */
    volatile __uint32_t dgx;					/* 0x0063 */
    volatile __uint32_t dbe;					/* 0x0064 */
    volatile __uint32_t dbx;					/* 0x0065 */
    volatile __uint32_t dae;					/* 0x0066 */
    volatile __uint32_t dax;					/* 0x0067 */
    volatile __uint32_t z_hi;	       				/* 0x0068 */
    volatile __uint32_t z_low;					/* 0x0069 */
    volatile __uint32_t dzx_hi;					/* 0x006a */
    volatile __uint32_t dzx_low;				/* 0x006b */
    volatile __uint32_t dze_hi;					/* 0x006c */
    volatile __uint32_t dze_low;				/* 0x006d */
    volatile __uint32_t _pad06e[0x70-0x6e];
    volatile __uint32_t char_h;					/* 0x0070 */
    volatile __uint32_t char_l;					/* 0x0071 */
    volatile __uint32_t _pad072[0x80-0x72];

    /* double buffered TE registers */
    volatile __uint32_t sw_u;					/* 0x0080 */
    volatile __uint32_t sw_l;					/* 0x0081 */
    volatile __uint32_t tw_dtx_u;				/* 0x0082 */
    volatile __uint32_t tw_dtx_l;				/* 0x0083 */
    volatile __uint32_t wi_u;					/* 0x0084 */
    volatile __uint32_t wi_l;					/* 0x0085 */
    volatile __uint32_t dwie_u;					/* 0x0086 */
    volatile __uint32_t dwie_l;					/* 0x0087 */
    volatile __uint32_t dwix_u;					/* 0x0088 */
    volatile __uint32_t dwix_l;					/* 0x0089 */
    volatile __uint32_t dwiy_u;					/* 0x008a */
    volatile __uint32_t dwiy_l;					/* 0x008b */
    volatile txscaleu txscale;					/* 0x008c */
    volatile __uint32_t _pad08d[0xc0-0x8d];
    volatile __uint32_t dswe_u;					/* 0x00c0 */
    volatile __uint32_t dswe_l;					/* 0x00c1 */
    volatile __uint32_t dtwe_u;					/* 0x00c2 */
    volatile __uint32_t dtwe_l;					/* 0x00c3 */
    volatile __uint32_t dswx_u;					/* 0x00c4 */
    volatile __uint32_t dswx_l;					/* 0x00c5 */
    volatile __uint32_t dtwx_u;					/* 0x00c6 */
    volatile __uint32_t dtwx_l;					/* 0x00c7 */
    volatile __uint32_t dswy_u;					/* 0x00c8 */
    volatile __uint32_t dswy_l;					/* 0x00c9 */
    volatile __uint32_t dtwy_u;					/* 0x00ca */
    volatile __uint32_t dtwy_l;					/* 0x00cb */
    volatile __uint32_t _pad0cc[0x100-0x0cc];

    /* single buffered shared RE/TE registers */
    volatile __uint32_t xfrabort;				/* 0x0100 */
    volatile __uint32_t _pad101;
    volatile xfrcontrolu xfrcontrol;				/* 0x0102 */
    volatile __uint32_t _pad103[0x110-0x103];
    volatile fillmodeu fillmode;				/* 0x0110 */
    volatile texmode1u texmode1;				/* 0x0111 */
    volatile configu config;					/* 0x0112 */
    volatile __uint32_t scissorx;				/* 0x0113 */
    volatile __uint32_t scissory;				/* 0x0114 */
    volatile __uint32_t xywin;					/* 0x0115 */
    volatile __uint32_t _pad116[0x140-0x116];

    /* single buffered RE registers */
    volatile __uint32_t bkgrd_rg;				/* 0x0140 */
    volatile __uint32_t bkgrd_ba;				/* 0x0141 */
    volatile texenvrgu txenv_rg;				/* 0x0142 */
    volatile __uint32_t txenv_b;				/* 0x0143 */
    volatile fogrgu fog_rg;					/* 0x0144 */
    volatile __uint32_t fog_b;					/* 0x0145 */
    volatile glineconfigu glineconfig;				/* 0x0146 */
    volatile __uint32_t scrmsk1x;				/* 0x0147 */
    volatile __uint32_t scrmsk1y;				/* 0x0148 */
    volatile __uint32_t scrmsk2x;				/* 0x0149 */
    volatile __uint32_t scrmsk2y;				/* 0x014a */
    volatile __uint32_t scrmsk3x;				/* 0x014b */
    volatile __uint32_t scrmsk3y;				/* 0x014c */
    volatile __uint32_t scrmsk4x;				/* 0x014d */
    volatile __uint32_t scrmsk4y;				/* 0x014e */
    volatile winmodeu winmode;					/* 0x014f */
    volatile __uint32_t _pad150[0x153-0x150];
    volatile __uint32_t xfrsize;				/* 0x0153 */
    volatile __uint32_t xfrinitfactor;				/* 0x0154 */
    volatile __uint32_t xfrfactor;				/* 0x0155 */
    volatile __uint32_t xfrmasklow;				/* 0x0156 */
    volatile __uint32_t xfrmaskhigh;				/* 0x0157 */
    volatile __uint32_t xfrcounters;				/* 0x0158 */
    volatile xfrmodeu xfrmode;					/* 0x0159 */
    volatile __uint32_t lspat;					/* 0x015a */
    volatile lscrlu lscrl;					/* 0x015b */
    volatile __uint32_t device_addr;				/* 0x015c */
    volatile __uint32_t device_data;				/* 0x015d */
    volatile statusu status;					/* 0x015e */
    volatile __uint32_t re_togglecntx;				/* 0x015f */
    volatile __uint32_t PixCmd;					/* 0x0160 */
    volatile ppfillmodeu pp1fillmode;				/* 0x0161 */
    volatile __uint32_t ColorMaskMSBs;				/* 0x0162 */
    volatile __uint32_t ColorMaskLSBsA;				/* 0x0163 */
    volatile __uint32_t ColorMaskLSBsB;				/* 0x0164 */
    volatile ppblendfactoru BlendFactor;			/* 0x0165 */
    volatile ppstencilmodeu Stencilmode;			/* 0x0166 */
    volatile ppstencilmasku Stencilmask;			/* 0x0167 */
    volatile ppzmodeu Zmode;					/* 0x0168 */
    volatile ppafuncmodeu Afuncmode;				/* 0x0169 */
    volatile ppaccmodeu Accmode;				/* 0x016a */
    volatile ppblendcolorrgu BlendcolorRG;			/* 0x016b */
    volatile ppblendcolorbau BlendcolorBA;			/* 0x016c */
    volatile ppdrbpointersu DRBpointers;			/* 0x016d */
    volatile ppdrbsizeu DRBsize;				/* 0x016e */
    volatile pptagmodeu TAGmode;				/* 0x016f */
    volatile pptagdata_ru TAGdata_R;				/* 0x0170 */
    volatile pptagdata_gu TAGdata_G;				/* 0x0171 */
    volatile pptagdata_bu TAGdata_B;				/* 0x0172 */
    volatile pptagdata_au TAGdata_A;				/* 0x0173 */
    volatile __uint32_t TAGdata_Z;				/* 0x0174 */
    volatile pixcoordinateu PixCoordinate;			/* 0x0175 */
    volatile __uint32_t PIXcolorR;				/* 0x0176 */
    volatile __uint32_t PIXcolorG;				/* 0x0177 */
    volatile __uint32_t PIXcolorB;				/* 0x0178 */
    volatile __uint32_t PIXcolorA;				/* 0x0179 */
    volatile __uint32_t PIXcolorAcc;				/* 0x017a */
    volatile ppwinmodeu pp1winmode;				/* 0x017b */
    volatile __uint32_t _pad17c[0x017e-0x017c];
    volatile __uint32_t MainBufSel;				/* 0x017e */
    volatile __uint32_t OlayBufSel;				/* 0x017f */

    /* single buffered TE registers */
    volatile texmode2u texmode2;				/* 0x0180 */
    volatile txsizeu txsize;					/* 0x0181 */
    volatile txtileu txtile;					/* 0x0182 */
    volatile txlodu txlod;					/* 0x0183 */
    volatile txbcolor_rgu txbcolor_rg;				/* 0x0184 */
    volatile txbcolor_bau txbcolor_ba;				/* 0x0185 */
    volatile txbaseu txbase;					/* 0x0186 */
    volatile tram_cntrlu tram_cntrl;				/* 0x0187 */
    volatile max_pixel_rgu max_pixel_rg;			/* 0x0188 */
    volatile max_pixel_bau max_pixel_ba;			/* 0x0189 */
    volatile mddma_cntrlu mddma_cntrl;				/* 0x018a */
    volatile __uint32_t texrbuffer;				/* 0x018b */
    volatile txclampsizeu txclampsize;				/* 0x018c */
    volatile __uint32_t _pad18d[0x190-0x18d];
    volatile txaddru txaddr;					/* 0x0190 */
    volatile txmipmapu txmipmap;				/* 0x0191 */
    volatile txdetailu txdetail;				/* 0x0192 */
    volatile txborderu txborder;				/* 0x0193 */
    volatile detailscaleu detailscale;				/* 0x0194 */
    volatile __uint32_t te_togglecntx;				/* 0x0195 */
    volatile __uint32_t teversion;				/* 0x0196 */
    volatile __uint32_t _pad197[0x1a0-0x197];
    volatile __uint32_t tl_wbuffer;				/* 0x01a0 */
    volatile tl_modeu tl_mode;					/* 0x01a1 */
    volatile tl_specu tl_spec;					/* 0x01a2 */
    volatile tl_addru tl_addr;					/* 0x01a3 */
    volatile tl_mipmapu tl_mipmap;				/* 0x01a4 */
    volatile tl_borderu tl_border;				/* 0x01a5 */
    volatile __uint32_t tl_vidcntrl;				/* 0x01a6 */
    volatile tl_baseu tl_base;					/* 0x01a7 */
    volatile __uint32_t tl_vidabort;				/* 0x01a8 */
    volatile __uint32_t tl_s_size;				/* 0x01a9 */
    volatile __uint32_t tl_t_size;				/* 0x01aa */
    volatile __uint32_t tl_s_left;				/* 0x01ab */
    volatile __uint32_t tl_t_bottom;				/* 0x01ac */
    volatile __uint32_t _pad1ad[0x1af-0x1ad];
    volatile __uint32_t reserved;				/* 0x01af */
} rss_single;

/*
**  single (paired) RSS registers
*/
typedef struct {
    /* double buffered shared RE/TE registers */
    volatile __uint64_t tri_x1_x0;				/* 0x0200 */
    volatile __uint64_t tri_ymax_x2;				/* 0x0202 */
    volatile __uint64_t tri_ymin_ymid;				/* 0x0204 */
    volatile __uint64_t tri_dxdy0;				/* 0x0206 */
    volatile __uint64_t tri_dxdy1;				/* 0x0208 */
    volatile __uint64_t tri_dxdy2;				/* 0x020a */
    volatile __uint64_t gline_ystartf_xstartf;			/* 0x020c */
    volatile __uint64_t gline_yendf_xendf;			/* 0x020e */
    volatile __uint64_t gline_dy_dx;				/* 0x0210 */
    volatile __uint32_t _pad212[0x242-0x212];

    /* double buffered RE registers */
    volatile __uint64_t xline_inc2_inc1;			/* 0x0242 */
    volatile __uint64_t ir_xline_error_oct;			/* 0x0244 */
    volatile __uint32_t _pad246[0x25c-0x246];
    volatile __uint64_t green_red;				/* 0x025c */
    volatile __uint64_t alpha_blue;				/* 0x025e */
    volatile __uint64_t drx_dre;				/* 0x0260 */
    volatile __uint64_t dgx_dge;				/* 0x0262 */
    volatile __uint64_t dbx_dbe;				/* 0x0264 */
    volatile __uint64_t dax_dae;				/* 0x0266 */
} rss_paired;

/*
** extended RSS registers
*/
typedef struct {
    /* double buffered RE registers */
    volatile __uint64_t z;	       				/* 0x0268 */
    volatile __uint64_t dzx;					/* 0x026a */
    volatile __uint64_t dze;					/* 0x026c */
    volatile __uint32_t _pad26e[0x280-0x26e];

    /* double buffered TE registers */
    volatile __uint64_t sw;					/* 0x0280 */
    volatile __uint64_t tw_dtx;					/* 0x0282 */
    volatile __uint64_t wi;					/* 0x0284 */
    volatile __uint64_t dwie;					/* 0x0286 */
    volatile __uint64_t dwix;					/* 0x0288 */
    volatile __uint64_t dwiy;					/* 0x028a */
    volatile __uint32_t _pad28c[0x2c0-0x28c];
    volatile __uint64_t dswe;					/* 0x02c0 */
    volatile __uint64_t dtwe;					/* 0x02c2 */
    volatile __uint64_t dswx;					/* 0x02c4 */
    volatile __uint64_t dtwx;					/* 0x02c6 */
    volatile __uint64_t dswy;					/* 0x02c8 */
    volatile __uint64_t dtwy;					/* 0x02ca */
} rss_extended;

/*
** raster subsystem address map
*/
typedef struct {
	rss_single 	s;					/* 0x000 */
    	volatile unsigned char pad0[4*0x200-sizeof(rss_single)];
    	rss_paired 	p;					/* 0x200 */
    	rss_extended 	x;					/* 0x268 */

    	/* Load and Execute */
    	volatile unsigned char pad1[4*0x200-sizeof(rss_extended)-sizeof(rss_paired)];
	
	struct {
		rss_single 	s;				/* 0x400 */
    		volatile unsigned char pad2[4*0x200-sizeof(rss_single)];
    		rss_paired	p;				/* 0x600 */
    		rss_extended 	x;				/* 0x668 */
	} go;
} raster;

/* Needed by Xserver and 0-GE GL for shadowing RE/PP1 regs for ctx switching*/
typedef struct {
   rss_single  s;             
   rss_paired  p;             
   rss_extended   x;
   unsigned char xmap_index;
} raster_shadow;

/**************************************************************************
 * DCB Structures and Macros
 **************************************************************************/

/* DCB constants */

#define MGRAS_DCB_CRS(x)	((x & 0x7) << 7)
#define MGRAS_DCB_ENDATAPACK	BIT(6)
#define MGRAS_DCB_ENCRSINC	BIT(5)
#define MGRAS_DCB_DATAWIDTH_1	(0x1 << 3)
#define MGRAS_DCB_DATAWIDTH_2	(0x2 << 3)
#define MGRAS_DCB_DATAWIDTH_3	(0x3 << 3)
#define MGRAS_DCB_DATAWIDTH_4	(0x0 << 3)

#define MGRAS_DCB_ENSYNCACK	BIT(0)
#define MGRAS_DCB_ENASYNCACK	BIT(1)
#define MGRAS_DCB_CSWIDTH(x)	((x & 0x1f) << 2)		 
#define MGRAS_DCB_CSHOLD(x)	((x & 0x1f) << 7) 		 
#define MGRAS_DCB_CSSETUP(x)	((x & 0x1f) << 12)

#define MGRAS_DCBCTRL_CMAPALL	(MGRAS_DCB_CSWIDTH(1) | \
				 MGRAS_DCB_CSHOLD(1)  | \
				 MGRAS_DCB_CSSETUP(1))	      /* 0x01084 */

#define MGRAS_DCBCTRL_CMAP0	(MGRAS_DCB_CSWIDTH(1) | \
				 MGRAS_DCB_CSHOLD(1)  | \
				 MGRAS_DCB_CSSETUP(1))	      /* 0x01084 */

#define MGRAS_DCBCTRL_CMAP1	(MGRAS_DCB_CSWIDTH(1) | \
				 MGRAS_DCB_CSHOLD(1)  | \
				 MGRAS_DCB_CSSETUP(1))	      /* 0x01084 */

#define MGRAS_DCBCTRL_VC3	(MGRAS_DCB_CSWIDTH(2) | \
				 MGRAS_DCB_CSHOLD(2)  | \
				 MGRAS_DCB_CSSETUP(2) | \
				 MGRAS_DCB_ENASYNCACK)	      /* 0x0210A */

#define MGRAS_DCBCTRL_DAC	(MGRAS_DCB_CSWIDTH(1) | \
				 MGRAS_DCB_CSHOLD(1)  | \
				 MGRAS_DCB_CSSETUP(1))	      /* 0x01084 */

#define MGRAS_DCBCTRL_PP1	(MGRAS_DCB_CSWIDTH(3) | \
				 MGRAS_DCB_CSHOLD(3)  | \
				 MGRAS_DCB_CSSETUP(3))	      /* 0x0318c */

#define MGRAS_DCBCTRL_PP1_RESYNC (MGRAS_DCB_CSWIDTH(4) | \
				  MGRAS_DCB_CSHOLD(0)  | \
				  MGRAS_DCB_CSSETUP(1))	      /* see PP1 spec */

#define MGRAS_DCBCTRL_BDVERS	(MGRAS_DCB_CSWIDTH(4) | \
				 MGRAS_DCB_CSHOLD(1)  | \
				 MGRAS_DCB_CSSETUP(1))	      /* 0x01090 */

/* To access data, just use flexdata.data32, or flexdata.data16, etc. */

typedef union {
    struct {
    	volatile __uint64_t	data;
    } bydouble;
    struct {
        volatile __uint32_t	pad;
        volatile __uint32_t	data;
    } byword;
    struct {
        volatile unsigned short pad[3];
        volatile unsigned short data;
    } byshort;
    struct {
        volatile unsigned char pad[7];
        volatile unsigned char data;
    } bybyte;
} dcbdata64;


/**************************************************************************
 * XMAP Structures and Macros
 **************************************************************************/

#define MGRAS_XMAP_PP1_SELECT	(MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_XMAP_INDEX	(MGRAS_DCB_CRS(1) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_XMAP_CONFIG_BYTE	(MGRAS_DCB_CRS(2) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_XMAP_CONFIG	(MGRAS_DCB_CRS(2) | MGRAS_DCB_DATAWIDTH_4)
#define MGRAS_XMAP_BUF_SELECT	(MGRAS_DCB_CRS(3) | MGRAS_DCB_DATAWIDTH_4)
#define MGRAS_XMAP_MAIN_MODE	(MGRAS_DCB_CRS(4) | MGRAS_DCB_DATAWIDTH_4)
#define MGRAS_XMAP_OVERLAY_MODE	(MGRAS_DCB_CRS(5) | MGRAS_DCB_DATAWIDTH_4)
#define MGRAS_XMAP_DIB		(MGRAS_DCB_CRS(6) | MGRAS_DCB_DATAWIDTH_4)
#define MGRAS_XMAP_DIB_DW	(MGRAS_DCB_CRS(6) | MGRAS_DCB_DATAWIDTH_4 | \
				 MGRAS_DCB_ENDATAPACK)
#define MGRAS_XMAP_RE_RAC	(MGRAS_DCB_CRS(7) | MGRAS_DCB_DATAWIDTH_4)

#define MGRAS_XMAP_WRITEALLPP1	BIT(0)
#define MGRAS_XMAP_AUTOINC	BIT(19)

#define MGRAS_CURSOR_CMAP_MSB		0x73f
#define MGRAS_XMAP_CURSORMSB(x)		(x & 0x7ff)
#define MGRAS_XMAP_CONST_ALPHA(x) 	((x & 0xff) << 11)
#define MGRAS_XMAP_NUMRSS(x)		(((x-1) & 0x3) << 20)
#define MGRAS_XMAP_FIFO_BYPASS_BIT      BIT(22)
#define MGRAS_XMAP_DISABLE_CBLANK_BIT   BIT(14)

/* default XMAP register values */
#define MGRAS_XMAP_RETRYTIME_DEFAULT          0x653
#define MGRAS_XMAP_REFCONTROL_DEFAULT         0x13ff801
#define MGRAS_XMAP_DIBSKIP_DEFAULT            0x3fffff
#define MGRAS_XMAP_DIBPTRS_DEFAULT            0x1c0c8240
#define MGRAS_XMAP_DIBTOPSCAN_DEFAULT         0x3ff
#define MGRAS_XMAP_DIBCONTROL0_DEFAULT        0x0bb
#define MGRAS_XMAP_DIBCONTROL1_DEFAULT        0x1bb
#define MGRAS_XMAP_DIBCONTROL2_DEFAULT        0x124
#define MGRAS_XMAP_DIBCONTROL3_DEFAULT        0x140
#define MGRAS_XMAP_MAINBUF_DEFAULT            0x0
#define MGRAS_XMAP_OVERLAYBUF_DEFAULT         0x0

/* RAC initialization values */
#define MGRAS_XMAP_RACCONTROL_REPPSYNC        0x50214
#define MGRAS_XMAP_RACCONTROL_REPPSYNC_SHIFT  0x70214
#define MGRAS_XMAP_RACCONTROL_DEFAULT         0x10214
#define MGRAS_XMAP_RACCONTROL(ctl,ccv,enb,ld) ((0x14)    | \
					       (ld<<8)   | \
					       (enb<<9)  | \
					       (ccv<<10) | \
					       (ctl<<16) )

/* Register bits */
#define MGRAS_XMAP_DISPLAY_ENABLE_BIT         BIT(8)

/* XXX alpha and MSB */
#define MGRAS_XMAP_CONFIG_DEFAULT(numRE)	\
	(MGRAS_XMAP_CURSORMSB(MGRAS_CURSOR_CMAP_MSB) | \
	MGRAS_XMAP_CONST_ALPHA(0x00) | \
	MGRAS_XMAP_AUTOINC | MGRAS_XMAP_NUMRSS(numRE) )

/* index register values for Xmap  registers */

#define MGRAS_XMAP_CONFIG_ADDR		0x0	
#define MGRAS_XMAP_CONFIG_BYTE_ADDR	0x1	
#define MGRAS_XMAP_REV_ADDR		0x4	
#define MGRAS_XMAP_FIFOCAP_ADDR		0x8	

#define MGRAS_XMAP_MAINBUF_ADDR		0x0
#define MGRAS_XMAP_OVERLAYBUF_ADDR      0x4

#define MGRAS_XMAP_MAINMODE_ADDR        0x0

#define MGRAS_XMAP_OVERLAYMODE_ADDR     0x0

#define MGRAS_XMAP_DIBPTRS_ADDR         0x0
#define MGRAS_XMAP_DIBTOPSCAN_ADDR      0x4
#define MGRAS_XMAP_DIBSKIP_ADDR         0x8
#define MGRAS_XMAP_DIBCONTROL0_ADDR     0xc
#define MGRAS_XMAP_DIBCONTROL1_ADDR     0x10
#define MGRAS_XMAP_DIBCONTROL2_ADDR     0x14
#define MGRAS_XMAP_DIBCONTROL3_ADDR     0x18
#define MGRAS_XMAP_RETRYTIME_ADDR       0x1c
#define MGRAS_XMAP_REFCONTROL_ADDR      0x20

#define MGRAS_XMAP_RERACCONTROL_ADDR    0x0
#define MGRAS_XMAP_RERACDATA_ADDR       0x4
#define MGRAS_XMAP_RERACDATA_CCV_ADDR   0x6
#define MGRAS_XMAP_RERACSIGNATURE_ADDR  0x7

#define MGRAS_XMAP_8BIT_CI		0x0
#define MGRAS_XMAP_8BIT_RGBA		0x4
#define MGRAS_XMAP_12BIT_RGB		0x8
#define MGRAS_XMAP_OVL_DISABLE		0x0

#define MGRAS_XMAP_SELECT_PP1(n)    ((n) << 1)
#define MGRAS_XMAP_SELECT_BROADCAST (0x1)

#define mgras_xmapSetPP1Select(base,data)			\
	{ base->xmap.pp1_sel = data; }

#define mgras_xmapSetAddr(base,addr)				\
	{ base->xmap.index = addr; }

#define mgras_xmapSetConfigByte(base,data)	       		\
	{ base->xmap.config_byte = (data & 0xff); } 

#define mgras_xmapSetConfig(base,data)				\
	{ base->xmap.config = data; } 

#define mgras_xmapGetConfig(base,data)				\
	{ data = base->xmap.config; }

#define mgras_xmapSetBufSelect(base,data)			\
	{ base->xmap.buf_sel = data; }

#define mgras_xmapSetMainMode(base,wid,data)			\
	{ mgras_xmapSetAddr(base,wid << 2);			\
	base->xmap.main_mode = data; }
	
/* The number must be less than 32 */
#define mgras_xmapSetOvlMode(base,wid,data)			\
	{ mgras_xmapSetAddr(base,(wid & 0x1f) << 2);		\
	base->xmap.ovl_mode = data; }

#define mgras_xmapSetDIBdata(base,data)				\
	{ base->xmap.dib = data; }

#define mgras_xmapGetDIBdata(base,data)				\
	{ data = base->xmap.dib; }

#define mgras_xmapSetDIBdataDW(base,data)				\
	{ base->xmap.dib_dw = data; }

#define mgras_xmapSetRetryTime(base,data)				\
	{ base->xmap.dib = data; }

#define mgras_xmapSetRefControl(base,data)				\
	{ base->xmap.dib = data; }

#define mgras_xmapSetRE_RAC(base, data)			\
	{ base->xmap.re_rac = data; }

#define mgras_xmapGetRE_RAC(base, data)			\
	{ data = base->xmap.re_rac; }
	    
#define mgras_xmapFIFOWAIT(base)				\
	{							\
	mgras_xmapSetAddr(base,MGRAS_XMAP_FIFOCAP_ADDR);	\
	while (base->xmap.config == 0)				\
	   mgras_xmapSetAddr(base,MGRAS_XMAP_FIFOCAP_ADDR);	\
	}

typedef struct {
	unsigned char pad0[MGRAS_XMAP_PP1_SELECT];
	volatile unsigned char pp1_sel;
	unsigned char pad1[MGRAS_XMAP_INDEX - MGRAS_XMAP_PP1_SELECT - 1];
	volatile unsigned char index;
	unsigned char pad2a[MGRAS_XMAP_CONFIG - MGRAS_XMAP_INDEX - 1];
	volatile __uint32_t config;
	unsigned char pad2b[MGRAS_XMAP_CONFIG_BYTE - MGRAS_XMAP_CONFIG - 4];
	volatile unsigned char config_byte;
	unsigned char pad3[MGRAS_XMAP_BUF_SELECT - MGRAS_XMAP_CONFIG_BYTE - 1];
	volatile __uint32_t buf_sel;
	unsigned char pad4[MGRAS_XMAP_MAIN_MODE - MGRAS_XMAP_BUF_SELECT - 4];
	volatile __uint32_t main_mode;
	unsigned char pad5[MGRAS_XMAP_OVERLAY_MODE - MGRAS_XMAP_MAIN_MODE - 4];
	volatile __uint32_t ovl_mode;
	unsigned char pad6[MGRAS_XMAP_DIB - MGRAS_XMAP_OVERLAY_MODE - 4];
	volatile __uint32_t dib;
	unsigned char pad7[MGRAS_XMAP_DIB_DW - MGRAS_XMAP_DIB - 4];
	volatile __uint64_t  dib_dw;
	unsigned char pad8[MGRAS_XMAP_RE_RAC - MGRAS_XMAP_DIB_DW -  8];
	volatile __uint32_t re_rac;
	unsigned char pad9[0x400 - MGRAS_XMAP_RE_RAC - 4];
} _pp1xmap;

/**************************************************************************
 * CMAP Structures and Macros
 **************************************************************************/

/* CMAP CTRL registers */
#define MGRAS_CMAP_ADDRLO  	(MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_CMAP_ADDR 	(MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_2 | MGRAS_DCB_ENCRSINC) 
#define MGRAS_CMAP_ADDRHI  	(MGRAS_DCB_CRS(1) | MGRAS_DCB_DATAWIDTH_1) 
#define MGRAS_CMAP_PAL		(MGRAS_DCB_CRS(2) | MGRAS_DCB_DATAWIDTH_3)
#define MGRAS_CMAP_CMD		(MGRAS_DCB_CRS(3) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_CMAP_STATUS 	(MGRAS_DCB_CRS(4) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_CMAP_REV		(MGRAS_DCB_CRS(6) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_CMAP_RESERVED	(MGRAS_DCB_CRS(7) | MGRAS_DCB_DATAWIDTH_1)

/***** CMAP revision registers *****/
#define MGRAS_CMAP_CMAP_REV(cmapArev)        EXTRACT_BITS(cmapArev, 2, 0)
#define MGRAS_CMAP_MONITOR_TYPE(cmapArev)    EXTRACT_BITS(cmapArev, 7, 4)
#define MGRAS_CMAP_RA_REV(cmapBrev)          EXTRACT_BITS(cmapBrev, 6, 4)
#define MGRAS_CMAP_NO_VIDEO(cmapBrev)        EXTRACT_BIT(cmapBrev, 7)

/* CMAP Command Register */
#define MGRAS_CMAP_CMD_RESET_0        BIT(0)
#define MGRAS_CMAP_CMD_RESET_1        BIT(1)
#define MGRAS_CMAP_CMD_SET_0          BIT(2)
#define MGRAS_CMAP_CMD_SET_1          BIT(3)
#define MGRAS_CMAP_CMD_AFB            BIT(4)
#define MGRAS_CMAP_CMD_UNMASK_PIPES   (MGRAS_CMAP_CMD_RESET_0 | \
				       MGRAS_CMAP_CMD_RESET_1 )

/* 
 * CMAP  macros
 */
#define MGRAS_CBLANK_ENABLE	BIT(0)

/* For normal operation, wait for BFIFO to emtpy, then check CMAP0 status */
#define mgras_cmapFIFOWAIT(base)					\
	{								\
		mgras_BFIFOWAIT(base,HQ3_BFIFO_MAX);			\
		while (!(base->cmap0.status & 0x08));			\
	}

#define mgras_cmapSetAddr(base, reg) 					\
{ base->cmapall.addr = ((reg << 8) & 0xff00) | ((reg >> 8) & 0xff); }
#define mgras_cmapSetRGB(base,r,g,b) 					\
{ base->cmapall.pal = (r << 24 ) | ( g << 16 ) | (b << 8 ); }
#define mgras_cmapSetCmd(base,data) 					\
{ base->cmapall.cmd = data; }
#define mgras_cmapGetCmd(base,data) 					\
{ data = (which ? base->cmap1.cmd : base->cmap0.cmd); } 
#define mgras_cmapGetStatus(base,status) 			        \
{ status = (which ? base->cmap1.status : base->cmap0.status); } 
#define mgras_cmapGetRev(base,data,which) 				\
{ data = (which ? base->cmap1.rev : base->cmap0.rev); } 

#define mgras_cmapGetRGB(base, which, r, g, b )				\
	 {								\
        __uint32_t _temp;						\
 	_temp = (which ? base->cmap1.pal : base->cmap0.pal);		\
 	r = (_temp & 0xff000000) >> 24;					\
        g = (_temp & 0x00ff0000) >> 16;					\
        b = (_temp & 0x0000ff00) >> 8;					\
        }

#define mgras_toggleCBlank(base, OnOff)                                 \
{ MGRAS_SET_BC1(base, (OnOff ? MGRAS_BC1_CBLANK_1 : MGRAS_BC1_CBLANK_0)); }
   

typedef struct {
	unsigned char pad0[MGRAS_CMAP_ADDRLO];
	volatile unsigned char addrlo;
	unsigned char pad1[MGRAS_CMAP_ADDR - MGRAS_CMAP_ADDRLO-1];
	volatile unsigned short addr;
	unsigned char pad2[MGRAS_CMAP_ADDRHI - MGRAS_CMAP_ADDR-2];
	volatile unsigned char addrhi;
	unsigned char pad3[MGRAS_CMAP_PAL - MGRAS_CMAP_ADDRHI-1];
	volatile __uint32_t pal;
	unsigned char pad4[MGRAS_CMAP_CMD - MGRAS_CMAP_PAL-4];
	volatile unsigned char cmd;
	unsigned char pad5[MGRAS_CMAP_STATUS  - MGRAS_CMAP_CMD-1];
	volatile unsigned char status;
	unsigned char pad6[MGRAS_CMAP_REV - MGRAS_CMAP_STATUS-1];
	volatile unsigned char rev;
	unsigned char pad7[MGRAS_CMAP_RESERVED - MGRAS_CMAP_REV-1];
	volatile unsigned char reserved;
	unsigned char pad8[0x400 - MGRAS_CMAP_RESERVED-1];
} _MScmap;

/**************************************************************************
 * DAC Structures and Macros
 **************************************************************************/

#define MGRAS_DAC_ADDR 	( MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_2)
#define MGRAS_DAC_PAL  	( MGRAS_DCB_CRS(1) | MGRAS_DCB_DATAWIDTH_2)
#define MGRAS_DAC_CMD  	( MGRAS_DCB_CRS(2) | MGRAS_DCB_DATAWIDTH_2)
#define MGRAS_DAC_MODE	( MGRAS_DCB_CRS(3) | MGRAS_DCB_DATAWIDTH_1)

#define MGRAS_DAC_PLL_LOCKTIME          (1000) /* 1 ms */

/* Address of registers in the Control Register */ 
#define MGRAS_DAC_ID_ADDR		0x3
#define MGRAS_DAC_PIXMASK_ADDR		0x4
#define MGRAS_DAC_CMD1_ADDR		0x5
#define MGRAS_DAC_CMD2_ADDR		0x6
#define MGRAS_DAC_CMD3_ADDR		0x7
#define MGRAS_DAC_CMD4_ADDR		0x8
#define MGRAS_DAC_PLL_CTL_ADDR		0x9
#define MGRAS_DAC_STATUS_ADDR		0xa
#define MGRAS_DAC_REV_ADDR		0xb
#define MGRAS_DAC_PLL_R_ADDR		0xc
#define MGRAS_DAC_CMD5_ADDR             0xd
#define MGRAS_DAC_PLL_V_ADDR		0xf
#define MGRAS_DAC_CURS_ADDR		0x204

#define MGRAS_DAC_MODE_INIT		0x03
#define MGRAS_DAC_PIXMASK_INIT		0xff
#define MGRAS_DAC_PIXMASK_BLANK	        0x00
#define MGRAS_DAC_CMD1_INIT		0x89
#define MGRAS_DAC_CMD2_INIT		0xec
#define MGRAS_DAC_CMD3_INIT		0xc0
#define MGRAS_DAC_CMD4_INIT		0x06
#define MGRAS_DAC_CURS_DISABLE		0x01

/* Mode register */
#define MGRAS_DAC_MODE_RESET_BIT        BIT(0)

#define mgras_dacSetAddr(base,address)				\
	{							\
	base->dac.addr.bybyte.b0 = (address & 0xff);			\
	}

#define mgras_dacSetAddr16(base,address)				\
	{							\
        base->dac.addr.byshort =  ((address << 8) & 0xff00) | ((address >> 8) & 0xff);		\
	}
	
#define mgras_dacSetRGB(base,r,g,b)				\
	{							\
		base->dac.pal =	(((r << 6) & 0xff00) | (r & 0x3)); \
		base->dac.pal =	(((g << 6) & 0xff00) | (g & 0x3));	\
		base->dac.pal =	(((b << 6) & 0xff00) | (b & 0x3));	\
	}

#define mgras_dacGetRGB(base,r,g,b)				\
	{							\
		unsigned short _tmp, _low, _high;		\
		_tmp = base->dac.pal;				\
		_low = (_tmp & 0xff);				\
		_high = (_tmp >> 8) & 0xff;			\
		r = (_high << 2) | (_low & 0x3);		\
		_tmp = base->dac.pal;				\
		_low = (_tmp & 0xff);				\
		_high = (_tmp >> 8) & 0xff;			\
		g = (_high << 2) | (_low & 0x3);		\
		_tmp = base->dac.pal;				\
		_low = (_tmp & 0xff);				\
		_high = (_tmp >> 8) & 0xff;			\
		b = (_high << 2) | (_low & 0x3);		\
	}
#define mgras_dacSetMode(base,data)				\
	base->dac.mode = data 

#define mgras_dacSetAddrCmd(base, addr, data)			\
	{							\
	mgras_dacSetAddr(base,addr);				\
	base->dac.cmd.bybyte.b1 = data;				\
	}

#define mgras_dacSetAddrCmd16(base, addr, data)			\
	{							\
	mgras_dacSetAddr16(base,addr);				\
	base->dac.cmd.bybyte.b1 = data;				\
	}

#define mgras_dacSetAddr16Cmd16(base, addr, data)		\
	{							\
	mgras_dacSetAddr16(base,addr);				\
	base->dac.cmd.byshort = data;				\
	}

typedef struct {
	unsigned char pad0[MGRAS_DAC_ADDR];
	union {
	  volatile short byshort;
	  struct {
	    volatile unsigned char b1;
	    volatile unsigned char b0;
	  } bybyte;
	} addr;
	unsigned char pad1[MGRAS_DAC_PAL - MGRAS_DAC_ADDR - 2];
	volatile unsigned short pal;
	unsigned char pad2[MGRAS_DAC_CMD - MGRAS_DAC_PAL - 2];
	union {
	  volatile unsigned short byshort;
	  struct {
	    volatile unsigned char b1;
	    volatile unsigned char b0;
	  } bybyte;
	} cmd;
	unsigned char pad3[MGRAS_DAC_MODE - MGRAS_DAC_CMD - 2];
	volatile unsigned char mode;
	unsigned char pad4[0x400 - MGRAS_DAC_MODE - 1];
} _ADVdac;

/**************************************************************************
 * VC3 Structures and Macros
 **************************************************************************/

#define MGRAS_VC3_INDEX 	(MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_1)  

#define MGRAS_VC3_INDEXDATA 	(MGRAS_DCB_CRS(0) | MGRAS_DCB_ENCRSINC | \
				 MGRAS_DCB_DATAWIDTH_3)	

#define MGRAS_VC3_DATA 		(MGRAS_DCB_CRS(1) | MGRAS_DCB_ENCRSINC | \
				 MGRAS_DCB_DATAWIDTH_2)

#define MGRAS_VC3_RAM_64BIT	(MGRAS_DCB_CRS(3) | MGRAS_DCB_DATAWIDTH_4)
#define MGRAS_VC3_RAM	 	(MGRAS_DCB_CRS(3) | MGRAS_DCB_DATAWIDTH_2)


/* 
 * VC3 constants
 */
/* Index values */
#define VC3_VID_EP		0	
#define VC3_CURS_EP		1
#define VC3_CURS_X_LOC		2
#define VC3_CURS_Y_LOC		3
#define VC3_DID_EP1		4
#define VC3_DID_EP2		5
#define VC3_SCAN_LENGTH		6
#define VC3_RAM_ADDR		7
#define VC3_VERT_LINE_COUNT 	0xb
#define VC3_CUR_CURSOR_X	0xe
#define VC3_DID_FRAMETAB1 	0x10
#define VC3_DID_LINETAB1  	0x11
#define VC3_DID_FRAMETAB2 	0x12
#define VC3_DID_LINETAB2  	0x13
#define VC3_CURS_CONTROL	0x1d	
#define VC3_DISPLAY_CONTROL 	0x1e
#define VC3_CONFIG		0x1f

/* VC3 Config register bits */
#define VC3_UNRESET	BIT(0)
#define VC3_SLOW_CLOCK	BIT(1)
#define VC3_CURS_ERR	BIT(2)
#define VC3_DID_ERR1	BIT(3)
#define VC3_DID_ERR2	BIT(4)
#define VC3_VTG_ERR	BIT(5)
#define VC3_REV_SHIFT	6
#define VC3_TEMP_HIBYTE_SHIFT		
#define VC3_TEMP_HIBYTE_MASK (0xff << VC3_TEMP_HIBYTE_SHIFT)					

/* VC3 Display bits */
#define VC3_VINTR	BIT(0)
#define VC3_BLACKOUT	BIT(1)
#define VC3_VIDEO_ENAB	BIT(2)
#define VC3_DID_ENAB1	BIT(3)
#define VC3_DID_ENAB2	BIT(4)
#define VC3_GENSYNC	BIT(5)
#define VC3_GENLOCK 	BIT(6)
#define VC3_INTERLACED	BIT(7)
#define VC3_FIELD_ONLY	BIT(8)
#define VC3_ODD_FIELD	BIT(9)
#define VC3_ODD_FIELD_TIMING BIT(10)

/* VC3 Cursor bits */
#define VC3_ENA_CURS_FUNCTION 		BIT(0)
#define VC3_ENA_CURS_DISPLAY  		BIT(1)
#define VC3_CURS_CROSSHAIR		BIT(2)
#define VC3_CURS_GLYPH_64		BIT(3)

#define VC3_CURS_XOFF			31
#define VC3_CURS_YOFF			31

#define mgras_vc3SetIndex(base,addr) 		base->vc3.index = addr 

#define mgras_vc3SetReg(base,reg ,data16) 				\
	base->vc3.indexdata = (reg << 24) | ((data16 & 0xffff) << 8)

#define mgras_vc3GetReg(base,reg,data16) 				\
	mgras_vc3SetIndex(base,reg);					\
	data16 = base->vc3.data

#define mgras_vc3SetRam(base,data)		base->vc3.ram = data;
#define mgras_vc3GetRam(base,data)		data = base->vc3.ram;

/* VC3 Memory Map
 * 16-bit short addresses
 */ 

#define VC3_VID_LINETAB_ADDR		0x0   /* 768 shorts for line table  */
#define VC3_VID_FRAMETAB_ADDR		0x380 /* 128 shorts for frame table */
#define VC3_CURS_GLYPH_ADDR		0x400 /* 64x64 glyph, needs to be on 256-word bndry */
 
#define VC3_MAX_DID_FRAME_LEN_SHORT	1086	/* HDTV (1085+1) shorts - 
						 * need end-of-frame entry
					   	 */
#define VC3_DID_FRAMETAB1_BASE		0x500
#define VC3_DID_FRAMETAB2_BASE		(VC3_DID_FRAMETAB1_BASE + \
					 VC3_MAX_DID_FRAME_LEN_SHORT)	
#define VC3_DID_LINETAB1_BASE		(VC3_DID_FRAMETAB2_BASE + \
					 VC3_MAX_DID_FRAME_LEN_SHORT)  
#define VC3_DID_LINETAB1_END		(VC3_DID_LINETAB1_BASE + \
					 ((0x8000 - VC3_DID_LINETAB1_BASE)/2))
/*7K entries for each buffer*/	
#define VC3_DID_LINETAB2_BASE		VC3_DID_LINETAB1_END
#define VC3_DID_LINETAB2_END		0x8000

typedef struct {
	unsigned char pad0[MGRAS_VC3_INDEX];
	volatile unsigned char index;
	unsigned char pad1[MGRAS_VC3_INDEXDATA - MGRAS_VC3_INDEX - 1];
	volatile __uint32_t indexdata;
	unsigned char pad2[MGRAS_VC3_DATA - MGRAS_VC3_INDEXDATA - 4];
	volatile unsigned short data;
	unsigned char pad3[MGRAS_VC3_RAM_64BIT - MGRAS_VC3_DATA - 2];
	volatile  __uint64_t ram64;
	unsigned char pad4[MGRAS_VC3_RAM - MGRAS_VC3_RAM_64BIT - 8];
	volatile unsigned short ram;
	unsigned char pad5[0x400 - MGRAS_VC3_RAM - 2];
} _vc3;

/**************************************************************************
 * HQ3 Structures and Macros
 **************************************************************************/
#define HQ_bit_field(lower,n)           (((1 << (n)) - 1) << (lower))

#define HQ3_ID_MASK		              0xffffff00
#define HQ3_ID			              0x48513300
#define HQ3_REV(data)		              ((data & 0xf0) >> 4)	

/* GIO ID register */
#define HQ3_GIO_ID_PRODUCT_ID(gio_id)         EXTRACT_BITS(gio_id,  6,  0)
#define HQ3_GIO_ID_PRODUCT_ID_W(gio_id)       EXTRACT_BIT(gio_id,   7)
#define HQ3_GIO_ID_PRODUCT_REV(gio_id)        EXTRACT_BITS(gio_id, 15,  8)
#define HQ3_GIO_ID_GIO_INTERFACE_SIZE(gio_id) EXTRACT_BIT(gio_id,  16)
#define HQ3_GIO_ID_GIO_ROM(gio_id)            EXTRACT_BIT(gio_id,  17)
#define HQ3_GIO_ID_MANUFACTURER_CODE(gio_id)  EXTRACT_BITS(gio_id, 31, 18)

/* hq_config register */
#define mgras_hq3SetHQConfig(mgbase, data)    { mgbase->hq_config = data; }
#define mgras_hq3GetHQConfig(mgbase, data)    { data = mgbase->hq_config; }
#define HQ3_HQ_CONFIG_DEFAULT		((63 << 9) | 0x008d)	/* num-ge must be 1 */
#define HQ3_NUMGE_MASK	(BIT(0) | BIT(1))
#define HQ3_UCODE_STALL	BIT(3)
#define HQ_HQCFG_UCODE_STALL_BIT 	(1 << 3)
#define HQ3_CONFIG_GEDIAG_MASK	0x30
#define HQ3_DIAG_READ_GE0	(2 << 4)
#define HQ3_DIAG_READ_GE1	(3 << 4)
#define HQ_HQCFG_GIO_IS_DIAG_MASTER_BIT (1 <<  7)
#define HQ_HQCFG_CTX_HOST_SRC_BIT       (1 <<  8)

/* gio_config register */
#define mgras_hq3SetGIOConfig(mgbase, data)   { mgbase->gio_config = data; }
#define mgras_hq3GetGIOConfig(mgbase, data)   { data = mgbase->gio_config; }
#define HQ3_GIO_CONFIG_DEFAULT	              0x765b900f
#define HQ_GIOCFG_HQBUS_BYPASS_BIT      (1 <<  6)

/* flag register */
#define HQ_FLAG_GE_FLAGS_BITS           HQ_bit_field(0,4)
#define HQ_FLAG_GE_SYSFLAGS_BITS        HQ_bit_field(4,4)
# define GE_FLAG_OUTPUT_FLUSHED		0x10
# define GE_FLAG_HQ3_CFIFO_NONEMPTY	0x20
# define GE_FLAG_RESTORE_DONE		0x40
#define HQ_FLAG_CTX_DONE_BIT            (1 << 8)
#define HQ_FLAG_UCODE_ERROR_BIT         (1 << 9)
#define HQ_FLAG_CP_FLAGS_BITS           HQ_bit_field(10,2)
#define HQ_FLAG_TLB_FAULT_BIT           (1 << 12)
#define HQ_FLAG_TLB_PTR_FAULT_BIT       (1 << 13)
#define HQ_FLAG_PRIV_VIOLATION_BIT      (1 << 14)
#define HQ_FLAG_DMA_FAULT_BIT           (1 << 15)
#define HQ_FLAG_CD_FLAG_BIT             (1 << 16)
#define HQ_FLAG_GE_READ_BIT             (1 << 17)
#define HQ_FLAG_GE_DIAG_READ_BIT        (1 << 18)
#define HQ_FLAG_CTX_SVMB_BIT            (1 << 19)
#define HQ_FLAG_HOST_FLAGS_BITS         HQ_bit_field(20,3)
#define HQ_FLAG_CP_IN_BITS              HQ_bit_field(23,2)

/* formatter/converter mode register values */
#define HQ_FC_NIBBLE			(0 << 0)
#define HQ_FC_BYTE			(1 << 0)
#define HQ_FC_SHORT			(2 << 0)
#define HQ_FC_WORD			(3 << 0)
#define HQ_FC_1_COMP_PER_PIXEL		(0 << 2)
#define HQ_FC_2_COMP_PER_PIXEL		(1 << 2)
#define HQ_FC_3_COMP_PER_PIXEL		(2 << 2)
#define HQ_FC_4_COMP_PER_PIXEL		(3 << 2)
#define HQ_FC_NIBBLE_EXPANSION_NONE	(2 << 8)
#define HQ_FC_ZERO_ALIGN_MASK		(1 << 10)
#define HQ_FC_INVALIDATE_TRAILING	(1 << 11)
#define HQ_FC_CANONICAL			(HQ_FC_NIBBLE_EXPANSION_NONE | \
					 HQ_FC_ZERO_ALIGN_MASK | \
					 HQ_FC_INVALIDATE_TRAILING)

/* bits in busy_dma register */
#define HQ_BUSY_HQ_SYNC_BIT             (1 << 0)
#define HQ_BUSY_HIF_BIT                 (1 << 1)
#define HQ_BUSY_CFIFO_BIT               (1 << 2)
#define HQ_BUSY_CDEC_BIT                (1 << 3)
#define HQ_BUSY_REIF_BIT                (1 << 4)
#define HQ_BUSY_HAG_BIT                 (1 << 5)
#define HQ_BUSY_FMT_BIT                 (1 << 6)
#define HQ_BUSY_CNV_BIT                 (1 << 7)
#define HQ_BUSY_CP_BIT                  (1 << 8)
#define HQ_BUSY_MSTR_DMA_BIT            (1 << 9)
#define HQ_BUSY_CDEC_DMA_BIT            (1 << 10)
#define HQ_BUSY_REIF_DMA_BIT            (1 << 11)
#define HQ_BUSY_HAG_DMA_BIT             (1 << 12)
#define HQ_BUSY_CP_DMA_BIT              (1 << 13)

/* bits in status register */
#define HQ_STATUS_REBUS_SYNC_BIT        (1 << 0)
#define HQ_STATUS_RSS_IDLE_BIT          (1 << 1)
#define HQ_STATUS_VERT_BLANK_BIT        (1 << 2)
#define HQ_STATUS_CFIFO_HW_BIT          (1 << 3)
#define HQ_STATUS_CFIFO_LW_BIT          (1 << 4)
#define HQ_STATUS_DFIFO_HW_BIT          (1 << 5)
#define HQ_STATUS_DFIFO_LW_BIT          (1 << 6)
#define HQ_STATUS_BFIFO_HW_BIT          (1 << 7)
#define HQ_STATUS_BFIFO_LW_BIT          (1 << 8)
#define HQ_STATUS_DMA_ACTIVE_BIT        (1 << 9)
#define HQ_STATUS_DFIFO_SELECT_BIT      (1 << 10)
#define HQ_STATUS_CTXSW_ACTIVE_BIT      (1 << 11)

/* bits in the dma_type register */
#define HQ_FB_TO_FB_MODE		1
#define HQ_REIF_DMA_DST_MASK		(3 << 1)
#define HQ_REIF_DMA_DST_SHIFT		1
#define HQ_REIF_DMA_DST_CP		(0 << 1)
#define HQ_REIF_DMA_DST_HOST		(1 << 1)
#define HQ_REIF_DMA_DST_RE		(2 << 1)
#define HQ_REIF_FINISH_DMA_ON_CTX_SWTCH	(1 << 3)

/*
 * in the intr_enab register,
 * lower 23 bits map to lower 23 bits of flags register,
 * upper 9 bits map to the lower 9 bits of status register.
 */
#define HQ_INTENB_FLAG_REG_MASK		(HQ_bit_field(0, 23))
#define HQ_INTENB_REBUS_SYNC_BIT	(HQ_STATUS_REBUS_SYNC_BIT << 23)
#define HQ_INTENB_RSS_IDLE_BIT		(HQ_STATUS_RSS_IDLE_BIT << 23)
#define HQ_INTENB_VERT_BLANK_BIT	(HQ_STATUS_VERT_BLANK_BIT << 23)
#define HQ_INTENB_CFIFO_HW_BIT		(HQ_STATUS_CFIFO_HW_BIT << 23)
#define HQ_INTENB_CFIFO_LW_BIT		(HQ_STATUS_CFIFO_LW_BIT << 23)
#define HQ_INTENB_DFIFO_HW_BIT		(HQ_STATUS_DFIFO_HW_BIT << 23)
#define HQ_INTENB_DFIFO_LW_BIT		(HQ_STATUS_DFIFO_LW_BIT << 23)
#define HQ_INTENB_BFIFO_HW_BIT		(HQ_STATUS_BFIFO_HW_BIT << 23)
#define HQ_INTENB_BFIFO_LW_BIT		(HQ_STATUS_BFIFO_LW_BIT << 23)
#define HQ_FLOW_CONTROL_INTENB		(HQ_INTENB_CFIFO_HW_BIT | \
					 HQ_INTENB_CFIFO_LW_BIT | \
					 HQ_INTENB_DFIFO_HW_BIT | \
					 HQ_INTENB_DFIFO_LW_BIT | \
					 HQ_INTENB_BFIFO_HW_BIT | \
					 HQ_INTENB_BFIFO_LW_BIT)

/* hq command word construction macro */
#define MGRAS_HQ_CMDTOKEN(cmd, bytes) \
	(((((unsigned int)bytes) & 0x00ff) <<  0) | \
	 ((((unsigned int)cmd)   & 0x1fff) <<  8) \
	 )

/* defines for the address field (bits 8-20) of the hq3 command word */
#define HQ_RSS_ADDR_FIRST		0x1000
#define HQ_RSS_EXTENDED_REG		0x0200
#define HQ_RSS_LOAD_AND_XCT		0x0400
#define HQ_RSS_WRITE_REG		0x0800
#define HQ_RSS_ADDR_LAST		0x1fff

/* hq command tokens */
#define HQ_HAG_IM_PG_LIST_0_WRITE	 MGRAS_HQ_CMDTOKEN(0x800, 4)
#define HQ_HAG_IM_PG_WIDTH_WRITE	 MGRAS_HQ_CMDTOKEN(0x804, 4)
#define HQ_HAG_IM_ROW_OFFSET_WRITE	 MGRAS_HQ_CMDTOKEN(0x805, 4)
#define HQ_HAG_IM_ROW_START_ADDR_WRITE	 MGRAS_HQ_CMDTOKEN(0x806, 4)
#define HQ_HAG_IM_LINE_CNT_WRITE	 MGRAS_HQ_CMDTOKEN(0x807, 4)
#define HQ_HAG_IM_FIRST_SPAN_WIDTH_WRITE MGRAS_HQ_CMDTOKEN(0x808, 4)
#define HQ_HAG_IM_LAST_SPAN_WIDTH_WRITE	 MGRAS_HQ_CMDTOKEN(0x809, 4)
#define HQ_HAG_IM_DMA_CTRL_WRITE	 MGRAS_HQ_CMDTOKEN(0x80b, 4)
#define HQ_REIF_SCAN_WIDTH_WRITE	 MGRAS_HQ_CMDTOKEN(0xa02, 4)
#define HQ_REIF_SCAN_SIZE_WRITE		 MGRAS_HQ_CMDTOKEN(0xa03, 4)
#define HQ_REIF_SCAN_MODE_WRITE		 MGRAS_HQ_CMDTOKEN(0xa04, 4)
#define HQ_REIF_SCAN_CTRL_WRITE		 MGRAS_HQ_CMDTOKEN(0xa05, 4)
#define HQ_REIF_DMA_TYPE_WRITE		 MGRAS_HQ_CMDTOKEN(0xa06, 4)
#define HQ_REIF_CLEAR_REBUS_SYNC	 MGRAS_HQ_CMDTOKEN(0xa07, 4)
#define HQ_FC_PIXEL_FORMAT_MODE_WRITE	 MGRAS_HQ_CMDTOKEN(0xc00, 4)
#define HQ_SET_CD_FLAG			 MGRAS_HQ_CMDTOKEN(0xe04, 0)

/* hq3 tokens */
#define HQ_CP_GE_SET_REBUS_SYNC		 MGRAS_HQ_CMDTOKEN(CP_GE_SET_REBUS_SYNC, 0)
#define HQ_CP_GE_NOSAVE_NEXT_SWITCH	 MGRAS_HQ_CMDTOKEN(CP_GE_NOSAVE_NEXT_SWITCH, 0)
#define HQ_CP_GE_SAVE_IDLE_HQ_STATE	 MGRAS_HQ_CMDTOKEN(CP_GE_SAVE_IDLE_HQ_STATE, 4)

/* HQ ucode stuff */
#define HQ3_UCODERAM_LEN	              (0x5000/4)
#define mgras_hq3GetPC(mgbase, data)          { data = mgbase->hqpc; }

#define mgras_QueryIntr(base,config) 	config = base->intr_enab_set
#define mgras_cfifoGetLevel(base,level) level = base->fifo_status & 0x7f
#define mgras_bfifoGetLevel(base,level) level = base->gio_status & 0x1f

/* other hq3 parameters */
#define HQ3_CFIFO_MAX		64	
#if defined(IP22)
#define HQ3_CFIFO_HIWATER	36
#else /* ie IP28, IP30 etc */
#define HQ3_CFIFO_HIWATER	32
#endif
#define HQ3_CFIFO_LOWATER	20
#define HQ3_CFIFO_DELAY	        100

#define HQ3_DFIFO_HIWATER	40
#define HQ3_DFIFO_LOWATER	20
#define HQ3_DFIFO_DELAY	        0xfff /* DO NOT change this value     */
				      /* This works around an HQ3 bug */

#define HQ3_BFIFO_MAX		16	
#define HQ3_BFIFO_HIWATER	6	
#define HQ3_BFIFO_LOWATER	2	
#define HQ3_BFIFO_DELAY	        140

#define HQ3_CFIFODEPTH(base)	EXTRACT_BITS(base->fifo_status,  6,  0)
#define HQ3_CFIFOVALID(base)	EXTRACT_BITS(base->fifo_status, 14,  7)
#define HQ3_DFIFODEPTH(base)	EXTRACT_BITS(base->fifo_status, 21, 15)
#define HQ3_CFIFOHWM(base)	EXTRACT_BITS(base->fifo_status, 28, 22)

#define HQ3_BFIFODEPTH(base)	(base->gio_status & 0x1f)
#define HQ3_DCBBUSY		BIT(5)
#define HQ3_HIF_CTX_REST	BIT(6)

/* hq hag dma control register values */
#define HQ_HAG_HOST_TO_RE_DMA_START(pool) \
	((1    << 0) |	/* Run flag */ \
	 (pool << 1) |	/* im pool */ \
	 (0    << 3) |	/* Host to gfx transfer */ \
	 (1    << 4) |	/* RE, not CP, is the destination */ \
	 (1    << 5) |	/* Always 1 */ \
	 (0    << 6) |	/* No decimation */ \
	 (1    << 7))	/* Complete without context switch */ \

#define HQ_HAG_RE_TO_HOST_DMA_START(pool) \
	((1    << 0) |	/* Run flag */ \
	 (pool << 1) |	/* im pool */ \
	 (1    << 3) |	/* Gfx to host transfer */ \
	 (1    << 4) |	/* RE, not CP, is the destination */ \
	 (1    << 5) |	/* Always 1 */ \
	 (0    << 6) |	/* No decimation */ \
	 (1    << 7))	/* Complete without context switch */ \

#define HQ_HAG_HOST_TO_GE_DMA_START(pool) \
	(HQ_HAG_HOST_TO_RE_DMA_START(pool) & ~(1 << 4) /* CP is destination */)
	
#define HQ_HAG_GE_TO_HOST_DMA_START(pool) \
	(HQ_HAG_RE_TO_HOST_DMA_START(pool) /* bit 4 is irrelevant */)

/* hq access macros */
#ifndef mgras_WRITECFIFO64
#define mgras_WRITECFIFO64(base, hiWord, loWord) \
	((base)->cfifo.bydouble.data = \
	((__uint64_t)(hiWord) << 32) | ((__uint32_t)(loWord)))
/*
 * It is critical that the above loword be cast only to uint32_t, not uint64_t.
 * If it is cast instead to uint64_t, then the compiler sign extends it for
 * some reason.  This means that a negative loword will sign extend through
 * the bits of the high word, so that the high word will appear to be -1.
 */
#endif

#define mgras_WRITECFIFO32(base, word) \
	((base)->cfifo.byword.data = (__uint32_t)(word))

#define mgras_WRITECFIFO32HI(base, word) \
	((base)->cfifo.byword.pad = (__uint32_t)(word))

#define mgras_WRITEHQ32(base, reg, word) \
	((base)->reg = word)

#define mgras_READHQ32(base, reg, word) \
	(word = (base)->reg)

/* wait until there are n entries avail below high-water mark */
#define mgras_BFIFOPOLL(base, n) \
	while (HQ3_BFIFODEPTH(base) > (HQ3_BFIFO_HIWATER-(n)))

#define mgras_BFIFOWAIT(base,n)			\
	while (HQ3_BFIFODEPTH(base) > (HQ3_BFIFO_MAX-n))

#define mgras_CFIFOWAIT(base,n)			\
	while (HQ3_CFIFODEPTH(base) > (HQ3_CFIFO_MAX-n))

#define mgras_cfifo(base,token,data)			\
	{ base->cfifo[token] = data; }

typedef struct _hag_base_addr {
    volatile __uint32_t hi;
    volatile __uint32_t lo;
} _hag_base_addr;

typedef struct _hag_addr_cnt {
    volatile __uint32_t addr;
    volatile __uint32_t cnt;
} _hag_addr_cnt;

typedef struct hq_hag {
    /* HQ - Host Address Generator (HAG)
     * These registers are READ ONLY. Write using control command tokens. 
     */
    volatile __uint32_t	im_pg_list[4];				/* 0x1fx50300*/
    volatile __uint32_t	im_pg_width;				/* 0x1fx50310*/
    volatile __uint32_t	im_row_offset;				/* 0x1fx50314*/
    volatile __uint32_t	im_row_start_addr;			/* 0x1fx50318*/
    volatile __uint32_t	im_line_cnt;				/* 0x1fx5031C*/
    volatile __uint32_t	im_first_span_width;			/* 0x1fx50320*/
    volatile __uint32_t	im_last_span_width;			/* 0x1fx50324*/
    volatile __uint32_t	im_y_dec_factor;			/* 0x1fx50328*/
    volatile __uint32_t	im_dma_ctrl;				/* 0x1fx5032C*/
    _hag_addr_cnt	dl_jump_no_wait;			/* 0x1fx50330*/
    _hag_addr_cnt	dl_jump_wait;				/* 0x1fx50338*/
    volatile __uint32_t	stack_ram_wd0[8];			/* 0x1fx50340*/
    volatile __uint32_t	stack_ram_wd1[8];			/* 0x1fx50360*/
    _hag_base_addr	tx_table_base[5];			/* 0x1fx50380*/
    volatile __uint32_t	dma_page_size;				/* 0x1fx503A8*/
    volatile __uint32_t	dma_pg_bits[5];				/* 0x1fx503AC*/
    volatile __uint32_t	tx_max_offset[5];			/* 0x1fx503C0*/
    volatile __uint32_t	hag_state_flags;			/* 0x1fx503D4*/
    volatile __uint32_t	im_y_dec_remainder;			/* 0x1fx503D8*/
    _hag_addr_cnt	dl_push;				/* 0x1fx503DC*/
    volatile __uint32_t	dl_return;				/* 0x1fx503E4*/
    volatile __uint32_t	dl_stack_ptr;				/* 0x1fx503E8*/
    __uint32_t 		pad0[(0x400-0x3EC)/4];
    volatile __uint32_t	restore_cfifo;				/* 0x1fx50400*/
    volatile __uint32_t	gio_restore_addrs;			/* 0x1fx50404*/
    __uint32_t 		pad1[(0x500-0x408)/4];
} hq_hag;

struct hq_reif {
    volatile __uint32_t	read_wd_0;	    			/* 0x1fx50200*/
    volatile __uint32_t	read_wd_1;	    			/* 0x1fx50204*/
    volatile __uint32_t	scan_width;	    			/* 0x1fx50208*/
    volatile __uint32_t	line_count;	    			/* 0x1fx5020C*/
    volatile __uint32_t	re_dma_modes;	    			/* 0x1fx50210*/
    volatile __uint32_t	re_dma_cntrl;	    			/* 0x1fx50214*/
    volatile __uint32_t	re_dma_type;	    			/* 0x1fx50218*/
    volatile __uint32_t	rebus_sync;	    			/* 0x1fx5021C*/
    volatile __uint32_t	reif_flags;	    			/* 0x1fx50220*/
    volatile __uint32_t	rebus_out_1;	    			/* 0x1fx50224*/
    volatile __uint32_t	rebus_out_0;	    			/* 0x1fx50228*/
    volatile __uint32_t	diag_read_wd_1;	    			/* 0x1fx5022C*/
    volatile __uint32_t	diag_read_wd_0;	    			/* 0x1fx50230*/
    __uint32_t		pad0[(0x300-0x234)/4];
};

/**************************************************************************
 * Multi-Channel Option Structures and Macros
 **************************************************************************/

/* CRS numbers */
#define MCO_FPGAPROG   0
#define MCO_IDREG      0
#define MCO_FPGACTRL   1
#define MCO_INDEX      2
#define MCO_DATAIO     3
#define MCO_VC2INDEX   4
#define MCO_VC2DATA    5
#define MCO_VC2TMP     6
#define MCO_VC2RAM     7

#define MGRAS_MCO_FPGA_CONFIG  (MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_MCO_FPGA_CONTROL (MGRAS_DCB_CRS(1) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_MCO_INDEX        (MGRAS_DCB_CRS(2) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_MCO_DATAIO       (MGRAS_DCB_CRS(3) | MGRAS_DCB_DATAWIDTH_1)

#define MGRAS_MCO_VC2_INDEX    (MGRAS_DCB_CRS(4) | MGRAS_DCB_DATAWIDTH_1)  

#define MGRAS_MCO_VC2_INDEXDATA	(MGRAS_DCB_CRS(4) | MGRAS_DCB_ENCRSINC | MGRAS_DCB_DATAWIDTH_3)	

#define MGRAS_MCO_VC2_DATA (MGRAS_DCB_CRS(5) | MGRAS_DCB_ENCRSINC | MGRAS_DCB_DATAWIDTH_2)

#define MGRAS_MCO_VC2_RAM_64BIT	(MGRAS_DCB_CRS(7) | MGRAS_DCB_DATAWIDTH_4)

#define MGRAS_MCO_VC2_RAM 	(MGRAS_DCB_CRS(7) | MGRAS_DCB_DATAWIDTH_2)



typedef struct {
    unsigned char pad0[MGRAS_MCO_FPGA_CONFIG];
    volatile unsigned char fpga_config;
    unsigned char pad1[MGRAS_MCO_FPGA_CONTROL - MGRAS_MCO_FPGA_CONFIG - 1];
    volatile unsigned char fpga_control;
    unsigned char pad2[MGRAS_MCO_INDEX - MGRAS_MCO_FPGA_CONTROL - 1];
    volatile unsigned char index;
    unsigned char pad3[MGRAS_MCO_DATAIO - MGRAS_MCO_INDEX - 1];
    volatile unsigned char dataio;
    unsigned char pad4[MGRAS_MCO_VC2_INDEX - MGRAS_MCO_DATAIO -1 ];
    volatile unsigned char vc2index;
    unsigned char pad6[MGRAS_MCO_VC2_DATA - MGRAS_MCO_VC2_INDEX - 1];
    volatile unsigned short vc2data;
    unsigned char pad8[MGRAS_MCO_VC2_RAM - MGRAS_MCO_VC2_DATA - 2];
    volatile unsigned short vc2ram;
    unsigned char pad9[0x400 - MGRAS_MCO_VC2_RAM - 2];
} _mco;

/* Generic MCO Macrodefinitions */

/* dcbctrl_11 setting for crs(0) and crs(1) */
#define MCO_DCBCTRL_PAL ((MGRAS_DCB_CSWIDTH(6) | \
			  MGRAS_DCB_CSHOLD(6)   | \
			  MGRAS_DCB_CSSETUP(0)) & \
			 ~(MGRAS_DCB_ENSYNCACK   | \
			   MGRAS_DCB_ENASYNCACK))

/* dcbctrl_11 setting for crs(2) and crs(3) */
#define MCO_DCBCTRL_IO_START  ((MGRAS_DCB_CSWIDTH(6)  | \
				MGRAS_DCB_CSHOLD(6)   | \
				MGRAS_DCB_CSSETUP(0)) & \
			       ~(MGRAS_DCB_ENSYNCACK   | \
				 MGRAS_DCB_ENASYNCACK))

#define MCO_DCBCTRL_IO ((MGRAS_DCB_CSWIDTH(6) | \
			 MGRAS_DCB_CSHOLD(6)   | \
			 MGRAS_DCB_CSSETUP(6)  | \
			 MGRAS_DCB_ENSYNCACK)  & \
			~(MGRAS_DCB_ENASYNCACK))

/* dcbctrl_11 setting for crs(4), crs(5), crs(6), and crs(7) */
#define MCO_DCBCTRL_VC2 ((MGRAS_DCB_CSWIDTH(6)  | \
			  MGRAS_DCB_CSHOLD(6)   | \
			  MGRAS_DCB_CSSETUP(6)  | \
		          MGRAS_DCB_ENASYNCACK)  & \
			 ~(MGRAS_DCB_ENSYNCACK))

#define NCONFIGRD(x)  ((x & 0x04) >> 2)
#define NCONFIGWR     (0x01)
#define NSTATUS(x)    ((x & 0x02) >> 1)
#define CONFDONE(x)   (x & 0x01)

#define MCO_SETINDEX(base, device, indexA)                             \
    base->mco_mpa1.mco.index = ((device | indexA) & 0xff)

#define MCO_PROG_WRITE(base, data)                      \
    base->mco_mpa1.mco.fpga_config = (data & 0xff)

#define MCO_ID_READ(base, in)                           \
    in = ((base->mco_mpa1.mco.fpga_config) & 0xff)

#define MCO_CTRL_WRITE(base, ctrl)                      \
    base->mco_mpa1.mco.fpga_control = (ctrl & 0xff)

#define MCO_CTRL_READ(base, in)                         \
    in = ((base->mco_mpa1.mco.fpga_control) & 0xff)

#define MCO_INDEX_WRITE(base, ind)                      \
    base->mco_mpa1.mco.index = (ind & 0xff)

#define MCO_INDEX_READ(base, in)                        \
    in = ((base->mco_mpa1.mco.index) & 0xff)

#define MCO_IO_WRITE(base, out)                                       \
    base->mco_mpa1.mco.dataio = (out & 0xff)

#define MCO_IO_READ(base, in)                                         \
    in = ((base->mco_mpa1.mco.dataio) & 0xff)

#define MCO_VC2_INDEX_WRITE(base, ind)                  \
    base->mco_mpa1.mco.vc2index = (ind & 0xff)

#define MCO_VC2_INDEX_READ(base, in)                    \
    in = ((base->mco_mpa1.mco.vc2index) & 0xff)

#define MCO_VC2_DATA_WRITE(base, out)                   \
    base->mco_mpa1.mco.vc2data = (out & 0xffff)

#define MCO_VC2_DATA_READ(base, out)                    \
    out = ((base->mco_mpa1.mco.vc2data) & 0xffff)

#define MCO_VC2_RAM_WRITE(base, out)                   \
    base->mco_mpa1.mco.vc2ram = (out & 0xffff)

#define MCO_VC2_RAM_READ(base, out)                    \
    out = ((base->mco_mpa1.mco.vc2ram) & 0xffff)

/* Miscellaneous MCO defines */
#define MCO_BOARD_ID_MASK       0xf0
#define MCO_BOARD_REV_MASK      0x0f
#define MCO_BOARD_ID            0xa0
#define MCO_BOARD_ID_P0	        0x01
#define MCO_BOARD_NOT_PRESENT	0x00

/* Mode definitions */
#define MCO_UNINITIALIZED       0x00
#define MCO_4SCREEN_MODE        0x10
#define OS_4SCREEN              (MCO_CTL1_OSMODEA | MCO_CTL1_OSMODEB)
#define IS_4SCREEN              0x00
#define MCO_2SCREEN_MODE        0x11
#define MCO_FIELD_SEQ_MODE      0x12
#define OS_FIELD_SEQ            (MCO_CTL1_OSMODEC)
#define IS_FIELD_SEQ            (MCO_CTL1_ISMODEB)
#define MCO_MINIFY_MODE         0x13
#define OS_MINIFY               (MCO_CTL1_OSMODEA | MCO_CTL1_OSMODEB)
#define IS_MINIFY               (MCO_CTL1_ISMODEC)

/* MCO.INDEX Definitions */

/* ADV473 RAMDAC */
#define MCO_ADV473A               (0x00)
#define MCO_ADV473B               (0x10)
#define MCO_ADV473_RAMWR          (0)          /* RAM Address Register */
#define MCO_ADV473_CPRAM          (1)          /* Color Palette RAM    */
#define MCO_ADV473_PIXRDREG       (2)          /* Pixel RdMsk Register */
#define MCO_ADV473_RAMRD          (3)          /* RAM Address Register */
#define MCO_ADV473_OWR            (4)          /* Overlay Address Reg  */
#define MCO_ADV473_OREG           (5)          /* Overlay Registers    */
#define MCO_ADV473_CMD            (6)          /* Command Register     */
#define MCO_ADV473_ORD            (7)          /* Overlay Address Reg  */

#define MCO_ADV473_CMD_INIT	 0x30

/* - Overlay Palette is ignored (signals on MCO are grounded)
   - Color Palette is used for gamma correction
   - Todo: Add blanking functionality
           Determine Initial Color Palette
	   Determine Pix Read Mask Value
*/   

#define MCO_ADV473_SETADDR(base, device, index) \
    MCO_SETINDEX(base, device, index)


#define MCO_ADV473_SET24BIT(base, device)  \
    MCO_ADV473_SETADDR(base, device, MCO_ADV473_CMD); \
    MCO_IO_WRITE(base, 0x00)


#define MCO_ADV473_SETCPALWR(base, device, cpal_address) \
    MCO_ADV473_SETADDR(base, device, MCO_ADV473_RAMWR); \
    MCO_IO_WRITE(base,cpal_address)

#define MCO_ADV473_WRCOLORPAL(r, g, b) \
    MCO_IO_WRITE(base,r); \
    MCO_IO_WRITE(base,g); \
    MCO_IO_WRITE(base,b);


#define MCO_ADV473_SETPALRD(base, device, cpal_address) \
    MCO_ADV473_SETADDR(base, device, MCO_ADV473_RAMRD); \
    MCO_IO_WRITE(base,cpal_address);

#define MCO_ADV473_RDCOLORPAL(r, g, b) \
    MCO_IO_READ(base,r); \
    MCO_IO_READ(base,g); \
    MCO_IO_READ(base,b);

#if 0
#define MCO_ADV473_SETPIXRDMSK(base, device, pixel_mask) \ 
    MCO_ADV473_SETADDR(base, device, MCO_ADV473_PMSKREG); \
    MCO_IO_WRITE(base, pixel_mask)
#endif

/* This should work for applying blanking to the ADV473.  
 * Remember, we CANNOT apply blanking the 'normal way' by blanking the PP1,
 * because that would yield blank pixels on the VIO, disrupting any other 
 * devices.  Thus, we blank via the Pixel Read Mask Register. */

#define MCO_ADV473_BLANK(base, device) \
    MCO_ADV473_SETPIXRDMSK(base, device, 0x0);

/* ADV7162 RAMDAC */
#define MCO_ADV7162A              (0x20)
#define MCO_ADV7162B              (0x30)
#define MCO_ADV7162_ADDR          (0)          /* Address Register         */
#define MCO_ADV7162_LUT           (1)          /* LkUpTble CPal or Overlay */
#define MCO_ADV7162_PAL           (1)          /* LkUpTble CPal or Overlay */
#define MCO_ADV7162_CTRL          (2)          /* Control Register         */
#define MCO_ADV7162_CMD           (2)          /* Control Register         */
#define MCO_ADV7162_MODE          (3)          /* Mode Register            */

#define MCO_ADV7162_SETADDR(base, device, index) \
    MCO_SETINDEX(base, device, index)

/* Address of registers in the Control Register */
#define MCO_ADV7162_ID_ADDR		0x3
#define MCO_ADV7162_PIXMASK_ADDR	0x4
#define MCO_ADV7162_CMD1_ADDR		0x5
#define MCO_ADV7162_CMD2_ADDR		0x6
#define MCO_ADV7162_CMD3_ADDR		0x7
#define MCO_ADV7162_CMD4_ADDR		0x8
#define MCO_ADV7162_PLL_CTL_ADDR	0x9
#define MCO_ADV7162_STATUS_ADDR		0xa
#define MCO_ADV7162_REV_ADDR		0xb
#define MCO_ADV7162_PLL_R_ADDR		0xc
#define MCO_ADV7162_CMD5_ADDR           0xd
#define MCO_ADV7162_PLL_V_ADDR		0xf
#define MCO_ADV7162_CURS_ADDR		0x204

#define MCO_ADV7162_ID                  0x79
#define MCO_ADV7162_MODE_INIT		0x03
#define MCO_ADV7162_PIXMASK_INIT	0xff
#define MCO_ADV7162_PIXMASK_BLANK       0x00
#define MCO_ADV7162_CMD1_INIT		0xB9 /* orig -- 0x89 */
#define MCO_ADV7162_CMD2_INIT		0xec /* orig -- 0xE4 */
#define MCO_ADV7162_CMD3_INIT		0x40 /* orig -- 0xc0 */
#define MCO_ADV7162_CMD3_INIT_FS	0xc0
#define MCO_ADV7162_CMD4_INIT		0x00 /* orig -- 0x06 */
#define MCO_ADV7162_CMD4_INIT_FS	0x06
#define MCO_ADV7162_CMD5_INIT		0x40
#define MCO_ADV7162_CURS_DISABLE	0x01

#define MCO_ADV7162_MODE_RESET_BIT      BIT(0)
#define MCO_ADV7162_INT_PLL_ENA         BIT(0)
#define MCO_ADV7162_RSEL_ENA            BIT(1)
#define MCO_ADV7162_VSEL_ENA            BIT(2)
#define MCO_ADV7162_PSEL0_ENA           BIT(4)
#define MCO_ADV7162_PSEL1_ENA           BIT(5)
#define MCO_ADV7162_PLL_S(x)            ((x & 0x03) << 6)

/* TMC22091 Encoders */
#define MCO_TMC22091A             (0x40)
#define MCO_TMC22091B             (0x50)
#define MCO_TMC22091C             (0x60)
#define MCO_TMC22091D             (0x70)
#define MCO_TMC22091_CTRPLTR      (0)          /* Control Register Ptr     */
#define MCO_TMC22091_CLUTREG      (1)          /* CLUT Address Register    */
#define MCO_TMC22091_CTRL         (2)          /* Control Registers        */
#define MCO_TMC22091_CLUT         (3)          /* CLUT location            */

/* 22091 Global Control Register */
#define MCO_ENC_GLOBALREG                  (0x00)
#define MCO_ENC_SRESET_N                   BIT(4)
#define MCO_ENC_PAL                        BIT(3)
#define MCO_ENC_LUMDIS                     BIT(2)
#define MCO_ENC_CHRDIS                     BIT(1)
#define MCO_ENC_HRESET                     BIT(0)

/* 22091 Format Control Register */
#define MCO_ENC_FORMATREG                  (0x01)
#define MCO_ENC_LCREN                      BIT(6)
#define MCO_ENC_RAMPEN                     BIT(5)
#define MCO_ENC_CB_N                       BIT(4)
#define MCO_ENC_FORMAT
#define MCO_ENC_INMODE

/* 22091 Interface Control Register */
#define MCO_ENC_INTERFACEREG               (0x02)
#define MCO_ENC_VITSEN                     BIT(7)
#define MCO_ENC_SHCY                       BIT(6)
#define MCO_ENC_TBASE
#define MCO_ENC_SOUT                       BIT(3)
#define MCO_ENC_FBDIS                      BIT(2)
#define MCO_ENC_PDCDIR                     BIT(1)
#define MCO_ENC_FLDLK                      BIT(0)

/* 22091 Test Control Register */
#define MCO_ENC_TESTREG                    (0x03)
#define MCO_ENC_LIMEN                      BIT(6)
#define MCO_ENC_TESTEN                     BIT(5)
#define MCO_ENC_HOLDEN                     BIT(4)
#define MCO_ENC_TSTMSB                     BIT(3)
#define MCO_ENC_LUMTST                     BIT(2)
#define MCO_ENC_8FSUBR                     BIT(1)
#define MCO_ENC_CHRTST                     BIT(0)

/* The meaning of Register 4 (i.e. Key Control or Layering Control)
 * is determened by Format Control Register Bit 6.  MCO_ENC_LCREN */

/* 22091 Key Control Register */
#define MCO_ENC_KEYREG                     (0x04)
#define MCO_ENC_HKEN                       BIT(6)
#define MCO_ENC_BUKEN                      BIT(5)
#define MCO_ENC_SKEXT                      BIT(4)
#define MCO_ENC_DKDIS                      BIT(3)
#define MCO_ENC_EKDIS                      BIT(2)
#define MCO_ENC_FKDIS                      BIT(1)
#define MCO_ENC_SKEN_KEY                   BIT(0)

/* 22091 Layering Control Register */
#define MCO_ENC_LAYREG                     (0x04)
#define MCO_ENC_LAYMODE_MSB                BIT(7)
#define MCO_ENC_HKEN                       BIT(6)
#define MCO_ENC_BUKEN                      BIT(5)
#define MCO_ENC_SKEXT                      BIT(4)
#define MCO_ENC_LAYMODE_LSB                
#define MCO_ENC_SKEN_LAY                   BIT(0)

/* 22091 Key Value Registers */
#define MCO_ENC_DKEYREG                    (0x05)
#define MCO_ENC_EKEYREG                    (0x06)
#define MCO_ENC_FKEYREG                    (0x07)

/* Registers 0x08 - 0x0D are reserved */

/* 22091 Misc. Control Registers */
#define MCO_ENC_MISCREG                    (0x0e)
#define MCO_ENC_EFEN                       BIT(7)
#define MCO_ENC_COMPDA                     BIT(6)
#define MCO_ENC_SVIDDA                     BIT(5)
#define MCO_ENC_FKREN                      BIT(4)
#define MCO_ENC_RATIO                      BIT(3)
#define MCO_ENC_TFLK                       BIT(2)
#define MCO_ENC_T512                       BIT(1)
#define MCO_ENC_CB100                      BIT(0)

/* 22091 Standards Control Register */
#define MCO_ENC_STDSREG                    (0x0f)
#define MCO_ENC_EFEN                       BIT(7)
#define MCO_ENC_SIX25                      BIT(6)
#define MCO_ENC_PALID                      BIT(5)
#define MCO_ENC_SETUP                      BIT(4)
#define MCO_ENC_YGAIN
#define MCO_ENC_CGAIN

/* 22091 Timing Registers */
#define MCO_ENC_SYREG                      (0x10)
#define MCO_ENC_BRREG                      (0x11)
#define MCO_ENC_BUREG                      (0x12)
#define MCO_ENC_CBPREG                     (0x13)
#define MCO_ENC_XBPREG                     (0x14)
#define MCO_ENC_VAREG                      (0x15)
#define MCO_ENC_VCREG                      (0x16)
#define MCO_ENC_VBREG                      (0x17)
#define MCO_ENC_XBPVAVCREG                 (0x18)
#define MCO_ENC_XBP
#define MCO_ENC_VA
#define MCO_ENC_VC
#define MCO_ENC_FPREG                      (0x19)
#define MCO_ENC_ELREG                      (0x1a)
#define MCO_ENC_EHREG                      (0x1b)
#define MCO_ENC_SLREG                      (0x1c)
#define MCO_ENC_SHREG                      (0x1d)
#define MCO_ENC_CBLREG                     (0x1e)

#define MCO_ENC_FIELDLTYPEREG              (0x1f)    
#define MCO_ENC_FIELD
#define MCO_ENC_LTYPE
 
/* MCO Control Registers */

/* MCO Control Register 1 */
#define MCO_CTRLREG1		(0x80)
#define MCO_CONTROL1            MCO_CTRLREG1
/* MCO Control Register 1 Bit definitions */
#define MCO_CTL1_OSMODEA	BIT(0)          /* Output Swizzle Mode A */
#define MCO_CTL1_OSMODEB	BIT(1)          /* Output Swizzle Mode B */
#define MCO_CTL1_OSMODEC	BIT(2)          /* Output Swizzle Mode C */
#define MCO_CTL1_ISMODEA	BIT(3)          /* Input Swizzle Mode A */
#define MCO_CTL1_ISMODEB	BIT(4)          /* Input Swizzle Mode B */
#define MCO_CTL1_ISMODEC	BIT(5)          /* Input Swizzle Mode C */

/* MCO Control Register 2 */
#define MCO_CTRLREG2		(0x90)
#define MCO_CONTROL2            MCO_CTRLREG2
/* MCO Control Register 2 Bit definitions */
#define MCO_CTL2_NTSC		BIT(0)
#define MCO_CTL2_PAL		BIT(1)
#define MCO_CTL2_SEL1		BIT(2)
#define MCO_CTL2_SEL2		BIT(3)
#define MCO_CTL2_SEL3		BIT(4)

/* MCO Control Register 3 */
#define MCO_CONTROL3               (0xA0)

/* Interface to mco vc2 */

#define MCO_VC2_SETREG(base,index,data16)                               \
    base->mco_mpa1.mco.vc2index = index;                                     \
    base->mco_mpa1.mco.vc2data = data16

#define MCO_VC2_READREG(base,index,data)                                \
    base->mco_mpa1.mco.vc2index = index;                                     \
    data = base->mco_mpa1.mco.vc2data

#define MCO_VC2_SETINDEX(base,index)                                    \
    base->mco_mpa1.mco.vc2index = index

#define MCO_VC2_GETINDEX(base,data)                                     \
    data = base->mco_mpa1.mco.vc2index

#define MCO_VC2_SETRAM(base,data)                                       \
    base->mco_mpa1.mco.vc2ram = data

#define MCO_VC2_GETRAM(base,data)                                       \
    data = base->mco_mpa1.mco.vc2ram
  
/* MPA1 -- mardigras presenter adapter */

#define MGRAS_MPA1_CRS0 (MGRAS_DCB_CRS(0) | MGRAS_DCB_DATAWIDTH_1)
#define MGRAS_MPA1_CRS1 (MGRAS_DCB_CRS(1) | MGRAS_DCB_DATAWIDTH_1)
/* 
 * stolen from dmedia/devices/galileo/kern/ev2/ev1_internals.h
 */
#define HQ3_MPA1_PROTOCOL_NOACK		\
  (MGRAS_DCB_CSSETUP(2) | MGRAS_DCB_CSHOLD(2) |  \
   MGRAS_DCB_CSWIDTH(10))

#define HQ3_MPA1_PROTOCOL_SYNCACK \
  (MGRAS_DCB_CSSETUP(2) | MGRAS_DCB_CSHOLD(2) |  \
   MGRAS_DCB_CSWIDTH(10) | MGRAS_DCB_ENSYNCACK)

#define HQ3_MPA1_PROTOCOL_ASYNCACK \
  (MGRAS_DCB_CSSETUP(2) | MGRAS_DCB_CSHOLD(2) |  \
   MGRAS_DCB_CSWIDTH(10) | MGRAS_DCB_ENASYNCACK)

typedef struct {
    unsigned char pad0[MGRAS_MPA1_CRS0];
    volatile unsigned char crs0;
    unsigned char pad1[MGRAS_MPA1_CRS1 - MGRAS_MPA1_CRS0 -1 ];
    volatile unsigned char crs1;
} _mpa1;

typedef union {
    _mco mco;
    _mpa1 mpa1;
    volatile unsigned char pad1[0x400];
} _mco_mpa1;

/**************************************************************************
 * HQ3 Address space (full MGRAS address range)
 **************************************************************************/

typedef struct mgras_hw {
    /* HQ - Kernel GIO Address Map */
    volatile __uint32_t gio_id;					/* 0x1fx00000 */
    volatile unsigned char pad2_hquc[0x40000-0x00000-4];
    volatile __uint32_t hquc[0x5000/4];		    		/* 0x1fx40000 */
    volatile __uint32_t window;					/* 0x1fx45000 */
    volatile unsigned char pad2_hqpc[0x6000-0x5000-4];
    volatile __uint32_t hqpc;				    	/* 0x1fx46000 */
    volatile unsigned char pad2_cp_data[0x7000-0x6000-4];      	
    volatile __uint32_t cp_data;		    	    	/* 0x1fx47000 */
    volatile unsigned char pad2_rss_diag[0xc000-0x7000-4];	    
    volatile __uint32_t rss_diag[(0x50000-0x4c000)/4];	    	/* 0x1fx4c000 */
    volatile __uint32_t hq_config;			    	/* 0x1fx50000 */
    volatile __uint32_t flag_ctx;				/* 0x1fx50004 */
    volatile __uint32_t flag_set_priv;				/* 0x1fx50008 */
    volatile __uint32_t flag_clr_priv;			    	/* 0x1fx5000C */
    volatile __uint32_t flag_enab_set;			    	/* 0x1fx50010 */
    volatile __uint32_t flag_enab_clr;			    	/* 0x1fx50014 */
    volatile __uint32_t intr_enab_set;			    	/* 0x1fx50018 */
    volatile __uint32_t intr_enab_clr;			    	/* 0x1fx5001C */
    volatile __uint32_t cfifo_hw;			    	/* 0x1fx50020 */
    volatile __uint32_t cfifo_lw;			    	/* 0x1fx50024 */
    volatile __uint32_t cfifo_delay;			    	/* 0x1fx50028 */
    volatile __uint32_t dfifo_hw;			    	/* 0x1fx5002C */
    volatile __uint32_t dfifo_lw;			    	/* 0x1fx50030 */
    volatile __uint32_t dfifo_delay;			    	/* 0x1fx50034 */
    volatile unsigned char pad2_ge0_diag[0x40-0x34-4];
    volatile __uint32_t ge0_diag_d;			    	/* 0x1fx50040 */
    volatile __uint32_t ge0_diag_a;			    	/* 0x1fx50044 */
    volatile __uint32_t ge1_diag_d;			    	/* 0x1fx50048 */
    volatile __uint32_t ge1_diag_a;			    	/* 0x1fx5004C */
    volatile __uint32_t ctxsw;
    volatile unsigned char pad2_cfifo_priv[0x80-0x50-4];	    
    dcbdata64 	cfifo_priv;				    	/* 0x1fx50080 */
    volatile unsigned char pad2_ctx_data[0x100-0x80-8];	    
    volatile __uint32_t ctx_data;			    	/* 0x1fx50100 */
    volatile unsigned char pad2_bfifo_hw[0x108-0x100-4];
    volatile __uint32_t bfifo_hw;			    	/* 0x1fx50108 */
    volatile __uint32_t bfifo_lw;			    	/* 0x1fx5010c */
    volatile __uint32_t bfifo_delay;			    	/* 0x1fx50110 */
    volatile __uint32_t gio_config;			    	/* 0x1fx50114 */
    volatile __uint32_t gio_bigendian;			    	/* 0x1fx50118 */
    volatile __uint32_t gio_litendian;			    	/* 0x1fx5011c */
    volatile __uint32_t mstr_delay;				/* 0x1fx50120 */
    volatile __uint32_t tlb_test;				/* 0x1fx50124 */
    volatile unsigned char pad2_reif_ctx[0x200-0x124-4];
    struct hq_reif	reif_ctx;    				/* 0x1fx50200*/
    hq_hag 		hag_ctx;				/* 0x1fx50300*/
    volatile __uint32_t hif_ctx[(0x600-0x500)/4];    	    	/* 0x1fx50500*/
    volatile __uint32_t gio_ctx[(0x700-0x600)/4];    	    	/* 0x1fx50600*/
    volatile unsigned char pad2_tlb[0x800-0x700];
    volatile __uint32_t tlb[(0x880-0x800)/4];		    	/* 0x1fx50800*/
    volatile __uint32_t tlb_curr_addr;			    	/* 0x1fx50880*/
    volatile unsigned char pad2_tlb_valids[0x900-0x880-4];
    volatile __uint32_t tlb_valids;		    	   	/* 0x1fx50900*/
    volatile unsigned char pad2_dcb_address_space[0x60000-0x50900-4];
    
    /* HQ - DCB GIO Address Map */
    /* DCB Dev #0 */
    volatile unsigned char pad_dcb_0[0x400];			/* 0x1fx60000 */
    /* DCB Dev #1 */
    volatile unsigned char pad_dcb_1[0x400];			/* 0x1fx60400 */
    /* DCB Dev #2 */
    volatile unsigned char pad_dcb_2[0x400];			/* 0x1fx60800 */
    /* DCB Dev #3 */
    _MScmap	cmapall;		    		   	/* 0x1fx60C00 */
    /* DCB Dev #4 */
    _MScmap	cmap0;		    			   	/* 0x1fx61000 */
    /* DCB Dev #5 */
    _MScmap 	cmap1;				 	   	/* 0x1fx61400 */
    /* DCB Dev #6 */
    _ADVdac	dac;		    			   	/* 0x1fx61800 */
    /* DCB Dev #7 */
    _pp1xmap	xmap;		    	    		   	/* 0x1fx61C00 */
    /* DCB Dev #8 */
    _vc3 	vc3;				 	   	/* 0x1fx62000 */
    /* DCB Dev #9 */
    _mgrasbd 	bdvers;				 	   	/* 0x1fx62400 */
    /* DCB Dev #10 used by galileo 1.5 ab1 */
    volatile unsigned char pad_dcb_10[0x400];			/* 0x1fx62800 */
    /* DCB Dev #11 Multichannel Option  */
    /*             AND Flat Panel (mpa1) Display for Mardi Gras*/
    _mco_mpa1    mco_mpa1;			                /* 0x1fx62C00 */
    /* DCB Dev #12 used by galileo 1.5 cc1 */
    volatile unsigned char pad_dcb_12[0x400];			/* 0x1fx63000 */
    /* DCB Dev #13 used by galileo 1.5 gal */
    volatile unsigned char pad_dcb_13[0x400];			/* 0x1fx63400 */
    /* DCB Dev #14 used by galileo 1.5 tmi/csc */
    volatile unsigned char pad_dcb_14[0x400];			/* 0x1fx63800 */
    /* DCB Dev #15 */
    volatile unsigned char pad_dcb_15[0x400];			/* 0x1fx63C00 */
    volatile unsigned char pad2_dcbctrl[0x8000-0x4000];
    /* DCB control registers */
    volatile __uint32_t dcbctrl_0;		    		/* 0x1fx68000 */
    volatile __uint32_t dcbctrl_1;		    		/* 0x1fx68004 */
    volatile __uint32_t dcbctrl_2;		    		/* 0x1fx68008 */
    volatile __uint32_t dcbctrl_cmapall;	    		/* 0x1fx6800c */
    volatile __uint32_t dcbctrl_cmap0;		    		/* 0x1fx68010 */
    volatile __uint32_t dcbctrl_cmap1;		    		/* 0x1fx68014 */
    volatile __uint32_t dcbctrl_dac;		    		/* 0x1fx68018 */
    volatile __uint32_t dcbctrl_pp1;		    		/* 0x1fx6801c */
    volatile __uint32_t dcbctrl_vc3;		    		/* 0x1fx68020 */
    volatile __uint32_t dcbctrl_bdvers;		    		/* 0x1fx68024 */
    volatile __uint32_t dcbctrl_ab1;		    		/* 0x1fx68028 */
    volatile __uint32_t dcbctrl_11;		    		/* 0x1fx6802c */
    volatile __uint32_t dcbctrl_cc1;		    		/* 0x1fx68030 */
    volatile __uint32_t dcbctrl_gal;		    		/* 0x1fx68034 */
    volatile __uint32_t dcbctrl_tmi;		    		/* 0x1fx68038 */
    volatile __uint32_t dcbctrl_15;		    		/* 0x1fx6803c */
    volatile unsigned char pad2_user_address_space[0x70000-0x6803c-4];

    /* HQ - User GIO Address Map */
    volatile __uint32_t status;		    	    		/* 0x1fx70000 */
    volatile __uint32_t fifo_status;		    		/* 0x1fx70004 */
    volatile __uint32_t flag_set;		    		/* 0x1fx70008 */
    volatile __uint32_t flag_clr;		    		/* 0x1fx7000c */
    volatile __uint32_t rebus_read_hi;		    		/* 0x1fx70010 */
    volatile __uint32_t rebus_read_lo;		    		/* 0x1fx70014 */
    volatile unsigned char pad2_cfifo[0x80-0x14-4];
    dcbdata64 cfifo;				    		/* 0x1fx70080 */
    volatile unsigned char pad2_gio_status[0x100-0x80-8];
    volatile __uint32_t gio_status;		    		/* 0x1fx70100 */
    volatile __uint32_t busy_dma;		    		/* 0x1fx70104 */
    volatile unsigned char pad2_rss[0xc000-0x0104-4];
    raster rss;					    		/* 0x1fx7c000 */
    volatile unsigned char pad2_cfifo_cmd[0x4000-sizeof(raster)];
    dcbdata64 cfifo_cmd[0x80000/8];    		    		/* 0x1fx80000 */
} mgras_hw;

/**************************************************************************
 * GE Microcode Structures and Macros
 **************************************************************************/

/* Settings for downloading ucode */ 
#define	MGRAS_UCODERAM_INITVAL	0xffffffff	
#define	GE11_UCODERAM_LEN	256*1024
#define	GE11_UCODE_BASE		0x200000 

#define	GE11_DIAG_EXEC_ADDR	0x40000
#define	GE11_GO 		BIT(0)
#define	GE11_UCODE_LAT_4 	~BIT(4) 
#define	GE11_UCODE_LAT_3 	BIT(4) 

#define GE11_DIAG_MODE_ADDR	0x40038
#define GE11_DIAG_MODE		BIT(0)
#define GE11_OFIFO_WM		(0x68 << 1)

#define GE11_DIAG_SEL_REG       0x40010
#define GE11_DIAG_INPUT         BIT(8)

#define GE11_COTM_CONFIG_REG	0x40018
#define GE11_COTM_CONFIG_VAL	((4<<13) | (6<<10) | (6<<7) | (6<<4) | 1)

/**************************************************************************
 * Emulation HW Structures and Macros
 **************************************************************************/

/*
** structs for emulation hardware
*/
typedef struct {
    volatile __uint32_t _pad00;
    volatile __uint32_t addr_bc;			    /* 0x1f500004 */
    volatile __uint32_t _pad08;
    volatile __uint32_t ctrl;				    /* 0x1f50000c */
    volatile __uint32_t _pad10;
    volatile __uint32_t stat;				    /* 0x1f500014 */
    volatile __uint32_t _pad18; 
    volatile __uint32_t pio_rd_add;			    /* 0x1f50001c */
} sgio_regs;

typedef struct {
    int slot;
    unsigned int arb_reg;
} ng1_setgio_args;	

/**************************************************************************
 * Miscellaneous Structures and Macros
 **************************************************************************/

#define MGRAS_WINLSB(x,y)	((y & 3) << 2) | (x & 3)
#define MGRAS_CIDMATCH(cid)	(1 << (4 + (cid)))

#define MGRAS_8BITCMAP_BASE	0
#define MGRAS_CMAP_NCOLMAPENT	8192

#endif /* _SYS_MGRASHW_H_ */
