#ifndef __SYS_MIPS_ELF_H__
#define __SYS_MIPS_ELF_H__

/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/
/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/
/*
 * Copyright 1992 Silicon Graphics,  Inc.
 * ALL RIGHTS RESERVED
 * 
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SGI
 * The copyright notice above does not evidence any  actual  or
 * intended  publication of this source code and material is an
 * unpublished work by Silicon  Graphics,  Inc.  This  material
 * contains CONFIDENTIAL INFORMATION that is the property and a
 * trade secret of Silicon Graphics, Inc. Any use,  duplication
 * or  disclosure  not  specifically  authorized  in writing by
 * Silicon Graphics is  strictly  prohibited.  THE  RECEIPT  OR
 * POSSESSION  OF  THIS SOURCE CODE AND/OR INFORMATION DOES NOT
 * CONVEY ANY RIGHTS TO REPRODUCE, DISCLOSE OR  DISTRIBUTE  ITS
 * CONTENTS,  OR  TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT
 * MAY DESCRIBE, IN WHOLE OR IN PART.
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND
 * Use, duplication or disclosure by the Government is  subject
 * to  restrictions  as  set  forth  in  FAR 52.227.19(c)(2) or
 * subparagraph (c)(1)(ii) of the Rights in Technical Data  and
 * Computer  Software  clause  at  DFARS 252.227-7013 and/or in
 * similar or successor clauses in the FAR, or the DOD or  NASA
 * FAR  Supplement.  Unpublished  --  rights reserved under the
 * Copyright Laws of the United States. Contractor/manufacturer
 * is Silicon Graphics, Inc., 2011 N. Shoreline Blvd., Mountain
 * View, CA 94039-7311
 */

/* WARNING:  This file is included in, and logically a part of,
 * /usr/include/elf.h.  The distribution of material between them
 * is artificial, generally reflecting the distinction between generic
 * and system-dependent, and may change.  Do not include this file
 * directly -- include /usr/include/elf.h instead.
 */

/*
 *	Any symbol definition in this file of the form
 *
 *		#define	  pre_MIPS_suf	pre_IRIX_suf
 *
 *	is included for backwards compatibility.  New code 
 *	should use the   pre_IRIX_suf   symbol.
 *
 *	See /usr/include/sys/elf.h for more information on
 *	the naming conventions and other file structure for
 *	the various elf.h files.
 */
#include <sys/elftypes.h> 

/* 
 * Random constants
 */

#define ELF_MIPS_MAXPGSZ ELF_IRIX_MAXPGSZ
#define ELF_MIPS_MINPGSZ ELF_IRIX_MINPGSZ


/* ====================================================================
 *
 * Elf header
 *
 * ====================================================================
 */

/*
 * e_flags
 */

#define EF_MIPS_NOREORDER	0x00000001
#define EF_MIPS_OPSEX		EF_MIPS_NOREORDER
#define EF_MIPS_PIC		0x00000002
#define EF_MIPS_CPIC		0x00000004
#define EF_MIPS_XGOT		0x00000008
#define EF_MIPS_OPTIONS_FIRST	0x00000080 
/* obsolete names */
#define EF_MIPS_UGEN_ALLOC	EF_MIPS_XGOT
#define EF_MIPS_UGEN_RESERVED	EF_MIPS_64BIT_WHIRL

#define EF_MIPS_ABI64		EF_IRIX_ABI64
	/* see explanation in /usr/include/sys/elf.h	*/

#define EF_MIPS_64BIT_WHIRL	EF_MIPS_ABI64
	/* This is here for compatibility with pre-v7.4 compiler objects. */

#define EF_MIPS_ABI2		0x00000020 
	/* indicates n32 as opposed to o32 */

/*
 *	The EF_MIPS_ARCH field of e_flags describes the ISA of the object.
 *		size:	4 bits
 *		type:	int
 */
#define EF_MIPS_ARCH		0xf0000000	/* mask: 4 bit field */
#define EF_MIPS_ARCH_1		0x00000000
#define EF_MIPS_ARCH_2          0x10000000
#define EF_MIPS_ARCH_3          0x20000000
#define EF_MIPS_ARCH_4          0x30000000
#define EF_MIPS_ARCH_5          0x40000000
#define EF_MIPS_ARCH_6          0x50000000

/*
 *	The EF_MIPS_ARCH_ASE field of e_flags describes the set of 
 *	Application Specific Extensions used by the object.
 *		size:	4 bits
 *		type:	bit-field
 */
#define EF_MIPS_ARCH_ASE	0x0f000000	/* mask: 4 bit field	*/
#define EF_MIPS_ARCH_ASE_MDMX	0x08000000	/* multi-media extensions*/
#define EF_MIPS_ARCH_ASE_M16	0x04000000	/* MIPS16 isa extensions */

/*
 *	Please reserve these 8 bits of e_flags for future
 *	expansion of the EF_MIPS_ARCH_ASE field;  increasing
 *	the field from 4 bits to 12 bits.
 *	
 *		0x00ff0000	
 *	
 *	If and when we expand it, we'll redefine the EF_MIPS_ARCH_ASE 
 *	macro to be:
 *	
 *		0x0fff0000.
 */

/* ====================================================================
 *
 * Program header
 *
 * ====================================================================
 */


/* 
 * special Program header types
 */

#define PT_MIPS_REGINFO		(PT_LOPROC + 0)	/* n32,n64 -- obsolete	*/
#define PT_MIPS_RTPROC		(PT_LOPROC + 1)	/* o32 -- runtime proc tbl */
#define PT_MIPS_OPTIONS		(PT_LOPROC + 2) /* irix/mips		*/


/* 
 * special p_flags
 */

#define PF_MIPS_LOCAL		0x10000000

/* ====================================================================
 *
 * Section Headers
 *
 * ====================================================================
 */

/* 
 * Special mips section indices
 */

#define SHN_MIPS_ACOMMON	SHN_IRIX_ACOMMON
#define SHN_MIPS_TEXT		SHN_IRIX_TEXT
#define SHN_MIPS_DATA		SHN_IRIX_DATA
#define SHN_MIPS_SCOMMON	SHN_IRIX_SCOMMON
#define SHN_MIPS_SUNDEFINED	SHN_IRIX_SUNDEFINED
#define SHN_MIPS_LCOMMON	SHN_IRIX_LCOMMON
#define SHN_MIPS_LUNDEFINED	SHN_IRIX_LUNDEFINED


/*
 * sh_type
 *
 *	These SHT_MIPS_* names are included for backwards
 *	compatibility. 
 */

#define SHT_MIPS_LIBLIST	SHT_IRIX_LIBLIST
#define SHT_MIPS_MSYM		SHT_IRIX_MSYM
#define SHT_MIPS_CONFLICT	SHT_IRIX_CONFLICT
#define SHT_MIPS_GPTAB		SHT_IRIX_GPTAB
#define SHT_MIPS_UCODE		SHT_IRIX_UCODE
#define SHT_MIPS_DEBUG		SHT_IRIX_DEBUG
#define SHT_MIPS_REGINFO	SHT_IRIX_REGINFO
#ifdef __osf__
#define SHT_MIPS_PACKAGE	SHT_IRIX_PACKAGE
#define SHT_MIPS_PACKSYM	SHT_IRIX_PACKSYM
#endif /* __osf__ */

#define SHT_MIPS_RELD		SHT_IRIX_RELD
#define SHT_MIPS_DONTUSE	SHT_IRIX_DONTUSE
/* Don't  use 10 until after the ragnarok beta */
#define SHT_MIPS_IFACE		SHT_IRIX_IFACE
#define SHT_MIPS_CONTENT	SHT_IRIX_CONTENT
#define SHT_MIPS_OPTIONS	SHT_IRIX_OPTIONS

#define SHT_MIPS_SHDR		SHT_IRIX_SHDR
#define SHT_MIPS_FDESC		SHT_IRIX_FDESC
#define SHT_MIPS_EXTSYM		SHT_IRIX_EXTSYM
#define SHT_MIPS_DENSE		SHT_IRIX_DENSE
#define SHT_MIPS_PDESC		SHT_IRIX_PDESC
#define SHT_MIPS_LOCSYM		SHT_IRIX_LOCSYM
#define SHT_MIPS_AUXSYM		SHT_IRIX_AUXSYM
#define SHT_MIPS_OPTSYM		SHT_IRIX_OPTSYM
#define SHT_MIPS_LOCSTR		SHT_IRIX_LOCSTR
#define SHT_MIPS_LINE		SHT_IRIX_LINE
#define SHT_MIPS_RFDESC		SHT_IRIX_RFDESC

#define SHT_MIPS_DELTASYM	SHT_IRIX_DELTASYM
#define SHT_MIPS_DELTAINST	SHT_IRIX_DELTAINST
#define SHT_MIPS_DELTACLASS	SHT_IRIX_DELTACLASS

#define SHT_MIPS_DWARF		SHT_IRIX_DWARF
#define SHT_MIPS_DELTADECL	SHT_IRIX_DELTADECL
#define SHT_MIPS_SYMBOL_LIB	SHT_IRIX_SYMBOL_LIB
#define SHT_MIPS_EVENTS		SHT_IRIX_EVENTS
#define SHT_MIPS_TRANSLATE	SHT_IRIX_TRANSLATE
#define SHT_MIPS_PIXIE		SHT_IRIX_PIXIE
#define SHT_MIPS_XLATE		SHT_IRIX_XLATE
#define SHT_MIPS_XLATE_DEBUG	SHT_IRIX_XLATE_DEBUG
#define SHT_MIPS_WHIRL		SHT_IRIX_WHIRL
#define SHT_MIPS_EH_REGION	SHT_IRIX_EH_REGION
#define SHT_MIPS_XLATE_OLD	SHT_IRIX_XLATE_OLD
#define SHT_MIPS_PDR_EXCEPTION	SHT_IRIX_PDR_EXCEPTION

#define SHT_MIPS_NUM		SHT_IRIX_NUM

/*
 * sh_flags
 */

#define SHF_MIPS_GPREL		SHF_IRIX_GPREL
#define SHF_MIPS_MERGE		SHF_IRIX_MERGE
#define SHF_MIPS_ADDR		SHF_IRIX_ADDR
#define SHF_MIPS_STRINGS	SHF_IRIX_STRINGS
#define SHF_MIPS_NOSTRIP 	SHF_IRIX_NOSTRIP
#define SHF_MIPS_LOCAL		SHF_IRIX_LOCAL
#define SHF_MIPS_NAMES		SHF_IRIX_NAMES
#define SHF_MIPS_NODUPE		SHF_IRIX_NODUPE

/*
 * special section names
 *
 *	These are the section names for those sections
 *	that exist in both Irix/mips and Irix/ia64 objects.
 */
#define MIPS_LIBLIST            ".liblist"
#define MIPS_MSYM               ".msym"
#define MIPS_CONFLICT           ".conflict"
#define MIPS_SDATA              ".sdata"
#define MIPS_REL_SDATA          ".rel.sdata"
#define MIPS_SRDATA             ".srdata"
#define MIPS_RDATA              ".rdata"
#define MIPS_SBSS               ".sbss"
#define MIPS_LIT4               ".lit4"
#define MIPS_LIT8               ".lit8"
#define MIPS_LIT16              ".lit16"
#define MIPS_EVENTS             ".MIPS.events"
#define MIPS_INTERFACES         ".MIPS.interfaces"
#define MIPS_OPTIONS            ".MIPS.options"
#define MIPS_SYMBOL_LIB         ".MIPS.symlib"
#define MIPS_DEBUG_INFO         ".debug_info"
#define MIPS_DEBUG_LINE         ".debug_line"
#define MIPS_DEBUG_ABBREV       ".debug_abbrev"
#define MIPS_DEBUG_FRAME        ".debug_frame"
#define MIPS_DEBUG_ARANGES      ".debug_aranges"
#define MIPS_DEBUG_PUBNAMES     ".debug_pubnames"
#define MIPS_DEBUG_STR          ".debug_str"
#define MIPS_DEBUG_FUNCNAMES    ".debug_funcnames"
#define MIPS_DEBUG_TYPENAMES    ".debug_typenames"
#define MIPS_DEBUG_VARNAMES     ".debug_varnames"
#define MIPS_DEBUG_WEAKNAMES    ".debug_weaknames"
#define MIPS_XLATE              ".MIPS.Xlate"
#define MIPS_XLATE_DEBUG	".MIPS.Xlate_debug"
#define MIPS_CONTENT		".MIPS.content"
#define MIPS_WHIRL              ".WHIRL"	/* Mips uses this name       */
/* #define MIPS_WHIRL           ".MIPS.WHIRL"*/ /* Should have use this name */

#ifdef __osf__
#define MIPS_PACKAGE            ".package"
#define MIPS_PACKSYM            ".packsym"
#endif /* __osf__ */

/*
 * special section names
 *
 *	These are the section names for sections
 *	that are unique to Irix/mips.
 */
#define MIPS_REGINFO		".reginfo"
#define MIPS_RHEADER		".rheader"
#define MIPS_GPTAB_SDATA	".gptab.sdata"
#define MIPS_GPTAB_DATA		".gptab.data"
#define MIPS_GPTAB_BSS		".gptab.bss"
#define MIPS_GPTAB_SBSS		".gptab.sbss"
#define MIPS_LBSS		".lbss"
#define MIPS_UCODE		".ucode"
#define MIPS_MDEBUG		".mdebug"
#define MIPS_COMPACT_RELOC	".compact_rel"
#define MIPS_DELTACLASS		".MIPS.dclass"
#define MIPS_DELTASYM		".MIPS.dsym"
#define MIPS_DELTAINST		".MIPS.dinst"
#define MIPS_DELTADECL		".MIPS.ddecl"
#define MIPS_REL_DELTA		".rel.delta"
#define MIPS_XLATE_DEBUG	".MIPS.Xlate_debug"
#define MIPS_XLATE_OLD		".MIPS.Xlate_old"
#define MIPS_PERF_FUNCTIONS    	".MIPS.Perf_function"
#define MIPS_PERF_WEAKNAMES	".MIPS.Perf_weak_names"
#define MIPS_PERF_CALLGRAPH	".MIPS.Perf_call_graph"
#define MIPS_PERF_BBOFFSETS	".MIPS.Perf_bb_offsets"
#define MIPS_PERF_TABLE		".MIPS.Perf_table"
#define MIPS_PERF_ARGTRACE  	".MIPS.Perf_argtrace"
#define MIPS_EH_REGION		"_MIPS_eh_region"
#define MIPS_EH_REGION_SUPP	"_MIPS_eh_region_supp"
#define MIPS_PDR_EXCEPTION	".MIPS.pdr_exception"


/* ====================================================================
 *
 * Symbol table
 *
 * ====================================================================
 */


/* ====================================================================
 *
 * .IRIX.options Section	
 *
 *	[ Mips specific information documented in this file. ]
 *
 * ====================================================================
 */

/*
 *	Elf_Options.kind == ODK_MIPS_EXCEPTIONS	
 *
 *      masks for Elf_Options.info
 */
#define OEX_PAGE0       0x10000 /* page zero must be mapped */
#define OEX_SMM         0x20000 /* Force sequential memory mode? */
#define OEX_FPDBUG      0x40000 /* Force floating point debug mode? */
#define OEX_PRECISEFP   OEX_FPDBUG
#define OEX_DISMISS     0x80000 /* Dismiss invalid address faults? */
#define OEX_FPU_MIN     0x1f    /* FPE's which MUST be enabled */
#define OEX_FPU_MAX     0x1f00  /* FPE's which MAY be enabled */
#define OEX_FPU_INVAL   0x10
#define OEX_FPU_DIV0    0x08
#define OEX_FPU_OFLO    0x04
#define OEX_FPU_UFLO    0x02
#define OEX_FPU_INEX    0x01

/*
 *	Elf_Options.kind == ODK_MIPS_HWPATCH
 *
 *      masks for Elf_Options.info
 */
#define OHW_R4KEOP      0x1     /* R4000 end-of-page patch */
#define OHW_R8KPFETCH   0x2     /* may need R8000 prefetch patch */
#define OHW_R5KEOP      0x4     /* R5000 end-of-page patch */
#define OHW_R5KCVTL     0x8     /* R5000 cvt.[ds].l bug.  clean=1 */
#define OHW_R10KLDL     0x10    /* R10000 requires LDL patch    */

/*
 *      Elf_Options.kind == ODK_MIPS_HWAND
 *
 *      masks for Elf_Options.info
 */
 	    /*
	     * Marks an executable as r4k end of page bug safe.
	     * Both bits must be AND'd for the executable to be
	     * certified safe.
	     *
	     * CHECKED  PRESENT	MEANING
	     *	    1	    1	    certified clean of r4k eop bug
	     * All other variations will render the executable
	     * suspect.
	     */
#define	OHWA0_R4KEOP_CHECKED	0x00000001
#define OHWA0_R4KEOP_CLEAN	0x00000002

/*
 *      Elf_Options.kind == ODK_MIPS_HWAND
 *
 *      masks for following Elf_Options.hwp_flags1
 */
/* #define OHWA1_... */


/*
 *      Elf_Options.kind == ODK_MIPS_HWAND
 *
 *      masks for following Elf_Options.hwp_flags2
 */
/* #define OHWA2_... */

/*
 *      Elf_Options.kind == ODK_MIPS_HWOR
 *
 *      masks for Elf_Options.info
 */
#define OHWO0_FIXADE	0x00000001	/* Object requires FIXADE call */


/*
 *      Elf_Options.kind == ODK_MIPS_HWOR
 *
 *      masks for following Elf_Options.hwp_flags1
 */
/* #define OHWO1_... */


/*
 *      Elf_Options.kind == ODK_MIPS_HWOR
 *
 *      masks for following Elf_Options.hwp_flags2
 */
/* #define OHWO2_... */



/* ====================================================================
 *
 * .gptab Section
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef union
{
	struct
	{
		Elf32_Word	gt_current_g_value;
		Elf32_Word	gt_unused;
	} gt_header;
	struct
	{
		Elf32_Word	gt_g_value;
		Elf32_Word	gt_bytes;
	} gt_entry;
} Elf32_Gptab;
#endif


/* ====================================================================
 *
 * .reginfo Section
 *
 *      The compiler may generate this section and the linker
 *      may generate the corresponding reginfo segment, but
 *      the information contained herein is ignored by Irix.
 *      Instead, Irix gets the appropriate information from
 *      the options segment.
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef struct
{
	Elf32_Word	ri_gprmask;
	Elf32_Word	ri_cprmask[4];
	Elf32_Sword	ri_gp_value;
} Elf32_RegInfo;

typedef struct
{
	Elf64_Word      ri_gprmask;     /* mask of general regs used */
	Elf64_Word      ri_pad;         /* for alignment */
	Elf64_Word      ri_cprmask[4];  /* mask of cop regs used */
	Elf64_Addr      ri_gp_value;    /* initial value of gp */
} Elf64_RegInfo;

#endif


/* ====================================================================
 *
 * .rel, .rela Section
 *
 * ====================================================================
 */

/*
 * relocation types
 */

#define R_MIPS_NONE		0
#define R_MIPS_16		1
#define R_MIPS_32		2
#define R_MIPS_ADD		R_MIPS_32
#define R_MIPS_REL		3
#define R_MIPS_REL32		R_MIPS_REL
#define R_MIPS_26		4
#define R_MIPS_HI16		5
#define R_MIPS_LO16		6
#define R_MIPS_GPREL		7
#define R_MIPS_GPREL16		R_MIPS_GPREL
#define R_MIPS_LITERAL		8
#define R_MIPS_GOT		9
#define R_MIPS_GOT16		R_MIPS_GOT
#define R_MIPS_PC16		10
#define R_MIPS_CALL		11
#define R_MIPS_CALL16		R_MIPS_CALL
#define R_MIPS_GPREL32		12

#define R_MIPS_SHIFT5		16
#define R_MIPS_SHIFT6		17
#define R_MIPS_64		18
#define R_MIPS_GOT_DISP		19
#define R_MIPS_GOT_PAGE		20
#define R_MIPS_GOT_OFST		21
#define R_MIPS_GOT_HI16		22
#define R_MIPS_GOT_LO16		23
#define R_MIPS_SUB		24
#define R_MIPS_INSERT_A		25
#define R_MIPS_INSERT_B		26
#define R_MIPS_DELETE		27
#define R_MIPS_HIGHER		28
#define R_MIPS_HIGHEST		29
#define R_MIPS_CALL_HI16	30
#define R_MIPS_CALL_LO16	31
#define R_MIPS_SCN_DISP		32
#define	R_MIPS_REL16		33
#define R_MIPS_ADD_IMMEDIATE    34
#define R_MIPS_PJUMP    	35
#define R_MIPS_RELGOT		36
#define R_MIPS_JALR	    	37

#define _R_MIPS_COUNT_		38	/* Number of relocations */
	/* _R_MIPS_COUNT_ is not a relocation type, it is
	** a count of relocation types. 
        ** Must be one greater than the highest relocation
        ** type.
	*/

#define R_MIPS_LOVEND0R		100 /* Vendor specific relocations */
#define R_MIPS_HIVENDOR		127



/* ====================================================================
 *
 * .MIPS.content Section
 *
 * sh_type:	SHT_MIPS_CONTENT
 * sh_link:	section header index of section classified
 * sh_info:	0
 * attributes:	SHF_ALLOC, SHF_MIPS_NOSTRIP
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
typedef union {
	struct {                            /* Normal descriptor */
		Elf64_Word      con_info;
		Elf64_Word      con_start;
	} con_y;
	Elf64_Xword con_xval;               /* Extension descriptor */
} Elf64_Content;

/* con_info masks: */
#define __CON64_EMASK   0x80000000
#define __CON64_ESHIFT  31
#define __CON64_KMASK   0x7f000000
#define __CON64_KSHIFT  24
#define __CON64_LMASK   0x00ffffff
#define __CON64_VMASK   0x7fffffffffffffffll

/* Access macros: */
#define ELF64_CON_EXTN(c) \
	(((c).con_y.con_info & __CON64_EMASK)>>__CON64_ESHIFT)
#define ELF64_CON_KIND(c) \
	(((c).con_y.con_info & __CON64_KMASK)>>__CON64_KSHIFT)
#define ELF64_CON_LENGTH(c)     ((c).con_y.con_info & __CON64_LMASK)
#define ELF64_CON_XVAL(c)       ((c).con_xval & __CON64_VMASK)

/* Content kind -- valid for ELF-32 and ELF-64: */
typedef enum {
    CK_NULL	= 0,	    /* Invalid, same as EK_NULL */
    CK_DEFAULT	= 0x30,	    /* Default type of data for section */
    CK_ALIGN	= 0x31,	    /* Alignment for described range */
    CK_INSTR	= 0x32,	    /* Instructions */
    CK_DATA	= 0x33,	    /* Non-address data */
    CK_SADDR_32	= 0x34,	    /* Simple 32-bit addresses */
    CK_GADDR_32	= 0x35,	    /* GP-relative 32-bit addresses */
    CK_CADDR_32	= 0x36,	    /* Complex 32-bit addresses */
    CK_SADDR_64	= 0x37,	    /* Simple 64-bit addresses */
    CK_GADDR_64	= 0x38,	    /* GP-relative 64-bit addresses */
    CK_CADDR_64	= 0x39,	    /* Complex 64-bit addresses */
    CK_NO_XFORM	= 0x3a,	    /* No transformations allowed in this range */
    CK_NO_REORDER = 0x3b,   /* No reordering allowed in this range */
    CK_GP_GROUP = 0x3c,	    /* Text/data in range with length given by
			       second argument references GP group given
			       by first. */
    CK_STUBS	= 0x3d	    /* Text in range is stub code. ULEB */
} Elf_MIPS_Content_Kind;
typedef	Elf_MIPS_Content_Kind	Elf_Content_Kind;


#endif


/* ====================================================================
 *
 * .MIPS.events Section
 *
 * sh_type:	SHT_MIPS_EVENTS
 * sh_link:	section header index of section whose events tracked
 * sh_info:	section header index of associated interface section
 * attributes:	SHF_ALLOC, SHF_MIPS_NOSTRIP
 *
 * ====================================================================
 */

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))
/* Event kind -- valid for ELF-32 and ELF-64: */
typedef enum {
    EK_NULL = 0x00,	    /* No valid information */
    EK_ADDR_RESET = 0x01,   /* Reset offset into associated text section */
    EK_INCR_LOC_EXT = 0x02, /* Increment offset into associated text section */
    EK_ENTRY = 0x03,	    /* Subprogram entrypoint */
    EK_IF_ENTRY = 0x04,	    /* Subprogram entrypoint with associated interface offset */
    EK_EXIT = 0x05,	    /* Subprogram exit */
    EK_PEND = 0x06,	    /* Subprogram end (last instruction) */

    EK_SWITCH_32 = 0x7,	    /* jr for switch stmt, table entries are 32bit */
    EK_SWITCH_64 = 0x8,	    /* jr for switch stmt, table entries are 64bit */
    EK_DUMMY = 0x09,	    /* empty slot */

    EK_BB_START = 0x0a,	    /* Basic block beginning */
    EK_INCR_LOC_UNALIGNED = 0x0b,    /* Increment unaligned byte offset */
    EK_GP_PROLOG_HI = 0x0c, /* Establish high 16bits of GP */
    EK_GP_PROLOG_LO = 0x0d, /* Establish low 16bits of GP */
    EK_GOT_PAGE = 0x0e,	    /* Compact relocation: GOT page pointer */
    EK_GOT_OFST = 0x0f,     /* Compact relocation: GOT page offset */
    EK_HI = 0x10,	    /* Compact relocation: high 16bits of abs. addr */
    EK_LO = 0x11,	    /* Compact relocation: low 16bits of abs. addr */
    EK_64_HIGHEST = 0x12,   /* Compact relocation: most significant 16 bits
			       of a 64bit absolute address */
    EK_64_HIGHER = 0x13,    /* Compact relocation: second most significant
			       16 bits of a 64bit absolute address */
    EK_64_HIGH = 0x14,	    /* Compact relocation: third most significant
			       16 bits of a 64bit absolute address */
    EK_64_LOW = 0x15,       /* Compact relocation: least significant 16 bits
			       of a 64bit absolute address */
    EK_GPREL = 0x16,        /* Compact relocation: GP relative reference */

    EK_DEF = 0x17,	    /* Define new event kind format */

    EK_FCALL_LOCAL = 0x18,	/* point-of-call (jalr) to a local procedure */
    EK_FCALL_EXTERN = 0x19,	/* jalr to extern procedure (small got case) */
    EK_FCALL_EXTERN_BIG = 0x1a,	/* jalr to extern procedure (large got case) */
    EK_FCALL_MULT = 0x1b,	/* jalr to more than one procedure */
    EK_FCALL_MULT_PARTIAL = 0x1c, /* jalr to multiple + unknown procedures */

    EK_LTR_FCALL = 0x1d,	/* jalr to rld lazy-text res.  index of
				   symbol associated. */
    EK_PCREL_GOT0 = 0x1e, 	/* immediate is hi 16 bits of 32-bit
				   constant.  Argument is offset to lo,
				   in instructions, not bytes*/

    /* The following events are reserved for supporting Purify-type tools: */
    EK_MEM_COPY_LOAD = 0x1f,    /* load only for copying data */
    EK_MEM_COPY_STORE = 0x20,   /* store only for copying data --
                                   LEB128 operand is word offset to
                                   paired load */
    EK_MEM_PARTIAL_LOAD = 0x21, /* load for reference to a subset of bytes --
                                   BYTE operand's 8 bits indicate which
                                   bytes are actually used */
    EK_MEM_EAGER_LOAD = 0x22,   /* load is speculative */
    EK_MEM_VALID_LOAD = 0x23,   /* load of data known to be valid */

				  
		/*
		 * Yet to be defined kinds with no fields (like EK_EXIT)
		 */
    EK_CK_UNUSED_NONE_0 = 0x50, /*  */
    EK_CK_UNUSED_NONE_1 = 0x51, /*  */
    EK_CK_UNUSED_NONE_2 = 0x52, /*  */
    EK_CK_UNUSED_NONE_3 = 0x53, /*  */
    EK_CK_UNUSED_NONE_4 = 0x54, /*  */

		/*
		 * Yet to be defined kinds with 1 16 bit field
		 */
    EK_CK_UNUSED_16BIT_0 = 0x55,
    EK_CK_UNUSED_16BIT_1 = 0x56,
    EK_CK_UNUSED_16BIT_2 = 0x57, /*  */
    EK_CK_UNUSED_16BIT_3 = 0x58, /*  */
    EK_CK_UNUSED_16BIT_4 = 0x59, /*  */

		/*
		 * Yet to be defined kinds with 1 32 bit field
		 */
    EK_CK_UNUSED_32BIT_0 = 0x5a, /*  */
    EK_CK_UNUSED_32BIT_1 = 0x5b, /*  */
    EK_CK_UNUSED_32BIT_2 = 0x5c, /*  */

		/*
		 * Yet to be defined kinds with 1 64 bit field
		 */

    EK_CK_UNUSED_64BIT_0 = 0x5d,
    EK_CK_UNUSED_64BIT_1 = 0x5e,
    EK_CK_UNUSED_64BIT_2 = 0x5f, /*  */
    EK_CK_UNUSED_64BIT_3 = 0x60, /*  */
    EK_CK_UNUSED_64BIT_4 = 0x61, /*  */

		/*
		 * Yet to be defined kinds with 1 uleb128 field
		 */
    EK_CK_UNUSED_ULEB128_0 = 0x62, /* */
    EK_CK_UNUSED_ULEB128_1 = 0x63, /*  */
    EK_CK_UNUSED_ULEB128_2 = 0x64, /*  */
    EK_CK_UNUSED_ULEB128_3 = 0x65, /*  */
    EK_CK_UNUSED_ULEB128_4 = 0x66, /*  */
    EK_CK_UNUSED_ULEB128_5 = 0x67, /*  */
    EK_CK_UNUSED_ULEB128_6 = 0x68, /*  */
    EK_CK_UNUSED_ULEB128_7 = 0x69, /*  */
    EK_CK_UNUSED_ULEB128_8 = 0x6a, /*  */
    EK_CK_UNUSED_ULEB128_9 = 0x6b, /*  */


    EK_INCR_LOC = 0x80	    /* Increment offset into associated text section */

} Elf_MIPS_Event_Kind;
typedef	Elf_MIPS_Event_Kind Elf_Event_Kind;

/* The following defines list the various types of operands that are 
 * supported with the EK_DEF event kind.
 */
#define EK_DEF_UCHAR	(1)	    /* unsigned char (8 bits) */
#define EK_DEF_USHORT	(2)	    /* unsigned short (16 bits) */
#define EK_DEF_UINT	(3)	    /* unsigned int (32 bits) */
#define EK_DEF_ULONG	(4)	    /* unsigned long (64 bits) */
#define EK_DEF_ULEB128	(5)	    /* unsigned LEB128 encoded number */
#define EK_DEF_CHAR	(6)	    /* signed char (8 bits) */
#define EK_DEF_SHORT	(7)	    /* signed short (16 bits) */
#define EK_DEF_INT	(8)	    /* signed int (32 bits) */
#define EK_DEF_LONG	(9)	    /* signed long (64 bits) */
#define EK_DEF_LEB128	(10)	    /* signed LEB128 encoded number */
#define EK_DEF_STRING	(11)	    /* null terminated string */
#define EK_DEF_VAR	(12)	    /* variable length field: the first 2
				       bytes is an unsigned short
				       specifying the total number of bytes
				       of this field including the first 2
				       bytes */
#define CK_DEF EK_DEF    

#endif


/* ====================================================================
 *
 * .dynamic Section
 *
 * --- WARNING ---- WARNING ---- WARNING ---- WARNING ---- WARNING ----
 *
 *      TBD:  The .dynamic section has not yet been prepared for
 *	      use in both Irix/mips and Irix/ia64
 *
 *	I believe that most of the tags will be the same in both
 *	Irix/mips and Irix/ia64, but it still needs to be reviewed.
 *	
 *	For now, we'll document them only in mips/elf.h
 * ====================================================================
 */

#define DT_MIPS_RLD_VERSION     0x70000001
#define DT_MIPS_TIME_STAMP      0x70000002
#define DT_MIPS_ICHECKSUM       0x70000003
#define DT_MIPS_IVERSION        0x70000004
#define DT_MIPS_FLAGS           0x70000005
#define DT_MIPS_BASE_ADDRESS    0x70000006
#define DT_MIPS_MSYM            0x70000007
#define DT_MIPS_CONFLICT        0x70000008
#define DT_MIPS_LIBLIST         0x70000009
#define DT_MIPS_LOCAL_GOTNO     0x7000000A
#define DT_MIPS_CONFLICTNO      0x7000000B
#define DT_MIPS_LIBLISTNO       0x70000010
#define DT_MIPS_SYMTABNO        0x70000011
#define DT_MIPS_UNREFEXTNO      0x70000012
#define DT_MIPS_GOTSYM          0x70000013

#ifndef __osf__

#define DT_MIPS_HIPAGENO        0x70000014
#define DT_MIPS_70000015	0x70000015 	  /* reserved */
#define DT_MIPS_RLD_MAP         0x70000016


/*
 *	The following tags are for DELTA_C_PLUS_PLUS only
 *	and are obsolete.  
 */
#if	(defined(_DELTA_C_PLUS_PLUS) && (_DELTA_C_PLUS_PLUS==1))

#define DT_MIPS_DELTA_CLASS 	0x70000017	/* DCC class definitions      */
#define DT_MIPS_DELTA_CLASS_NO 	0x70000018	/* No. DCC class definitions  */
#define DT_MIPS_DELTA_INSTANCE	0x70000019	/* DCC class instances        */
#define DT_MIPS_DELTA_INSTANCE_NO 0x7000001A	/* No. DCC class instances    */
#define DT_MIPS_DELTA_RELOC	0x7000001B	/* DCC relocations            */
#define DT_MIPS_DELTA_RELOC_NO	0x7000001C	/* No.  DCC relocations       */
#define DT_MIPS_DELTA_SYM	0x7000001D	/* DCC symbols Refered to by
						 * DCC relocations            */
#define DT_MIPS_DELTA_SYM_NO	0x7000001E	/* No. DCC symbols	      */
#define DT_MIPS_DELTA_CLASSSYM	0x70000020	/* DCC class declarations     */
#define DT_MIPS_DELTA_CLASSSYM_NO 0x70000021	/* No. DCC class declarations */

#endif	/* _DELTA_C_PLUS_PLUS */


#define	DT_MIPS_CXX_FLAGS	0x70000022	/* Flags:  C++ flavor         */
#define	DT_MIPS_PIXIE_INIT	0x70000023	/* Init code for pixie	      */
#define	DT_MIPS_SYMBOL_LIB	0x70000024
#define DT_MIPS_LOCALPAGE_GOTIDX 0x70000025
#define DT_MIPS_LOCAL_GOTIDX	0x70000026
#define DT_MIPS_HIDDEN_GOTIDX	0x70000027
#define DT_MIPS_PROTECTED_GOTIDX 0x70000028
#define DT_MIPS_OPTIONS		0x70000029	/* Address of .options        */
#define DT_MIPS_INTERFACE	0x7000002a	/* Address of .interface      */
#define DT_MIPS_DYNSTR_ALIGN	0x7000002b
#define DT_MIPS_INTERFACE_SIZE	0x7000002c	/* size of the .interface     */
#define	DT_MIPS_RLD_TEXT_RESOLVE_ADDR 0x7000002d /* Addr of rld_text_resolve  */
#define	DT_MIPS_PERF_SUFFIX	0x7000002e	/* suffix to be added to dso
						 * name before dlopen() call  */
#define DT_MIPS_COMPACT_SIZE	0x7000002f	/* (O32)Size of .compact_rel  */
#define DT_MIPS_GP_VALUE	0x70000030	/* gp value for aux gots      */
#define DT_MIPS_AUX_DYNAMIC	0x70000031      /* Address of aux .dynamic    */

#else  /* __osf__ */

#define DT_MIPS_PACKAGE        	0x70000014
#define DT_MIPS_PACKAGENO       0x70000015
#define DT_MIPS_PACKSYM		0x70000016
#define DT_MIPS_PACKSYMNO	0x70000017
#define	DT_MIPS_IMPACKNO	0x70000018
#define	DT_MIPS_EXPACKNO	0x70000019
#define	DT_MIPS_IMPSYMNO	0x7000001A
#define	DT_MIPS_EXPSYMNO	0x7000001B
#define DT_MIPS_HIPAGENO        0x7000001C

#endif /* __osf__ */

#define RHF_NONE                    0x00000000
			/* A handy name for 'no flags at all'.
			*/

#define RHF_QUICKSTART              0x00000001
			/* RHF_QUICKSTART is turned on by ld if
			   ld determines that at link time the
			   object is quickstartable.
			*/
#define RHF_NOTPOT                  0x00000002 	
			/* RHF_NOTPOT bit non-zero if elf hash 
			   table element count
			   is NOT a Power Of Two 
			   If 0, rld uses mask, else rld uses %
			   (mod) operator to turn hash into index.
			*/
#define RHF_NO_LIBRARY_REPLACEMENT  0x00000004
			/* RHF_NO_LIBRARY_REPLACEMENT
			   is set by ld -no_library_replacement
			   and tells rld not to  honor _RLD*_ROOT
			   or _RLD*_PATH environment variables.
			   from the time detected (during the
			   program execution).
			*/

#define RHF_NO_MOVE                 0x00000008
			/* If RHF_NO_MOVE  is on and a DSO is moved
			   from the address it was linked at, rld
			   will give a fatal error and exit.
			   If it was a dlopen()-type call, the call
			   will fail (but the app will not exit).
			*/
#define RHF_SGI_ONLY                0x00000010
			/* RHF_SGI_ONLY enables various flags, such as the
			   liblist flags so that rld pays
			   attention tothe flags
			*/


#define RHF_GUARANTEE_INIT	    0x00000020
			/* RHF_GUARANTEE_INIT, if on, provokes a fatal
			   error if rld detects a call into a DSO
			   for which the -init code has not been run.

			   This could happen if dso B has dso C on B's
			   liblist and DSO C's init code calls something
			   int DSO B and DSO B has RHF_GUARANTEE_INIT on.
			   This was designed to help implement C++ 
			   initializations in an early C++ release (5.3?).
			   Not used by C++ for quite some time.
			*/
#define RHF_DELTA_C_PLUS_PLUS	    0x00000040
			/* RHF_DELTA_C_PLUS_PLUS tells rld that the
			   executable/DSO is Delta C++.
			*/
#define RHF_GUARANTEE_START_INIT    0x00000080
			/* RHF_GUARANTEE_START_INIT, if on, causes rld to
			   run a target DSO's -init  code
			   in result of a call into that DSO.

			   This could happen if dso B has dso C on B's
			   liblist and DSO C's init code calls something
			   int DSO B and DSO B has 
			   RHF_GUARANTEE_START_INIT on.
			   rld will start running DSO C's -init code and
			   when that calls a DSO  B function, 
			   DSO B's -init code is run before the
			   call to B is allowed to run.
			*/

#define RHF_PIXIE    		    0x00000100
                /* RHF_PIXIE is set in pixified DSOs/executables.
		   Notifies rld(1) to do special handling of
		   dlopen() strings and to save/restore pixie
		   registers (which save/restore is actually not
		   needed as of SpeedShop 1.3).
                */

	
#define	RHF_DEFAULT_DELAY_LOAD	    0x00000200
			/* If DSO B is linked into  DSO A
			   and RHF_DEFAULT_DELAY_LOAD is on in DSO B,
			   ld will mark (in the liblist of DSO A) that
			   DSO B should be delay-loaded by rld, not
			   loaded at the same time A is.
			   (data-dependencies may turn the delay-load off).
			*/
#define	RHF_REQUICKSTART	    0x00000400
			/* If RHF_REQUICKSTART is on, rqs will
			   process the object. If off, 
			   rqs will not process the object unless
			   -force_requickstart  option used.
			   Set by ld or rqs.
			   ld sets it if ld believes the object should
			   allow rqs-ing.  -abi objects do not have
			   this turned on by ld.
			*/
#define	RHF_REQUICKSTARTED          0x00000800
			/* RHF_REQUICKSTARTED  bit non-zero if
			   rqs has processed the object
			   Set by rqs.
			*/
#define RHF_CORD                    0x00001000
			/* RHF_CORD bit is non-zero if
			   the object  has been cord(1)ed
			   set by cord.
			*/
#define RHF_NO_UNRES_UNDEF	    0x00002000
			/* RHF_NO_UNRES_UNDEF is non-zero if
			   every external symbol is resolved
			   (ie, no externals are undefined)
			   Set by ld or rqs.
			*/
#define RHF_RLD_ORDER_SAFE	    0x00004000
			/* If on, RHF_RLD_ORDER_SAFE tells rld it
			   can stop searching for UNDEFineds to
			   resolve when the
			   first non-ABS, non-UNDEF is found
			   (saving time in rld).
			   Set by ld.
			*/
#endif /* __SYS_MIPS_ELF_H__ */
