#ifndef __SYS_NG1_H__
#define __SYS_NG1_H__

/*
 *  $Revision: 1.42 $
 */

#define GFX_NAME_NEWPORT	"NG1"	/* Newport Rex3 graphics board */

#define BROOKTREE_BUG		/* driver and server do stuff to compensate
				 * for BT445 rev A bug. Special timing
				 * tables needed as well. */

typedef struct {
    	int flags;
        short w, h;		/* monitor resolution */
        short fields_sec;	/* fields/sec */
} ng1_vof_info_t;

struct ng1_info {
			/* device independent information */
	struct gfx_info gfx_info; 
			/* ng1 specific information */
	unsigned char boardrev;
	unsigned char rex3rev;
	unsigned char vc2rev;
	unsigned char monitortype;
	unsigned char videoinstalled;
	unsigned char mcrev;
	unsigned char bitplanes;	/* 8 or 24 */
	unsigned char xmap9rev;
	unsigned char cmaprev;
	ng1_vof_info_t ng1_vof_info;
	unsigned char bt445rev;
	unsigned char paneltype;
};

#define NEWPORT_SYNC_VALUE 0

#ifdef _KERNEL

#define NEWPORT_MAP_SIZE 8192		/* 2 pages */

#define MAXNG1          2


/*
 * Status of each gio slot in a system -
 * have we probed, did we find ng1 there.
 * Redundant probes for graphics wreak havoc
 * on fddi.
 */
#define NG1_UNTOUCHED   0
#define NG1_PROBED      1
#define NG1_FOUND       2

struct newportData {
	struct gfx_data	gfx_data;	/* Common gfx driver above */
	struct rex3chip	*rex3;		/* K1 seg board base address */
	struct ng1_info *info;		/* board info for host */
	unsigned int 	attaches;
	void *		Retrace;	/* retrace handler data structure */
	unsigned short 	cursor_xhot;
	unsigned short 	cursor_yhot;
	unsigned int 	cursor_x;
	unsigned int 	cursor_y;
	int		dmainprogress;	/* XXX Don't touch dcb if set */
	void *          dmadap;         /* space for DMA descriptor array */
	char		boardnum;	/* index into various arrays */
	void *		ev1ptr;		/* NULL if no ev1 hw attached */
	int		topline;	/* (topscan % 1024) + 1	*/
					/* can change with setmon */
        int             firsttop;       /* (topscan % 1024) + 1 */
                                        /* topline after initialization,
                                           doesn't change after setmon */
#ifdef NG1FIFOINT
	int		timeout_pending; /* waiting for fifo low intr. */
	int		gfifo_level;
	int		bfifo_level;
	int		ftcallid;
#endif
	unsigned int 	loadedmodes[32];	/* XXX */
};

/*
 * device dependent rendering node
 */
struct newport_rnode {
	short xorg, yorg;	/* geometry at last clip */
	short xsize, ysize;
	short smask_llx, smask_lly; /* GL screen mask */
	short smask_urx, smask_ury;
	char swaps_canceled;	/* for ng1_frameinfo */
};

/*
 * Rex3 regs saved/restored at context switch time.
 */

struct newport_cx {

	/* rex3regs */

    unsigned int drawmode1;        /* extra mode bits for GL       */
    unsigned int drawmode0;        /* command register             */
    unsigned int lsmode;           /* line stipple mode            */
    unsigned int lspattern;        /* 32 bit pixel lspattern       */
    unsigned int lspatsave;        /* save register for lspattern  */
    unsigned int zpattern;         /* 32 bit pixel zpattern        */
    unsigned int colorback;        /* background color             */
    unsigned int colorvram;        /* fast vram clear color        */
    unsigned int alpharef;         /* afunction reference value    */
    unsigned int smask0x;          /* screen mask 0, window rel,   */
    unsigned int smask0y;          /* exclusively for the GL       */
    unsigned int _xstart;          /* 16.4(6) current x            */
    unsigned int _ystart;          /* 16.4(6) current y            */
    unsigned int _xend;            /* 16.4(6)                      */
    unsigned int _yend;            /* 16.4(6)                      */
    unsigned int xsave;            /* 16                      	    */
    unsigned int xymove;           /* x,y copy dest offset         */
    unsigned int bresd;            /* s17.8 bres d error term      */
    unsigned int bress1;           /* s1.15 bres s coverage term   */
    unsigned int bresoctinc1;      /* 3(4),17.3 octant+inc1 value  */
    unsigned int bresrndinc2;      /* s17.3 bres inc2 value        */
    unsigned int brese1;           /* 1.15 bres e1 (minor slope)   */
    unsigned int bress2;           /* s18.8 bres s2 coverage term  */

    unsigned int aweight0;         /* antialiasing weights         */
    unsigned int aweight1;         /* antialiasing weights         */
    unsigned int colorred;         /* o12.11 red (also foreground) */
    unsigned int coloralpha;       /* o8.11 alpha                  */
    unsigned int colorgrn;         /* o8.11 green                  */
    unsigned int colorblue;        /* o8.11 blue                   */
    unsigned int slopered;         /* s9.11                        */
    unsigned int slopealpha;       /* s9.11                        */
    unsigned int slopegrn;         /* s9.11                        */
    unsigned int slopeblue;        /* s9.11                        */
    unsigned int wrmask;           /* writemask                    */
    unsigned int hostrw0;          /* host PIO/DMA port (msw)      */
    unsigned int hostrw1;          /* host PIO/DMA port (lsw)      */

	/* configregs */

    unsigned int smask1x;          /* screenmask1 right,left edges */
    unsigned int smask1y;          /* screenmask1 bottom,top edges */
    unsigned int smask2x;          /* screenmask2 right,left edges */
    unsigned int smask2y;          /* screenmask2 bottom,top edges */
    unsigned int smask3x;          /* screenmask3 right,left edges */
    unsigned int smask3y;          /* screenmask3 bottom,top edges */
    unsigned int smask4x;          /* screenmask4 right,left edges */
    unsigned int smask4y;          /* screenmask4 bottom,top edges */
    unsigned int topscan;          /* y coord of top screen line   */
    unsigned int xywin;            /* window offset                */
    unsigned int clipmode;         /* cid,smask settings           */
    unsigned int config;           /* miscellaneous  config bits   */

	/* dcb registers */
    unsigned int dcbmode;          /* display ctrl bus mode reg    */
    unsigned int dcbdata0;         /* display ctrl bus mode data   */
    unsigned int dcbdata1;         /* display ctrl bus mode data   */
};

#endif /* _KERNEL */

/*
 *   newport private ioctls
 */
#define NG1_BASE			21000
#define NG1_SET_CURSOR_HOTSPOT		(NG1_BASE+1)
#define NG1_PIXELDMA			(NG1_BASE+2)
#define NG1_SET_DESTINATION_PLANES	(NG1_BASE+3)
/* XXX NG1_BASE+4 available */
#define NG1_GET_GEOMETRY		(NG1_BASE+5)
#define NG1_SETDISPLAYMODE              (NG1_BASE+6)
#define NG1_SETGAMMARAMP0               (NG1_BASE+7)
#define NG1_SETGAMMARAMP1               (NG1_BASE+8)
#define NG1_SETGAMMARAMP2               (NG1_BASE+9)
#define NG1_SETVIDEOTIMING		(NG1_BASE+10)
#define NG1_I2C_CMD			(NG1_BASE+11)
#define NG1_SET_SLAVE			(NG1_BASE+12)
#define NG1_GET_FRAMEINFO		(NG1_BASE+13)
#define NG1_SET_FBDEPTH			(NG1_BASE+99)

struct ng1_set_fbdepth {
	int depth;		/* 8/24, allow user to override what kernel says fb depth is */
};

struct ng1_set_cursor_hotspot {
	unsigned short xhot;
	unsigned short yhot;
};

struct ng1_pixeldma_args {
	int	xlen;		/* number of pixels (in bytes) to be DMA'ed */
	int	ylen;		/* number of lines */
	int	flags;		/* NG1_WRITE, NG1_STRIDE, NG1_ZOOM */
        int 	pmstride;       /* Number of bytes in stride */ 
	void 	*gfxaddr;	/* Graphics address to DMA to/from */
        char 	*buf;           /* Data buffer */
	int 	xzoom;		/* x zoom factor (may not be implemented) */
	int 	yzoom;		/* y zoom factor (may not be implemented) */
};

/* Flags for graphics DMA */
#define NG1_WRITE       0x01            /* DMA direction: host to gfx */
#define NG1_STRIDE      0x02            /* Stride DMA */
#define NG1_ZOOM        0x04            /* Pixel Zooming */
#define NG1_BUFLOCKED   0x08            /* Don't need to userdma() */

#define NG1_DMA_MAXSIZE (6*256*1024) 	/* 1.5Mb limit */
#define NG1_DMA_MAXXLEN	(NG1_XSIZE*4)
#define NG1_DMA_MAXYLEN	(NG1_YSIZE)

struct ng1_set_destination_planes {
	unsigned int planes; /* bits to set in aux2 */
};

struct ng1_get_geometry {
	int xorigin, yorigin;
	int xsize, ysize;
};

struct ng1_setdisplaymode_args {
	int wid;
	unsigned int mode;
};

struct ng1_setgammaramp_args {
	/*
	 * Could possibly add another field for which [0,1,2] gamma ramp
	 */
	unsigned char red[256];
	unsigned char green[256];
	unsigned char blue[256];
};

/* Definitions for ng1_timing_info.flags */

#define NG1_VOF_UNKNOWNMON        0x0001          /* unknown monitor */
#define NG1_VOF_INTERLACED        0x0002          /* interlaced output */
#define NG1_VOF_STEREO            0x0004          /* stereo format */
#define NG1_VOF_DO_GENSYNC        0x0008          /* enable incoming sync */
#define NG1_VOF_GENLOCK_SRC1      0x0010          /* genlock source is 1 or 0 */

/* number of extra black pixels at start of scan,
 * divided by 2.  workaround for bt445 rev a bug. 
 * Previous sw rev assumed 2 pixels, and bit 5 set
 * in flags to indicate this.  By using bits 5-7 to
 * record the number of pixels, old timing tables
 * should still work with the new code.
 */

#define NG1_VOF_EXTRAPIX_SHIFT	  5
#define NG1_VOF_EXTRAPIX_MASK	  (7 << NG1_VOF_EXTRAPIX_SHIFT)
/* Timing table channels moved around in ng1 rev 4 */
#define NG1_VOF_NEWSYNC		  (8 << NG1_VOF_EXTRAPIX_SHIFT)

#define NG1_VOF_PRESENTER	  0x200		/* flat panel timing table */
#define NG1_MASTER_SYNC		  0x400         /* board is master sync */
#define NG1_SLAVE_SYNC		  0x800         /* board is slave of framelock*/

struct ng1_timing_info
{
	int flags;          	/* see #defines above */
        short w, h;		/* monitor resolution */
        short fields_sec;	/* fields/sec */
        short cfreq;		/* pixel clock frequency (MHz) */
        unsigned short *ftab;	/* video timing frame table */
        short ftab_len;		/* length (in shorts) of *ftab */
        unsigned short *ltab;	/* video timing line table */
        short ltab_len;		/* length (in shorts) of *ltab */
        unsigned short *plltab;	/* Bt445 pixel clock parameters [ M, N, L ] */
        unsigned char *icsplltab;/* ICS1562 pixel clock parameters [  0,1,2,3,4,5,6 ] */
};

#ifndef _STANDALONE
#include <sys/dmcommon.h> /* for stamp_t */

struct ng1_frameinfo {
        stamp_t ust;            /* UST acquired atomically with 'line' */
	int field;		/* current field number */
	int flags;		/* same as ng1_timing_info */
	short fieldrate;	/* fields per sec */
	short linesperframe;	/* lines per frame, including blanking */
	short line;		/* current scanline being displayed */
	char swap_pending;	/* does caller have a swapbuf request pending */
	char swaps_canceled;	/* counts canceled swap requests */
};
#ifdef _KERNEL
extern void newportFrameInfo (struct newportData *, struct rrm_rnode *,
			      struct ng1_frameinfo *);
#endif
#endif


/*
 * This structure is used to shadow the XMAP9 mode table.
 */

typedef struct _Xmap9ModeTable {
    struct ng1_setdisplaymode_args  didMode;
    int                             hwiMode;
} Xmap9ModeTableRec, *Xmap9ModeTablePtr;

#ifdef _KERNEL
extern int newportProbe(struct rex3chip *, int index);
extern int newportPositionCursor(struct gfx_data *, int, int);
extern void newportInit(struct rex3chip *, struct ng1_info *, int, int);
extern int Ng1PixelDma(struct gfx_gfx *, struct ng1_pixeldma_args *);
extern int newportRetraceHandler(struct newportData *);
extern void newportRetraceInit(struct newportData *);
extern void newportRetraceCloseRN(struct newportData *,struct rrm_rnode *);
extern int newportSetDisplayMode(struct gfx_gfx *, int, unsigned int);
extern int newportSetGammaRamp(struct gfx_gfx *,  unsigned char *,  
				unsigned char *,  unsigned char *, int);
extern int ng1_setvideotiming(struct newportData *, struct ng1_timing_info *);
extern void ng1_error (void);

#ifdef FLATPANEL
extern void fp_reinit (struct newportData *);
extern void ng1_fp_probe (struct ng1_info *);
extern int check_for_panel_again[];
#endif /* FLATPANEL */
#endif /* _KERNEL */

#endif /* __SYS_NG1_H__ */

