
#ifndef __SYS_NG1_CMAP_H__
#define __SYS_NG1_CMAP_H__
/**************************************************************************
 *               Copyright (C) 1990, Silicon Graphics, Inc.               *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 **************************************************************************/

/*
 *   $Revision: 1.27 $
 */

#include "sys/ng1hw.h"

/*******************************************************
 * 		CMAP + RAMDAC definitions and macros
 *******************************************************/

/*
 * These bits go into the DCBCRS field of the Rex3 DCBMODE register
 */


#define CMAP_CRS_ADDR_REG       (0 << DCB_CRS_SHIFT)
#define CMAP_CRS_ADDR_LO        (0 << DCB_CRS_SHIFT)
#define CMAP_CRS_ADDR_HI        (1 << DCB_CRS_SHIFT)
#define CMAP_CRS_PAL_BUF        (2 << DCB_CRS_SHIFT)
#define CMAP_CRS_CMD_REG        (3 << DCB_CRS_SHIFT)
#define CMAP_CRS_STATUS_REG     (4 << DCB_CRS_SHIFT)
#define CMAP_CRS_REV_REG     	(6 << DCB_CRS_SHIFT)
/* Following is only valid on revD.3 cmap. */
#define CMAP_CRS_DIAG_READ 	(7 << DCB_CRS_SHIFT)


/*
 * Define the dcb transfer protocol used by the CMAP.
 */


#define DCB_CMAP_PROTOCOL ((4 << DCB_CSWIDTH_SHIFT) | (1 << DCB_CSHOLD_SHIFT) | (1 << DCB_CSSETUP_SHIFT))

	/*
	 *  Macros to set the CMAP palette buffer
	 */

#define cmapSetAddr( rex3, addr )                                            \
	rex3->set.dcbmode = DCB_CMAP_ALL | DCB_CMAP_PROTOCOL |               \
        	DCB_SWAPENDIAN | DCB_ENCRSINC |				     \
		CMAP_CRS_ADDR_REG | DCB_DATAWIDTH_2 ;  	     		     \
	rex3->set.dcbdata0.byshort.s1 = (addr);                        	     \
	rex3->set.dcbmode = DCB_CMAP_ALL | DCB_CMAP_PROTOCOL |               \
                                  CMAP_CRS_PAL_BUF | DCB_DATAWIDTH_3 ;

#define cmapSetRGB( rex3, r, g, b ) \
	rex3->set.dcbdata0.byword = ( (r) << 24 ) | ( (g) << 16 ) | ( (b) << 8 )

#define cmapFIFOWAIT( rex3 )                                            \
	{                                                               \
	unsigned int tmpmodesave = rex3->set.dcbmode;			\
	rex3->set.dcbmode = DCB_CMAP0 | DCB_CMAP_PROTOCOL |             \
                            CMAP_CRS_STATUS_REG | DCB_DATAWIDTH_1 ;     \
	BFIFOWAIT( rex3 );                                              \
	while ( !(rex3->set.dcbdata0.bybyte.b3 & 0x08) );               \
	rex3->set.dcbmode = tmpmodesave;				\
	}

#define cmapFIFOEMPTY( rex3 )						\
	{								\
	int gt=0;						\
	unsigned int tmpmodesave = rex3->set.dcbmode;			\
	rex3->set.dcbmode = DCB_CMAP0 | DCB_CMAP_PROTOCOL |		\
				CMAP_CRS_STATUS_REG | DCB_DATAWIDTH_1 ;	\
	BFIFOWAIT( rex3 );						\
	while ( rex3->set.dcbdata0.bybyte.b3 & 0x04) ; 			\
	rex3->set.dcbmode = tmpmodesave;				\
	}

/*
 * Some macros for diagnostics.
 */

#define cmapSetReg(rex3, reg, data)					\
	rex3->set.dcbmode = DCB_CMAP_ALL | DCB_CMAP_PROTOCOL |		\
        		reg | DCB_DATAWIDTH_1 ;    			\
	rex3->set.dcbdata0.bybyte.b3 = (data)

/*
 * cmapSetOneReg is like cmapSetReg, but writes to a selected cmap
 * instead of broadcasting to both cmaps.
 */
#define cmapSetOneReg(rex3, reg, chip, data)                            \
        rex3->set.dcbmode = ((chip) ? DCB_CMAP1 : DCB_CMAP0) |  \
                        DCB_CMAP_PROTOCOL |                             \
                        reg | DCB_DATAWIDTH_1 ;                         \
        rex3->set.dcbdata0.bybyte.b3 = (data)

#define cmapGetReg(rex3, reg, which, data)				\
	rex3->set.dcbmode = (which ? DCB_CMAP1 : DCB_CMAP0) | 		\
        	DCB_CMAP_PROTOCOL | (reg) | DCB_DATAWIDTH_1 ;    	\
	data = rex3->set.dcbdata0.bybyte.b3


#define cmapSetReadAddr( rex3, which, addr )				\
	rex3->set.dcbmode = ((which) ? DCB_CMAP1 : DCB_CMAP0) | 	\
        	DCB_CMAP_PROTOCOL | DCB_SWAPENDIAN | DCB_ENCRSINC |	\
		CMAP_CRS_ADDR_REG | DCB_DATAWIDTH_2 ;  	     		\
	rex3->set.dcbdata0.byshort.s1 = (addr);                        	\
	rex3->set.dcbmode = ((which) ? DCB_CMAP1 : DCB_CMAP0) | 	\
		DCB_CMAP_PROTOCOL | CMAP_CRS_PAL_BUF | DCB_DATAWIDTH_3 

#define cmapGetRGB( rex3, r, g, b ) 					  \
	{								  \
        int _temp = rex3->set.dcbdata0.byword;				  \
        r = (_temp & 0xff000000) >> 24;					  \
        g = (_temp & 0x00ff0000) >> 16;					  \
        b = (_temp & 0x0000ff00) >> 8;					  \
	}

#define cmapFIFOWAITCHIP( rex3, which )                                 \
        {                                                               \
        unsigned int tmpmodesave = rex3->set.dcbmode;                   \
        rex3->set.dcbmode = (which ? DCB_CMAP1 : DCB_CMAP0) |  		\
				DCB_CMAP_PROTOCOL |             	\
                            CMAP_CRS_STATUS_REG | DCB_DATAWIDTH_1 ;     \
        BFIFOWAIT( rex3 );                                              \
        while ( !(rex3->set.dcbdata0.bybyte.b3 & 0x08) );               \
        rex3->set.dcbmode = tmpmodesave;                                \
        }

/***********************************************************
 * 	Brooktree 445 RAMDAC definitions and macros        *
 ***********************************************************/

/*
 * These bits go into the DCBCRS field of the Rex3 DCBMODE register
 */

#define RDAC_CRS_ADDR_REG       (0 << DCB_CRS_SHIFT)
#define RDAC_CRS_PRIM_PAL       (1 << DCB_CRS_SHIFT)
#define RDAC_CRS_REVISION       (2 << DCB_CRS_SHIFT)
#define RDAC_CRS_OLAY_PAL       (3 << DCB_CRS_SHIFT)
#define RDAC_CRS_RGB_CTRL       (5 << DCB_CRS_SHIFT)
#define RDAC_CRS_SETUP          (6 << DCB_CRS_SHIFT)

/*
 * Define the Brooktree internal registers.
 */

#define RDAC_REV_REG		0x01
#define RDAC_COMMAND_REG0	0x06
#define RDAC_COMMAND_TEST0	0x07

#define RDAC_RED_MSB_POS	0x0
#define RDAC_RED_WIDTH		0x1
#define RDAC_RED_DISPLAY_ENA	0x2
#define RDAC_RED_BLINK_ENA	0x3

#define RDAC_GREEN_MSB_POS	0x8
#define RDAC_GREEN_WIDTH	0x9
#define RDAC_GREEN_DISPLAY_ENA	0xa
#define RDAC_GREEN_BLINK_ENA	0xb

#define RDAC_BLUE_MSB_POS	0x10
#define RDAC_BLUE_WIDTH		0x11
#define RDAC_BLUE_DISPLAY_ENA	0x12
#define RDAC_BLUE_BLINK_ENA	0x13

#define RDAC_OVL_MSB_POS	0x18
#define RDAC_OVL_WIDTH		0x19
#define RDAC_OVERLAY_CTRL	0x1a
#define RDAC_OVL_BLINK_ENA	0x1b

#define RDAC_CURS_MSB_POS	0x20
#define RDAC_CURS_WIDTH		0x21
#define RDAC_CURSOR_CTRL	0x22
#define RDAC_CURS_BLINK_ENA	0x23


#define RDAC_COMMAND_REG1	0x01
#define RDAC_VIDCLK_CTRL	0x03
#define RDAC_PLL_RATE0		0x05
#define RDAC_PLL_RATE1		0x06
#define RDAC_PLL_CTRL		0x07
#define RDAC_PXL_LOAD_CTRL	0x08
#define RDAC_PXL_START_POS	0x09
#define RDAC_FORMAT_CTRL	0x0A
#define RDAC_MPX_RATE		0x0B
#define RDAC_DEPTH_CTRL		0x0D
#define RDAC_PAL_BYPASS_POS	0x0E

/*
 * Define the value of RAMDAC Command reg1
 */

#define RDAC_COMMAND_REG1_VAL	0xc1
#define RDAC_COMMAND_REG1_GSYNC	0x80

/*
 * Define the dcb transfer protocol used by the RAMDAC.
 */

#define DCB_RDAC_PROTOCOL ((3 << DCB_CSWIDTH_SHIFT) | (1 << DCB_CSSETUP_SHIFT) | (1 <<DCB_CSHOLD_SHIFT))


#define Bt445Set( rex3, crs, dac_reg, data)                               \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL |               \
                            RDAC_CRS_ADDR_REG | DCB_DATAWIDTH_1;          \
	rex3->set.dcbdata0.bybyte.b3 = ( dac_reg );                        \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL | ( crs ) |     \
                            DCB_DATAWIDTH_1;                              \
	rex3->set.dcbdata0.bybyte.b3 = ( data );

#define Bt445Get( rex3, crs, dac_reg, data)                               \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL |               \
                            RDAC_CRS_ADDR_REG | DCB_DATAWIDTH_1;          \
	rex3->set.dcbdata0.bybyte.b3 = ( dac_reg );                        \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL | ( crs ) |     \
                            DCB_DATAWIDTH_1;                              \
	data = rex3->set.dcbdata0.bybyte.b3;

	/*
	 * The following two macros should only be used for setting the
	 * Gamma.
	 */

#define Bt445SetAddr( rex3, addr )                                        \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL |               \
                            RDAC_CRS_ADDR_REG | DCB_DATAWIDTH_1;          \
	rex3->set.dcbdata0.bybyte.b3 = ( addr );                           \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL |               \
                            RDAC_CRS_PRIM_PAL | DCB_DATAWIDTH_3; 

#define Bt445GetAddr(rex3, addr)                                        \
	rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL |             \
                            RDAC_CRS_ADDR_REG | DCB_DATAWIDTH_1;        \
	addr = rex3->set.dcbdata0.bybyte.b3;                           	\

#define Bt445SetRGB( rex3, r, g, b ) \
	rex3->set.dcbdata0.byword = ( (r) << 24 ) | ( (g) << 16 ) | ( (b) << 8 )

#define Bt445GetRGB( rex3, r, g, b ) 					  \
	{								  \
        int _temp = rex3->set.dcbdata0.byword;				  \
        r = (_temp & 0xff000000) >> 24;					  \
        g = (_temp & 0x00ff0000) >> 16;					  \
        b = (_temp & 0x0000ff00) >> 8;					  \
	}

#define Bt445GetByte( rex3, crs, dac_reg, data)                           \
        rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL |               \
                            RDAC_CRS_ADDR_REG | DCB_DATAWIDTH_1;          \
        rex3->set.dcbdata0.bybyte.b3 = ( dac_reg );                        \
        rex3->set.dcbmode = DCB_BT445 | DCB_RDAC_PROTOCOL | ( crs ) |     \
                            DCB_DATAWIDTH_1;                              \
        data = rex3->set.dcbdata0.bybyte.b3

#endif  /* __SYS_NG1_CMAP_H__ */
