/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989-1994 Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef __SYS_PFDAT_H__
#define __SYS_PFDAT_H__
#ident	"$Revision: 3.86 $"

#include "sys/sema.h"
#ifdef USE_LGPAGES
#include <sys/immu.h>
#endif

/*
 *	pf_next, pf_prev	Free list pointers or private pointers
 *				otherwise (buffer cache or getpages).
 *				These _must_ be first in structure.
 *
 *	pf_hchain		Hash chain list.
 *
 *	pf_forw, pf_prev	p_un.vp object list.
 */

typedef struct pfdat {
	struct pfdat	*pf_next;	/* Next free pfdat.	*/
	struct pfdat	*pf_prev;	/* Previous free pfdat.	*/
	int		pf_vcolor:8,	/* Virtual cache color. */
					/* bit 0: PE_RAWWAIT rawwait */
					/* bits 1..7: PE_COLOR  */
			pf_flags:24;	/* Page flags.		*/
	cnt_t		pf_use;		/* Share use count.	*/
	unsigned short	pf_rawcnt;	/* Count of processes	*/
					/* doing raw I/O to page*/
	unsigned long	pf_pageno;	/* Object page number	*/
	union {
		struct vnode	*vp;	/* Page's incore vnode.	*/
		void 		*tag;	/* Generic hash tag.	*/
	} p_un;
	struct pfdat	*pf_hchain;	/* Hash chain link.	*/
} pfd_t, pfde_t;

#define pf_vp		p_un.vp
#define pf_tag		p_un.tag
#define pf_pchain	pf_next
#define pf_vchain	pf_prev

/*
 * pf_flags flags
 */
#define	P_QUEUE		0x0001	/* Page on free queue		*/
#define	P_BAD		0x0002	/* Bad page (parity error, etc.)*/
#define	P_HASH		0x0004	/* Page on hash queue		*/
#define P_DONE		0x0008	/* I/O to read page is done.	*/
#define	P_SWAP		0x0010	/* Page on swap (not file).	*/
#define	P_WAIT		0x0020	/* Waiting for P_DONE		*/
#define	P_DIRTY		0x0040	/* Page stored to programatically */
#define	P_DQUEUE	0x0080	/* Page on vnode dirty queue	 */
#define P_ANON		0x0100	/* Anonymous page		*/
#define P_SQUEUE	0x0200	/* Page on shared anon free queue */
#define P_CW		0x0400	/* Force copy on write		*/
#define P_DUMP		0x0800	/* Dump this page on panics.	*/
#define	P_HOLE		0x1000	/* Page has no backing store	*/
#define	P_CACHESTALE	0x2000	/* Page may be stale in primary caches */

#define P_LPG_256K      0x4000  /* 256K page */
#define P_LPG_1M        0x8000  /* 1M page */
#define P_LPG_4M        0x10000 /* 4M page */
#define P_LPG_16M       0x20000 /* 16M page */

#define P_LARGEPG (P_LPG_256K | P_LPG_1M | P_LPG_4M | P_LPG_16M)

#define P_RESERVED      0x40000

#ifdef _VCE_AVOIDANCE
#define PE_RAWWAIT	0x01	/* some waiting for pf_rawcnt	*/
				/* to change			*/
#define PE_COLOR   	0xFE	/* current virtual color, or -1	*/
				/* to indicate none assigned 	*/

#define PE_COLOR_SHIFT	1

#define PFD_WAKEUP_RAWCNT(pfd) pfd_wakeup_rawcnt(pfd)
#else
#define PFD_WAKEUP_RAWCNT(pfd) ((void) NULL)
#endif

#define PG_ISMOD(P)	((P)->pf_flags & P_DIRTY)

/*
 * Non-existent memory is marked by setting the use count to -1
 * (cause we're out of flag bits).  These pages are never put on
 * a free list and are never referenced.  Some day we'll have more
 * flags, so use the macros below so we can change the implementation.
 */

#define PG_MARKHOLE(P)	((P)->pf_use = -1)
#define PG_ISHOLE(P)	((P)->pf_use == -1)

#if EVEREST

/* On Everest TFP systems, no need for a stale or in-transit queue since
 * the coherency protocol on the bus keeps all physical addresses up to date
 * and there is no danger of VCEs occuring due to 16K page size and 16K
 * d-cache & i-cache sizes.
 *
 * On non-TFP systems we will implement a scheme to avoid VCEs on
 * the kernel stack and take other VCEs. All page lists will be treated
 * as clean and the flushcaches will be avoided.
 */
#define	CLEAN_ASSOC		0	/* Clean, disk association */
#define	CLEAN_NOASSOC		1	/* Clean, no disk association */
#define STALE_ASSOC	CLEAN_ASSOC
#define STALE_NOASSOC	CLEAN_NOASSOC
#define INTRANS_ASSOC	CLEAN_ASSOC
#define INTRANS_NOASSOC	CLEAN_NOASSOC

#define	PH_NLISTS		2

#if !defined(TFP) && !defined(R10000)
#define KSTACKPOOL		1
#define MIN_KSTACK_PAGES 	6
#define KSTACK_TIMEOUT  	(3 * HZ)
#endif /* !TFP && !R10000 */

#else /* !EVEREST */
/*
 * The six free page lists define the state of the page, I-cache wise
 */
#define	CLEAN_ASSOC		0	/* Clean, disk association */
#define	INTRANS_ASSOC		1	/* Being cleaned, no disk association */
#define	STALE_ASSOC		2	/* Stale, disk association */
#define	CLEAN_NOASSOC		3	/* Clean, no disk association */
#define	INTRANS_NOASSOC		4	/* Being cleaned, disk association */
#define	STALE_NOASSOC		5	/* Stale, no disk association */

#define	PH_NLISTS		6
#endif

/*
 * List header type for free page table -- match link pointers in pfd_t.
 */
typedef struct plist {
	struct pfdat	*pf_next;
	struct pfdat	*pf_prev;
} plist_t;

/*
 * Free page table entry
 */
typedef struct phead {
	plist_t	ph_list[PH_NLISTS];	/* list headers */
	int	ph_count;		/* total count on all lists */
	int	ph_flushcount;
	int	ph_flags;
	int	dummy;
} phead_t;

#ifdef _KERNEL

#ifdef USE_LGPAGES

#define LPG_BASESIZE            0x40000         /* 256K page */
#define LPG_BASESHIFT           0x12

#define LPG_SZSHIFT_256K        LPG_BASESHIFT
#define LPG_SZSHIFT_1M          LPG_BASESHIFT+2
#define LPG_SZSHIFT_4M          LPG_BASESHIFT+4
#define LPG_SZSHIFT_16M         LPG_BASESHIFT+6

#define CTOB(shft, x)       ((x)<<(BPCSHIFT+shft))
#define BTOC(shft, x)       (((__psunsigned_t)(x)+(NBPC-1))>>(BPCSHIFT+shft))
#define BTOCT(shft, x)      ((__psunsigned_t)(x)>>(BPCSHIFT+shft))

/* macro to convert page size to mask for C0_PGMASK */
#define pgsize_to_pmask(size)   ((((size) - 1) >> 12) << 13)

/* values to put into pagemask register */
#define PAGE_256K_MASK          0x7E000
#define PAGE_1M_MASK            0x1FE000
#define PAGE_4M_MASK            0x7FE000
#define PAGE_16M_MASK           0x1FFE000

#define MAX_WIRED_LPGSLOTS      16

/* structure for special pages (eg, very large pages)
 * which will get wired into the random slots of the TLB
 */
typedef struct specpg {
	__psunsigned_t 	sp_vaddr;
	int     	sp_pgshft;
	tlbpde_t	sp_pte;
} specpg_t;

typedef struct special_pglist {
	specpg_t        sp_pages[MAX_WIRED_LPGSLOTS];
	short           sp_entries;
	short           sp_last;
} spec_pglist_t;

 /* this tuneable represents the max tlb
  * slots to spend wiring large pages.
  */
extern int max_lpg_tlbslots;

extern int nlpages_16m;         /* 16 Mbyte pages */
extern int nlpages_4m;          /* 4 Mbyte pages */
extern int nlpages_1m;          /* 1 Mbyte pages */
extern int nlpages_256k;        /* 256 Kbyte pages */

extern pfd_t *flpages_16m;
extern pfd_t *flpages_4m;
extern pfd_t *flpages_1m;
extern pfd_t *flpages_256k;


extern void     setup_large_pages(void);
extern int      get_lpage_size(pfd_t *);
extern pfd_t    *lpagealloc(int);
extern void     lpgfree(pfd_t *, int);
extern void     lpguse_dec(pfd_t *, int);
extern void     lpgunresv(pfd_t *, int);
extern void     add_lgpage(spec_pglist_t *, __psunsigned_t, uint, int);
extern void     delete_lgpage(pfd_t *);
extern void     wire_lgpages(spec_pglist_t *, unsigned char);
extern void     clr_lgpages(void);
extern void     free_lgpages(spec_pglist_t *);

#endif /* USE_LGPAGES */

/*
 * Page wait semaphores.
 */
typedef struct pagewait {
	sema_t	pwt_sema;		/* page wait semaphore	*/
	int	pwt_count;		/* wait counter		*/
} pfdwt_t;

/*
 * This is a tuneable used to control whether we do the
 * userdma tracking and error reporting of concurrent
 * user level dma operations into the same memory.
 */
extern int io4ia_userdma_war;

#ifdef EVEREST

#define	PC_NUM_WORDS		4
#define	PC_COUNTERS_PER_WORD	32
typedef struct page_counter {
	__uint64_t	pc_words[PC_NUM_WORDS];
} page_counter_t;

struct proc;
typedef struct dma_info {
	struct proc	*dma_proc;
	caddr_t		dma_uaddr;
	size_t		dma_count;
	caddr_t		dma_kaddr;
	caddr_t		dma_firstkva;
	caddr_t		dma_lastkva;
	struct dma_info	*dma_uhash;
	struct dma_info	*dma_khash;
} dma_info_t;
#endif

extern mutex_t	pinsert_lock;	/* lock on page hash additions	*/
extern lock_t	memory_lock;	/* lock for manipulationg memory*/

/* locks for manipulationg rawcnts -- Everest needs more than one */
#if MP
#define	RAW_CNT	8		/* number of raw locks (must be power of 2) */
#define	RAW_MSK	(RAW_CNT-1)	/* mask for choosing proper raw lock */
extern lock_t	rawmemory_lock[][128/sizeof(lock_t)]; /* cacheline per lock */
#else
extern lock_t	rawmemory_lock;
#endif
extern pfd_t	*pfdat;		/* page frame database --	*/
				/* allocated at bootup		*/
#ifdef EVEREST
extern page_counter_t	*page_counters;
extern pfn_t		num_page_counters;
#endif
extern phead_t	*phead;		/* free page headers		*/
extern pfdwt_t	pfdwait[];	/* page wait objects		*/
extern pfd_t	*pfd_direct;	/* highest directly-mappable pfd*/
extern pfd_t	**phash;	/* hash buckets for active pages*/
extern pfd_t	**phashlast;	/* last hash bucket		*/
extern int	pheadmask;	/* free page hash mask		*/
extern int	phashmask;	/* active page hash mask	*/
extern mutex_t	upage_lock;	/* lock for upage allocations	*/
#ifdef _VCE_AVOIDANCE
extern int	vce_avoidance;	/* set of VCE avoidance enabled */

#define COLOR_VALIDATION(pfd,color,flags,vmflags) \
	(vce_avoidance ? color_validation(pfd,color,flags,vmflags) : 0)
#else
#define COLOR_VALIDATION(pfd,color,flags,vmflags) (0)
#endif 

#define	pfdattophead(pfd)	(&phead[((pfd_t *)pfd - pfdat) & pheadmask])
#define	NPWAIT			64
#define	PWAITMASK		(NPWAIT-1)
#define	pfdattopwt(pfd)		(&pfdwait[((pfd_t *)pfd - pfdat) & PWAITMASK])

#define PHASH(PGNO,I)		(phash[((PGNO) + ((long)(I)>>5)) & phashmask])
#define NEXTPHASH(PL)		((PL) >= phashlast ? &phash[0] : ((PL) + 1))

#define PGNULL		0	/* pf_pageno null value		*/

#define pinsertlock()	mutex_lock(&pinsert_lock, PMEM)
#define pinsertunlock() mutex_unlock(&pinsert_lock)
#define upagelock()	mutex_lock(&upage_lock, PZERO)
#define upageunlock()	mutex_unlock(&upage_lock)

#if MP
#define	rawmemlock(_PFN)	\
	mutex_spinlock(&rawmemory_lock[(_PFN) & RAW_MSK][0])
#define	rawmemunlock(_PFN,T)	\
	mutex_spinunlock(&rawmemory_lock[(_PFN) & RAW_MSK][0], T)
#else
#define	rawmemlock(_PFN)	mutex_spinlock(&rawmemory_lock)
#define	rawmemunlock(_PFN,T)	mutex_spinunlock(&rawmemory_lock, T)
#endif


#define vcache(key,lpn)	(uint)((uint)(key) + lpn)

#ifdef _VCE_AVOIDANCE
#define pfn_to_pfde(pfn) pfntopfdat((pfn))
#define pfd_to_pfde(pfd) (pfd)

#define pfde_to_vcolor(pfd) ((pfd)->pf_vcolor >> PE_COLOR_SHIFT)
#define pfde_set_vcolor(pfd,color) ((pfd)->pf_vcolor = \
				     ((color << PE_COLOR_SHIFT) | \
				      ((pfd)->pf_vcolor & PE_RAWWAIT)))
#define	pfd_to_vcolor(pfd)  (pfde_to_vcolor(pfd_to_pfde(pfd)))
#define	pfn_to_vcolor(pfn)  (pfde_to_vcolor(pfn_to_pfde(pfn)))

#define pfde_to_rawwait(pfde) ((pfd)->pf_vcolor & PE_RAWWAIT)
#define pfde_set_rawwait(pfde) ((pfd)->pf_vcolor |= PE_RAWWAIT)
#define pfde_clear_rawwait(pfde) ((pfd)->pf_vcolor &= ~PE_RAWWAIT)
#endif

/* Function Prototypes */

struct region;
struct proc;
struct vfs;
struct vnode;

extern pfd_t *pagealloc(uint, int);
extern int pagefree(pfd_t *);
extern void pagefreeanon(pfd_t *);
extern void pagefreesanon(pfd_t *, int);
extern pfd_t *pgetsanon(int);
extern pfd_t *pglock(pfd_t *);
extern void pagewait(pfd_t *);
extern void pagedone(pfd_t *);
extern void pagesrelease(pfd_t *, int, int);
extern void pageshole(pfd_t *, int);
extern int pagespatch(pfd_t *, int);    
extern pfd_t *pfind(void *, pgno_t, int);
extern void pinsert(pfd_t *, void *, pgno_t, unsigned);
extern pfd_t *pinsert_try(pfd_t *, void *, pgno_t);
extern void prehash(pfd_t *, void *, pgno_t, unsigned);
extern void pcrehash(void *, pgno_t, void *, pgno_t);
extern int premove(pfd_t *);
extern void pagebad(pfd_t *);
extern void ptagremove(void *, pgno_t, pgno_t);
extern caddr_t pattach(caddr_t);
extern void prelease(caddr_t);
extern caddr_t pflip(caddr_t, caddr_t);
extern void ptossvp(struct vnode *, off_t, off_t);
extern void pflushinvalvp(struct vnode *, off_t, off_t);
extern int pflushvp(struct vnode *, off_t, int);
extern void pinvalfree(struct vnode *, off_t);
extern void pageuseinc(pfd_t *);
extern void pageflags(pfd_t *, int, int);
extern void pageflagsinc(pfd_t *, int, int);
extern void pdinsert(pfd_t *);
extern void pdflush(struct vnode *, int);
extern void pflushinvalvfsp(struct vfs *);

extern int reservermem(int);
extern int reservemem(int, int, int);
extern void unreservemem(int, int, int);
extern void setsxbrk(struct proc *);

extern void	ghostpagedone(int);
extern void	ghostpagewait(struct region *, int);

extern caddr_t	page_mapin(pfd_t *, int, int);
extern caddr_t	page_mapin_pfn(pfd_t *, int, int, uint);
extern void	page_mapout(caddr_t);
extern void	page_copy(pfd_t *, pfd_t *, int, int);
extern void	page_zero(pfd_t *, int, uint, uint);

#ifdef _VCE_AVOIDANCE
extern int pfd_wait_rawcnt(pfd_t *,int);
extern void pfd_wakeup_rawcnt(pfd_t *);
extern void pfd_set_vcolor(pfd_t *,int);
extern int color_validation(register pfd_t *,register int,int,int);
#endif 


#if KSTACKPOOL
extern void	kstack_pool_init(void);
extern pfd_t	*kstack_alloc(void);
extern void	kstack_free(pfd_t *);
extern void	kstack_timeout(void);
#endif

#ifdef EVEREST
extern int	page_counter_incr(pfn_t, int, int);
extern void	page_counter_decr(pfn_t, int, int);
extern int	page_counter_set(pfn_t, int, int);
extern void	page_counter_clear(pfn_t, int, int);
extern void	page_counter_undo(caddr_t, size_t, int);

extern void		dma_info_insert_user(struct proc *, caddr_t,
					     size_t);
extern dma_info_t	*dma_info_lookup_user(struct proc *, caddr_t,
					      size_t, int);
extern void		dma_info_insert_kern(dma_info_t *, caddr_t);
extern dma_info_t	*dma_info_lookup_kern(caddr_t, size_t, int);
extern void		dma_info_free(dma_info_t *);
#endif

#endif /* _KERNEL */
#endif /* !__SYS_PFDAT_H__ */
