/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989-1992 Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _SYS_POLL_H	/* wrapper symbol for kernel use */
#define _SYS_POLL_H	/* subject to change without notice */

#ident "$Revision: 1.24 $"

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long nfds_t;

/*
 * Structure of file descriptor/event pairs supplied in
 * the poll arrays.
 */
struct pollfd {
	int fd;				/* file desc to poll */
	short events;			/* events of interest on fd */
	short revents;			/* events that occurred on fd */
};

/*
 * Testable select events
 */
#define POLLIN		0x0001		/* fd is readable */
#define POLLPRI		0x0002		/* high priority info at fd */
#define	POLLOUT		0x0004		/* fd is writeable (won't block) */
#define POLLRDNORM	0x0040		/* normal data is readable */
#define POLLWRNORM	POLLOUT
#define POLLRDBAND	0x0080		/* out-of-band data is readable */
#define POLLWRBAND	0x0100		/* out-of-band data is writeable */

#define POLLNORM	POLLRDNORM

/*
 * Non-testable poll events (may not be specified in events field,
 * but may be returned in revents field).
 */
#define POLLERR		0x0008		/* fd has error condition */
#define POLLHUP		0x0010		/* fd has been hung up on */
#define POLLNVAL	0x0020		/* invalid pollfd entry */

#if defined(_KERNEL) || defined(_KMEMUSER)
#include <sys/sema.h>

/*
 * Poll list head structure.  A pointer to this is passed
 * to pollwakeup() from the caller indicating the event has
 * occurred.  NOTE: First two pointers correspond to first
 * two elements in polldat structure.
 */
struct pollhead {
	struct polldat	*ph_list;	/* list of pollers */
	lock_t		ph_lock;	/* protects list and events */
	unsigned long	ph_gen;		/* generation number */
	short		ph_events;	/* events pending on list */
	short		ph_user;	/* scratch for owner of handle */
};

#endif /* _KERNEL || _KMEMUSER */

#ifdef _KERNEL
/*
 * Data necessary to notify process sleeping in poll(2)
 * when an event has occurred.
 */
struct polldat {
	struct polldat *pd_next;	/* next in poll list */
	short		pd_events;	/* events being polled */
	short		pd_rotorhint;	/* used in pollwakeup() */
	struct pollhead *pd_headp;	/* backpointer to head of list */
	void		*pd_arg;	/* data saved for pollwakeup */
};

/*
 * Number of pollfd entries to read in at a time in poll.
 * The larger the value the better the performance, up to the
 * maximum number of open files allowed.  Large numbers will
 * use excessive amounts of kernel stack space.
 */
#define NPOLLFILE 32

/*
 * Routine called to notify a process of the occurrence
 * of an event.
 */
extern void pollwakeup	(struct pollhead *, short);
extern void initpollhead	(struct pollhead *);

#else	/* !_KERNEL */

extern int poll(struct pollfd *, nfds_t, int);

#endif	/* !_KERNEL */

#ifdef __cplusplus
}
#endif

#endif	/* _SYS_POLL_H */
