/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ifndef __SYS_REG_H__
#define __SYS_REG_H__
#ident	"$Revision: 3.16 $"

#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)

/*
 * Location of the users' stored
 * registers in exception frame.
 * Usage is u.u_ar0[XX] or USER_REG(x).
 */

#define	EF_ARGSAVE0	0		/* arg save for c calling seq */
#define	EF_ARGSAVE1	1		/* arg save for c calling seq */
#define	EF_ARGSAVE2	2		/* arg save for c calling seq */
#define	EF_ARGSAVE3	3		/* arg save for c calling seq */
#define	EF_AT		4		/* r1:  assembler temporary */
#define	EF_V0		5		/* r2:  return value 0 */
#define	EF_V1		6		/* r3:  return value 1 */
#define	EF_A0		7		/* r4:  argument 0 */
#define	EF_A1		8		/* r5:  argument 1 */
#define	EF_A2		9		/* r6:  argument 2 */
#define	EF_A3		10		/* r7:  argument 3 */
#if (_MIPS_SIM == _MIPS_SIM_ABI32)
#define	EF_T0		11		/* r8:  caller saved 0 */
#define	EF_T1		12		/* r9:  caller saved 1 */
#define	EF_T2		13		/* r10: caller saved 2 */
#define	EF_T3		14		/* r11: caller saved 3 */
#define	EF_T4		15		/* r12: caller saved 4 */
#define	EF_T5		16		/* r13: caller saved 5 */
#define	EF_T6		17		/* r14: caller saved 6 */
#define	EF_T7		18		/* r15: caller saved 7 */
#endif
#if (_MIPS_SIM == _MIPS_SIM_ABI64 || _MIPS_SIM == _MIPS_SIM_NABI32)
#define	EF_A4		11		/* r8:  argument 4 */
#define	EF_A5		12		/* r9:  argument 5 */
#define	EF_A6		13		/* r10: argument 6 */
#define	EF_A7		14		/* r11: argument 7 */
#define	EF_T0		15		/* r12: caller saved 0 */
#define	EF_T1		16		/* r13: caller saved 1 */
#define	EF_T2		17		/* r14: caller saved 2 */
#define	EF_T3		18		/* r15: caller saved 3 */
#endif
#define	EF_S0		19		/* r16: callee saved 0 */
#define	EF_S1		20		/* r17: callee saved 1 */
#define	EF_S2		21		/* r18: callee saved 2 */
#define	EF_S3		22		/* r19: callee saved 3 */
#define	EF_S4		23		/* r20: callee saved 4 */
#define	EF_S5		24		/* r21: callee saved 5 */
#define	EF_S6		25		/* r22: callee saved 6 */
#define	EF_S7		26		/* r23: callee saved 7 */
#define	EF_T8		27		/* r24: code generator 0 */
#define	EF_T9		28		/* r25: code generator 1 */
#define	EF_K0		29		/* r26: kernel temporary 0 */
#define	EF_K1		30		/* r27: kernel temporary 1 */
#define	EF_GP		31		/* r28: global pointer */
#define	EF_SP		32		/* r29: stack pointer */
#define	EF_FP		33		/* r30: frame pointer */
#define	EF_RA		34		/* r31: return address */
#define	EF_SR		35		/* status register */
#define	EF_MDLO		36		/* low mult result */
#define	EF_MDHI		37		/* high mult result */
#define	EF_BADVADDR	38		/* bad virtual address */
#define	EF_CAUSE	39		/* cause register */
#define	EF_EPC		40		/* program counter */
#define	EF_SAVEDPC	41		/* saved program counter (for dbx) */
#if TFP
#define EF_FP4		42		/* FP temp register */
#else
#define EF_LIOMSK	42		/* IP6 local interrupt mask reg */
#endif
#define EF_CONFIG	43		/* TFP C0_CONFIG register (for SMM) */
#define EF_CEL		44		/* Everest Curr Exec Lvl */
#define EF_CRMMSK       44              /* MOOSEHEAD interrupt mask */

#define EF_SIZE		sizeof(struct eframe_s)
/*
 * some instruction operations work much better by assuming an array of
 * registers
 */
#define	USER_REG(x)	(((k_machreg_t *)&u.u_eframe)[(x)])

/*
 * We make everything the same size - even though a few of the registers
 * will never be larger than 32 bits
 * WARNING: some code still accesses this as an array!
 */
typedef struct eframe_s {
	k_machreg_t	ef_argsave0;
	k_machreg_t	ef_argsave1;
	k_machreg_t	ef_argsave2;
	k_machreg_t	ef_argsave3;
	k_machreg_t	ef_at;
	k_machreg_t	ef_v0;
	k_machreg_t	ef_v1;
	k_machreg_t	ef_a0;
	k_machreg_t	ef_a1;
	k_machreg_t	ef_a2;
	k_machreg_t	ef_a3;
#if (_MIPS_SIM == _MIPS_SIM_ABI32)
	k_machreg_t	ef_t0;
	k_machreg_t	ef_t1;
	k_machreg_t	ef_t2;
	k_machreg_t	ef_t3;
	k_machreg_t	ef_t4;
	k_machreg_t	ef_t5;
	k_machreg_t	ef_t6;
	k_machreg_t	ef_t7;
#endif
#if (_MIPS_SIM == _MIPS_SIM_ABI64 || _MIPS_SIM == _MIPS_SIM_NABI32)
	k_machreg_t	ef_a4;
	k_machreg_t	ef_a5;
	k_machreg_t	ef_a6;
	k_machreg_t	ef_a7;
	k_machreg_t	ef_t0;
	k_machreg_t	ef_t1;
	k_machreg_t	ef_t2;
	k_machreg_t	ef_t3;
#define	ef_ta0	ef_a4
#define	ef_ta1	ef_a5
#define	ef_ta2	ef_a6
#define	ef_ta3	ef_a7
#endif
	k_machreg_t	ef_s0;
	k_machreg_t	ef_s1;
	k_machreg_t	ef_s2;
	k_machreg_t	ef_s3;
	k_machreg_t	ef_s4;
	k_machreg_t	ef_s5;
	k_machreg_t	ef_s6;
	k_machreg_t	ef_s7;
	k_machreg_t	ef_t8;
	k_machreg_t	ef_t9;
	k_machreg_t	ef_k0;
	k_machreg_t	ef_k1;
	k_machreg_t	ef_gp;
	k_machreg_t	ef_sp;
	k_machreg_t	ef_fp;
	k_machreg_t	ef_ra;
	k_machreg_t	ef_sr;
	k_machreg_t	ef_mdlo;
	k_machreg_t	ef_mdhi;
	k_machreg_t	ef_badvaddr;
	k_machreg_t	ef_cause;
	k_machreg_t	ef_epc;
	k_machreg_t	ef_savedpc;
#if TFP
	k_machreg_t	ef_fp4;
#else
	k_machreg_t	ef_liomsk;
#endif
	k_machreg_t	ef_config;
#if defined(_K32U32)
	k_machreg_t	ef_unused;	/* padding in 32 bit mode */
#endif
#if IP32
	__uint64_t	ef_crmmsk;	/* CRIME intmask at time of excep. */
#else
	__uint64_t	ef_cel;
#endif
} eframe_t;

#endif /* _LANGUAGE_C || _LANGUAGE_C_PLUS_PLUS */
#endif /* __SYS_REG_H__ */
