/**************************************************************************
 *									  *
 * 		 Copyright (C) 1992-1993 Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ifndef _RUNQ_PRIVATE_H
#define _RUNQ_PRIVATE_H

#include <sys/sema.h>
#include <sys/systm.h>

# if defined(MP)
# define RQ_BIGCONFIG
# define RQ_AGGRESSIVE_GANGS
# else
# define RQ_UP
# endif

/* new-era tracing */
/* turn on here for runq stats (needs ml/trace.s too) */
#if 0
# define RQ_NEWSTATS
#endif

/*
 * SPL level we need to protect ourselves.
 */
# define	splrunq		splhi

# define NDISCIPLINES	3

typedef char	cpucnt_t;

/*
 * By convention, processor set name PSET_ALL means all processors,
 * while set names 0 through BV_MAX indicate a set with only a bit
 * corresponding to the name, i.e., a processor set with only itself in it.
 */
# define	PSET_ALL	0xffffffff
# define	PSET_ACTIVE	0xfffffffe
# define	PSET_QUEUE	0xffffff00	/* and below ... */

/*****************************************************************************/
/*
 * Processor set interface.
 */
/*****************************************************************************/

typedef unsigned long long sbv_t;
typedef sbv_t	*bv_t;
typedef uint	bvname_t;

struct qds;
typedef void	(*invoke_func)(struct qds *, cpuid_t, int);

# define	BV_MAX		64
# define	BV_VEC		1

# define	BVF_SYS		0x1

# define	_bvfSys(b)	((b)->bv_flags |= BVF_SYS)
# define	_bvfUnsys(b)	((b)->bv_flags &= ~BVF_SYS)
# define	_bvfIssys(b)	((b)->bv_flags & BVF_SYS)

/*
 * Implementation dependent definitions. To allow for more bits, just need to
 * change these definitions, and perhaps implement the routines in 
 * assembly code for speed.
 */

/*
 * Macros that must be changed if bit vector is not an integer. I recommend
 * that this not be done for a long time.
 */
#define	copyvBv(b,n)	((n)->bv_vec[0] = *b)
#define	zerovBv(b)	((b)->bv_vec[0] = 0)
#define	isBv(h1, h2)	(((h1)->bv_vec[0])&((h2)->bv_vec[0]))
#define	is3Bv(h1,h2,h3)	(((h1)->bv_vec[0])&((h2)->bv_vec[0])& \
							((h3)->bv_vec[0]))
#if !defined(R4000) && !defined(TFP) && !defined(R10000)
	/*
	 * Won't ever have > 32 on a 32-bit machine.
	 */
#define	isoneBv(h1,n)	(((h1)->bv_vec[0])&((sbv_t)((int)1 << n)))
#define	oneBv(h1,n)	((h1)->bv_vec[0] = (sbv_t)((int)1 << n))
#define	onesetBv(h1,n)	((h1)->bv_vec[0] |= (sbv_t)((int)1 << n))
#define	oneclearBv(h1,n)	((h1)->bv_vec[0] &= \
						(sbv_t) ~((int)1 << n))
#else
#define	isoneBv(h1,n)	(((h1)->bv_vec[0])&((sbv_t)1 << n))
#define	oneBv(h1,n)	((h1)->bv_vec[0] = ((sbv_t)1 << n))
#define	onesetBv(h1,n)	((h1)->bv_vec[0] |= ((sbv_t)1 << n))
#define	oneclearBv(h1,n)	((h1)->bv_vec[0] &= ~((sbv_t)1 << n))
#endif

#define	orvBv(h1, h2)	((h1)->bv_vec[0]|=*(h2))
#define	nullvBv(h1, h2)	((h1)->bv_vec[0]&=~(*(h2)))

#define	copyBv(h1, h2)	((h2)->bv_vec[0]=(h1)->bv_vec[0])
#define	orBv(h1, h2)	((h1)->bv_vec[0]|=(h2)->bv_vec[0])
#define	andBv(h1, h2)	((h1)->bv_vec[0]&=(h2)->bv_vec[0])
#define	allBv(h1)	((h1)->bv_vec[0] = (sbv_t) -1)
#define	compBv(h1)	((h1)->bv_vec[0]=~((h1)->bv_vec[0]))
#define	zeroBv(h1)	((h1)->bv_vec[0] = 0)
#define	nullBv(h1,h2)	((h1)->bv_vec[0]&=~((h2)->bv_vec[0]))

/*
 * Macros that shouldn't need to ever be changed.
 */
#define	nameBv(h1)	((h1)->bv_name)

/*
 * Return the maximum size of a bit vector (the highest bit number, starting
 * from zero).
 */
# define sizeBv()		BV_MAX

typedef struct bvinfo_s {
/*--0--*/
	bvname_t	bv_name;	/* name of the bit vector */
	sbv_t		bv_vec[BV_VEC];	/* the bit vector */
	int		bv_ref;		/* reference count */
	struct bvinfo_s	*bv_next;	/* next vector */
/*--4--*/
	int		bv_flags;	/* control flags */
} bvinfo_t;

/*****************************************************************************/
/*
 * Time is kept in basic clock and microseconds.
 */
/*****************************************************************************/

#define	basictohz(x)	((x) ? (x) / Runq._timer_clkperhz : 0)
#define	basictoHZ(x)	((x) ? (x) / Runq._timer_clkperHZ : 0)
#define	HZtobasic(x)	((x) ? (x) * Runq._timer_clkperHZ : 0)
#define	HZtohz(x)	((x) ? (x) * fasthz : 0)
#define	basictime(x,y)	(x >= y ? (x - y) : (TIMER_MAX - (y - x)))

typedef ulong		basictime_t;
typedef unsigned	micro_t;

/*****************************************************************************/
/*
 * Process queues in the scheduler are represented by a lightweight queue
 * object. This lightweight queue has only a few, fast operations.
 */
/*****************************************************************************/

typedef struct affq_s {
	lock_t		_afflock;
	struct proc	*_arunq;
	struct proc	*_arunqe;	/* seldom used */
	unchar		_highest_pri;
	cpuid_t		_owner;
	int		_ospl;
	int		_nq;
	struct affq_s	*_link;
# ifdef RUNQDEBUG
	micro_t		_maxwait;
	long		_maxlen;
	long		_kicked;
	long		_moves;
# endif
} affq_t;

# define	RQ_AFF_ALL	1
# define	RQ_AFF_EXPIRED	2

/*****************************************************************************/
/*
 * This is the structure that we maintain per CPU out of the PDA for
 * CPU specific scheduling information.
 */
/*****************************************************************************/

typedef struct runqpda {
/*--0--*/
	void		(*rp_pfunc)(void);	/* call on resched */
	int		rp_ospl;	/* saved interrupt level */
	struct qds	*rp_queue;	/* queue owning processor */
	bvinfo_t	*rp_pset;	/* our processor set */
/*--4--*/
        basictime_t     rp_affinity;    /* affinity constant for this CPU */
	basictime_t     rp_warmcache;   /* time to warm up a cache */
	struct pda_s	*rp_pda;	/* pointer to his pda */
	char		rp_tse;		/* if TSE routine queued */
/*--8--*/
	ulong		rp_extime;	/* microseconds executed */
	ulong		rp_nscheds;	/* processes scheduled */
	struct runqpda	*rp_next;	/* to link them together */
	struct gangblock *rp_gb;	/* gang running */
# ifdef RUNQDEBUG
	cpuid_t		rp_owner;
# endif
	affq_t		*rp_localq[1];
} runqpda_t;

# define	p_warmcache		p_runqpda->rp_warmcache
# define	p_affinity		p_runqpda->rp_affinity
# define	p_gb			p_runqpda->rp_gb
# define	p_pfunc			p_runqpda->rp_pfunc
# define	p_queue			p_runqpda->rp_queue
# define	p_ospl			p_runqpda->rp_ospl
# define	p_cpset			p_runqpda->rp_pset
# define	p_pda			p_runqpda->rp_pda
# define	p_pdanext		p_runqpda->rp_next
# define	p_tse			p_runqpda->rp_tse
# define	p_extime		p_runqpda->rp_extime
# define	p_exnscheds		p_runqpda->rp_nscheds
# define	p_localq		p_runqpda->rp_localq
# ifdef RUNQDEBUG
#	define	p_owner			p_runqpda->rp_owner
# endif

/*****************************************************************************/
/*
 * This is the basic interface to the private data we keep for each proc
 * structure while a process is active. This data is dynamically allocated
 * when the process is created, and destroyed when it exits.
 */
/*****************************************************************************/

/*
 * We use the following strategy to "fast-path" process selection from queues.
 * The runqinfo field "ri_fcond" contains a set of flags that are set when
 * exception conditions arise. In general, we expect that a process will have
 * no exception conditions, so that a runnable check becomes a simple test
 * if the bit-vector containing the flags is zero.
 */

/*
 * Exception conditions that might be set.
 */
# if NDISCIPLINES > 3
# define	RQC_MAXCOND	4	/* maximum number of conditions */
# define	RQF_NUNIX	0x8	/* other discipline set */

# define	_isNunix(p)		((p)->p_runq_fcond & RQF_NUNIX)
# define	_makeNunix(p)		(p)->p_runq_fcond |= RQF_NUNIX
# define	_unNunix(p)		(p)->p_runq_fcond &= ~RQF_NUNIX

# else
# define	RQC_MAXCOND	3	/* maximum number of conditions */
# endif

# define	RQF_GFX		0x1	/* process required graphics */
# define	RQF_PSET	0x2	/* non-standard processor set */
# define	RQF_MUSTRUN	0x4	/* mustrun is set */

# define	_isGfx(p)		((p)->p_runq_fcond & RQF_GFX)
# define	_makeGfx(p)		(p)->p_runq_fcond |= RQF_GFX
# define	_unGfx(p)		(p)->p_runq_fcond &= ~RQF_GFX
# define	_isPset(p)		((p)->p_runq_fcond & RQF_PSET)
# define	_makePset(p)		(p)->p_runq_fcond |= RQF_PSET
# define	_unPset(p)		(p)->p_runq_fcond &= ~RQF_PSET
# define	_isMustrun(p)		((p)->p_runq_fcond & RQF_MUSTRUN)
# define	_makeMustrun(p)		(p)->p_runq_fcond |= RQF_MUSTRUN
# define	_unMustrun(p)		(p)->p_runq_fcond &= ~RQF_MUSTRUN


/*
 * Indexes for the overall process time counters.
 */
# define	RQ_TS_RUN	0
# define	RQ_TS_WAIT	1
# define	RQ_TS_QUEUE	2
# define	RQ_TSMAX	3

typedef struct runqinfo {
/*--0--*/
	/* control flags */
	ushort		ri_flags;	/* private flags to runq */
	unchar		ri_fcond;	/* condition flags */
	char		ri_tactive;	/* currently active timer */
	char		ri_tlast;	/* previous timer */
	cpuid_t		ri_endset;	/* CPU expecting TSE */
	char		ri_exwait;	/* waiting on timeout to run */

	/* affinity information */
	basictime_t	ri_btlast;	/* last basic time */
	struct pda_s	*ri_prinfo;	/* info on processor running us */
/*--4--*/
	time_t		ri_sanity;	/* timer sanity check */
	basictime_t	ri_affinity;	/* affinity countdown timer */
	basictime_t	ri_stamp;	/* timer stamp */
	ulong		ri_exstamp;	/* sched extime stamp */

	/* processors can run on */
	bvinfo_t	*ri_pset;	/* processor bit vector */
/*--8--*/
	unsigned	ri_HZsnap;	/* overflow snap */
	int		ri_scheds;	/* number of rescheds */
	int		ri_affhits;	/* number of affinity hits */

	/* deadline scheduler information */
	basictime_t	ri_deadline;	/* deadline period */
	basictime_t	ri_dlcnt;	/* time-to-deadline */
	basictime_t	ri_dlwant;	/* allocation */
	basictime_t	ri_dlrun;	/* actual execution time */
	basictime_t	ri_dlstamp;	/* period timestamp */
	int		ri_dlmiss;	/* missed periods */
	int		ri_period;	/* new period flag */

	/*
	 * Frame Scheduler
	 */
        lock_t		ri_frslock;   /* lock to protect access to */
				      /* following fields */
	uint            ri_frsflags;  /* flags  for the frame scheduler */
        uint            ri_frsrefs;   /* references from frs queues */
	void*           ri_frs;       /* frame scheduler for this process */

} runqinfo_t;

# define p_runq_fcond		p_runqinfo->ri_fcond
# define p_runq_flags		p_runqinfo->ri_flags
# define p_runq_deadline	p_runqinfo->ri_deadline
# define p_runq_dlwant		p_runqinfo->ri_dlwant
# define p_runq_dlrun		p_runqinfo->ri_dlrun
# define p_runq_dlcnt		p_runqinfo->ri_dlcnt
# define p_runq_dlstamp		p_runqinfo->ri_dlstamp
# define p_runq_dlmiss		p_runqinfo->ri_dlmiss
# define p_runq_HZsnap		p_runqinfo->ri_HZsnap
# define p_runq_tactive		p_runqinfo->ri_tactive
# define p_runq_affinity	p_runqinfo->ri_affinity
# define p_runq_period		p_runqinfo->ri_period
# define p_runq_cycles		p_runqinfo->ri_cycles
# define p_runq_tsend		p_runqinfo->ri_tsend
# define p_runq_endset		p_runqinfo->ri_endset
# define p_runq_pset		p_runqinfo->ri_pset
# define p_runq_stamp		p_runqinfo->ri_stamp
# define p_runq_exstamp		p_runqinfo->ri_exstamp
# define p_runq_sanity		p_runqinfo->ri_sanity
# define p_runq_tlast		p_runqinfo->ri_tlast
# define p_runq_btlast		p_runqinfo->ri_btlast
# define p_runq_prinfo		p_runqinfo->ri_prinfo
# define p_runq_exwait		p_runqinfo->ri_exwait
# define p_runq_exproc		p_runqinfo->ri_exproc
# define p_runq_exfired		p_runqinfo->ri_exfired
# define p_runq_scheds		p_runqinfo->ri_scheds
# define p_runq_affhits		p_runqinfo->ri_affhits
# define p_runq_frslock         p_runqinfo->ri_frslock
# define p_runq_frsflags        p_runqinfo->ri_frsflags
# define p_runq_frsrefs         p_runqinfo->ri_frsrefs
# define p_runq_frs             p_runqinfo->ri_frs

/*
 * Allowable p_runq_flags bits to be set.
 */
# define	RQF_RELEASED	0x1	/* process has been released */
# define	RQF_NONPRIV	0x2	/* also in non-priv alloc bucket */
# define	RQF_ONLY	0x4	/* only run during allocation */
# define	RQF_WAITING	0x8	/* waiting for next allocation */
# define	RQF_DEADLINE	0x10	/* deadline scheduling on */
# define	RQF_NOGANG	0x20	/* temporary suppress gang */
# define	RQF_EQUAL	0x40	/* running SGS_EQUAL process */
# define	RQF_TACTIVE	0x80	/* state timers active */
# define	RQF_EVTIMER	0x100	/* wait for timer event */
# define	RQF_JOINED	0x200	/* have join semaphore */

# define	_isDeadline(p)		((p)->p_runq_flags & RQF_DEADLINE)
# define	_makeDeadline(p)	(p)->p_runq_flags |= RQF_DEADLINE
# define	_unDeadline(p)		(p)->p_runq_flags &= ~RQF_DEADLINE
# define	_isWaiting(p)		((p)->p_runq_flags & RQF_WAITING)
# define	_makeWaiting(p)		(p)->p_runq_flags |= RQF_WAITING
# define	_unWaiting(p)		(p)->p_runq_flags &= ~RQF_WAITING
# define	_isNonpriv(p)		((p)->p_runq_flags & RQF_NONPRIV)
# define	_makeNonpriv(p)		(p)->p_runq_flags |= RQF_NONPRIV
# define	_unNonpriv(p)		(p)->p_runq_flags &= ~RQF_NONPRIV
# define	_isOnly(p)		((p)->p_runq_flags & RQF_ONLY)
# define	_makeOnly(p)		(p)->p_runq_flags |= RQF_ONLY
# define	_unOnly(p)		(p)->p_runq_flags &= ~RQF_ONLY
# define	_isReleased(p)		((p)->p_runq_flags & RQF_RELEASED)
# define	_makeReleased(p)	(p)->p_runq_flags |= RQF_RELEASED
# define	_unReleased(p)		(p)->p_runq_flags &= ~RQF_RELEASED
# define	_isTactive(p)		((p)->p_runq_flags & RQF_TACTIVE)
# define	_makeTactive(p)		(p)->p_runq_flags |= RQF_TACTIVE
# define	_unTactive(p)		(p)->p_runq_flags &= ~RQF_TACTIVE
# define	_isSlppr(p)		((p)->p_runq_flags & RQF_SLPPR)
# define	_makeSlppr(p)		(p)->p_runq_flags |= RQF_SLPPR
# define	_unSlppr(p)		(p)->p_runq_flags &= ~RQF_SLPPR
# define	_isEvtimer(p)		((p)->p_runq_flags & RQF_EVTIMER)
# define	_makeEvtimer(p)		(p)->p_runq_flags |= RQF_EVTIMER
# define	_unEvtimer(p)		(p)->p_runq_flags &= ~RQF_EVTIMER
/*
 * Frame Scheduler State
 */
#define         FRSF_ON         0x001    /* frame scheduled process */
#define         FRSF_CONTROLLER 0x002    /* frame scheduler controller process */
#define         FRSF_RUN        0x004    /* process has been run within minor frame */
#define         FRSF_YIELD      0x008    /* process has yielded */
#define         FRSF_JOIN       0x010    /* process has joined */
  
#define         _isFrameScheduled(p)    ((p)->p_runq_frsflags & FRSF_ON)
#define         _makeFrameScheduled(p)  (p)->p_runq_frsflags |= FRSF_ON
#define         _unFrameScheduled(p)    (p)->p_runq_frsflags &= ~FRSF_ON
#define         _initFrameScheduled(p)  (p)->p_runq_frsflags = FRSF_ON
#define         _endFrameScheduled(p)   (p)->p_runq_frsflags = 0

#define         _isFRSController(p)     ((p)->p_runq_frsflags & FRSF_CONTROLLER)
#define         _makeFRSController(p)   (p)->p_runq_frsflags |= FRSF_CONTROLLER
#define         _unFRSController(p)     (p)->p_runq_frsflags &= ~FRSF_CONTROLLER
#define         _endFRSController(p)    (p)->p_runq_frsflags = 0

#define         _isFSRun(p)             ((p)->p_runq_frsflags & FRSF_RUN)
#define         _makeFSRun(p)           (p)->p_runq_frsflags |= FRSF_RUN
#define         _unFSRun(p)             (p)->p_runq_frsflags &= ~FRSF_RUN

#define         _isFSYield(p)           ((p)->p_runq_frsflags & FRSF_YIELD)
#define         _makeFSYield(p)         (p)->p_runq_frsflags |= FRSF_YIELD
#define         _unFSYield(p)           (p)->p_runq_frsflags &= ~FRSF_YIELD

#define         _isFSJoin(p)            ((p)->p_runq_frsflags & FRSF_JOIN)
#define         _makeFSJoin(p)          (p)->p_runq_frsflags |= FRSF_JOIN
#define         _unFSJoin(p)            (p)->p_runq_frsflags &= ~FRSF_JOIN

#define         _isFSMaster(p)          ((p)->p_runq_frsflags & FRSF_MASTER)
#define         _makeFSMaster(p)        (p)->p_runq_frsflags |= FRSF_MASTER
#define         _unFSMaster(p)          (p)->p_runq_frsflags &= ~FRSF_MASTER

#define         _isFSAllocedFrsmpr(p)   ((p)->p_runq_frsflags & FRSF_AFRSMPR)
#define         _makeFSAllocedFrsmpr(p) (p)->p_runq_frsflags |= FRSF_AFRSMPR
#define         _unFSAllocedFrsmpr(p)   (p)->p_runq_frsflags &= ~FRSF_AFRSMPR

#define         _isFSAllocedFrssync(p)  ((p)->p_runq_frsflags & FRSF_AFRSSYNC)
#define         _makeFSAllocedFrssync(p) (p)->p_runq_frsflags |= FRSF_AFRSSYNC
#define         _unFSAllocedFrssync(p)  (p)->p_runq_frsflags &= ~FRSF_AFRSSYNC

  
/*****************************************************************************/
/*
 * A QueueDesc is a descriptor for creation/management of a queue of
 * runnable processes. The low and high priorities provide a first-pass
 * indication of candidate queues for a process. The actual queue insertion
 * routine may choose to reject a process if other conditions are not met.
 * This allows several different queues to cover the same priority ranges
 * but with different parameters.
 */
/*****************************************************************************/

/*
 * Can no processors be assigned to the queue?
 */
# define	QDEMPTYSET	1
# define	QDNONEMPTYSET	2

/*
 * Process priority update interval, in clock ticks. Typically used only
 * for gang scheduling.
 */
# define	QDV_UPINT	50

/*
 * In large configurations, does this queue support a local queue as well?
 */
# define	QDLOCALQ	1
# define	QDNOLOCALQ	2

/*
 * Available lock states.
 */

# define	QDS_AVAIL	0
# define	QDS_READ	1
# define	QDS_WRITE	2

/*
 * The types of queues available.
 */
# define	QDABS		1	/* absolute */
# define	QDAFFINE	2	/* processor affinity */
# define	QDDEADLINE	3	/* deadline/timeslice */
# define	QDGANG		4	/* gang */
# define	QDBGANG		5	/* batch gang */
# define	QDBAFFINE	6	/* batch affine */
# define	QDRT		7	/* user real-time */

struct qds;
struct proc;
struct shaddr_s;
typedef int (*infoproc_t)(struct qds *, int *, struct proc *, int, ...);
typedef struct qds {
	affq_t		_globalq;
	char		_lockstate;
	cpucnt_t	_nreaders;
	cpucnt_t	_writewait;
	int		_avail;
	int		_ospl;
	int		_nonq;
	int		_upint;

	__int32_t	_spacer[26];

	char		_iflocalq;
	struct gangblock *_gangq;
	struct gangblock *_gangqe;
	cpucnt_t	_ncpu;
	unchar		_lowval;
	unchar		_highval;
	char		_type;
	char		_nullset;
	bvinfo_t	*_pset;

	bvinfo_t	_activeset;	/* processors active to queue */
	bvinfo_t	_runset;	/* processors running from queue */
	bvinfo_t	_avset;		/* processors available to queue */

	int		(*_putq)(struct qds *, struct proc *);
	struct proc	*(*_getq)(struct qds *, int);
	int		(*_timeq)(struct qds *);
	void		(*_exitq)(struct qds *, struct proc *);
	void		(*_forkq)(struct proc *);
	struct proc	*(*_delq)(struct qds *, struct proc *);
	infoproc_t	_infoq;
	void		(*_endshaddrq)(struct qds *, struct shaddr_s *);
	void		(*_joinq)(struct qds *, cpuid_t, int);
	void		(*_leaveq)(struct qds *, cpuid_t);

	/* deadline info */
	int		_dlnonpriv;
	int		_dluse;

	/* link (when needed) */
	struct qds	*_next;
	struct qds	*_link;
	char		_spass;
	char		_qnum;
} QueueDesc;

# define _runq		_globalq._arunq
# define _runqe		_globalq._arunqe
# define _qlock		_globalq._afflock

/*****************************************************************************/
/*
 * There is one Runq structure in the system. It holds the information
 * necessary to manage scheduling of processes. The run queue will only
 * access the following fields of the process structure:
 *
 *	p_rtpri		- non-degrading priority process
 *	p_upri		- user mode priority
 *	p_pri		- kernel mode priority
 *	p_dvec		- scheduling discipline vector
 *	p_flink		- runq linked list
 *	p_blink		- runq linked list backward pointer
 *	p_mustrun	- absolutely, positively on this processor
 *	p_lastrun	- last processor this process ran on
 *	p_sonproc	- processor the process is assigned to
 *	p_onrq		- process is on the run queue
 *
 * The Runq also access fields of the private data area for each processor.
 * The fields accessed are:
 *
 *	p_cpuid		- processor id
 *	p_lticks	- time slice for next process
 *	p_runrun	- force dispatch cycle
 */
/*****************************************************************************/

struct sched_disc {		/* information per discipline */
	int	(*sd_disc)(struct proc *);	/* discipline entry point */
	struct bvinfo_s	*sd_pset;		/* processors it can run on */
};

typedef struct runq_s {
	QueueDesc	*_usertsq;
	QueueDesc	*_userrtq;
	QueueDesc	*_batchq;
	QueueDesc	*_deadq;

	lock_t 		_synclock;

	micro_t		_default_slice;
	volatile char	_running;		/* non-zero when active */
	unchar		_do_affinity;		/* if affinity is to be done */
	cpuid_t		_highest_active;	/* highest number active CPU */
	unchar		_spass;			/* for initializing */

	QueueDesc	*_qlist;		/* available queues */
	QueueDesc	*_active_gangq;		/* active gang queues */

	bvinfo_t	*_active_pset;		/* all active CPU's */
	struct runqpda	*_pdalist[BV_MAX];	/* information for active */

	/* HZ based time */
	unsigned	_timer_maxHZ;
	int		_timer_clkperHZ;
	int		_timer_clkperhz;

	/*
	 * changing stuff below
	 */
	__int32_t	_spacer[24];

	/*
	 * Track lowest priority CPU.
	 * XXX Keep lowest cpu and lowest cpu priority in a single word
	 * XXX to avoid locking.
	 */
	int		_lowu;
# define _lowupri(x)	((x) & 0xff)
# define _lowucpu(x)	(((x) >> 8) & 0xff)
# define _makelowu(c,p)	((((c)&0xff)<<8)|(p)&0xff)

	time_t		_timestamp;

	/* global statistics */
	__int64_t	_scheds;		/* number of scheduling trips */
	__int64_t	_affhits;		/* affinity hits */
	__int64_t	_totq;			/* total q wait (usecs) */

	/* synchronization and mutual exclusion */
	lock_t		_runqlock;
	mutex_t		_runqsema;
	sv_t		_waitsem;

	/* acceptable disciplines XXX Will go away eventually */
	struct sched_disc 	_sdisc[NDISCIPLINES];
} runq_t;

/*****************************************************************************/
/*
 * The gang control block. One of these is allocated when a share group
 * enters gang scheduling mode. It is deallocated when all members of
 * the share group have exited.
 */
/*****************************************************************************/

typedef enum {
	GANG_ACTIVE,		/* gang is running */
	GANG_START,		/* gang is starting up */
	GANG_STOP,		/* gang is shutting down */
	GANG_READY		/* gang is ready to run */
} gang_state;

typedef struct gangblock {
	/* update lock */
	lock_t		g_lock;
	int		g_ospl;
# ifdef RUNQDEBUG
	int		g_owner;
# endif

	/* main key - the shared address block */
	struct shaddr_s	*g_sa;		/* shared address block */
	int		g_sequence;

	/* stuff to do with maintaining the runnable processes */
	struct proc	*g_runq;	/* runnable members of the gang */
	struct proc	*g_runqe;	/* end of runnable list */
	int		g_runcnt;	/* number on the list */
	unchar		g_flags;	/* gang flags */
	unchar		g_pri;		/* highest priority member queued */
	QueueDesc	*g_queue;	/* owning queue */

	/* scheduling parameters */
	short		g_mincpu;	/* minimum CPU's before running */
	short		g_maxcpu;	/* maximum CPU's in gang */

	/* reservation accounting */
	gang_state	g_state;
	int		g_reserved;	/* count of reserved CPU's */
	ulong		g_stamp;	/* how much of time slice has passed */
	basictime_t	g_tslice;	/* how big is a time slice? */
	basictime_t	g_mslice;	/* microsecond time slice */

	/* list linkage */
	struct gangblock *g_next;	/* next gang */
	struct gangblock *g_last;	/* previous gang */
} gangblock_t;

# define	RQFG_ROUND	1	/* force round-robin (override aff) */
# define	RQFG_ROUNDPRED	2	/* predict affinity override */
# define	RQFG_EQUAL	3	/* equal process running */

# define	_isGround(g)		((g)->g_flags & RQFG_ROUND)
# define	_makeGround(g)		(g)->g_flags |= RQFG_ROUND
# define	_unGround(g)		(g)->g_flags &= ~RQFG_ROUND
# define	_isGrPred(g)		((g)->g_flags & RQFG_ROUNDPRED)
# define	_makeGrPred(g)		(g)->g_flags |= RQFG_ROUNDPRED
# define	_unGrPred(g)		(g)->g_flags &= ~RQFG_ROUNDPRED
# define	_isEqual(g)		((g)->g_flags & RQFG_EQUAL)
# define	_makeEqual(g)		(g)->g_flags |= RQFG_EQUAL
# define	_unEqual(g)		(g)->g_flags &= ~RQFG_EQUAL

/*****************************************************************************/
/*
 * Scheduler statistics interface.
 */
/*****************************************************************************/

# define SBUFSZ		1024
# ifdef RQ_NEWSTATS

extern int	runq_dl_log;

# define	TR_GANG			101	/* gang event */
# define	TR_DEAD			102	/* deadline event */
# define	TR_RUNQ			103	/* runq event */
# define	TR_FUNC			104	/* function event */

extern void	_trgang(int, gangblock_t *, proc_t *, sbv_t);
# define trgang(a, g, p, s) \
	if (runq_dl_log) _trgang(a, g, p, s); else
extern void	_trdead(int, proc_t *, basictime_t);
# define trdead(a, p, t)	\
	if (runq_dl_log) _trdead(a, p, t); else
extern void	_trrunq(int, proc_t *, int);
# define trrunq(a, p, c)	\
	if (runq_dl_log) _trrunq(a, p, c); else

void	_tr_on(void);
void	_tr_off(void);

# define	GNGACTIVATE		20	/* gang activation */
# define	GNGDEACTIVATE		21	/* gang deactivation */
# define	GNGNEW			23	/* gang creation */
# define	GNGEND			24	/* gang destruction */
# define	GNGRUN			25	/* run a gang member */
# define	GNGYIELD		26	/* yield a processor */

# define	DEDPERIOD		30	/* new period */
# define	DEDALLOC		31	/* used all allocated time */
# define	DEDTIMERSTART		32	/* timer post */
# define	DEDTIMERSTOP		33	/* timer expire */
# define	DEDONQ			34	/* queued a deadline process */
# define	DEDRUN			35	/* run a deadline process */
# define	DEDPUTQ			36	/* examine a deadline process */
# define	DEDTIMERSKIP		37	/* skip wait on timer */

# define	RNQNOW			40	/* run it right away */
# define	RNQRUN			41	/* run a process */
# define	RNQWAT			42	/* wait on event */
# define	RNQQUE			43	/* enter process on queue */

# define	FNCENTRY		10	/* enter function */
# define	FNCEXIT			11	/* exit function */

typedef struct {
	basictime_t		timestamp;	/* time record made */
	char			type;		/* record type */
	cpuid_t			cpu;		/* processer doing it */
	ushort			flags;		/* active flags */
	union {
		struct {
			char	type;		/* function event type */
			int	ordinal;	/* function ID */
		} function;
		struct {
			char	type;		/* gang event type */
			cpuid_t	prnum;		/* processor number */
			pid_t	pid;		/* process involved */
			int	sequence;	/* gang sequence number */
			sbv_t	procs;		/* activate/deactivate procs */
		} gang;
		struct {
			char	type;		/* deadline event type */
			cpuid_t	cpu;		/* processor involved */
			pid_t	pid;		/* process involved */
			basictime_t tleft;	/* time left */
		} deadline;
		struct {
			char	type;		/* runq event type */
			cpuid_t	cpu;		/* processor involved */
			pid_t	pid;		/* process involved */
		} runq;
	} tr;
} rqtrace_t;

# else

/*
 * If no statistics interface defined.
 */
#	define	trgang(a, b, c, d)
#	define	trdead(a, p, t)
#	define	trrunq(a, p, c)
#	define	_tr_on()
#	define	_tr_off()
# endif
# endif  /* _RUNQ_PRIVATE_H */
