/**************************************************************************
 *									  *
 * 		 Copyright (C) 1986, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
#ifndef __SYS_SCHEDCTL_H__
#define __SYS_SCHEDCTL_H__

#ifdef __cplusplus
extern "C" {
#endif

#ident	"$Revision: 3.34 $"

#include "sys/sysmp.h"
#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)
#include "sys/time.h"
#endif

/* defines for schedctl() commands */
#define	NDPRI		MPTS_RTPRI
#define	SLICE		MPTS_SLICE
#define	RENICE		MPTS_RENICE
#define DEADLINE	MPTS_DEADLINE
#define EVENT		MPTS_EVENT

/*
 * Additional schedctl commands.  These give precise emulation of the BSD4.3
 * getpriority and setpriority system calls.  They may be invoked through the
 * schedctl interface or by the direct system call hooks.
 */
#define RENICE_PROC	MPTS_RENICE_PROC
#define RENICE_PGRP	MPTS_RENICE_PGRP
#define RENICE_USER	MPTS_RENICE_USER
#define	GETNICE_PROC	MPTS_GTNICE_PROC
#define GETNICE_PGRP	MPTS_GTNICE_PGRP
#define GETNICE_USER	MPTS_GTNICE_USER

/* More schedctl commands */
#define SETHINTS	MPTS_SETHINTS
#define SCHEDMODE	MPTS_SCHEDMODE
#define SETMASTER	MPTS_SETMASTER
#define SETPSET		MPTS_PSETPROC
#define UNSETPSET	MPTS_PSETUNPROC
#define SETQPSET	MPTS_PSETQ
#define PSETDISC	MPTS_PSETDISC

/* schedctl inquiry codes */
#define GETNDPRI	MPTS_GETRTPRI

#ifdef _VM_WS_SWAPPING
/* schedctl commands to control working set properties. */
#define WS_PRIORITY	MPTS_WS_PRIORITY
#endif


/* possible non-degrading priorities */

/* these priorities are higher than ALL normal user process priorities */
#define NDPHIMAX	30
#define NDPHIMIN	39
#define ishiband(x)	((x) >= NDPHIMAX && (x) <= NDPHIMIN)

/* these priorities overlap normal user process priorities */
#define NDPNORMMAX	40
#define NDPNORMMIN	127

/* these priorities are below ALL normal user process priorities */
#define NDPLOMAX	128
#define NDPLOMIN	254

/* values for share group scheduling mode (schedctl(SCHEDMODE)) */
#define SGS_FREE	1	/* all procs schedule independently */
#define SGS_SINGLE	2	/* only one process runs */
#define SGS_EQUAL	3	/* all processes get equal share of cpu */
#define SGS_GANG	4	/* all processes gang schedule */

/* event scheduling information */
#define EV_PERIOD	1	/* return to deadline mode */
#define EV_TIMER	3	/* timer event */
#define EV_NEXTWAKE	4	/* next wakeup event */

#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)
/* deadline scheduling information */
#define DL_RELEASE	((struct sched_deadline *) 1L)
#define DL_BLOCK	((struct sched_deadline *) 2L)
#define DL_ONLY		((struct sched_deadline *) 3L)
#define DL_ANY		((struct sched_deadline *) 4L)
struct sched_deadline {
	timestruc_t		dl_period;	/* size of deadline interval */
	timestruc_t		dl_alloc;	/* cpu wanted in interval */
	/*
	 * These fields are placeholders for future functionality.
	 * They currently should be set to zero.
	 */
	timestruc_t		dl_deadline;	/* if deadline/period differ */
	int			dl_flags;	/* control flags */
};
#endif

#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)
#ifndef _KERNEL
ptrdiff_t schedctl(int, ...);
#endif
#endif

/*
 * Scheduling discipline definitions.
 * NOTE: the SD_UNIX_SCHED and SD_MP_SCHED definitions are provided solely
 *	 for backward compatibility.
 */
#define SD_UNIX_SCHED	0	/* normal unix scheduling */
#define SD_GFX_SCHED	1	/* graphics scheduling discipline */
#define SD_MP_SCHED	2	/* shared process group scheduling disc */
#define SD_UNIX_MASK	(1<<SD_UNIX_SCHED)
#define SD_GFX_MASK	(1<<SD_GFX_SCHED)
#define SD_MP_MASK	(1<<SD_MP_SCHED)
#define SD_MASK_ALL	(SD_UNIX_MASK|SD_GFX_MASK|SD_MP_MASK)

#ifdef _KERNEL
/* system configurable parameter */
extern unsigned ndpri_hilim, ndpri_lolim;

typedef struct irix5_sched_deadline {
   irix5_timestruc_t dl_period;
   irix5_timestruc_t dl_alloc;
   /*
    * These fields are placeholders for future functionality
    *
    */ 
   irix5_timestruc_t dl_deadline;
   int dl_flags;
} irix5_sched_deadline_t;

int irix5_to_sched_deadline(enum xlate_mode, void *,
                              int, struct xlate_info_s *); 
#endif

#ifdef __cplusplus
}
#endif

#endif /* __SYS_SCHEDCTL_H__ */
