#ifndef __SYS_SPLOCK_H__
#define __SYS_SPLOCK_H__

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1990-1993, Silicon Graphics, Inc.	  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ident "$Revision: 3.25 $"

#define	SPLOCKNAMSIZ	8

#include <sys/avl.h>

typedef struct k_splockmeter {
	avlnode_t m_avlnode;		/* tree linkage -- must be first */
	char	m_inuse;		/* use count */
	char	m_type;			/* type of spinlock */
	short	m_owner;		/* current owner of lock */
	uint	m_ts;			/* timestamp when got lock */
	void	*m_addr;		/* lock address */
	char	m_name[SPLOCKNAMSIZ];	/* text name for lock */
	ulong 	m_wait;			/* count of times locker had to wait */
	ulong 	m_lcnt;			/* count of 'lock' operations */
	ulong 	m_ucnt;			/* count of 'unlock' operations */
	inst_t	*m_callpc;		/* return pc of last lock */
	uint	m_elapsed;		/* longest time held lock */
	short	m_elapsedcpu;		/* cpu holding lock */
	short	m_waitcpu;		/* cpu waiting for lock */
	inst_t	*m_elapsedpc;		/* and the culprit */
	inst_t	*m_elapsedpcfree;	/* freeing pc of longest time held */
	void	*m_splevel;		/* spl function used to set spl */
	inst_t	*m_waitpc;		/* and the victim */
	uint	m_waittime;		/* longest time wait for lock */
	int	m_racers;		/* number of cpus waiting for lock */
	int	m_racers_num;		/* times lock has been waited for */
	int	m_racers_total;		/* total cpus that have waited */
	int	m_racers_max;		/* max cpus waiting simultaneously */
	int	m_gotfirst;		/* #times got lock on first try... */
	int	m_gotfirst_num;		/* ...out of this many calls */
} k_splockmeter_t;

#define next_lockmeter(m)	((k_splockmeter_t *)(m)->m_avlnode.avl_nextino)

extern struct k_splockmeter *lockmeter_find(void *);
extern struct k_splockmeter *lockmeter_chain;

extern int nsplock;			/* number of kernel spin locks */
extern int splocktype;			/* spin lock implementation */
extern int splockmeter;			/* if non-zero, do spin lock metering */

#define	SPLOCK_NONE	0		/* no spinlocks (SIMPLEX kernel) */
#define	SPLOCK_SOFT	1		/* software spinlocks */
#define	SPLOCK_HARD	2		/* hardware locks */

#define SPIN_LOCK	0x1
#define SPIN_INIT	0x2

#define SPINTYPE_SPIN	1
#define SPINTYPE_BIT	2
#define SPINTYPE_64BIT	3

#define	OWNER_NONE	-1

#define	MAP_LOCKID	0		/* spinlock map index */
#define SPLOCK_MAX	8192		/* number of preallocated spinlock */
					/* metering structures */

#endif /* !__SYS_SPLOCK_H__ */
