/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _FS_STATVFS_H	/* wrapper symbol for kernel use */
#define _FS_STATVFS_H	/* subject to change without notice */

#ifdef __cplusplus
extern "C" {
#endif

/*#ident	"@(#)uts-3b2:fs/statvfs.h	1.2"*/
#ident	"$Revision: 1.9 $"
/*
 * Structure returned by statvfs(2).
 */

#include <sys/types.h>

#define	FSTYPSZ	16

typedef struct statvfs {
	ulong_t	f_bsize;	/* fundamental file system block size */
	ulong_t	f_frsize;	/* fragment size */
	fsblkcnt_t f_blocks;	/* total # of blocks of f_frsize on fs */
	fsblkcnt_t f_bfree;	/* total # of free blocks of f_frsize */
	fsblkcnt_t f_bavail;	/* # of free blocks avail to non-superuser */
	fsfilcnt_t f_files;	/* total # of file nodes (inodes) */
	fsfilcnt_t f_ffree;	/* total # of free file nodes */
	fsfilcnt_t f_favail;	/* # of free nodes avail to non-superuser */
	ulong_t	f_fsid;		/* file system id (dev for now) */
	char	f_basetype[FSTYPSZ]; /* target fs type name, null-terminated */
	ulong_t	f_flag;		/* bit-mask of flags */
	ulong_t	f_namemax;	/* maximum file name length */
	char	f_fstr[32];	/* filesystem-specific string */
	ulong_t	f_filler[16];	/* reserved for future expansion */
} statvfs_t;

#if _SGIAPI
typedef struct statvfs64 {
	ulong_t	f_bsize;	/* fundamental file system block size */
	ulong_t	f_frsize;	/* fragment size */
	__uint64_t f_blocks;	/* total # of blocks of f_frsize on fs */
	__uint64_t f_bfree;	/* total # of free blocks of f_frsize */
	__uint64_t f_bavail;	/* # of free blocks avail to non-superuser */
	__uint64_t f_files;	/* total # of file nodes (inodes) */
	__uint64_t f_ffree;	/* total # of free file nodes */
	__uint64_t f_favail;	/* # of free nodes avail to non-superuser */
	ulong_t	f_fsid;		/* file system id (dev for now) */
	char	f_basetype[FSTYPSZ]; /* target fs type name, null-terminated */
	ulong_t	f_flag;		/* bit-mask of flags */
	ulong_t	f_namemax;	/* maximum file name length */
	char	f_fstr[32];	/* filesystem-specific string */
	ulong_t	f_filler[16];	/* reserved for future expansion */
} statvfs64_t;
#endif

/*
 * Flag definitions.
 */

#define	ST_RDONLY	0x01	/* read-only file system */
#define	ST_NOSUID	0x02	/* does not support setuid/setgid semantics */
#define ST_NOTRUNC	0x04	/* does not truncate long file names */
#define ST_DMI          0x08    /* DMI is enabled for current mount of FS */
#define	ST_NODEV	0x20000000	/* disallow opening of device files */
#define	ST_GRPID	0x40000000	/* group-ID assigned from directory */
#define	ST_LOCAL	0x80000000	/* local filesystem, for find */

#ifndef _KERNEL
int statvfs(const char *, struct statvfs *);
int fstatvfs(int, struct statvfs *);
#if _SGIAPI
int statvfs64(const char *, struct statvfs64 *);
int fstatvfs64(int, struct statvfs64 *);
#endif
#endif

#ifdef _KERNEL
#define STATVFS_VERS_32	0
#define	STATVFS_VERS_64	1

typedef struct irix5_statvfs {
	app32_ulong_t	f_bsize;
	app32_ulong_t	f_frsize;
	app32_ulong_t	f_blocks;
	app32_ulong_t	f_bfree;
	app32_ulong_t	f_bavail;
	app32_ulong_t	f_files;
	app32_ulong_t	f_ffree;
	app32_ulong_t	f_favail;
	app32_ulong_t	f_fsid;	
	char		f_basetype[FSTYPSZ];
	app32_ulong_t	f_flag;
	app32_ulong_t	f_namemax;
	char		f_fstr[32];
	app32_ulong_t	f_filler[16];
} irix5_statvfs_t;

typedef struct irix5_statvfs64 {
	app32_ulong_t		f_bsize;
	app32_ulong_t		f_frsize;
	app32_ulong_long_t	f_blocks;
	app32_ulong_long_t	f_bfree;
	app32_ulong_long_t	f_bavail;
	app32_ulong_long_t	f_files;
	app32_ulong_long_t	f_ffree;
	app32_ulong_long_t	f_favail;
	app32_ulong_t		f_fsid;	
	char			f_basetype[FSTYPSZ];
	app32_ulong_t		f_flag;
	app32_ulong_t		f_namemax;
	char			f_fstr[32];
	app32_ulong_t		f_filler[16];
} irix5_statvfs64_t, irix5_n32_statvfs_t;
#endif

#ifdef __cplusplus
}
#endif

#endif	/* _FS_STATVFS_H */
