/**************************************************************************
 *									  *
 * 		 Copyright (C) 1990-1991 Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*
 * sys/usync.h -- structures and defines for usync interface
 */
#ifndef __SYS_USYNC_H__
#define __SYS_USYNC_H__

#ident "$Revision: 1.1 $"


/*
 * usync commands.
 */

#define USYNC_BLOCK           	1	/* block the process		*/ 
#define USYNC_INTR_BLOCK      	2	/* interruptible block		*/ 
#define USYNC_TIMED_BLOCK    	3	/* timed block			*/ 
#define USYNC_UNBLOCK         	4	/* unblock a process		*/ 
#define USYNC_NOTIFY_REGISTER 	5	/* register notification	*/ 
#define USYNC_NOTIFY          	6	/* send notification		*/ 
#define USYNC_NOTIFY_DELETE     7	/* remove the proc's notification */ 
#define USYNC_NOTIFY_CLEAR      8	/* remove any proc's notification */ 
#define USYNC_RTLOCK_BLOCK    	9	/* real-time lock block   	*/
#define USYNC_RTLOCK_UNBLOCK  	10	/* real-time lock unblock 	*/
#define USYNC_GET_STATE  	11	/* get waiter count or prepost value */

#define	USYNC_VERSION_1	1001

#ifndef _KERNEL
extern int usync_cntl(int, void *);
#endif

typedef struct usync_sigevent {
        int             ss_signo;	/* signal to be sent	*/
        int      	ss_si_code;	/* si_code for siginfo */
        __uint64_t     	ss_value;	/* holds the value for union sigval */
} usync_sigevent_t;

/*
 * usync_arg
 *	structure used to pass arguments for the various ioctl commands
 *	defined so that it's size and field-offsets are the same in the
 *	ABIs - o32, n32 and 64.
 */

typedef struct usync_arg {
	int ua_version;		/* version of the usync interface */
	__uint64_t ua_addr;		/* address of object */
	ushort_t ua_policy;		/* queuing policy */
#define USYNC_POLICY_PRIORITY	1
#define USYNC_POLICY_FIFO	2
	ushort_t ua_flags;
#define USYNC_FLAGS_PREPOST_CONSUME 1		
	usync_sigevent_t ua_sigev;
	__uint64_t ua_reserved[2];		/* reserved fields */
} usync_arg_t;

#endif	/* __SYS_USYNC_H__ */
