#ifndef __SYS_VC2_H__
#define __SYS_VC2_H__
/**************************************************************************
 *               Copyright (C) 1990, Silicon Graphics, Inc.               *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 **************************************************************************/

/*
 *   $Revision: 1.16 $
 */

#include "sys/ng1hw.h"


/***************************************************
 *              VC2 definitions and macros
 ***************************************************/

/*
 * These bits go into the DCB_CRS field of the Rex DCBMODE register
 */

#define VC2_CRS_INDEX   (0 << DCB_CRS_SHIFT)
/* Assumes we write 16 bit data to vc2 register with ENCRSINC in dcbmode */
#define VC2_CRS_REGISTER (1 << DCB_CRS_SHIFT)
#define VC2_CRS_RAM      (3 << DCB_CRS_SHIFT)

/*
 * VC2 index register values.
 */

#define VC2_VIDEO_ENTRY_PTR     0
#define VC2_CURS_ENTRY_PTR      1
#define VC2_CURS_X_LOC          2
#define VC2_CURS_Y_LOC          3
#define VC2_CURRENT_CURS_X_LOC  4
#define VC2_DID_ENTRY_PTR       5
#define VC2_SCAN_LENGTH         6
#define VC2_RAM_ADDR            7
#define VC2_VT_FRAME_PTR        8
#define VC2_VT_LINESEQ_PTR      9
#define VC2_VT_LINES_IN_RUN     10
#define VC2_VERTICAL_LINE_CTR   11
#define VC2_CURS_TABLE_PTR      12
#define VC2_WORKING_CURS_Y      13
#define VC2_DID_FRAME_PTR       14
#define VC2_DID_LINE_TABLE_PTR  15
#define VC2_DC_CONTROL          16
#define VC2_CONFIG              31

/*
 * CONFIG register bits
 */

#define VC2_RESET       BIT(0)
#define VC2_SLOW_CLOCK  BIT(1)
#define VC2_CUR_ERR  	BIT(2)
#define VC2_DID_ERR  	BIT(3)
#define VC2_VTG_ERR  	BIT(4)
#define VC2_REVISION_SHIFT      5
#define VC2_REVISION_MASK       (0xf << VC2_REVISION_SHIFT)
#define VC2_TEMP_HIBYTE_SHIFT	8
#define VC2_TEMP_HIBYTE_MASK	(0xff << VC2_TEMP_HIBYTE_SHIFT)


/*
 * DC_CONTROL register bits
 */

#define VC2_ENA_VINTR           BIT(0)
#define VC2_ENA_DISPLAY         BIT(1)
#define VC2_ENA_VIDEO           BIT(2)
#define VC2_ENA_DIDS            BIT(3)
#define VC2_ENA_CURS_FUNCTION   BIT(4)
#define VC2_ENA_GENSYNC         BIT(5)
#define VC2_ENA_INTERLACE       BIT(6)
#define VC2_ENA_CURS_DISPLAY    BIT(7)
#define VC2_ENA_CROSS_CURS      BIT(8)
#define VC2_CURS_GLYPH_64       BIT(9)
#define VC2_GENLOCK_SELECT      BIT(10)



/*
 * Define the dcb transfer protocol used by the vc2.
 * XXX I picked the simplest one I could find.
 * XXX Ask the hw people about this, or refer to latest spec.
 * (See Rex3 spec, section 3.13)
 */

#define DCB_VC2_PROTOCOL (DCB_ENASYNCACK | (0x1 << DCB_CSSETUP_SHIFT) | \
			  (0x1 << DCB_CSHOLD_SHIFT) )

/*
 * Load a vc2 register with 16 bit data.
 */

#define vc2SetReg(rex,reg,data16)                                      \
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_INDEX | DCB_DATAWIDTH_3 | \
	                           DCB_ENCRSINC | DCB_VC2_PROTOCOL;    \
	rex->set.dcbdata0.byword = ((reg) << 24) | ((data16 & 0xffff) << 8)
/*
 * Read a vc2 register
 */

#define vc2GetReg(rex,reg,data)						\
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_INDEX | DCB_DATAWIDTH_1 |	\
	                           DCB_ENCRSINC | DCB_VC2_PROTOCOL;	\
	rex->set.dcbdata0.bybyte.b3 = (reg);				\
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_REGISTER | DCB_DATAWIDTH_2 |\
	                           DCB_ENCRSINC | DCB_VC2_PROTOCOL;	\
	data = rex->set.dcbdata0.byshort.s1
/*
 * Read the vc2 index register
 */
#define vc2GetIndex(rex,data)						\
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_INDEX | DCB_DATAWIDTH_1 |	\
	                           DCB_VC2_PROTOCOL;	\
	data = rex->set.dcbdata0.bybyte.b3
/*
 * Set the vc2 index register
 */
#define vc2SetIndex(rex,data)						\
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_INDEX | DCB_DATAWIDTH_1 |	\
	                           DCB_VC2_PROTOCOL;	\
	rex->set.dcbdata0.bybyte.b3 = data


/*
 * Load the vc2 shared ram with 16 bit data
 */

#define _vc2LoadSram(rex,addr,data16,nwords) \
	{ 								\
	int _n = nwords;						\
	short * _d; 							\
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_INDEX | DCB_DATAWIDTH_3 |	\
			DCB_ENCRSINC | DCB_VC2_PROTOCOL;            	\
	rex->set.dcbdata0.byword = (VC2_RAM_ADDR << 24) | ((addr) << 8);	\
	rex->set.dcbmode = DCB_VC2 | VC2_CRS_RAM | DCB_DATAWIDTH_2 |    \
			DCB_VC2_PROTOCOL;				\
	while (_n-- > 0)						\
		rex->set.dcbdata0.byshort.s1 = *_d++;			\
	}

#define vc2SetupRamAddr( rex, addr )      \
    vc2SetReg( rex, VC2_RAM_ADDR, addr ); \
    rex->set.dcbmode=DCB_VC2 | VC2_CRS_RAM | DCB_DATAWIDTH_2 | DCB_VC2_PROTOCOL;

#define vc2SetRam( rex, data ) \
    rex->set.dcbdata0.byshort.s1 = (data)

#define vc2GetRam(rex,data) \
	data = rex->set.dcbdata0.byshort.s1

/* MCO-specific Macros */
#define MCO_VC2_SETREG(base,index,data16)                               \
    base->mco_mpa1.mco.vc2index = index;                                     \
    base->mco_mpa1.mco.vc2data = data16

#define MCO_VC2_GETREG(base,index,data16)                                \
    base->mco_mpa1.mco.vc2index = index;                                     \
    data16 = base->mco_mpa1.mco.vc2data;

#define MCO_VC2_SETINDEX(base,index)                                    \
    base->mco_mpa1.mco.vc2index = index

#define MCO_VC2_GETINDEX(base,data)                                     \
    data = base->mco_mpa1.mco.vc2index

#define MCO_VC2_SETRAM(base,data)                                       \
    base->mco_mpa1.mco.vc2ram = data

#define MCO_VC2_GETRAM(base,data)                                       \
    data = base->mco_mpa1.mco.vc2ram


/* For the MCO, we do not have to share the memory w/ the DIDs so we can
   give the VTG full access to the entire SRAM.  However, it is unlikely
   that we will ever generate a timing table so huge.  
*/

/*
 *  Define the VC2 memory map
 *
 *  These address are 16 bit WORD addresses
 */

#define VC2_VID_LINE_TAB_ADDR		0x0
#define VC2_VID_FRAME_TAB_ADDR		0x400 /* 2K bytes for Vid line tab   */
#define VC2_CURSOR_GLYPH_ADDR		0x500 /* 512 bytes for Vid frame tab */

/* max length in 16 bit words of the DID frame table */
#define VC2_DID_FRAME_TAB_MAX_LEN_WORD	( 1025 )

/* first instance of DID frame table */
#define VC2_DID_FRAME_TAB_ADDR		0x600 /* 512 bytes for cursor glyph  */
#define VC2_DID_FRAME_TAB_END_ADDR	VC2_DID_FRAME_TAB_ADDR + \
					VC2_DID_FRAME_TAB_MAX_LEN_WORD

/* second instance of DID frame table */
#define VC2_DID_FRAME_TAB_ADDR2		VC2_DID_FRAME_TAB_END_ADDR
#define VC2_DID_FRAME_TAB_END_ADDR2	VC2_DID_FRAME_TAB_ADDR2 + \
					VC2_DID_FRAME_TAB_MAX_LEN_WORD

/* just for now hard code these */
#define VC2_DISP_CTRL_ADDR		0x0F00 /* Genlock and Interlace info */
#define VC2_DID_LINE_TAB_ADDR		0x1000 /* frame tab2 ends at 0x0e02  */
#define VC2_DID_LINE_TAB_ADDR2		0x3000 /* 16K bytes for DID line tab */

/* so we can check overflow */
#define VC2_DID_LINE_TAB_LEN_WORDS	0x2000

#define VC2_EXTERNAL_RAM_NBYTES		( 64 * 1024 )

	
#endif /* __SYS_VC2_H__ */

