/**************************************************************************
 *									  *
 * 		 Copyright (C) 1992, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
 * $Revision: 1.15 $
 */

#ifndef _HOST_MPG_H_
#define _HOST_MPG_H_

#define VENICE_MPG

#ifdef IP17
#define VENICE_BOARDNAME	"REC"	/* Reality Engine - Crimson */
#else
#define VENICE_BOARDNAME	"RE"    /* Reality Engine */
#endif

#define VENICE_3WAY_SIZE	0x2000	/* size of 3way hardware to map */
#define VENICE_USER_FIFO_SIZE	0x1000	/* size of fifo hardware to map */
#define VENICE_PRIV_FIFO_SIZE	0x1000	/* size of priviledged fifo for diags */
#define VENICE_MPG_REGS_SIZE	0x1000	/* size of MPG regs for diags */
#define VENICE_RDATA_SIZE	0x2000	/* size of return data area */
#define VENICE_MAP_SIZE		(VENICE_3WAY_SIZE+VENICE_USER_FIFO_SIZE+\
				 VENICE_PRIV_FIFO_SIZE+VENICE_MPG_REGS_SIZE+VENICE_RDATA_SIZE)
#define VENICE_DIAG_MAP_SIZE	VENICE_MAP_SIZE
#define VENICE_BASE_ADDR	0x2000
#define VENICE_USER_FIFO_ADDR   0x4000
#define VENICE_PRIV_FIFO_ADDR   0x5000
#define VENICE_MPG_REGS_ADDR    0x6000
#define VENICE_RDATA_ADDR	0x7000

/* configuration limitations */
#define VENICE_FIFO_LENGTH	255	/* #entries in upper fifo */

#define VENICE_FIFO_HIWATER	190	/* hardware checked hiwater */
#define VENICE_FIFO_LOWATER	4	/* hardware checked lowater */
#define VENICE_FIFO_SOFTWATER	140	/* software checked lowater */
#define VENICE_FIFO_SPINCNT	30	/* in terms of usec */

#define MPG_BOARD_ID_ADDR	(0x0000>>2)	/* board ID offset	    */
#define   MPG_BOARD_ID_VALUE	    0x000740ee	  /* board ID value	    */
#define MPG_ID_ADDR		(0x0004>>2)	/* chip ID offset	    */
#ifdef MPG1_B
#define   MPG_ID_VALUE		    0x00000001	  /* MPG1 ID value	    */
#else
#define   MPG_ID_VALUE		    0x00000000	  /* MPG1 ID value	    */
#endif
#define MPG_RESET_ADDR		(0x0008>>2)	/* reset register offset    */
#define MPG_SLAVE_LE_ADDR	(0x000c>>2)	/* little endian	    */
#define MPG_KERNEL_LE_ADDR	(0x0010>>2)	/* scatter-gather endian    */
#define MPG_IDMA_CNTL_ADDR	(0x0100>>2)	/* how to stop an IDMA	    */
#define   MPG_IDMA_CNTL_CPCMD_MASK	 0x0ff	  /* forced CP cmd with intr*/
#define   MPG_IDMA_CNTL_PASSTHRU_MASK    0x400	  /* enter passthru mode    */
#define   MPG_IDMA_CNTL_UNSTOPPABLE_MASK 0x800	  /* is it OK to intr       */
#define MPG_IDMA_SEQID_ADDR	(0x0104>>2)	/* IDMA sequence ID 	    */
#define   MPG_IDMA_SEQID_MASK	    0xffff	  /* only 16 bits valid	    */
#define MPG_ODMA_COUNT_ADDR	(0x0200>>2)	/* ODMA word count	    */
#define	    MPG_ODMA_COUNT_MAX_POS	0	    /* Max word count	    */
#define	    MPG_ODMA_COUNT_MAX_MASK	0x7fffff
#define	    MPG_ODMA_COUNT_INTR_EN_POS	23	    /* overrun interrupt    */
#define	    MPG_ODMA_COUNT_INTR_EN_MASK	0x800000
#define MPG_VIDEO0_INTR_ADDR	(0x0300>>2)	/* configure video intr 0   */
/* due to MPG design oversights, the following (VIDEO1) should not be used */
#define MPG_VIDEO1_INTR_ADDR	(0x0304>>2)	/* configure video intr 1   */
/* due to MPG design oversights, the previous (VIDEO1) should not be used */
#define MPG_FIFO_HIGH_INTR_ADDR	(0x0308>>2)	/* configure hi water intr  */
#define MPG_FIFO_LOW_INTR_ADDR	(0x030c>>2)	/* configure lo water intr  */
#define MPG_FIFO_OVERFLOW_INTR_ADDR (0x0310>>2)	/* configure OVERFLOW intr  */
/*
 * The following does not exists
 * #define MPG_GEF_INTR_ADDR	(0x0318>>2)	/ * configure GEF intr	    */

#define MPG_GEF_INTR_CLEAR_ADDR	(0x031c>>2)	/* clear GEF intr (protocol)*/
#define MPG_ODMA_INTR_ADDR	(0x0320>>2)	/* configure ODMA intr	    */
#define MPG_FIFO_HIGH_MARK_ADDR	(0x0500>>2)	/* configure hi water mark  */
#define MPG_FIFO_LOW_MARK_ADDR	(0x0504>>2)	/* configure lo water mark  */
#define MPG_FIFO_READ_PTR_ADDR	(0x0508>>2)	/* fifo read address	    */
#define MPG_FIFO_WRITE_PTR_ADDR	(0x050c>>2)	/* fifo write address	    */
#define MPG_FIFO_COUNT_ADDR	(0x0510>>2)	/* how full is the fifo?    */
#define MPG_3WAY_COUNT_ADDR	(0x0600>>2)	/* 3way count for cxt switch*/
#define MPG_VCWRITE_ADDR_ADDR	(0x0700>>2)	/* vcbus write address	    */
#define MPG_VCWRITE_DATA_ADDR	(0x0704>>2)	/* vcbus write data	    */
#define MPG_VCWRITE_FIFO_ADDR	(0x0708>>2)	/* vcbus write fifo data    */
#define MPG_VCREAD_DATA_ADDR	(0x0018>>2)	/* vcbus read data	    */
#define MPG_VCREAD_STATUS_ADDR	(0x001c>>2)	/* vcbus status		    */
#define   MPG_VC_READ_VALID_MASK    0x00000001	  /* read data valid	    */
#define   MPG_VC_FIFO_NOTFULL_MASK  0x00000002	  /* VC fifo not full	    */
#define MPG_VC_STATUS_ADDR	MPG_VCREAD_STATUS_ADDR /* XXX OBSOLETE NAME */
#define MPG_VCREAD_ADDR_ADDR	(0x0720>>2)	/* vcbus read address	    */
#define MPG_VCREAD_TRIGGER_ADDR	(0x0724>>2)	/* vcbus read trigger	    */
#define MPG_UBUS_ADDR0		(0x0740>>2)	/* low 8 bits of EEPROM addr*/
#define MPG_UBUS_ADDR1		(0x0744>>2)	/* bits 8-15 of EEPROM addr */
#define MPG_UBUS_ADDR2		(0x0748>>2)	/* jtag ctrl & upper EEPROM */
#define   MPG_ADDR2_EEPROM_MASK	    0x03	  /* bits 16-17 EEPROM addr */
#define MPG_UBUS_CNTL_ADDR	(0x0748>>2)	/* jtag ctrl & upper EEPROM */
#define   MPG_CTL_JSEL_RM0	    0x00	  /* select RM 0 chain	    */
#define   MPG_CTL_JSEL_RM1	    0x04	  /* select RM 1 chain	    */
#define   MPG_CTL_JSEL_RM2	    0x08	  /* select RM 2 chain	    */
#define   MPG_CTL_JSEL_RM3	    0x0c	  /* select RM 3 chain	    */
#define   MPG_CTL_JSEL_DG	    0x10	  /* select DG chain	    */
#define   MPG_CTL_JSEL_GE	    0x30	  /* select GE chain	    */
#define   MPG_CTL_JTAG_SHIFT1	    0x80	  /* shift 1 bit not 8	    */
#define   MPG_CTL_JTAG_TESTMODE	    0x40	  /* test mode		    */
#define MPG_UBUS_EEPROM_DATA_ADDR (0x074C>>2)	/* EEPROM read/write data   */
#define MPG_UBUS_JTAG_DATA_ADDR	(0x0750>>2)	/* jtag shift data          */
#define MPG_UBUS_JTAG_TMS_ADDR	(0x0754>>2)	/* jtag TMS data            */
#define MPG_UBUS_STATUS_ADDR	(0x0758>>2)	/* board rev & jtag status  */
#define   MPG_MISC_BD_ID_POS	    0
#define   MPG_MISC_BD_ID_MASK	    0x07	    /* board revision       */
#define     MPG_MISC_BD_ID_8GES		0x7	    /* set by jumpers	    */
#define     MPG_MISC_BD_ID_4GES		0x6	    /* to determine GE	    */
#define     MPG_MISC_BD_ID_2GES		0x5	    /* count at init time   */
#define     MPG_MISC_BD_ID_1GE		0x1
#define   MPG_GPI_CABLE_ID_POS	    3
#define   MPG_GPI_CABLE_ID_MASK	    0x08
#define   MPG_JTAG_GE_TMS_POS	    4
#define   MPG_JTAG_GE_TMS_MASK	    0x10	  /* ge tms bit             */
#define   MPG_JTAG_TB_TMS_POS	    5
#define   MPG_JTAG_TB_TMS_MASK	    0x20	  /* tbus tms bit           */
#define   MPG_JTAG_TDO_POS	    6
#define   MPG_JTAG_TDO_MASK	    0x40	  /* ctrler TDO bit         */
#define   MPG_JTAG_READY_POS	    7
#define   MPG_JTAG_READY_MASK	    0x80	  /* controller ready       */
#define MPG_VER_RET_SEQ_BUF_ADDR (0x0800>>2)	/* retrace seqid for vlist  */
#define MPG_FIFO_ADDR		(0x2000>>2)	/* input fifo; user mapped  */

#define MPG_GEF_CNTL_ADDR	MPG_UBUS_ADDR0

/* interrupt configuration registers from above all use these bit defs */
#define MPG_INTR_VECTOR_POS	    0		/* interrupt status	    */
#define MPG_INTR_VECTOR_MASK	    0x000000ff
#define MPG_INTR_DST_POS	    8		/* interrupt destination    */
#define MPG_INTR_DST_MASK	    0x00000f00
#define MPG_INTR_ENABLE_POS	    12		/* enable interrupt	    */
#define MPG_INTR_ENABLE_MASK	    0x00001000

#define MPG_INTR_CONFIG(vector,dst,enable)\
	(((vector) & MPG_INTR_VECTOR_MASK) |\
	(((dst) << MPG_INTR_DST_POS) & MPG_INTR_DST_MASK) |\
	(((enable) << MPG_INTR_ENABLE_POS) & MPG_INTR_ENABLE_MASK))

#endif /* _HOST_FCG_H_ */
