/*	Copyright (c) 1990, 1991 UNIX System Laboratories, Inc.	*/
/*	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF     	*/
/*	UNIX System Laboratories, Inc.                     	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 * +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * 		PROPRIETARY NOTICE (Combined)
 * 
 * This source code is unpublished proprietary information
 * constituting, or derived under license from AT&T's UNIX(r) System V.
 * In addition, portions of such source code were derived from Berkeley
 * 4.3 BSD under license from the Regents of the University of
 * California.
 * 
 * 
 * 
 * 		Copyright Notice 
 * 
 * Notice of copyright on this source code product does not indicate 
 * publication.
 * 
 * 	(c) 1986,1987,1988,1989  Sun Microsystems, Inc
 * 	(c) 1983,1984,1985,1986,1987,1988,1989  AT&T.
 * 	          All rights reserved.
 *  
 */

#ifndef _FS_VFS_H	/* wrapper symbol for kernel use */
#define _FS_VFS_H	/* subject to change without notice */

/*#ident	"@(#)uts-3b2:fs/vfs.h	1.6"*/
#ident	"$Revision: 1.26 $"

/*
 * Data associated with mounted file systems.
 */

#include <sys/types.h>
#include <sys/sema.h>

/*
 * File system identifier. Should be unique (at least per machine).
 */
typedef struct {
	uint val[2];			/* file system id type */
} fsid_t;

/*
 * File identifier.  Should be unique per filesystem on a single
 * machine.  This is typically called by a stateless file server
 * in order to generate "file handles".
 */
#define MAXFIDSZ	46
#define freefid(fidp) \
  kmem_free((caddr_t)(fidp), sizeof (struct fid) - MAXFIDSZ + (fidp)->fid_len)

typedef struct fid {
	u_short		fid_len;		/* length of data in bytes */
	char		fid_data[MAXFIDSZ];	/* data (variable length) */
} fid_t;

/*
 * Structure per mounted file system.  Each mounted file system has
 * an array of operations and an instance record.  The file systems
 * are kept on a doubly linked list headed by "rootvfs" and terminated
 * by NULL.
 *
 * MP locking protocols:
 *	vfs_next, vfs_prevp			vfslock global spinlock
 *	vfs_vnodecovered			vfs_lock/vfs_unlock
 *	vfs_flag 				vfslock global spinlock
 *	vfs_nsubmounts				vfs_lock/vfs_unlock
 *	vfs_bcount				unprotected
 *	vfs_busycnt				vfslock global spinlock
 *	all others				readonly once mounted
 */
typedef struct vfs {
	struct vfs	*vfs_next;		/* next VFS in VFS list */
	struct vfs	**vfs_prevp;		/* ptr to previous vfs_next */ 
	struct vfsops	*vfs_op;		/* operations on VFS */
	struct vnode	*vfs_vnodecovered;	/* vnode mounted on */
	void		*vfs_data;		/* private data */
	int		vfs_dcount;		/* count of dirty pages */
	u_int		vfs_flag;		/* flags */
	dev_t		vfs_dev;		/* device of mounted VFS */
	u_short		vfs_nsubmounts;		/* immediate sub-mount count */
	short		vfs_busycnt;		/* # of accesses in progress */
	sv_t		vfs_wait;		/* busy mount point sync */
	u_int		vfs_bsize;		/* native block size */
	int		vfs_fstype;		/* file system type index */
	fsid_t		vfs_fsid;		/* file system id */
	u_long		vfs_bcount;		/* I/O count (accounting) */
	struct mac_vfs	*vfs_mac;		/* MAC Plan G information */
	struct acl_vfs	*vfs_acl;		/* ACL Plan G information */
	struct cap_vfs	*vfs_cap;		/* Capability Plan G info */
	struct inf_vfs	*vfs_inf;		/* Info-label Plan G info */
	fsid_t		*vfs_altfsid;		/* An ID fixed for life of FS */
} vfs_t;

/*
 * VFS flags.
 */
#define VFS_RDONLY	0x0001		/* read-only vfs */
#define VFS_NOSUID	0x0002		/* setuid disallowed */
#define VFS_NODEV	0x0004		/* disallow opening of device files */
#define VFS_GRPID	0x0008		/* group-ID assigned from directory */
#define VFS_REMOUNT	0x0010		/* modify mount options only */
#define VFS_NOTRUNC	0x0020		/* does not truncate long file names */
#define VFS_UNLINKABLE	0x0040		/* unlink(2) can be applied to root */
#define VFS_BADBLOCK	0x0080		/* disk based fs has bad block */
#define VFS_MLOCK	0x0100		/* lock vfs so that subtree is stable */
#define VFS_MWAIT	0x0200		/* waiting for access lock */
#define VFS_MWANT	0x0400		/* waiting for update */
#define VFS_LOCAL	0x1000		/* local filesystem, for find */
#define VFS_OFFLINE	0x2000		/* filesystem is being unmounted */
#define VFS_DMI		0x4000		/* filesystem has the DMI enabled */


#ifdef _KERNEL
/*
 * Argument structure for mount(2).
 */
struct mounta {
	char	*spec;
	char	*dir;
	sysarg_t flags;
	char	*fstype;
	char	*dataptr;
	sysarg_t datalen;
};
#endif

/*
 * Reasons for calling the vfs_mountroot() operation.
 */
enum whymountroot { ROOT_INIT, ROOT_REMOUNT, ROOT_UNMOUNT };
typedef enum whymountroot whymountroot_t;
struct vnode;
struct statvfs;
struct mounta;
struct cred;

/*
 * Operations supported on virtual file system.
 */
typedef struct vfsops {
	int	(*vfs_mount)(struct vfs *, struct vnode *,
			     struct mounta *, struct cred *);
					/* mount file system */
	int	(*vfs_unmount)(struct vfs *, int, struct cred *);
					/* unmount file system */
	int	(*vfs_root)(struct vfs *, struct vnode **);
					/* get root vnode */
	int	(*vfs_statvfs)(struct vfs *, struct statvfs *, struct vnode *);
					/* get file system statistics */
	int	(*vfs_sync)(struct vfs *, short, struct cred *);
					/* flush files */
	int	(*vfs_vget)(struct vfs *, struct vnode **, struct fid *);
					/* get vnode from fid */
	int	(*vfs_mountroot)(struct vfs *, enum whymountroot);
					/* mount the root filesystem */
	int	(*vfs_swapvp)();	/* return vnode for swap */
} vfsops_t;

#define VFS_MOUNT(vfsp, mvp, uap, cr) \
		(*(vfsp)->vfs_op->vfs_mount)(vfsp, mvp, uap, cr)
#define VFS_UNMOUNT(vfsp,f,cr)	(*(vfsp)->vfs_op->vfs_unmount)(vfsp, f, cr)
#define VFS_ROOT(vfsp, vpp)	(*(vfsp)->vfs_op->vfs_root)(vfsp, vpp)
#define VFS_STATVFS(vfsp, sp, vp) (*(vfsp)->vfs_op->vfs_statvfs)(vfsp, sp, vp)
#define VFS_SYNC(vfsp, flag, cr) \
		(*(vfsp)->vfs_op->vfs_sync)(vfsp, flag, cr)
#define VFS_VGET(vfsp, vpp, fidp) \
		(*(vfsp)->vfs_op->vfs_vget)(vfsp, vpp, fidp)
#define VFS_MOUNTROOT(vfsp, why) \
		(*(vfsp)->vfs_op->vfs_mountroot)(vfsp, why)
#define VFS_SWAPVP(vfsp, vpp, nm) \
		(*(vfsp)->vfs_op->vfs_swapvp)(vfsp, vpp, nm)

/*
 * Filesystem type switch table.
 */
typedef struct vfssw {
	char		*vsw_name;		/* type name string */
	int		(*vsw_init)(struct vfssw *, int); /* init routine */
	struct vfsops	*vsw_vfsops;		/* filesystem operations */
	struct vnodeops	*vsw_vnodeops;		/* file operations */
	void		*vsw_fill;		/* was dying vnodeops - lboot
						 * still wants to fill in
						 */
	long		vsw_flag;		/* flags */
} vfssw_t;

#ifdef _KERNEL

/*
 * Public operations.
 */
struct vnode;
union rval;
extern void	vfs_mountroot(void);	/* mount the root */
extern void	vfs_add(struct vnode *, struct vfs *, int);
extern void	vfs_simpleadd(struct vfs *);
					/* add a new vfs to mounted vfs list */
extern void	vfs_remove(struct vfs *);
					/* remove a vfs from mounted vfs list */
extern int	vfs_lock(struct vfs *);	/* lock and unlock a vfs */
extern void	vfs_unlock(struct vfs *);
extern int	vfs_busy(struct vfs *);	/* mark busy for serial sync/unmount */
extern vfs_t	*vfs_busydev(dev_t);	/* to keep dev from unmounting */
extern void	vfs_unbusy(struct vfs *);
extern vfs_t 	*getvfs(fsid_t *);	/* return vfs given fsid */
extern vfs_t 	*vfs_devsearch(dev_t);	/* find vfs given device */
extern vfs_t 	*vfs_devsearch_nolock(dev_t); /* find vfs given device:nolock */
extern vfssw_t 	*vfs_getvfssw(char *);	/* find vfssw ptr given fstype name */
extern u_long	vf_to_stf(u_long);	/* map VFS flags to statfs flags */
extern int	dounmount(struct vfs *, int, struct cred *);
					/* unmount a vfs */
extern int	cmount(struct mounta *, union rval *, char *);

/*
 * Callers must use kmem_zalloc to allocate a vfs.
 */
#define VFS_INIT(vfsp, op, data) { \
	(vfsp)->vfs_op = (op); \
	(vfsp)->vfs_data = (data); \
	sv_init(&(vfsp)->vfs_wait, SV_DEFAULT, 0); \
}

#define VFS_FREE(vfsp) sv_destroy(&(vfsp)->vfs_wait);

/*
 * Set and clear vfs_flag bits.
 */
#define vfs_setflag(vfsp,f)	{ int _vfs_s = mp_mutex_spinlock(&vfslock); \
				  (vfsp)->vfs_flag |= (f); \
				  mp_mutex_spinunlock(&vfslock, _vfs_s); }
#define vfs_clrflag(vfsp,f)	{ int _vfs_s = mp_mutex_spinlock(&vfslock); \
				  (vfsp)->vfs_flag &= ~(f); \
				  mp_mutex_spinunlock(&vfslock, _vfs_s); }

#define vfs_spinlock()		mp_mutex_spinlock(&vfslock)
#define vfs_spinunlock(s)	mp_mutex_spinunlock(&vfslock,s)
#define vfsp_wait(V,P,S)	mp_sv_wait(&(V)->vfs_wait,P,&vfslock,s)
#define vfsp_waitsig(V,P,S)	mp_sv_wait_sig(&(V)->vfs_wait,P,&vfslock,s)

/*
 * Globals.
 */
extern struct vfs *rootvfs;		/* ptr to root vfs structure */
extern lock_t vfslock;			/* protects rootvfs and vfs_flag */
extern struct vfssw vfssw[];		/* table of filesystem types */
extern char rootfstype[];		/* name of root fstype */
extern int nfstype;			/* # of elements in vfssw array */

extern int imon_enabled;
extern void (*imon_broadcast)(dev_t, int);
#define IMON_BROADCAST(dev,ev) if(imon_enabled){(*imon_broadcast)(dev,ev);}

#endif /* _KERNEL */

/*
 * VFS_SYNC flags.
 */
#define SYNC_NOWAIT	0		/* start delayed writes */
#define SYNC_ATTR	0x01		/* sync attributes */
#define SYNC_CLOSE	0x02		/* close file system down */
#define SYNC_DELWRI	0x04		/* look at delayed writes */
#define SYNC_WAIT	0x08		/* wait for i/o to complete */
#define SYNC_BDFLUSH	0x10		/* BDFLUSH is calling -- don't block */
#define SYNC_FSDATA	0x20		/* flush fs data (e.g. superblocks) */
#define SYNC_PDFLUSH	0x40		/* push v_dpages */

#ifdef _KERNEL
extern void vfs_syncall(int);
extern void vfs_syncroot(int);
#endif

#endif	/* _FS_VFS_H */
