/****************************************************************************
 *
 * Copyright 1993, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 *
 ***************************************************************************/

#ifndef __SYS_VINO_H__
#define __SYS_VINO_H__

/*
 *  vino.h - interface file for vino video driver
 */

/*
 * 525 lines total -> active plus blanking;
 * 625 lines total -> active plus blanking;
 *
 * Betacam/MII/composite tape formats use square pixel analog timings;
 *	  525 == NTSC; 625 == PAL;
 * D1 tape formats use CCIR601 timings;
 * D2 tape formats use the 4 times subcarrier timings.
 */

/* Note all of these *must* match the corresponding VL_TIMING_XXX */
#define V_TIMING_525_SQ_PIX  0	/*analog (525 lines) 12.27 MHz; 646x486 */
#define V_TIMING_625_SQ_PIX  1	/*analog (625 lines) 14.75 MHz; 768x576 */

/* these are only valid on the digital port */
#define V_TIMING_525_CCIR601 2	/*digital component  13.50 MHz; 720x486 */
#define V_TIMING_625_CCIR601 3	/*digital component  13.50 MHz; 720x576 */

#define	V_TIMING_INDYCAM	4	/* Position in table for IndyCam */
/*
 *  Vino private ioctl definitions
 */

#define VINOIOC           ('v'<<24|'i'<<16)
#define VINOIOCTL(x)      (VINOIOC|x)
#define VINOIOCTLN(x)     (x & 0xffff)

#define VINO_SETUP_CONNECTION	VINOIOCTL(0)
#define VINO_DESTROY_CONNECTION	VINOIOCTL(1)
#define VINO_SETUP_GET_FRAME	VINOIOCTL(2)
#define VINO_GET_FRAME		VINOIOCTL(3)
#define VINO_FINISH_GET_FRAME	VINOIOCTL(4)
#define VINO_GET_REG		VINOIOCTL(5)
#define VINO_SET_REG		VINOIOCTL(6)
#define VINO_GET_CNTRL		VINOIOCTL(7)
#define VINO_SET_CNTRL		VINOIOCTL(8)
#define VINO_SET_POLL_SEL	VINOIOCTL(9)
#define VINO_GET_POLL_SEL	VINOIOCTL(10)
#define VINO_GET_STATUS		VINOIOCTL(11)
#define VINO_SET_ANALOG_INPUT	VINOIOCTL(12)
#define VINO_GET_ANALOG_INPUT	VINOIOCTL(13)
#define VINO_GET_FRONTIER_MSC   VINOIOCTL(14)
#define VINO_GET_UST_MSC_PAIR   VINOIOCTL(15)

#ifndef	IOCTLS_ONLY

/*
 *  Vino common vid ioctl definitions
 */

/***************************************************
 *
 *  GETBOARDINFO ioctl
 *
 *  The following struct defines the data returned 
 *  by this ioctl.
 *
 ***************************************************/

#define VINO_BOARDNAME	"vino"
#define	VINO_SW_REVISION	3	/* driver/daemon interface version */ 
					/* ... bump when changing interface */
typedef struct {
    struct vid_info	vid_info;	/* Device independent info */
    int			bdrev;		/* vino board rev */
    int			bdflags;	/* board specific info */
    int			swrev;		/* driver/daemon interface version */
} VinoInfo;

/****************************************************
 *
 *  VINO_SETUP_CONNECTION ioctl
 *
 *  The following defines and structures are used in
 *  setting up a connection between a source and
 *  drain node in vino.
 *
 *****************************************************/

/*
 *  Vino source video node definitions
 *
 * (These must match the MUXSWITCH controls in dev_vino.h!)
 */

#define VINO_SRC_DV_IN		0	/* Digital video in */
#define VINO_SRC_AV_IN		1	/* Analog video in */

/*
 *  Vino drain memory node definitions
 */

#define VINO_DRN_MEM_0		0
#define VINO_DRN_MEM_1		1

/*
 *  Vino channel definitions
 */

#define CHANNEL_A		0
#define CHANNEL_B		1
#define CHANNEL_A_OR_B		2

typedef struct {
    int src_node;
    int drn_node;
    int pref_ch;
    int	format;		/* mux switch */
    int	timing;		/* ntsc or pal timing */
} VinoSetupConnectionArgs;

/****************************************************
 *
 *  VINO_DESTROY_CONNECTION ioctl
 *
 *  The following defines and structures are used
 *  to tell vino to destroy a connection.
 *
 *****************************************************/

typedef struct {
    int	    src_node;
    int     drn_node;
} VinoDestroyConnectionArgs;

/****************************************************
 *
 *  VINO_SETUP_GET_FRAME ioctl
 *
 *  The following defines and structures are used in
 *  setting up vino for do get frame operations.
 *
 *****************************************************/

/*
 *  Mode bits for VINO_GET_FRAME.
 */

#define ODD_FIELD		(1 << 0)
#define EVEN_FIELD		(1 << 1)
#define FRAME       		(ODD_FIELD | EVEN_FIELD)
#define INTERLEAVED		(1 << 2)
#define CONTINUOUS	        (1 << 3)
#define DMRINGBUFFER		(1 << 5)

/*
 *  Timings allowed for VINO_GET_FRAME
 */

#if 0
#define VINO_NTSC_TIMING	1
#define VINO_PAL_TIMING		0
#endif

/*
 *  Frame rates allowed for VINO_GET_FRAME
 */

#define VINO_FR_05	0x003
#define VINO_FR_10	0x0c3
#define VINO_FR_15	0x333
#define VINO_FR_20	0x3cf
#define VINO_FR_25	0x3ff
#define VINO_FR_30	0xfff		/* only valid for NTSC */
#define	VINO_FR_ODD_FIELDS 	0x0aab
#define	VINO_FR_EVEN_FIELDS	0x1555

/*
 *  Decimation factors allowed for VINO_GET_FRAME
 */

#if 0	/* decimation factor is one less than zoom factor */
#define VINO_DECIM_NONE		0
#define VINO_DECIM_HALF		1
#define	VINO_DECIM_THIRD	2
#define VINO_DECIM_FOURTH	3
#endif

#define VINO_DECIM_XY		0
#define VINO_DECIM_X_ONLY	1

/*
 *  Pixel Formats allowed for VINO_SETUP_GET_FRAME
 */

#define VINO_RGBA		0
#define VINO_RGB_332		1
#define VINO_YUV		2
#define VINO_YUV_LUMA_ONLY	3

/*
 *  Pixel format sizes in bytes
 */

#define VINO_RGBA_SIZE		4
#define VINO_RGB_332_SIZE	1
#define VINO_YUV_SIZE		2
#define VINO_YUV_LUMA_ONLY_SIZE	1

/*
 *  VINO_SETUP_GET_FRAME ioctl arguments
 */

typedef struct {
	int	mem_node;	/* mem node number */
	int	vid_node;	/* video node */
	char   *buffer;		/* pointer to user data buffer */

	int     buflen;		/* length of buffer in bytes */
	int     numframes;	/* number of fields */
	int     decim_sf;	/* one of the above factors */
	int	decim_x;	/* decim in x direction only */
	int	frame_rate;	/* frame rate for continuous capture */
	int	frames_per_sec;	/* frame rate for continuous capture */
	int	frame_count;	/* requested frame count */
	int	frame_len;	/* length in bytes of a frame */

	int	timing;		/* ntsc or pal timing */
	int     pix_format;	/* pixel format conversion control */
	int     alpha;		/* alpha value for rgba pixels */

	int     mode;		/* dma transfer mode bits */
	int     left;		/* Not implemented yet */
	int     right;		/* Not implemented yet */
	int	line_size;	/* for h/w interleave */
	
	/*
	*  Note that top_even and top_odd must be within
	*  1 scan line of each other or the call is rejected.
	*/

	int     top_even;	/* top scan line - even field */
	int     top_odd;	/* top scan line - odd field */
	int     bottom_even;	/* bottom scan line - even field */
	int     bottom_odd;	/* bottom scan line - odd field */

	int	fifo_thres;	/* setting for the fifo threshold */
	int	key;

	int	media_offset;	/* TOC entry offsets */
	int	image_offset;

} VinoSetupGetFrameArgs;

/****************************************************
 *
 *  VINO_GET_FRAME ioctl
 *
 *  The following defines and structures are used
 *  to tell vino to do get frame operations.
 *
 *****************************************************/

typedef struct {
    int     mem_node;		/* mem node number */
    char   *buffer;		/* pointer to user data buffer */
} VinoGetFrameArgs;

/****************************************************
 *
 *  VINO_FINISH_GET_FRAME ioctl
 *
 *  The following defines and structures are used
 *  to tell vino to do finish get frame operations.
 *
 *  The argument to VINO_FINISH_GET_FRAME is an
 *  int specifying the mem_node.
 *
 *****************************************************/

/****************************************************
 *
 *  VINO_SET_POLL_SEL ioctl
 *  VINO_GET_POLL_SEL ioctl
 *
 *  The following defines and structures are used
 *  to specify which events are pollable.
 *
 *****************************************************/

#define A_EVEN_VID_RETRACE	(1 << 0) /* ch A even field starting */
#define B_EVEN_VID_RETRACE	(1 << 1) /* ch B even field starting */

#define A_ODD_VID_RETRACE	(1 << 2) /* ch A odd field starting */
#define B_ODD_VID_RETRACE	(1 << 3) /* ch B odd field starting */

#define A_DROP_FIELD		(1 << 4) /* ch A dropped a field */
#define B_DROP_FIELD		(1 << 5) /* ch B dropped a field */

#define A_TRANSFER_EVENT	(1 << 6) /* ch A transfer complete */
#define B_TRANSFER_EVENT	(1 << 7) /* ch A transfer complete */

#define A_TRANSFER_ABORTED	(1 << 8) /* ch A transfer aborted */
#define B_TRANSFER_ABORTED	(1 << 9) /* ch A transfer aborted */


#define DIGITAL_VIDEO_SYNC_LOST		(1 << 12) /* IndyCam unplugged */
#define COMPOSITE_VIDEO_SYNC_LOST	(1 << 13) /* HPLL unlocked in 7191 */
#define SVIDEO_VIDEO_SYNC_LOST		(1 << 14) /* HPLL unlocked in 7191 */
#define	SYNC_LOST_MESSAGE		(1 << 15)

#define	SYNC_LOST		(DIGITAL_VIDEO_SYNC_LOST|\
				 COMPOSITE_VIDEO_SYNC_LOST|\
				 SVIDEO_VIDEO_SYNC_LOST)

#define A_RETRACE_EVENT		(A_EVEN_VID_RETRACE | A_ODD_VID_RETRACE)
#define B_RETRACE_EVENT		(B_EVEN_VID_RETRACE | B_ODD_VID_RETRACE)

#define RETRACE_EVENT		(A_RETRACE_EVENT | B_RETRACE_EVENT)
#define DROP_FIELD		(A_DROP_FIELD | B_DROP_FIELD)
#define TRANSFER_EVENT		(A_TRANSFER_EVENT | B_TRANSFER_EVENT)
#define TRANSFER_ABORTED	(A_TRANSFER_ABORTED | B_TRANSFER_ABORTED)

#define	A_EVENTS	(A_RETRACE_EVENT|A_DROP_FIELD|A_TRANSFER_EVENT|\
			 A_TRANSFER_ABORTED|SYNC_LOST)

#define	B_EVENTS	(B_RETRACE_EVENT|B_DROP_FIELD|B_TRANSFER_EVENT|\
			 B_TRANSFER_ABORTED|SYNC_LOST)

#define	SINGLE_EVENTS	(DROP_FIELD|TRANSFER_ABORTED|SYNC_LOST)

typedef struct {
    int cur_mask;
    int cur_event;
} VinoPollSelArgs;

/********************************************************************
 *
 *  VINO_GET_REG ioctl and VINO_SET_REG ioctl arguments 
 *
 *  Register/value pair structure.
 *
 ********************************************************************/

typedef struct {
        int reg;
        int val;
} RegVal;

/**************************************************************************
 *
 *  Vino register definitions used with V_GET_REG and V_SET_REG ioctls.
 *
 **************************************************************************/

/*
 * Vino ASIC registers.
 */

#define V_VINO_REG_BASE			0

#define V_VINO_REG_REV_ID		V_VINO_REG_BASE + 0
#define V_VINO_REG_CONTROL		V_VINO_REG_BASE + 1
#define V_VINO_REG_INTR_STATUS		V_VINO_REG_BASE + 2
#define V_VINO_REG_I2C_CONTROL		V_VINO_REG_BASE + 3
#define V_VINO_REG_I2C_DATA		V_VINO_REG_BASE + 4

#define V_VINO_REG_A_ALPHA		V_VINO_REG_BASE + 5
#define V_VINO_REG_A_CLIP_START		V_VINO_REG_BASE + 6
#define V_VINO_REG_A_CLIP_END		V_VINO_REG_BASE + 7
#define V_VINO_REG_A_FRAME_RATE		V_VINO_REG_BASE + 8
#define V_VINO_REG_A_FIELD_CNTR		V_VINO_REG_BASE + 9
#define V_VINO_REG_A_LINE_SIZE		V_VINO_REG_BASE + 10
#define V_VINO_REG_A_LINE_COUNT		V_VINO_REG_BASE + 11
#define V_VINO_REG_A_PAGE_INDEX		V_VINO_REG_BASE + 12
#define V_VINO_REG_A_NEXT_4_DESC	V_VINO_REG_BASE + 13
#define V_VINO_REG_A_DESC_TBL_PTR	V_VINO_REG_BASE + 14
#define V_VINO_REG_A_DESC_0		V_VINO_REG_BASE + 15
#define V_VINO_REG_A_DESC_1		V_VINO_REG_BASE + 16
#define V_VINO_REG_A_DESC_2		V_VINO_REG_BASE + 17
#define V_VINO_REG_A_DESC_3		V_VINO_REG_BASE + 18
#define V_VINO_REG_A_FIFO_THRSHLD	V_VINO_REG_BASE + 19
#define V_VINO_REG_A_FIFO_READ		V_VINO_REG_BASE + 20
#define V_VINO_REG_A_FIFO_WRITE		V_VINO_REG_BASE + 21

#define V_VINO_REG_B_ALPHA		V_VINO_REG_BASE + 22
#define V_VINO_REG_B_CLIP_START		V_VINO_REG_BASE + 23
#define V_VINO_REG_B_CLIP_END		V_VINO_REG_BASE + 24
#define V_VINO_REG_B_FRAME_RATE		V_VINO_REG_BASE + 25
#define V_VINO_REG_B_FIELD_CNTR		V_VINO_REG_BASE + 26
#define V_VINO_REG_B_LINE_SIZE		V_VINO_REG_BASE + 27
#define V_VINO_REG_B_LINE_COUNT		V_VINO_REG_BASE + 28
#define V_VINO_REG_B_PAGE_INDEX		V_VINO_REG_BASE + 29
#define V_VINO_REG_B_NEXT_4_DESC	V_VINO_REG_BASE + 30
#define V_VINO_REG_B_DESC_TBL_PTR	V_VINO_REG_BASE + 31
#define V_VINO_REG_B_DESC_0		V_VINO_REG_BASE + 32
#define V_VINO_REG_B_DESC_1		V_VINO_REG_BASE + 33
#define V_VINO_REG_B_DESC_2		V_VINO_REG_BASE + 34
#define V_VINO_REG_B_DESC_3		V_VINO_REG_BASE + 35
#define V_VINO_REG_B_FIFO_THRSHLD	V_VINO_REG_BASE + 36
#define V_VINO_REG_B_FIFO_READ		V_VINO_REG_BASE + 37
#define V_VINO_REG_B_FIFO_WRITE		V_VINO_REG_BASE + 38

/*
 *  7191 DMSD registers.
 */

#define V_DMSD_REG_BASE			V_VINO_REG_BASE + 39

#define V_DMSD_REG_IDEL                 V_DMSD_REG_BASE + 0
#define V_DMSD_REG_HSYB			V_DMSD_REG_BASE + 1
#define V_DMSD_REG_HSYS			V_DMSD_REG_BASE + 2
#define V_DMSD_REG_HCLB			V_DMSD_REG_BASE + 3
#define V_DMSD_REG_HCLS			V_DMSD_REG_BASE + 4
#define V_DMSD_REG_HPHI			V_DMSD_REG_BASE + 5
#define V_DMSD_REG_LUMA                 V_DMSD_REG_BASE + 6
#define V_DMSD_REG_HUEC			V_DMSD_REG_BASE + 7
#define V_DMSD_REG_CKTQ			V_DMSD_REG_BASE + 8
#define V_DMSD_REG_CKTS			V_DMSD_REG_BASE + 9
#define V_DMSD_REG_PLSE			V_DMSD_REG_BASE + 10
#define V_DMSD_REG_SESE			V_DMSD_REG_BASE + 11
#define V_DMSD_REG_GAIN			V_DMSD_REG_BASE + 12
#define V_DMSD_REG_STDC			V_DMSD_REG_BASE + 13
#define V_DMSD_REG_IOCK			V_DMSD_REG_BASE + 14
#define V_DMSD_REG_CTL3			V_DMSD_REG_BASE + 15
#define V_DMSD_REG_CTL4			V_DMSD_REG_BASE + 16
#define V_DMSD_REG_CHCV                 V_DMSD_REG_BASE + 17
#define V_DMSD_REG_HS6B			V_DMSD_REG_BASE + 18
#define V_DMSD_REG_HS6S			V_DMSD_REG_BASE + 19
#define V_DMSD_REG_HC6B			V_DMSD_REG_BASE + 20
#define V_DMSD_REG_HC6S			V_DMSD_REG_BASE + 21
#define V_DMSD_REG_HP6I			V_DMSD_REG_BASE + 22

/*
 *  Camera controller registers.
 */

#define V_INDYCAM_REG_BASE		V_DMSD_REG_BASE + 23

#define V_INDYCAM_REG_CTRL_STATUS	V_INDYCAM_REG_BASE + 0
#define V_INDYCAM_REG_SHUTTER_SPEED	V_INDYCAM_REG_BASE + 1
#define V_INDYCAM_REG_GAIN		V_INDYCAM_REG_BASE + 2
#define V_INDYCAM_REG_BRIGHTNESS	V_INDYCAM_REG_BASE + 3
#define V_INDYCAM_REG_RED_BAL		V_INDYCAM_REG_BASE + 4
#define V_INDYCAM_REG_BLUE_BAL		V_INDYCAM_REG_BASE + 5
#define V_INDYCAM_REG_RED_SAT		V_INDYCAM_REG_BASE + 6
#define V_INDYCAM_REG_BLUE_SAT		V_INDYCAM_REG_BASE + 7
#define V_INDYCAM_REG_VERSION		V_INDYCAM_REG_BASE + 8
#define V_INDYCAM_REG_MASTER_RESET	V_INDYCAM_REG_BASE + 9

/*
 *  The V_MAX_REG must have a value equal to the highest 
 *  numbered register.
 */

#define V_MAX_REG			V_INDYCAM_REG_BASE + 9

/*********************************************************************
 *
 *  VINO_SET_CNTRL ioctl
 *  VINO_GET_CNTRL ioctl
 *
 *  Structures and definitions used with the VINO_SET_CNTRL ioctl
 *  and the VINO_SET_CNTRL ioctl.  On set control the value is the
 *  value to be set.  On get control the value is the current value
 *  of the control.
 *
 *********************************************************************/

typedef struct {
    int type;
    int value;
    int node_num;
    int addr;
    int offset;
} VinoControlArgs;

/*
 *  Device Independent Controls
 */

#define V_BLEND        0
#define V_CHROMAKEY    1
#define V_BRIGHTNESS   2
#define V_CONTRAST     3
#define V_H_PHASE      4
#define V_HUE          5
#define V_RED_SETUP    6
#define V_GRN_SETUP    7
#define V_BLU_SETUP    8
#define V_V_PHASE      9
#define V_SIGNAL       10
#define V_FREEZE       11
#define V_CHROMA_TRAP  12
/*
 *  numbers 13 and 14 are not used
 */
#define V_MUXSWITCH    15
#define V_FORMAT       16
#define V_TIMING       17
#define V_SYNC         18
#define V_SYNC_DELAY   19
#define V_ORIGIN       20
#define V_OFFSET       21
#define V_SIZE         22

/* number 23 unused. */

#define V_I2CREG       24
#define V_COUNT        25
#define V_RATE         26
#define V_ADDRESS      27
#define V_CAP_TYPE     28
#define V_WINDOW       29
#define V_PACKING      30
#define V_ALPHAVALUE   31
#define	V_COLOR_MODE   32

#define V_NUM_CONTROLS 32

/*
 *  V_MUXSWITCH control type definitions
 *
 * (These *must* match the MUXSWITCH inputs in dev_vino.h)
 */

/* analog switch */
#define	V_COMPOSITE		0
#define	V_SVIDEO		1
#define	V_MUXSWITCH_MAX		V_SVIDEO

/* digital "logical" switch */
/* INDYCAM input selected restricts timing, etc. */
#define	V_INDYCAM		0
#define	V_CCIR601		1
#define	V_DMUXSWITCH_MAX	V_CCIR601

/*
 * COLOR MODE types
 * (These *must* match VL_VINO_COLOR_MODE_XXX entries dev_vino.h)
 */
#define	V_COLOR_MODE_AUTO	0
#define	V_COLOR_MODE_COLOR	1
#define	V_COLOR_MODE_MONO	2

/*
 * Special Value Constant for V_I2CREG operation.
 * (initiates I2C register read operation - coordinate value in vl/dev_vino.h)
 */
#define	V_I2CREG_READ		0xf1f2

/* IndyCam controls */

#define	V_INDYCAM_BASE			100
#define V_INDYCAM_AGCENA		V_INDYCAM_BASE+0
#define V_INDYCAM_AWBCTL		V_INDYCAM_BASE+1
#define V_INDYCAM_SHUTTER		V_INDYCAM_BASE+2
#define V_INDYCAM_SHUTTER_SNAP		V_INDYCAM_BASE+3
#define V_INDYCAM_BRIGHTNESS		V_INDYCAM_BASE+4
#define V_INDYCAM_GAIN			V_INDYCAM_BASE+5
#define V_INDYCAM_RED_BALANCE		V_INDYCAM_BASE+6
#define V_INDYCAM_BLUE_BALANCE		V_INDYCAM_BASE+7
#define V_INDYCAM_RED_SATURATION	V_INDYCAM_BASE+8
#define V_INDYCAM_BLUE_SATURATION	V_INDYCAM_BASE+9

/*
 *  VINO 7191 device dependent control definitions.
 */

#define	V_VINO_DMSD_BASE	200
#define V_VINO_IDEL		V_VINO_DMSD_BASE+0
#define V_VINO_HSYB		V_VINO_DMSD_BASE+1
#define V_VINO_HSYS		V_VINO_DMSD_BASE+2
#define V_VINO_HCLB		V_VINO_DMSD_BASE+3
#define V_VINO_HCLS		V_VINO_DMSD_BASE+4
#define V_VINO_HPHI		V_VINO_DMSD_BASE+5
#define V_VINO_BYPS		V_VINO_DMSD_BASE+6
#define V_VINO_PREF		V_VINO_DMSD_BASE+7
#define V_VINO_BPSS		V_VINO_DMSD_BASE+8
#define V_VINO_CORI		V_VINO_DMSD_BASE+9
#define V_VINO_APER		V_VINO_DMSD_BASE+10
#define V_VINO_HUEC		V_VINO_DMSD_BASE+11
#define V_VINO_CKTQ		V_VINO_DMSD_BASE+12
#define V_VINO_CKTS		V_VINO_DMSD_BASE+13
#define V_VINO_PLSE		V_VINO_DMSD_BASE+14
#define V_VINO_SESE		V_VINO_DMSD_BASE+15
#define V_VINO_COLO		V_VINO_DMSD_BASE+16
#define V_VINO_LFIS		V_VINO_DMSD_BASE+17
#define V_VINO_VTRC		V_VINO_DMSD_BASE+18
#define V_VINO_NFEN		V_VINO_DMSD_BASE+19
#define V_VINO_HRMV		V_VINO_DMSD_BASE+20
#define V_VINO_SECS		V_VINO_DMSD_BASE+21
#define V_VINO_HPLL		V_VINO_DMSD_BASE+22
#define V_VINO_CHRS		V_VINO_DMSD_BASE+23
#define V_VINO_GPSW		V_VINO_DMSD_BASE+24
#define V_VINO_AUFD		V_VINO_DMSD_BASE+25
#define V_VINO_FSEL		V_VINO_DMSD_BASE+26
#define V_VINO_SXCR		V_VINO_DMSD_BASE+27
#define V_VINO_SCEN		V_VINO_DMSD_BASE+28
#define V_VINO_YDEL		V_VINO_DMSD_BASE+29
#define V_VINO_VNOI		V_VINO_DMSD_BASE+30
#define V_VINO_CHCV		V_VINO_DMSD_BASE+31
#define V_VINO_HS6B		V_VINO_DMSD_BASE+32
#define V_VINO_HS6S		V_VINO_DMSD_BASE+33
#define V_VINO_HC6B		V_VINO_DMSD_BASE+34
#define V_VINO_HC6S		V_VINO_DMSD_BASE+35
#define V_VINO_HP6I		V_VINO_DMSD_BASE+36

#define V_DD_CTL_VINO_MAX	36

/********************************************************************
 *
 *  VINO_GET_FRONTIER_MSC ioctl
 *
 ********************************************************************/

#include "sys/dmcommon.h"

typedef struct vinoGetFrontierMSCArgs
{
  int mem_node;
  stamp_t *return_ptr;
  int is_conversion;
  unsigned int cvt_buffer_sender_id;
} vinoGetFrontierMSCArgs;


/********************************************************************
 *
 *  VINO_GET_UST_MSC_PAIR ioctl
 *
 ********************************************************************/

typedef struct vinoGetUSTMSCPairArgs
{
  int vid_node;
  int mem_node;
  USTMSCpair *return_ptr;
} vinoGetUSTMSCPairArgs;


/****************************************************
 *
 *  VINO_GET_STATUS ioctl
 *
 *  This ioctl returns the current vino driver and
 *  hardware status.
 *
 *****************************************************/

typedef struct {
    char composite_present;
    char composite_timing;
    char svideo_present;
    char svideo_timing;
    char indycam_present;
} VinoGetStatusArgs;

/*
 * VinoInfo.flags bits
 */
#define	V_BUG_DOASSERT		(1<<4)	/* vino assert's cause panic */
#define	V_BUG_DOINTSTATLOG	(1<<5)	/* keep log of interrupt states */

#define	V_BUG_DAPJEOQWAR	(1<<8)	/* need jump at end of each quad */
#define	V_BUG_DAPJEOPWAR	(1<<9)	/* need jump at end of each page */
#define	V_BUG_DAPNREGBAD	(1<<10)	/* next4 reg bad, fill extra dap pages*/
#define	V_BUG_NOCLIPRIGHT	(1<<11)	/* can't clip from right */

#define	V_BUG_DECIMBAD		(1<<12)	/* decimation below 1/3 doesn't work */
#define	V_BUG_UPDATESTARTREG	(1<<13)	/* start reg doesn't update properly */
#define	V_BUG_RATE_MIN_5	(1<<14)	/* rate minimum of 5fps */

#ifdef VINO_REG_DEFINES_OBSOLETE

#define VINO_DD_CTL_BASE	400

/************************************************************************
 *
 * Phillips 7191 DMSD - dd controls
 *
 * Refer to the Phillips Data Book for specific meaning of the
 * following device dependent controls.  The controls are set in
 * the 7191 register unchanged.  For registers containing multiple
 * controls, the control value are shifted as appropriate to be or-ed 
 * into the registers.
 *
 ************************************************************************/

/*
 *  Increment Delay register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_IDEL		(VINO_DD_CTL_BASE + 0)

/*
 *  H sync begin, 50 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HSYB		(VINO_DD_CTL_BASE + 1)

/*
 *  H sync stop, 50 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HSYS		(VINO_DD_CTL_BASE + 2)

/*
 *  H clamp begin, 50 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HCLB		(VINO_DD_CTL_BASE + 3)

/*
 *  H clamp stop, 50 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HCLS		(VINO_DD_CTL_BASE + 4)

/*
 *  H sync after PHI1, 50 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HPHI		(VINO_DD_CTL_BASE + 5)

/*
 *  Luminance Control register
 *
 *  BYPS
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  PREF
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  BPSS
 *   type: int
 *   values: 0 - 3
 *
 *  CORI
 *   type: int
 *   values: 0 - 3
 *
 *  APER
 *   type: int
 *   values: 0 - 3
 */

#define VL_VINO_BYPS		(VINO_DD_CTL_BASE + 6)
#define VL_VINO_PREF		(VINO_DD_CTL_BASE + 7)
#define VL_VINO_BPSS		(VINO_DD_CTL_BASE + 8)
#define VL_VINO_CORI		(VINO_DD_CTL_BASE + 9)
#define VL_VINO_APER		(VINO_DD_CTL_BASE + 10)

/*
 *  Hue control register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HUEC		(VINO_DD_CTL_BASE + 11)

/* 
 *  Colour-killer threshold QAM register
 *
 *  type:   int
 *  values: 0x00 - 0x1F
 */

#define VL_VINO_CKTQ		(VINO_DD_CTL_BASE + 12)

/*
 *  Colour-killer threshold SECAM register
 *
 *  type:   int
 *  values: 0x00 - 0x1F
 */

#define VL_VINO_CKTS		(VINO_DD_CTL_BASE + 13)

/*
 *  PAL switch sensitivity register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_PLSE		(VINO_DD_CTL_BASE + 14)

/*
 *  SECAM switch sensitivity register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_SESE		(VINO_DD_CTL_BASE + 15)

/*
 *  Gain control register
 *
 *  COLO
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  GAIN
 *   type: int
 *   values: 0 - 3
 */

#define VL_VINO_COLO		(VINO_DD_CTL_BASE + 16)
#define VL_VINO_GAIN		(VINO_DD_CTL_BASE + 17)

/*
 *  Standard/mode control register
 *
 *  VTRC
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  NFEN
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  HRMV
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  SECS
 *   type: VLBoolean
 *   values: 0 - 1
 */

#define VL_VINO_VTRC		(VINO_DD_CTL_BASE + 18)
#define VL_VINO_NFEN		(VINO_DD_CTL_BASE + 19)
#define VL_VINO_HRMV		(VINO_DD_CTL_BASE + 20)
#define VL_VINO_SECS		(VINO_DD_CTL_BASE + 21)

/*
 *  I/O and clock control register
 *
 *  HPLL
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  CHRS
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  GPSW
 *   type: int
 *   values: 0 - CVBS
 *	     2 - SVIDEO
 */

#define VL_VINO_HPLL		(VINO_DD_CTL_BASE + 22)
#define VL_VINO_CHRS		(VINO_DD_CTL_BASE + 23)
#define VL_VINO_GPSW		(VINO_DD_CTL_BASE + 24)

/*
 *  Control #3 register
 *
 *  AUFD
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  FSEL
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  SXCR
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  SCEN
 *   type: VLBoolean
 *   values: 0 - 1
 *
 *  YDEL
 *   type: int
 *   values: 0 - 7
 */

#define VL_VINO_AUFD		(VINO_DD_CTL_BASE + 25)
#define VL_VINO_FSEL		(VINO_DD_CTL_BASE + 26)
#define VL_VINO_SXCR		(VINO_DD_CTL_BASE + 27)
#define VL_VINO_SCEN		(VINO_DD_CTL_BASE + 28)
#define VL_VINO_YDEL		(VINO_DD_CTL_BASE + 29)

/*
 *  Control #4 register
 *
 *  VNOI
 *   type: int
 *   values: 0 - 3
 */

#define VL_VINO_VNOI		(VINO_DD_CTL_BASE + 30)

/*
 *  Colour control register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_CHCV		(VINO_DD_CTL_BASE + 31)

/*
 *  H sync begin, 60 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HS6B		(VINO_DD_CTL_BASE + 32)

/*
 *  H sync end, 60 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HS6S		(VINO_DD_CTL_BASE + 33)

/*
 *  H clamp begin, 60 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HC6B		(VINO_DD_CTL_BASE + 34)

/*
 *  H clamp end, 60 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HC6S		(VINO_DD_CTL_BASE + 35)

/*
 *  H sync after PHI1, 60 Hz register
 *
 *  type:   int
 *  values: 0x00 - 0xFF
 */

#define VL_VINO_HP6I		(VINO_DD_CTL_BASE + 36)
#endif

#endif	/* IOCTLS_ONLY */
#endif /* __SYS_VINO_H__ */
