#ifndef __SYS_XLV_LOCK_H__
#define __SYS_XLV_LOCK_H__

/**************************************************************************
 *                                                                        *
 *            Copyright (C) 1993-1994, Silicon Graphics, Inc.             *
 *                                                                        *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *                                                                        *
 **************************************************************************/
#ident "$Revision: 1.2 $"

#ifdef _KERNEL

/*
 * The xlv_io_lock[] is acquired in access mode whenever an I/O is
 * started on a subvolume. It is released after the I/O (and any
 * intermediate error recoveries) are done.
 * The xlv_io_lock[] must be acquired in update mode and xlv_config_lock[]
 * acquired in access mode if you want to block new I/O and block
 * configuration changes.
 * Both xlv_io_lock[] and xlv_config_lock[] must be acquired in update
 * mode if you want to change the current subvolume configuration.
 * There is a locking protocol that must be followed to do this.
 *
 * The xlv_io_lock[] and xlv_config_lock[] are statically allocated.
 * They must have at least the same number of entries as the xlv_tab.
 */
typedef struct {
	int		xlvd_lock_count;
	mrlock_t	lock;
} xlv_lock_entry_t;

extern xlv_lock_entry_t xlv_io_lock[];
extern xlv_lock_entry_t	xlv_config_lock[];



/* PZERO is a non-breakable priority, thus we will never be interrupted
   from a lock-wait. */
#define XLV_IO_LOCK(entry, access) \
	(mrlock (&(xlv_io_lock[entry].lock), access, PZERO))

#define XLV_IO_UNLOCK(entry) (mrunlock (&(xlv_io_lock[entry].lock)))

#endif /* KERNEL */

#endif /* SYS_XLV_LOCK_H__ */
