#ifndef __SYS_XMAP9_H__
#define __SYS_XMAP9_H__
/**************************************************************************
 *               Copyright (C) 1990, Silicon Graphics, Inc.               *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 **************************************************************************/


/*
 *   $Revision: 1.29 $
 */

#include "sys/ng1hw.h"

/*****************************************
 * 		XMAP9 definitions and macros
 *****************************************/

/*
 * These bits go into the DCBCRS field of the Rex DCBMODE register
 */


#define XM9_CRS_CONFIG 		(0 << DCB_CRS_SHIFT)
#define XM9_CRS_REVISION	(1 << DCB_CRS_SHIFT)
#define XM9_CRS_FIFO_AVAIL	(2 << DCB_CRS_SHIFT)
#define XM9_CRS_CURS_CMAP_MSB	(3 << DCB_CRS_SHIFT)
#define XM9_CRS_PUP_CMAP_MSB 	(4 << DCB_CRS_SHIFT)
#define XM9_CRS_MODE_REG_DATA	(5 << DCB_CRS_SHIFT)
#define XM9_CRS_MODE_REG_INDEX	(7 << DCB_CRS_SHIFT)

/*
 * XM9_CONFIG register bits
 */

#define XM9_PUPMODE             BIT(0)
#define XM9_ODD_PIXEL           BIT(1)
#define XM9_8_BITPLANES         BIT(2)
#define XM9_VIDEO_RGBMAP_SHIFT  4
#define XM9_VIDEO_RGBMAP_MASK   (0x3 << XM9_VIDEO_RGBMAP_SHIFT)
#define XM9_EXPRESS_VIDEO       BIT(6)
#define XM9_VIDEO_OPTION        BIT(7)

/*
 * XM9_MODE register bits
 */

#define XM9_BUF_SEL             BIT(0)
#define XM9_OVL_BUF_SEL         BIT(1)
#define XM9_GAMMA_BYPASS        BIT(2)
#define XM9_MSB_CMAP_SHIFT      3
#define XM9_MSB_CMAP_MASK       (0x1f << XM9_MSB_CMAP_SHIFT)

#define XM9_PIX_MODE_SHIFT      8
#define XM9_PIX_SIZE_SHIFT      10
#define XM9_VIDEO_MODE_SHIFT    12

#define XM9_PIX_MODE_MASK       (0x3 << XM9_PIX_MODE_SHIFT)
#define XM9_PIX_SIZE_MASK       (0x3 << XM9_PIX_SIZE_SHIFT)
#define XM9_VIDEO_MODE_MASK     (0x3 << XM9_VIDEO_MODE_SHIFT)
#define XM9_VIDEO_DITHER_BYPASS BIT(14)
#define XM9_ALPHA_ENABLE        BIT(15)
#define XM9_AUX_PIX_MODE_SHIFT  16
#define XM9_AUX_PIX_MODE_MASK   (0x7 << XM9_AUX_PIX_MODE_SHIFT)
#define XM9_AUX_MSB_CMAP_SHIFT  19
#define XM9_AUX_MSB_CMAP_MASK   (0x1f << XM9_AUX_MSB_CMAP_SHIFT)
#define XM9_PIXSIZE_4		(0 << XM9_PIX_SIZE_SHIFT)
#define XM9_PIXSIZE_8		(1 << XM9_PIX_SIZE_SHIFT)
#define XM9_PIXSIZE_12		(2 << XM9_PIX_SIZE_SHIFT)
#define XM9_PIXSIZE_24		(3 << XM9_PIX_SIZE_SHIFT)

/*
 * XM9_MODE register field value defines
 */

#define XM9_PIX_BUF1            1
#define XM9_AUX_MODE_OLAY       ( 2 << XM9_AUX_PIX_MODE_SHIFT )
#define XM9_AUX_MODE_ULAY       ( 1 << XM9_AUX_PIX_MODE_SHIFT )

/*
 * XM9_FIFO_AVAIL register bits
 */

#define XM9_FIFO_0_AVAIL 0
#define XM9_FIFO_1_AVAIL 1
#define XM9_FIFO_2_AVAIL 3
#define XM9_FIFO_3_AVAIL 2
#define XM9_FIFO_FULL	 XM9_FIFO_0_AVAIL
#define XM9_FIFO_EMPTY	 XM9_FIFO_3_AVAIL

/*
 * Define the dcb transfer protocol used by the xmap9.
 * XXX We may need to program the DCBMODE CS{WIDTH, HOLD, SETUP} bits
 * (See Rex3 spec, section 3.13)
 */

#define W_DCB_XMAP9_PROTOCOL ( (2 << DCB_CSSETUP_SHIFT) |    \
                          (1 << DCB_CSHOLD_SHIFT) | (0 << DCB_CSWIDTH_SHIFT))

#define WSLOW_DCB_XMAP9_PROTOCOL ( (5 << DCB_CSSETUP_SHIFT) |    \
                          (5 << DCB_CSHOLD_SHIFT) | (0 << DCB_CSWIDTH_SHIFT))

#define WAYSLOW_DCB_XMAP9_PROTOCOL ( (12 << DCB_CSSETUP_SHIFT) |    \
                          (12 << DCB_CSHOLD_SHIFT) | (0 << DCB_CSWIDTH_SHIFT))

#define R_DCB_XMAP9_PROTOCOL ( (2 << DCB_CSSETUP_SHIFT) |    \
                          (1 << DCB_CSHOLD_SHIFT) | (3 << DCB_CSWIDTH_SHIFT))

	/*
	 * Macros to read the various XMAP9 registers
	 */
/*
 * Ex: config = xmapConfig(rex);
 */

#define xmapConfig(rex)							   \
        (rex->set.dcbmode = DCB_XMAP0 | XM9_CRS_CONFIG | DCB_DATAWIDTH_1 | \
                        R_DCB_XMAP9_PROTOCOL,                                \
        (rex->set.dcbdata0.bybyte.b3 & 0xff))

/*
 * Ex: xrev = xmapRevision(rex);
 */

#define xmapRevision(rex)						     \
        (rex->set.dcbmode = DCB_XMAP0 | XM9_CRS_REVISION | DCB_DATAWIDTH_1 | \
                        R_DCB_XMAP9_PROTOCOL,                                  \
        (rex->set.dcbdata0.bybyte.b3 & 7))


/*
 * XXX We just check one xmap now - which one is the best to test ?
 *
 * Ex:		if (xmap9FifoFull(rex)) 
 *				...
 */

#define xmap9FifoFull(rex)						\
        (rex->set.dcbmode = DCB_XMAP0 | XM9_CRS_FIFO_AVAIL |		\
			DCB_DATAWIDTH_1 | R_DCB_XMAP9_PROTOCOL,	\
		(rex->set.dcbdata0.bybyte.b3 & 3) == XM9_FIFO_FULL)


#define xmap9FIFOWait(rex)                                             \
	{                                                               \
        rex->set.dcbmode = DCB_XMAP0 | XM9_CRS_FIFO_AVAIL |             \
			DCB_DATAWIDTH_1 | R_DCB_XMAP9_PROTOCOL;           \
	BFIFOWAIT( rex );                                              \
	while ( (rex->set.dcbdata0.bybyte.b3 & 3) != XM9_FIFO_EMPTY  ) ; \
	}

	/*
	 * Macros to load the various XMAP9 registers
	 */

/*
 * Set up the xmap9 config register.
 * This loads both xmaps, setting xmap 0 to
 * handle 'even' pixels, xmap 1 to handle odd.
 */

#define xmap9LoadConfig(rex,data) 					    \
        rex->set.dcbmode =  DCB_XMAP0 |  XM9_CRS_CONFIG | DCB_DATAWIDTH_1 | \
                        W_DCB_XMAP9_PROTOCOL; 				    \
        rex->set.dcbdata0.bybyte.b3 = ((data) & ~ XM9_ODD_PIXEL);	    \
        rex->set.dcbmode =  DCB_XMAP1 |  XM9_CRS_CONFIG | DCB_DATAWIDTH_1 | \
                        W_DCB_XMAP9_PROTOCOL; 				    \
        rex->set.dcbdata0.bybyte.b3 = ((data) | XM9_ODD_PIXEL)


/*
 *  Write to any xmap9 register other than mode table.
 */

#define xmap9SetReg( rex, x9reg, data ) \
	rex->set.dcbmode = DCB_XMAP_ALL | W_DCB_XMAP9_PROTOCOL | x9reg |  \
                                                     DCB_DATAWIDTH_1;   \
        rex->set.dcbdata0.bybyte.b3 = (data)

/*
 * Load mode register(s)
 */

#define xmap9SetModeReg(rex,modereg,data24,cfreq) 			\
	if (cfreq > 119 )						\
	    rex->set.dcbmode =  DCB_XMAP_ALL |  XM9_CRS_MODE_REG_DATA | \
                        DCB_DATAWIDTH_4 | W_DCB_XMAP9_PROTOCOL;	    	\
	else if (cfreq > 59) \
	    rex->set.dcbmode =  DCB_XMAP_ALL |  XM9_CRS_MODE_REG_DATA | \
                        DCB_DATAWIDTH_4 | WSLOW_DCB_XMAP9_PROTOCOL;    	\
	else \
	    rex->set.dcbmode =  DCB_XMAP_ALL |  XM9_CRS_MODE_REG_DATA | \
                        DCB_DATAWIDTH_4 | WAYSLOW_DCB_XMAP9_PROTOCOL;  	\
        rex->set.dcbdata0.byword = ((modereg) << 24) | (data24 & 0xffffff)


/*
 * Some macros for diagnostics.
 */

#define xmap9GetReg(rex, reg, which,data)				\
        rex->set.dcbmode =  ((which) ? DCB_XMAP1 : DCB_XMAP0) |  	\
		reg | DCB_DATAWIDTH_1 | R_DCB_XMAP9_PROTOCOL ;		\
        data = rex->set.dcbdata0.bybyte.b3

#define xmap9GetMode(rex, reg, which, data)				\
	{ int _r = (reg) << 2;						\
	  int _which = ((which) ? DCB_XMAP1 : DCB_XMAP0);		\
	  data = getmodebyte(rex, _r, _which); _r++;			\
	  data |= getmodebyte(rex, _r, _which) << 8; _r++;		\
	  data |= getmodebyte(rex, _r, _which) << 16;			\
	}

#define getmodebyte(rex, _r, _which)					\
        (rex->set.dcbmode =  _which | XM9_CRS_MODE_REG_INDEX |		\
				DCB_DATAWIDTH_1 | W_DCB_XMAP9_PROTOCOL, \
	rex->set.dcbdata0.bybyte.b3 = _r,				\
        rex->set.dcbmode =  _which | XM9_CRS_MODE_REG_DATA |		\
			     DCB_DATAWIDTH_1 | R_DCB_XMAP9_PROTOCOL,	\
	rex->set.dcbdata0.bybyte.b3)

#endif /* __SYS_XMAP9_H__ */
